#pragma once

#include "core/plot_core.h"

double max(const QVector<double> &data);
double min(const QVector<double> &data);
QPair<double, double> minmax(const QVector<double> &data);
double rms(const QVector<double> &data);
double avg(const QVector<double> &data);

namespace plot {

const QPen DefaultPenForInactiveCurves(QColor(220, 220, 220), 1, Qt::DashDotLine);



class TCurveData
{
public:
    QVector<double> XSeq;
    QVector<double> YSeq;
    QPen Pen;
    bool isVerticaled;

    TCurveData();
    void detach();
    void clear();
    void AddY(double y);
    void AddPoint(double x, double y);

    int size() const;
    bool isCompressedXCurve() const;
    bool isCompressedYCurve() const;
    void setXCalib(double x0, double dx);
    double getXat(int i) const;
    double getYat(int i) const;

    ~TCurveData();

    static TCurveData* clone(TCurveData *orig, QPen *pen = nullptr);
    TCurveData* clone(QPen *pen = nullptr);
};

typedef QVector<QWeakPointer<TCurveData> > TCurvesData;
typedef QVector<QSharedPointer<TCurveData> > TCurvesDataOwner;



class TCurves
{
public:
    void DeleteAllCurves();
    void ReSyncDataAndQwtCurves();
    void DefineLimits(QVector<double> &MaxX, QVector<double> &MaxY, QVector<double> &MinX, QVector<double> &MinY, int N);

    TCurves(QwtPlot *_Plot);
    ~TCurves();
    virtual void SetActiveCurve(int Active);
    void VerifyActiveCurve();

    void GetClosestPoint(double *, double *, const QPoint &Pos, int N = 0) const;
    void UnselectCurves();
    void SelectCurve(int N);

    void CleanData();

    bool vertical;
    QwtPlot *Plot;
    TCurvesData SourceData;
    TCurvesData PaintData;
    TCurvesDataOwner MathData;
    QVector<QwtPlotCurve *> QwtCurves;
    QwtPlotCurve *MCCurve;
    QVector<int> LastPaintSize; //incremental painter
    int ActiveCurve;//-1 - all curves; 0,1,... - n of active curve
};
}
