#include "plot_3dWidgets.h"

#include "../../src/gui/flowlayout.hpp"
#include <QSettings>

namespace plot {

ControlPanelHistory::ControlPanelHistory()
{
    BDetailedMarkersInfoIsChecked = BCrossectionIsChecked = BMarkersIsChecked = BAngleIsChecked = false;
    BGridIsChecked = BCrossIsChecked = BFixOXDirIsChecked = BFixOYDirIsChecked = false;
    MakeCorrectLineByMedianaXDirIsChecked = MakeCorrectLineByMedianaYDirIsChecked = false;
    BFixOXDirIsEnabled = BFixOYDirIsEnabled = true;
    NOfPalette = 0;
    CurrentSubtrOrder = CurrentYFitOrder = CurrentXFitOrder = -1;
    GaussianFilterSize = MedianValueFilterSize = MeanValueFilterSize = -1;
    MinSliderVal = 0; MaxSliderVal = 1;
    HistogramEqualization = false;
    AutoZScaleOn = true;
}

//================================================================================================================================
TFilterSizeWidget::TFilterSizeWidget(QWidget *parent) : QDialog(parent)
{
    QVBoxLayout *MainLayout = new QVBoxLayout(this);
    QHBoxLayout *l1 = new QHBoxLayout();
    QHBoxLayout *l2 = new QHBoxLayout();
    MainLayout->addLayout(l1);
    MainLayout->addLayout(l2);

    SFilterSize = new QwtSlider(this, Qt::Horizontal, QwtSlider::BottomScale);
    SFilterSize->setHandleSize(10, 20);
    SFilterSize->setMinimumWidth(85);
    SFilterSize->setRange(2, 30, 1);
    SFilterSize->setTracking(false);
    connect(SFilterSize, SIGNAL(valueChanged(double)), this, SLOT(onSliderMoved(double)));

    LFilterSize = new QLabel;
    LFilterSize->setMinimumWidth(QFontMetricsF(QFont()).size(Qt::TextSingleLine, "10 px").width());

    l1->addWidget(SFilterSize);
    l1->addWidget(LFilterSize);

    QPushButton *ok = new QPushButton(tr("Apply"));
    QPushButton *nok = new QPushButton(tr("Cancel"));
    connect(ok, SIGNAL(clicked(bool)), this, SLOT(accept()));
    connect(nok, SIGNAL(clicked(bool)), this, SLOT(reject()));

    l2->addWidget(ok);
    l2->addWidget(nok);
}
void TFilterSizeWidget::onSliderMoved(double val)
{
    LFilterSize->setText(QString::number((int)val) + " px");
    emit SliderValueChanged((int)val);
}
void TFilterSizeWidget::SetFilterSize(int V)
{
    if (SFilterSize->minValue > V || V > SFilterSize->maxValue) return;
    SFilterSize->setValue(V);
    onSliderMoved(V);
}
int TFilterSizeWidget::FilterSize() const
{
    return (int)SFilterSize->value;
}

//================================================================================================================================
void T3dPlotControl::GUICreate()
{
    TCommonPlotControl::GUICreate();
    BAutoScale->hide();

    CreateActionsAndMenus();

    BCross = new TwoStateButton(":/plot/point.png", ":/plot/pointOn.png");
    BCross->setToolTip(QObject::tr("Point selection"));
    BCross->setCheckable(true);

    BAreaZzoom = new TwoStateButton(":/plot/Pall_Zoom_Of.png", ":/plot/Pall_Zoom_On.png");
    BAreaZzoom->setToolTip(QObject::tr("Set Z range by selection"));
    BAreaZzoom->setCheckable(true);

    BCrossection = new QPushButton;
    BCrossection->setIcon(makeHugeIcon(":/plot/cross_section.png"));
    BCrossection->setCheckable(true);
    BCrossection->setToolTip(QObject::tr("Cross section"));
    BCrossection->hide();

    addDelimeterH(ControlMainLayout, BMarkers->sizeHint().width());
    ControlMainLayout->addWidget(TBFontSize);
    ControlMainLayout->addWidget(TBPalettes);

    ControlMainLayout->insertWidget(3, BCrossection);
    ControlMainLayout->insertWidget(4, BFixOXDir);
    ControlMainLayout->insertWidget(5, BFixOYDir);

    ControlMainLayout->insertWidget(16, BCross);

    ControlMainLayout->insertWidget(19, TBMath2);

    ControlMainLayout->insertWidget(26, BAreaZzoom);

    BAreaZzoom->show();

    setTitle("3D");
}
void T3dPlotControl::CreateActionsAndMenus()
{
    MMath2 = new QMenu();
    QMenu *MXFitOrders = new QMenu(tr("X Fitting"));
    QMenu *MYFitOrders = new QMenu(tr("Y Fitting"));
    QMenu *MSubtr = new QMenu(tr("Subtract"));

    AGXFitOrders = new QActionGroup(this);
    AGYFitOrders = new QActionGroup(this);
    AGSubtr      = new QActionGroup(this);

    const QString FitOrders[] = {tr("None"), tr("0 order"), tr("1st order"), tr("2nd order"), tr("3rd order"), tr("4th order"), tr("5th order")};
    for (auto &fitOrder:FitOrders)
    {
        AGXFitOrders->addAction(MXFitOrders->addAction(fitOrder))->setCheckable(true);
        AGYFitOrders->addAction(MYFitOrders->addAction(fitOrder))->setCheckable(true);
        AGSubtr->addAction(MSubtr->addAction(fitOrder))->setCheckable(true);
    }
    AGXFitOrders->actions()[0]->setChecked(true); //CurrentXFitOrder = -1;
    AGYFitOrders->actions()[0]->setChecked(true); //CurrentYFitOrder = -1;
    AGSubtr     ->actions()[0]->setChecked(true); //CurrentSubtrOrder = -1;

    //=================================================QMenu's=================================================
    MMathProcessing->addAction(tr("Fit Lines X"))->setCheckable(true);
    MMathProcessing->addAction(tr("Fit Lines Y"))->setCheckable(true);
    MMathProcessing->addMenu(MSubtr);

    MMath2->addMenu(MXFitOrders);
    MMath2->addMenu(MYFitOrders);
    MMath2->addSeparator();
    MMath2->addAction(NameOfMeanValueFilter())->setCheckable(true);
    MMath2->addAction(NameOfMedianValueFilter())->setCheckable(true);
    MMath2->addAction(NameOfGaussianFilter())->setCheckable(true);
    MMath2->addAction(NameOfHistogramFilter())->setCheckable(true);

    //=================================================QToolButton's=================================================
    TBMath2 = new QToolButton;
    TBMath2->setIcon(makeHugeIcon(":/plot/math_extra.png"));
    TBMath2->setToolTip(QObject::tr("Additional Math"));
    TBMath2->setPopupMode(QToolButton::InstantPopup);
    TBMath2->setMenu(MMath2);

    TBPalettes = new QToolButton;
    TBPalettes->setIcon(makeHugeIcon(":/plot/palette.png"));
    TBPalettes->setToolTip(QObject::tr("Palettes"));
    TBPalettes->setPopupMode(QToolButton::InstantPopup);
    QMenu *MPalettes = new QMenu;
    TBPalettes->setMenu(MPalettes);
    AGPalettes = new QActionGroup(this);

    BFixOXDir = new QPushButton;
    BFixOXDir->setCheckable(true);
    BFixOXDir->hide();
    BFixOXDir->setIcon(makeHugeIcon(":/plot/markers_Y.png"));
    BFixOXDir->setToolTip(QObject::tr("markers vertical mode"));

    BFixOYDir = new QPushButton;
    BFixOYDir->setCheckable(true);
    BFixOYDir->hide();
    BFixOYDir->setIcon(makeHugeIcon(":/plot/markers_X.png"));
    BFixOYDir->setToolTip(QObject::tr("markers horizontal mode"));
}
void T3dPlotControl::SetSelectionModeEnabled(const bool Enabled)
{
    TCommonPlotControl::SetSelectionModeEnabled(Enabled);
    BCross->setVisible(!Enabled);
}
int T3dPlotControl::GetNumOfAction(const QString &ActName)
{
    if (ActName == NameOfMeanValueFilter()) return 3;
    if (ActName == NameOfMedianValueFilter()) return 4;
    if (ActName == NameOfGaussianFilter()) return 5;
    if (ActName == NameOfHistogramFilter()) return 6;
    return 0;
}
void T3dPlotControl::on_BApply_clicked()
{
    BSelectByRect->setVisible(true);
    BSelectByRect->setChecked(false);
}
void T3dPlotControl::ResetAllFilters()
{
    AGXFitOrders->actions()[0]->setChecked(true); //CurrentXFitOrder = -1;
    AGYFitOrders->actions()[0]->setChecked(true); //CurrentYFitOrder = -1;
    AGSubtr     ->actions()[0]->setChecked(true); //CurrentSubtrOrder = -1;
    MMathProcessing->actions()[0]->setChecked(false);
    MMathProcessing->actions()[1]->setChecked(false);
    MMath2->actions()[GetNumOfAction(NameOfMeanValueFilter())]->setChecked(false);
    MMath2->actions()[GetNumOfAction(NameOfMedianValueFilter())]->setChecked(false);
    MMath2->actions()[GetNumOfAction(NameOfGaussianFilter())]->setChecked(false);
    MMath2->actions()[GetNumOfAction(NameOfHistogramFilter())]->setChecked(false);
}

//================================================================================================================================
void T3dMarkersInfo::GUICreate()
{
    TMarkersInfo::GUICreate();

    LEdQ->hide();
    LQv->hide();

    LZAUnit = new QLabel;
    LZBUnit = new QLabel;
    LDeltaZUnit = new QLabel;

    QLabel *LZA = new QLabel(tr("z ="));
    LEdZA = new QLineEdit;
    LEdZA->setMaximumWidth(MaxWidth);
    LEdZA->setReadOnly(true);

    QLabel *LZB = new QLabel(tr("z ="));
    LEdZB = new QLineEdit;
    LEdZB->setMaximumWidth(MaxWidth);
    LEdZB->setReadOnly(true);

    QLabel *LDeltaZ = new QLabel(QString(QChar(0x2206)) + tr("z ="));
    LEdDeltaZ = new QLineEdit;
    LEdDeltaZ->setMaximumWidth(MaxWidth);
    LEdDeltaZ->setReadOnly(true);

    GLMarkerA->addWidget(LZA, 2, 0, Qt::AlignRight);
    GLMarkerA->addWidget(LEdZA, 2, 1);
    GLMarkerA->addWidget(LZAUnit, 2, 2);

    GLMarkerB->addWidget(LZB, 2, 0, Qt::AlignRight);
    GLMarkerB->addWidget(LEdZB, 2, 1);
    GLMarkerB->addWidget(LZBUnit, 2, 2);

    GLDownL->addWidget(LDeltaZ, 2, 0, Qt::AlignRight);
    GLDownL->addWidget(LEdDeltaZ, 2, 1);
    GLDownL->addWidget(LDeltaZUnit, 2, 2);
}

}
