#include "plot_3dplot.h"
#include "math/plot_MathProcessing.h"

#include <qwt_plot_layout.h>
#include <qwt_plot_marker.h>
#include <qwt_plot_canvas.h>

#include <QtEvents>
#include <QDrag>
#include <QApplication>
#include <QSplitter>
#include "widgets/fixedsplitter.h"
#include <QVBoxLayout>
#include <QFileDialog>
#include <QInputDialog>
#include <QMessageBox>
#include <QGestureEvent>
#include <QDebug>
#include "widgets/threedoverlaybuttons.h"
#include "2d/plot_2dpure.h"
#include "3d/plot_3dpure.h"
#include "widgets/twodoverlaybuttons.h"

namespace plot {

//================================================================================================================================
T3dPlot::T3dPlot(int N, QObject *_Parent, QWidget *wparent) : QWidget(wparent)
{
    overlayButtons = new ThreeDOverlayButtons();

    Parent = _Parent;
    TwoDPlotSPM = nullptr;
    TwoDPlot = nullptr;
    TwoDPlotVertical = nullptr;
    MainLayout = nullptr;
    MarkersInfoLayout = nullptr;
    PlotVerticalLay = nullptr;
    orderChannel = nullptr;
    positionChannel = nullptr;
    plots2D3DLay = nullptr;
    ControlLayout = nullptr;

    GUICreate(N);
    InitPalettes();

    ActivePlot = -1;
    SetActivePlot(0);
    installEventFilter(this);
    overlayButtons->setParentPlot(this);
}
void T3dPlot::SetFullWindowPlot(int plot, bool full)
{
#ifndef _ADROID

    currFullWindPlot = full ? plot : -1;

    if (full)
    {
        for (int i=0; i<numTabsFor3DPlot; i++)
        {
            auto layTab = mainTab->widget(i)->layout();
            for (int j=0; j<layTab->count(); j++)
            {
                if (layTab->itemAt(j)->widget() == ThreeDPlots[plot])
                {
                    layTab->removeWidget(ThreeDPlots[plot]);
                    goto exit2For;
                }
            }
        }
        exit2For:

        if (showingScan2DPlot)
            TwoDPlot->hide();
        else
            TwoDPlotSPM->hide();
        if (TwoDPlot->overlayButtons)
            TwoDPlot->overlayButtons->set2DPlot(TwoDPlotVertical);
        SplitPlotTab->hide();
        TwoDPlotVertical->setHidden(false);
        plots2D3DLay->addWidget(ThreeDPlots[plot]);
        ThreeDPlots[plot]->s_Refresh();
        ThreeDPlots[plot]->secondParent = SplitPlotTab;

        ThreeDPlots[plot]->ResizeQPlot();

        TwoDPlotVertical->update();
        TwoDPlotVertical->Pure2dPlot->Refresh();

        if (TwoDPlot->showPlotControl)
        {
            TwoDPlot->PlotControl->show();
            TwoDPlot->PlotControl->hide();
        }

        TwoDPlotVertical->Pure2dPlot->SetSelectWidth(
                    TwoDPlotVertical->Pure2dPlot->CurrentXLimits[1] - TwoDPlotVertical->Pure2dPlot->CurrentXLimits[0]);
    }
    else
    {
        TwoDPlotVertical->setHidden(true);
        ThreeDPlots[plot]->secondParent = nullptr;
        plots2D3DLay->removeWidget(ThreeDPlots[plot]);
        SplitPlotTab->show();
        if (showingScan2DPlot)
        {
            TwoDPlot->setParent(SplitPlotTab);
            TwoDPlot->show();
        }
        else
            TwoDPlotSPM->show();
        if (TwoDPlot->overlayButtons)
            TwoDPlot->overlayButtons->set2DPlot(TwoDPlot);
        for (int i=0; i<numTabsFor3DPlot; i++)
        {
            auto layTab = mainTab->widget(i)->layout();
            if (layTab->count() == 2)
                continue;

            if (orderChannel->at(plot)%2 != 0 || ThreeDPlots[plot]->singleInTabPage)
            {
                layTab->addWidget(ThreeDPlots[plot]);
                layTab->itemAt(0)->setAlignment(Qt::AlignLeft);
                if (!ThreeDPlots[plot]->singleInTabPage)
                    layTab->itemAt(1)->setAlignment(Qt::AlignLeft);
            }
            else {
                auto wid = layTab->itemAt(0)->widget();
                layTab->removeWidget(wid);
                layTab->addWidget(ThreeDPlots[plot]);
                layTab->addWidget(wid);
                layTab->itemAt(0)->setAlignment(Qt::AlignLeft);
                layTab->itemAt(1)->setAlignment(Qt::AlignLeft);
            }
        }
        fixedSplit->setSizes(QList<int>() << fixedSplit->width()-1 << 1);
        TwoDPlot->Pure2dPlot->SetSelectHeight(
                    TwoDPlot->Pure2dPlot->CurrentYLimits[1] - TwoDPlot->Pure2dPlot->CurrentYLimits[0]);
    }
    if (showingScan2DPlot)
        TwoDPlot->on_BDetailedMarkersInfo_clicked();
    else
        TwoDPlotSPM->on_BDetailedMarkersInfo_clicked();
    TwoDPlotVertical->on_BDetailedMarkersInfo_clicked();

#endif
}
void T3dPlot::SwapPlots(int p1, int p2)
{
    int tabIndexes[2] = {-1, -1};
    QLayout *tabLays[2] = {nullptr, nullptr};
    int pJ[2] = {-1, -1}; //places in their tabs

    {
        int t = orderChannel->at(p1);
        orderChannel->replace(p1, orderChannel->at(p2));
        orderChannel->replace(p2, t);

        for (int i=0; i<positionChannel->size(); i++)
        {
            for (int j=0; j<orderChannel->size(); j++)
            {
                if (orderChannel->at(j) == i)
                {
                    positionChannel->replace(i, j);
                    break;
                }
            }
        }
    }

    for (int i=0; i<numTabsFor3DPlot; i++)
    {
        auto layTab = mainTab->widget(i)->layout();
        for (int j=0; j<layTab->count(); j++)
        {
            if (layTab->itemAt(j)->widget() == ThreeDPlots[p1])
            {
                tabIndexes[0] = i;
                tabLays[0] = layTab;
                pJ[0] = j;
            }
            else if (layTab->itemAt(j)->widget() == ThreeDPlots[p2])
            {
                tabIndexes[1] = i;
                tabLays[1] = layTab;
                pJ[1] = j;
            }
        }
    }

    bool oneTabWith4Plots = (numTabsFor3DPlot == 1 && (tabIndexes[0] == -1 || tabIndexes[1] == -1));

    if (tabIndexes[0] == tabIndexes[1])
    {
        if (oneTabWith4Plots)
            return;
        auto wid = tabLays[0]->itemAt(0)->widget();
        tabLays[0]->removeWidget(wid);
        tabLays[0]->addWidget(wid);
        tabLays[0]->itemAt(0)->setAlignment(Qt::AlignLeft);
        tabLays[0]->itemAt(1)->setAlignment(Qt::AlignLeft);
        QString str1, str2;
        if (wid == ThreeDPlots[p1])
        {
            str1 = QString('A' + p2);
            str2 = QString('A' + p1);
        }
        else {
            str1 = QString('A' + p1);
            str2 = QString('A' + p2);
        }
        mainTab->setTabText(tabIndexes[0],
                tr("Channel") + " " + str1 +
                " " + tr("and") + " " +
                tr("Channel") + " " + str2);

        return;
    }

    if (oneTabWith4Plots)
    {
        auto lay = mainTab->widget(0)->layout();
        while (lay->count() > 0)
            lay->removeWidget(lay->itemAt(0)->widget());
        int first = 0, second = 1;
        for (int i=0; i<orderChannel->count(); i++)
        {
            ThreeDPlots[i]->hide();
            ThreeDPlots[i]->setParent(nullptr);
            if (orderChannel->at(i) == 0)
                first = i;
            else if (orderChannel->at(i) == 1)
                second = i;
        }
        lay->addWidget(ThreeDPlots[first]);
        lay->addWidget(ThreeDPlots[second]);
        ThreeDPlots[first]->show();
        ThreeDPlots[second]->show();
    }
    else
    {
        if (tabLays[0] != nullptr)
            tabLays[0]->removeWidget(ThreeDPlots[p1]);
        if (tabLays[1] != nullptr)
            tabLays[1]->removeWidget(ThreeDPlots[p2]);
        if (tabLays[0] != nullptr)
        {
            if (pJ[0] == 1)
            {
                tabLays[0]->addWidget(ThreeDPlots[p2]);
            }
            else {
                auto wid = tabLays[0]->itemAt(0)->widget();
                tabLays[0]->removeWidget(wid);
                tabLays[0]->addWidget(ThreeDPlots[p2]);
                tabLays[0]->addWidget(wid);
            }
        }

        if (tabLays[1] != nullptr)
        {
            if (pJ[1] == 1)
            {
                tabLays[1]->addWidget(ThreeDPlots[p1]);
            }
            else {
                auto wid = tabLays[1]->itemAt(0)->widget();
                tabLays[1]->removeWidget(wid);
                tabLays[1]->addWidget(ThreeDPlots[p1]);
                tabLays[1]->addWidget(wid);
            }
        }
    }

    for (int i=0; i<2; i++)
    {
        if (tabLays[i] == nullptr)
            break;
        tabLays[i]->itemAt(0)->setAlignment(Qt::AlignLeft);
        tabLays[i]->itemAt(1)->setAlignment(Qt::AlignLeft);

        int t1 = 0, t2 = 0;
        auto wid1 = tabLays[i]->itemAt(0)->widget();
        auto wid2 = tabLays[i]->itemAt(1)->widget();
        for (int j=0; j<ThreeDPlots.size(); j++)
        {
            if (ThreeDPlots[j] == wid1)
                t1 = j;
            else if (ThreeDPlots[j] == wid2)
                t2 = j;
        }
        mainTab->setTabText(tabIndexes[i],
                tr("Channel") + " " + QString('A' + t1) +
                " " + tr("and") + " " +
                tr("Channel") + " " + QString('A' + t2));
    }
}
void T3dPlot::ConnectPlotControlSlots()
{
    connect(PlotControl->BMarkers,             SIGNAL(toggled(bool)), this, SLOT(on_BMarkers_clicked(bool)));
    connect(PlotControl->BDetailedMarkersInfo, SIGNAL(toggled(bool)), this, SLOT(on_BDetailedMarkersInfo_clicked()));
    connect(PlotControl->BAngle,               SIGNAL(toggled(bool)), this, SLOT(on_BAngle_clicked()));
    connect(PlotControl->BGrid,                SIGNAL(toggled(bool)), this, SLOT(on_BGrid_clicked()));
    connect(PlotControl->BFixX,                SIGNAL(toggled(bool)), this, SLOT(on_BFixX_toggled()));
    connect(PlotControl->BFixY,                SIGNAL(toggled(bool)), this, SLOT(on_BFixY_toggled()));

    connect(PlotControl->BRestoreScale,        SIGNAL(clicked()), this, SLOT(on_BRestoreScale_clicked()));
    connect(PlotControl->BCross,               SIGNAL(state_changed(bool)), this, SLOT(on_BCross_clicked()));
    connect(PlotControl->BSelectByRect,        SIGNAL(clicked()), this, SLOT(on_BSelectByRect_clicked()));
    connect(PlotControl->BExpandSelectRect,    SIGNAL(clicked()), this, SLOT(on_BExpandSelectRect_clicked()));
    connect(PlotControl->BCancel,              SIGNAL(clicked()), this, SLOT(on_BCancel_clicked()));
    connect(PlotControl->BApply,               SIGNAL(clicked()), this, SLOT(on_BApply_clicked()));

    connect(PlotControl->BAreaZzoom,           SIGNAL(state_changed(bool)), this, SLOT(on_BAreaZzoom_clicked()));

    connect(PlotControl->BSavePlotPixmap,      SIGNAL(clicked()), this, SLOT(on_BSavePlotPixmap_clicked()));
    connect(PlotControl->BSave,                SIGNAL(clicked()), this, SLOT(on_BSave_clicked()));
    connect(PlotControl->BCrossection,         SIGNAL(toggled(bool)), this, SLOT(on_BCrossection_clicked()));
    connect(PlotControl->BFixOXDir,            SIGNAL(clicked()), this, SLOT(on_BFixOXDir_clicked()));
    connect(PlotControl->BFixOYDir,            SIGNAL(clicked()), this, SLOT(on_BFixOYDir_clicked()));

    connect(PlotControl->MMathProcessing->actions()[0], SIGNAL(toggled(bool)), this, SLOT(on_GwyXFit_triggered(bool)));
    connect(PlotControl->MMathProcessing->actions()[1], SIGNAL(toggled(bool)), this, SLOT(on_GwyYFit_triggered(bool)));

    connect(PlotControl->MMath2->actions()[PlotControl->GetNumOfAction(PlotControl->NameOfMeanValueFilter())],   SIGNAL(triggered(bool)), this, SLOT(on_MeanValue_triggered(bool)));
    connect(PlotControl->MMath2->actions()[PlotControl->GetNumOfAction(PlotControl->NameOfMedianValueFilter())], SIGNAL(triggered(bool)), this, SLOT(on_MedianValue_triggered(bool)));
    connect(PlotControl->MMath2->actions()[PlotControl->GetNumOfAction(PlotControl->NameOfGaussianFilter())],    SIGNAL(triggered(bool)), this, SLOT(on_Gaussian_triggered(bool)));
    connect(PlotControl->MMath2->actions()[PlotControl->GetNumOfAction(PlotControl->NameOfHistogramFilter())],   SIGNAL(toggled(bool)), this, SLOT(on_Histogram_triggered(bool)));

    connect(PlotControl->AGXFitOrders, SIGNAL(triggered(QAction*)), this, SLOT(on_FitXOrder_changed()));
    connect(PlotControl->AGYFitOrders, SIGNAL(triggered(QAction*)), this, SLOT(on_FitYOrder_changed()));
    connect(PlotControl->AGSubtr, SIGNAL(triggered(QAction*)), this, SLOT(on_SubtrOrder_changed()));
    connect(PlotControl->AGFontSizes, SIGNAL(triggered(QAction*)), this, SLOT(on_FontSize_changed()));
}
void T3dPlot::ConnectSlots()
{
    ConnectPlotControlSlots();
    connect(TwoDPlot, SIGNAL(AddDataToContainer(plot::TCurvesDataOwner, plot::TMetaStorage)),
            this, SIGNAL(AddDataToContainer(plot::TCurvesDataOwner, plot::TMetaStorage)));
    connect(Slider->Sliders, SIGNAL(ValuesChanged(double, double)), this, SLOT(on_Sliders_ValuesChanged(double, double)));
    connect(Slider->Sliders, SIGNAL(AutoChanged(bool)), this, SLOT(on_AutoZScale_changed(bool)));

    for (auto ThreeDPlot : ThreeDPlots)
        connect(ThreeDPlot, SIGNAL(UpdateAllMarkersInfo()), this, SLOT(UpdateAllMarkersInfo()));
}
void T3dPlot::plotDoubleClicked()
{
    if (currFullWindPlot != -1)
    {
        SetFullWindowPlot(currFullWindPlot, false);
        return;
    }

    for (int i=0; i<ThreeDPlots.size(); i++)
    {
        if(sender() == ThreeDPlots[i])
        {
            SetFullWindowPlot(i, true);
            break;
        }
    }
}
void T3dPlot::setScanSPM2DPlot(bool setScanPlot)
{
    if (showingScan2DPlot == setScanPlot)
        return;
    showingScan2DPlot = setScanPlot;

    if (plot2dFullWind)
        plot2dDoubleClicked();


    //auto splitSizes = SplitPlotTab->sizes();
    //int first = splitSizes.first();
    //int second = splitSizes.last();
    if (showingScan2DPlot)
    {
        SplitPlotTab->addWidget(TwoDPlot);
        TwoDPlotSPM->setParent(nullptr);
        TwoDPlot->setParent(SplitPlotTab);
        //ControlLayout->removeWidget(TwoDPlotSPM->PlotControl);
        if (TwoDPlotSPM->PlotControl->isVisible() && TwoDPlot->showPlotControl)
            TwoDPlot->PlotControl->show();
        TwoDPlotSPM->PlotControl->hide();
        MarkersInfoLayout->removeWidget(TwoDPlotSPM->MarkersInfo);
        MarkersInfoLayout->addWidget(TwoDPlot->MarkersInfo, 0, Qt::AlignBottom);
        if (TwoDPlot->overlayButtons)
            TwoDPlot->overlayButtons->set2DPlot(TwoDPlot);
    }
    else
    {
        SplitPlotTab->addWidget(TwoDPlotSPM);
        TwoDPlot->setParent(nullptr);
        TwoDPlotSPM->setParent(SplitPlotTab);
        if (TwoDPlot->PlotControl->isVisible())
        {
            TwoDPlot->PlotControl->hide();
            TwoDPlotSPM->PlotControl->show();
        }
        //ControlLayout->addWidget(TwoDPlotSPM->PlotControl);
        MarkersInfoLayout->removeWidget(TwoDPlot->MarkersInfo);
        MarkersInfoLayout->addWidget(TwoDPlotSPM->MarkersInfo, 0, Qt::AlignBottom);
    }
    //SplitPlotTab->setSizes(splitSizes);
    //SplitPlotTab->setSizes(QList<int>() << first << second);
}
void T3dPlot::GUICreate(int N)
{
    int YNum = qSqrt(N);
    if (N % YNum != 0)
        YNum = 1;
    int XNum = N / YNum;

    numPlotByX = (uint)XNum;
    numPlotByY = (uint)YNum;

    QWidget *plots2D3DWid = new QWidget();
    plots2D3DLay = new QVBoxLayout(plots2D3DWid);

    ControlLayout = new QVBoxLayout;

    mainTab = new QTabWidget();
    mainTab->setStyleSheet(""
           "QTabWidget::tab-bar {left: 8px;} "
           "QTabWidget::pane { border: 1px solid #B4B4B4; border-style: outset; border-width: 2px; border-radius: 10px; border-color: #B4B4B4; } "
           "QTabWidget { background: #F0F0F0; } "
           );
    QWidget **tabsWid = new QWidget*[numPlotByY];
    QHBoxLayout **hLay_tabs = new QHBoxLayout*[numPlotByY];
    orderChannel = new QVector<int>();
    positionChannel = new QVector<int>();
    char word = 'A';
    for (int i=0; i<YNum; i++)
    {
        tabsWid[i] = new QWidget();
        hLay_tabs[i] = new QHBoxLayout(tabsWid[i]);
        mainTab->addTab(tabsWid[i],
                        tr("Channel") + " " + QString(word) +
                        " " + tr("and") + " " +
                        tr("Channel") + " " + QString(word+1));
        word += 2;
    }

    MainLayout = new QHBoxLayout(this);
    MainLayout->setContentsMargins(0, 0, 6, 0);
    MainLayout->setSpacing(0);

    SplitPlotTab = new QSplitter(this);
    fixedSplit = new FixedSplitter();
    fixedSplit->setOrientation(Qt::Horizontal);
    SplitPlotTab->setOrientation(Qt::Vertical);

    MarkersInfo = new T3dMarkersInfo;
    Slider = new TDoubleSlider;

    QSettings settingsGui("Param/gui.conf", QSettings::IniFormat);
    showSlider = settingsGui.value("gui/show_scan_slider", true).toBool();

    MarkersInfoLayout = new QVBoxLayout();
    MarkersInfoLayout->addWidget(MarkersInfo, 0, Qt::AlignTop);

    QWidget *sliderAndVert2DPlot = new QWidget();
    QHBoxLayout *sliderAndvert2DPlotLay = new QHBoxLayout(sliderAndVert2DPlot);

    MainLayout->addLayout(MarkersInfoLayout, 0);
    fixedSplit->addWidget(plots2D3DWid);
    fixedSplit->addWidget(sliderAndVert2DPlot);
    fixedSplit->setStretchFactor(1, 1);
    MainLayout->addWidget(fixedSplit);

    plots2D3DLay->addWidget(SplitPlotTab);

    SplitPlotTab->addWidget(mainTab);

    ThreeDPlots.resize(N);
    for (int i = 0; i < YNum; i++)
    {
        for (int j = i * XNum; j < (i + 1) * XNum && j < ThreeDPlots.size(); j++)
        {
            ThreeDPlots[j] = new TPure3dPlot(j, this, Parent, tabsWid[i]);
            hLay_tabs[i]->addWidget(ThreeDPlots[j], 0, Qt::AlignLeft);
            orderChannel->append(j);
            positionChannel->append(j);
            connect(ThreeDPlots[j], SIGNAL(doubleClicked()), this, SLOT(plotDoubleClicked()));

            connect(ThreeDPlots[j], SIGNAL(onResized(int, QSize)),
                    overlayButtons, SLOT(ChannelResized(int, QSize)));
            connect(ThreeDPlots[j], SIGNAL(MouseEventSignal(QEvent*, TPure3dPlot*, int)),
                    overlayButtons, SLOT(MouseEvent(QEvent*, TPure3dPlot*, int)));
        }
        numTabsFor3DPlot++;
    }
    if (mainTab->count() == 1)
    {
#if (QT_VERSION >= QT_VERSION_CHECK(5, 0, 0))
        mainTab->tabBar()->setHidden(true);
#else
        QTabBar *tabBar = qFindChild<QTabBar *>(mainTab);
        tabBar->hide();
#endif
        ThreeDPlots[0]->singleInTabPage = true;
    }

    for(int i=0; i<positionChannel->size(); i++)
    {
        for (int j=0; j<orderChannel->size(); j++)
        {
            if (orderChannel->at(j) == i)
            {
                positionChannel->replace(i, j);
                break;
            }
        }
    }

    QFont font = qApp->font();
    font.setPointSize(ThreeDPlots.at(0)->FontSize);
    Slider->Title->setFont(font);
    Slider->Sliders->QwtBar->setFont(font);

    bool isChild = true;
#ifdef _ADROID
    isChild = false;
#endif

    #ifdef _ADROID
    TwoDOverlayButtons *TwoDOverlay = new TwoDOverlayButtons(false);
#else
    TwoDOverlayButtons *TwoDOverlay = new TwoDOverlayButtons();
#endif
    TwoDPlot = new T2dPlot(this, isChild, nullptr, false, TwoDOverlay);
    TwoDPlotVertical = new T2dPlot(this, true, this, true, TwoDOverlay);
    TwoDPlot->depent2dPlot = TwoDPlotVertical;
    TwoDPlotVertical->master2dPlot = TwoDPlot;
    TwoDPlotVertical->MainLayout->removeWidget(TwoDPlot->MarkersInfo);
    TwoDPlotVertical->overlayButtons = TwoDOverlay;
    TwoDPlotVertical->setHidden(true);
    TwoDPlot->overlayButtons = TwoDOverlay;
    TwoDPlot->overlayButtons->set2DPlot(TwoDPlot);
    TwoDPlot->MainLayout->removeWidget(TwoDPlot->MarkersInfo);
    MarkersInfoLayout->addWidget(TwoDPlot->MarkersInfo, 0, Qt::AlignBottom);

    SplitPlotTab->addWidget(TwoDPlot);
    TwoDPlot->setParent(SplitPlotTab);

    SplitPlotTab->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    SplitPlotTab->setStretchFactor(1, 1);
    lastSplitPlotSizes[0] = this->height()*0.75;
    lastSplitPlotSizes[1] = this->height()*0.25;
    SplitPlotTab->setSizes(QList<int>() << lastSplitPlotSizes[0] << lastSplitPlotSizes[1]);
    TwoDPlot->PlotControl->hide();
    connect(TwoDPlot->Pure2dPlot, SIGNAL(ShowPlotControl()), SLOT(Show2dPlotControl()));
    connect(TwoDPlot->Pure2dPlot, SIGNAL(doubleClicked()), this, SLOT(plot2dDoubleClicked()));

    PlotControl = new T3dPlotControl;

    PlotVerticalLay = new QVBoxLayout();
    PlotVerticalLay->setContentsMargins(5, 0, 0, 0);

    MainLayout->addLayout(ControlLayout);

    if (showSlider)
        sliderAndvert2DPlotLay->addWidget(Slider, 0, Qt::AlignTop);
    sliderAndvert2DPlotLay->addLayout(PlotVerticalLay);

    ControlLayout->addWidget(PlotControl);
    ControlLayout->addWidget(TwoDPlot->PlotControl);

    if (showPlotControl)
        PlotControl->hide();

    MarkersInfo->GUICreate();

    TwoDPlotVertical->Pure2dPlot->MathProcessing->vertical = true;
#ifndef _ADROID

    PlotVerticalLay->addWidget(TwoDPlotVertical);
    TwoDPlot->PlotControl->GUICreate();
    TwoDPlot->ConnectPlotControlSlots();
    ConnectPlot2dWithVertical2d();
    MarkersInfoLayout->addWidget(TwoDPlotVertical->MarkersInfo, 0, Qt::AlignBottom);
    //TwoDPlotVertical->PlotControl->GUICreate();
    //TwoDPlotVertical->ConnectPlotControlSlots();
#endif
    TwoDPlotVertical->PlotControl = TwoDPlot->PlotControl;
    PlotControl->GUICreate();

    ConnectSlots();
}
void T3dPlot::plot2dDoubleClicked()
{
    if (plot2dFullWind)
    {
        if (showingScan2DPlot)
        {
            plots2D3DLay->removeWidget(TwoDPlot);
            SplitPlotTab->show();
#if (QT_VERSION >= QT_VERSION_CHECK(5, 9, 0))
            SplitPlotTab->replaceWidget(1, TwoDPlot);
#else
            SplitPlotTab->widget(1)->hide();
            SplitPlotTab->insertWidget(1, TwoDPlot);
#endif
        }
        else {
            plots2D3DLay->removeWidget(TwoDPlotSPM);
            SplitPlotTab->show();
#if (QT_VERSION >= QT_VERSION_CHECK(5, 9, 0))
            SplitPlotTab->replaceWidget(1, TwoDPlotSPM);
#else
            SplitPlotTab->widget(1)->hide();
            SplitPlotTab->insertWidget(1, TwoDPlotSPM);
#endif
        }
        SplitPlotTab->setSizes(last2D3DPlotSplitSizes);
    }
    else
    {
        last2D3DPlotSplitSizes = SplitPlotTab->sizes();
        QWidget *tWid = new QWidget();
#if (QT_VERSION >= QT_VERSION_CHECK(5, 9, 0))
        SplitPlotTab->replaceWidget(1, tWid);
#else
        SplitPlotTab->insertWidget(1, tWid);
#endif
        SplitPlotTab->hide();
        if (showingScan2DPlot)
            plots2D3DLay->addWidget(TwoDPlot);
        else
            plots2D3DLay->addWidget(TwoDPlotSPM);
    }
    plot2dFullWind = !plot2dFullWind;
}
void T3dPlot::ConnectPlot2dWithVertical2d()
{
#ifndef _ADROID
    connect(TwoDPlot->PlotControl->BAutoScaleX, SIGNAL(clicked()), TwoDPlotVertical->Pure2dPlot, SLOT(on_BAutoScaleX_clicked()));
    connect(TwoDPlot->PlotControl->BAutoScaleY, SIGNAL(clicked()), TwoDPlotVertical->Pure2dPlot, SLOT(on_BAutoScaleY_clicked()));
    connect(TwoDPlot->PlotControl->BRestoreScale, SIGNAL(clicked()), TwoDPlotVertical->Pure2dPlot, SLOT(BRestoreScale_clicked()));
    connect(TwoDPlot->PlotControl->BExpandSelectRect, SIGNAL(clicked()), TwoDPlotVertical->Pure2dPlot, SLOT(on_BExpandSelectRect_clicked()));

    connect(TwoDPlot->PlotControl->BMarkers, SIGNAL(toggled(bool)), TwoDPlotVertical, SLOT(on_BMarkers_clicked(bool)));
    connect(TwoDPlot->PlotControl->BDetailedMarkersInfo, SIGNAL(toggled(bool)), TwoDPlotVertical, SLOT(on_BDetailedMarkersInfo_clicked()));
    connect(TwoDPlot->PlotControl->BGrid, SIGNAL(toggled(bool)), TwoDPlotVertical->Pure2dPlot, SLOT(BGrid_clicked(bool)));
    connect(TwoDPlot->PlotControl->BFixX, SIGNAL(toggled(bool)), TwoDPlotVertical, SLOT(on_BFixX_toggled()));
    connect(TwoDPlot->PlotControl->BFixY, SIGNAL(toggled(bool)), TwoDPlotVertical, SLOT(on_BFixY_toggled()));

    connect(TwoDPlot->PlotControl->BSelectByRect, SIGNAL(clicked()), TwoDPlotVertical, SLOT(on_BSelectByRect_clicked()));
    //connect(TwoDPlot->PlotControl->BCancel, SIGNAL(clicked()), TwoDPlotVertical, SLOT(on_BCancel_clicked()));
    connect(TwoDPlot->PlotControl->BApply, SIGNAL(clicked()), TwoDPlotVertical, SLOT(on_BApply_clicked()));

    connect(TwoDPlot->PlotControl->BAutoScale, SIGNAL(toggled(bool)), TwoDPlotVertical, SLOT(on_BAutoScale_toggled()));
    //connect(PlotControl->BSavePlotPixmap, SIGNAL(clicked()), this, SLOT(on_BSavePlotPixmap_clicked()));
    //connect(PlotControl->BSave,  SIGNAL(clicked()), this, SLOT(on_BSave_clicked()));

    connect(TwoDPlot->PlotControl->CBChooseActiveCurve, SIGNAL(highlighted(int)), TwoDPlotVertical, SLOT(on_CBChooseActiveCurve_highlighted(int)));
    connect(TwoDPlot->PlotControl->CBChooseActiveCurve, SIGNAL(activated(int)), TwoDPlotVertical, SLOT(on_CBChooseActiveCurve_activated(int)));

    connect(TwoDPlot->PlotControl->BResetMCCurve, SIGNAL(clicked()), TwoDPlotVertical->Pure2dPlot, SLOT(ResetMCCurve()));

    connect(TwoDPlot->PlotControl->AGCurveColours, SIGNAL(triggered(QAction*)), TwoDPlotVertical, SLOT(on_CurveColour_changed()));
    connect(TwoDPlot->PlotControl->AGCurveThickness, SIGNAL(triggered(QAction*)), TwoDPlotVertical, SLOT(on_CurveThickness_changed()));
    connect(TwoDPlot->PlotControl->AGFitOrders, SIGNAL(triggered(QAction*)), TwoDPlotVertical, SLOT(on_FitOrder_changed()));
    connect(TwoDPlot->PlotControl->AGFontSizes, SIGNAL(triggered(QAction*)), TwoDPlotVertical, SLOT(on_FontSize_changed()));

#endif
}
void T3dPlot::AdjustAxisLegends()
{
    auto activePlot = ThreeDPlots.at(ActivePlot);
    TwoDPlot->Pure2dPlot->SetAutoLegendEnabled(activePlot->AutoLegendEnabled);
    if (activePlot->AutoLegendEnabled)
    {
        TwoDPlot->Pure2dPlot->XAxisUnit = activePlot->XYAxisUnit;
        TwoDPlot->Pure2dPlot->YAxisUnit = activePlot->ZAxisUnit;
    }
    else
    {
        if (activePlot->XLegend == activePlot->YLegend)
            TwoDPlot->Pure2dPlot->SetXLegend(activePlot->XLegend);
        else
            TwoDPlot->Pure2dPlot->SetXLegend("Not available");
        TwoDPlot->Pure2dPlot->SetYLegend(activePlot->ZLegend);
    }

    TwoDPlotVertical->Pure2dPlot->SetAutoLegendEnabled(activePlot->AutoLegendEnabled);
    if (activePlot->AutoLegendEnabled)
    {
        TwoDPlotVertical->Pure2dPlot->YAxisUnit = activePlot->XYAxisUnit;
        TwoDPlotVertical->Pure2dPlot->XAxisUnit = activePlot->ZAxisUnit;
    }
    else
    {
        if (activePlot->XLegend == activePlot->YLegend)
            TwoDPlotVertical->Pure2dPlot->SetYLegend(activePlot->XLegend);
        else
            TwoDPlotVertical->Pure2dPlot->SetYLegend("Not available");
        TwoDPlotVertical->Pure2dPlot->SetXLegend(activePlot->ZLegend);
    }
}
void T3dPlot::ResizeSliders()
{
    if (ThreeDPlots.size() == 1)
        Slider->setMaximumHeight(qBound(ThreeDPlots.at(ActivePlot)->Height(), 200, height()));
}
void T3dPlot::AdjustTwoDPlot()
{
    AdjustAxisLegends();
    if (PlotControl->BCrossection->isChecked() && PlotControl->BMarkers->isChecked())
        MakeCrossection();
}
void T3dPlot::SetActivePlot(int N)
{
    if (N < 0 || N > 3)
        return;

    if (showPlotControl)
        PlotControl->show();
    TwoDPlot->PlotControl->hide();
    if (!showingScan2DPlot)
        TwoDPlotSPM->PlotControl->hide();

    if (ActivePlot == N) return;

    RestoreEnvironmentState(ActivePlot = N);

    for (int i = 0; i < ThreeDPlots.size(); i++)
        ThreeDPlots[i]->SetPlotActive(i == ActivePlot);
    ThreeDPlots[ActivePlot]->setFocus(Qt::MouseFocusReason);
    AdjustTwoDPlot();
    emit ActivePlotChanged(N);
}
void T3dPlot::RestoreEnvironmentState(int N)
{
	auto plot = ThreeDPlots.at(N);
    bool block = plot->blockRefresh;
	plot->blockRefresh = true;

    PlotControl->BMarkers-> setChecked(plot->BMarkersIsChecked);
    PlotControl->BAngle->   setChecked(plot->BAngleIsChecked);
    PlotControl->BGrid->    setChecked(plot->BGridIsChecked);
    PlotControl->BFixX->    setChecked(plot->BFixXIsChecked);
    PlotControl->BFixY->    setChecked(plot->BFixYIsChecked);

    PlotControl->BFixOXDir->setChecked(plot->BFixOXDirIsChecked);
    PlotControl->BFixOYDir->setChecked(plot->BFixOYDirIsChecked);
    PlotControl->BFixOXDir->setEnabled(plot->BFixOXDirIsEnabled);
    PlotControl->BFixOYDir->setEnabled(plot->BFixOYDirIsEnabled);
    if (PlotControl->BFixOXDir->isChecked())
        PlotControl->BFixOXDir->setIcon(makeHugeIcon(":/plot/markers_Y_On.png"));
    else
        PlotControl->BFixOXDir->setIcon(makeHugeIcon(":/plot/markers_Y.png"));
    if (PlotControl->BFixOYDir->isChecked())
        PlotControl->BFixOYDir->setIcon(makeHugeIcon(":/plot/markers_X_On.png"));
    else
        PlotControl->BFixOYDir->setIcon(makeHugeIcon(":/plot/markers_X.png"));

    PlotControl->BFixOXDir->setVisible(PlotControl->BMarkers->isChecked());
    PlotControl->BFixOYDir->setVisible(PlotControl->BMarkers->isChecked());

    if (plot->xScanning == -1)
    {
        PlotControl->AGSubtr->actions()[plot->CurrentSubtrOrder + 1]->setChecked(true);
        PlotControl->AGXFitOrders->actions()[plot->CurrentXFitOrder + 1]->setChecked(true);
        PlotControl->AGYFitOrders->actions()[plot->CurrentYFitOrder + 1]->setChecked(true);
        PlotControl->MMathProcessing->actions()[0]->setChecked(plot->MakeCorrectLineByMedianaXDirIsChecked);
        PlotControl->MMathProcessing->actions()[1]->setChecked(plot->MakeCorrectLineByMedianaYDirIsChecked);
        PlotControl->MMath2->actions()[PlotControl->GetNumOfAction(PlotControl->NameOfMeanValueFilter())]->setChecked(plot->MeanValueFilterSize >= 0);
        PlotControl->MMath2->actions()[PlotControl->GetNumOfAction(PlotControl->NameOfMedianValueFilter())]->setChecked(plot->MedianValueFilterSize >= 0);
        PlotControl->MMath2->actions()[PlotControl->GetNumOfAction(PlotControl->NameOfGaussianFilter())]->setChecked(plot->GaussianFilterSize >= 0);
        PlotControl->MMath2->actions()[PlotControl->GetNumOfAction(PlotControl->NameOfHistogramFilter())]->setChecked(plot->HistogramEqualization);
    }

    SetAllFilterSizesTextInAction();

	plot->blockRefresh = block;

    //if (PlotControl->BDetailedMarkersInfo->isChecked() != plot->BDetailedMarkersInfoIsChecked)
    {
        PlotControl->BDetailedMarkersInfo->setChecked(plot->BDetailedMarkersInfoIsChecked);
        on_BDetailedMarkersInfo_clicked();
    }
    PlotControl->BCrossection->setChecked(plot->BCrossectionIsChecked);
    if (plot->NOfPalette < PlotControl->AGPalettes->actions().size())
        PlotControl->AGPalettes->actions()[plot->NOfPalette]->setChecked(true);

    ResizeSliders();

    UpdateMinMaxInSlider();
    Slider->Sliders->SetMinMaxRelSlidersValues(plot->MinSliderVal, plot->MaxSliderVal);
    Slider->Sliders->setAutoMode(plot->AutoZScaleOn);
    UpdateSliderHistogram(plot->zhistogram);
}


void T3dPlot::InitPalettes()
{
    QFile plfile(QFile::exists("Param/palette.txt") ? "Param/palette.txt" : ":/config/palette.txt");
    plfile.open(QIODevice::ReadOnly | QIODevice::Text);
    QTextStream stream(&plfile);
    TPalette cv;
    QString name;
    int mode = 0;

    QVector<QString> names;

    if (stream.atEnd()) {
        cv.append(QColor(0, 0, 0));
        cv.append(QColor(190, 190, 190));
        cv.append(QColor(255, 255, 255));
        Palettes.append(cv);
        names.append("Gray");
        cv.clear();
    }
    while (!stream.atEnd()) {
        QString param;
        stream >> param;

        if (param == "#start") {
            stream >> param;
            if (param == "#RGB") {
                mode = 0;
            } else if (param == "#HEX") {
                mode = 1;
            }

            stream.readLine();//skip eol
            name = stream.readLine();
        } else if (param == "#end") {
            Palettes.append(cv);
            names.append(name);
            cv.clear();
        } else {
            if (mode == 0) {
                int r = param.toInt();

                stream >> param;
                int g = param.toInt();

                stream >> param;
                int b = param.toInt();

                cv.append(QColor(r, g, b));
            } else if (mode == 1) {
                cv.append(QColor(param));
            }
        }
    }


    names.append("Spectral 4d");
    for (int i = 0; i < names.size(); i++)
    {
        PlotControl->AGPalettes->addAction(PlotControl->TBPalettes->menu()->addAction(names.at(i)));
        PlotControl->AGPalettes->actions()[i]->setCheckable(true);
    }
    connect(PlotControl->AGPalettes, SIGNAL(triggered(QAction*)), this, SLOT(on_MPalettes_trigered()));

    for (auto &ThreeDPlot:ThreeDPlots)
        ThreeDPlot->SetPalette(Palettes.first(), 0, 1);
}
void T3dPlot::on_MPalettes_trigered()
{
    int NOfPalette = FindCheckedItemIn(PlotControl->AGPalettes);
    if (NOfPalette < 0)
        return;

    ThreeDPlots[ActivePlot]->NOfPalette = NOfPalette;
    on_Sliders_ValuesChanged(ThreeDPlots.at(ActivePlot)->MinSliderVal,
                             ThreeDPlots.at(ActivePlot)->MaxSliderVal);
}
void T3dPlot::on_Sliders_ValuesChanged(double Min, double Max)
{
    ThreeDPlots[ActivePlot]->MinSliderVal = Min;
    ThreeDPlots[ActivePlot]->MaxSliderVal = Max;

    TPalette palette;
    if (ThreeDPlots[ActivePlot]->NOfPalette >= Palettes.size()) //isSpectral
        palette = GetSpectralPalette(TwoDPlot->Pure2dPlot->PlotsXLimits,
                                     TwoDPlot->Pure2dPlot->GetSelectedRect().left(),
                                     TwoDPlot->Pure2dPlot->GetSelectedRect().right());
    else
        palette = Palettes.at(ThreeDPlots.at(ActivePlot)->NOfPalette);

    ThreeDPlots[ActivePlot]->SetPalette(palette, Min, Max);
    if (ThreeDPlots.at(ActivePlot)->HistogramEqualization && ThreeDPlots.at(ActivePlot)->isFiltering)
        ColorMapHistogramEqualization(ThreeDPlots.at(ActivePlot)->zhistogram, ThreeDPlots.at(ActivePlot)->colormap, Min, Max);

    ThreeDPlots[ActivePlot]->ReplotChanged();
    Slider->Sliders->SetColourMap(ThreeDPlots.at(ActivePlot)->colormap);
    
    emit PaletteChanged(palette);
}
TPalette T3dPlot::GetSpectralPalette(const Range &ZLimits, double X1, double X2)
{
    double Centre = (X1 + X2) / 2.;
    if (ZLimits[0] == ZLimits[1] || Centre < ZLimits[0] || Centre > ZLimits[1]) {
        TPalette cv;
        cv.append(QColor(0, 0, 0));
        cv.append(QColor(190, 190, 190));
        cv.append(QColor(255, 255, 255));
        return cv;
    }

    double k = 300 / (ZLimits[1] - ZLimits[0]);
    double HalfDeltaH = k * (X2 - X1) / 2.;
    double H = 300 - k * (Centre - ZLimits[0]);

    TPalette palette;
    int Size = 20;
    for (int i = 0; i < Size; i++)
    {
        double p = (double)i / (Size - 1);
        QColor Colour;
        double h = H - HalfDeltaH + 2*HalfDeltaH*p;
        if (HalfDeltaH < 40) {
            if (i < Size / 2)
                Colour.setHsv(h, 255, 255*p*2);
            else
                Colour.setHsv(h, 255*2*(1 - p), 255);
        } else
            Colour.setHsv(h, 255, 255);
        palette.append(Colour);
    }

    return palette;
}
bool T3dPlot::eventFilter(QObject *Object, QEvent *Event)
{
    if (Event->type() == QEvent::Resize)
        fixedSplit->setSizes(QList<int>() << fixedSplit->width()-1 << 1);

    if (Event->type() == TSelectEvent::id)
        if (ThreeDPlots.at(ActivePlot)->NOfPalette >= Palettes.size()) //isSpectral
            //TSelectEvent *e = (TSelectEvent *)Event; e->SelectedRect.left(), e->SelectedRect.right();
            on_Sliders_ValuesChanged(ThreeDPlots.at(ActivePlot)->MinSliderVal,
                                     ThreeDPlots.at(ActivePlot)->MaxSliderVal);

    if (Event->type() == TSelectEvent::id || Event->type() == TCrossEvent::id || Event->type() == TScalesChangedEvent::id)
    {
        if (Parent) QCoreApplication::sendEvent(Parent, Event);
    }

    return QWidget::eventFilter(Object, Event);
}


void T3dPlot::on_BGrid_clicked()
{
    ThreeDPlots[ActivePlot]->BGrid_clicked(PlotControl->BGrid->isChecked());
    ThreeDPlots[ActivePlot]->BGridIsChecked = PlotControl->BGrid->isChecked();
}
void T3dPlot::on_BDetailedMarkersInfo_clicked()
{
    MarkersInfo->setVisible(PlotControl->BDetailedMarkersInfo->isChecked());
    ThreeDPlots[ActivePlot]->BDetailedMarkersInfoIsChecked = PlotControl->BDetailedMarkersInfo->isChecked();
    UpdateAllMarkersInfo();
}
void T3dPlot::on_BAngle_clicked()
{
    ThreeDPlots[ActivePlot]->BAngle_clicked(PlotControl->BAngle->isChecked());
    ThreeDPlots[ActivePlot]->BAngleIsChecked = PlotControl->BAngle->isChecked();
}
void T3dPlot::on_BMarkers_clicked(bool on)
{
    if (on)
        PlotControl->BCross->change_state_man_with_sig(false);

    ThreeDPlots[ActivePlot]->BMarkersIsChecked = on;
    ThreeDPlots[ActivePlot]->BMarkers_clicked(on);

    if (!on) {
        PlotControl->BDetailedMarkersInfo->setChecked(false);
        PlotControl->BAngle->setChecked(false);
        //PlotControl->BCrossection->setChecked(false);
    }
    PlotControl->BMarkers->setIcon(makeHugeIcon(on ? ":/plot/markersOn.png" : ":/plot/markers.png"));
    PlotControl->BDetailedMarkersInfo->setVisible(on);
    PlotControl->BAngle->setVisible(on);
    PlotControl->BCrossection->setChecked(on);
    PlotControl->BFixOYDir->setVisible(on);
    PlotControl->BFixOXDir->setVisible(on);
}
void T3dPlot::on_BFixX_toggled()
{
    for (auto &ThreeDPlot:ThreeDPlots)
        ThreeDPlot->BFixXIsChecked = PlotControl->BFixX->isChecked();
}
void T3dPlot::on_BFixY_toggled()
{
    for (auto &ThreeDPlot:ThreeDPlots)
        ThreeDPlot->BFixYIsChecked = PlotControl->BFixY->isChecked();
}
void T3dPlot::on_BRestoreScale_clicked()
{
    for (auto &ThreeDPlot:ThreeDPlots)
    {
        if (overlayButtons->blockResizeTimes < 2)
            overlayButtons->blockResizeTimes = 2; //fix for wrong resize buttons bug
        ThreeDPlot->BRestoreScale_clicked();
    }

    if (Parent)
        QCoreApplication::postEvent(Parent,
                                    new TScalesChangedEvent(ThreeDPlots.at(0)->PlotsXLimits,
                                                            ThreeDPlots.at(0)->PlotsYLimits,
                                                            TScalesChangedEvent::R_BRestoreScale_clicked));
}
void T3dPlot::on_BAreaZzoom_clicked()
{
    for (auto &ThreeDPlot:ThreeDPlots)
        ThreeDPlot->AreaZzoom = PlotControl->BAreaZzoom->isChecked();
}
void T3dPlot::on_BCross_clicked()
{
    for (auto &ThreeDPlot:ThreeDPlots)
    {
        ThreeDPlot->Cross->setVisible(ThreeDPlot->BCrossIsChecked = PlotControl->BCross->isOn);
        ThreeDPlot->Replot();
    }
}
void T3dPlot::on_BSelectByRect_clicked()
{
    PlotControl->SetSelectionModeEnabled(PlotControl->BSelectByRect->isChecked());
    if (PlotControl->BSelectByRect->isChecked())
    {
        for (int i = 0; i < ThreeDPlots.size(); i++)
        {
            ThreeDPlots[i]->QPlot->hide();
            ThreeDPlots[i]->SetSelectionModeEnabled(i == ActivePlot);
            ThreeDPlots[i]->SetPlotEnabled(i == ActivePlot);
        }
        if (Parent) QCoreApplication::postEvent(Parent, new TSelectEvent(ThreeDPlots.at(ActivePlot)->GetSelectedRect(), TSelectEvent::St_SelectionStarted));
    }
    else
    {
        ThreeDPlots[ActivePlot]->SetSelectionModeEnabled(false);
        for (auto &ThreeDPlot:ThreeDPlots)
        {
            ThreeDPlot->SetPlotEnabled(true);
            ThreeDPlot->QPlot->show();
            ThreeDPlot->ReAssignPlotOptions();
        }
        ThreeDPlots[ActivePlot]->unsetCursor();
    }
    //ThreeDPlots[ActivePlot]->setFocus(Qt::MouseFocusReason);
    ThreeDPlots[ActivePlot]->ResizeQPlot();
    ThreeDPlots[ActivePlot]->setFocus();
}
void T3dPlot::on_BApply_clicked()
{
    if (Parent) QCoreApplication::postEvent(Parent, new TSelectEvent(ThreeDPlots.at(ActivePlot)->GetSelectedRect(), TSelectEvent::St_SelectionApplied));
    PlotControl->on_BApply_clicked();
    on_BSelectByRect_clicked();
    ThreeDPlots[ActivePlot]->SelectDone();
    for (int i = 0; i < ThreeDPlots.size(); i++)
    {
        if (i == ActivePlot) continue;
        if (!ThreeDPlots[i]->BFixXIsChecked)
            ThreeDPlots[i]->SetPlotsXScales(ThreeDPlots.at(ActivePlot)->PlotsXLimits[0], ThreeDPlots.at(ActivePlot)->PlotsXLimits[1]);
        if (!ThreeDPlots[i]->BFixYIsChecked)
            ThreeDPlots[i]->SetPlotsYScales(ThreeDPlots.at(ActivePlot)->PlotsYLimits[0], ThreeDPlots.at(ActivePlot)->PlotsYLimits[1]);
        ThreeDPlots[i]->ResizeQPlot();
    }
    if (ThreeDPlots[ActivePlot]->ZoomBySelection && Parent)
        QCoreApplication::postEvent(Parent,
                                    new TScalesChangedEvent(ThreeDPlots.at(ActivePlot)->PlotsXLimits,
                                                            ThreeDPlots.at(ActivePlot)->PlotsYLimits,
                                                            TScalesChangedEvent::R_BApply_clicked));
    ThreeDPlots[ActivePlot]->ResizeQPlot();
}
void T3dPlot::on_BCancel_clicked()
{
    Range *tXLims = new Range[ThreeDPlots.size()];
    Range *tYLims = new Range[ThreeDPlots.size()];
    for (int i = 0; i < ThreeDPlots.size(); i++)
    {
        tXLims[i] = Range(ThreeDPlots[i]->GetCurXLims()->arr[0], ThreeDPlots[i]->GetCurXLims()->arr[1]);
        tYLims[i] = Range(ThreeDPlots[i]->GetCurYLims()->arr[0], ThreeDPlots[i]->GetCurYLims()->arr[1]);
    }
    PlotControl->BSelectByRect->setVisible(true);
    PlotControl->BSelectByRect->setChecked(false);
    on_BSelectByRect_clicked();
    if (Parent) QCoreApplication::postEvent(Parent, new TSelectEvent(ThreeDPlots.at(ActivePlot)->GetSelectedRect(), TSelectEvent::St_SelectionCancled));
    ThreeDPlots[ActivePlot]->ResizeQPlot();
}


void T3dPlot::on_BSavePlotPixmap_clicked()
{
    QString SelectedFilter;
    QString FileName;

#ifdef _ADROID
    FileName = "/sdcard/";
#endif

    FileName = QFileDialog::getSaveFileName(this,
                                            tr("Save plot as picture..."),
                                            FileName,
                                            tr("*.png;;*.jpg;;*.tiff;;*.bmp"),
                                            &SelectedFilter,
                                            QFileDialog::DontUseNativeDialog);

    if (FileName.isEmpty())
        return;

    if (!FileName.endsWith(SelectedFilter.remove(0, 1)))
        FileName += SelectedFilter;

    QImage graph = QPixmap::grabWidget(ThreeDPlots[ActivePlot]).toImage();
    int imgWidth = graph.width();
    QImage slider;

    if (Slider->isVisible())
    {
        slider = QPixmap::grabWidget(Slider).toImage().scaledToHeight(graph.height(), Qt::SmoothTransformation);
        imgWidth += slider.width();
    }

    QImage composition(imgWidth, graph.height(), graph.format());
    QPainter painter(&composition);
    painter.drawImage(0, 0, graph);
    if (Slider->isVisible())
        painter.drawImage(graph.width(), 0, slider);

    composition.save(FileName, SelectedFilter.remove(0, 1).toAscii().data());
}

void T3dPlot::appendAttrib(plot::TMetaStorage &attributes, int N)
{
    auto plot = ThreeDPlots.at(N);
    bool isFiltering = plot->isFiltering;
    attributes.insert("XFit", (isFiltering ? plot->CurrentXFitOrder : -1) + 1);
    attributes.insert("YFit", (isFiltering ? plot->CurrentYFitOrder : -1) + 1);
    attributes.insert("Substract", (isFiltering ? plot->CurrentSubtrOrder : -1) + 1);
    attributes.insert("gwyXFit", isFiltering ? plot->MakeCorrectLineByMedianaXDirIsChecked : false);
    attributes.insert("gwyYFit", isFiltering ? plot->MakeCorrectLineByMedianaYDirIsChecked : false);

    attributes.insert("Mean", (isFiltering && plot->MeanValueFilterEnabled) ? plot->MeanValueFilterSize : -1);
    attributes.insert("Median", (isFiltering && plot->MedianValueFilterEnabled) ? plot->MedianValueFilterSize : -1);
    attributes.insert("Gaussian", (isFiltering && plot->GaussianValueFilterEnabled) ? plot->GaussianFilterSize : -1);
    attributes.insert("HistEq", isFiltering ? plot->HistogramEqualization : false);

    attributes.insert("Palette", plot->NOfPalette);
    attributes.insert("ZMin View", plot->MinSliderVal);
    attributes.insert("ZMax View", plot->MaxSliderVal);
    attributes.insert("ZViewAuto", plot->AutoZScaleOn);
}
void T3dPlot::restoreAttributes(const plot::TMetaStorage &attributes, int N)
{
	auto plot = ThreeDPlots.at(N);
    bool block = plot->blockRefresh;
	plot->blockRefresh = true;

    SetTitle(attributes.value("Name", tr("Scan")).toString(), N);
    SetXYDescAndSIUnit(attributes.value("XYLabel", "").toString(),
                       attributes.value("XYUnit", tr("m")).toString(), N);
    SetZDescAndSIUnit(attributes.value("ZLabel", "").toString(),
                      attributes.value("ZUnit", tr("Counts")).toString(), N);
    SetAllXScales(attributes.value("XMin", 0).toDouble(),
                  attributes.value("XMax", 1).toDouble());
    SetAllYScales(attributes.value("YMin", 0).toDouble(),
                  attributes.value("YMax", 1).toDouble());

    plot->CurrentXFitOrder = attributes.value("XFit", 0).toInt() - 1;
    plot->CurrentYFitOrder = attributes.value("YFit", 0).toInt() - 1;
    plot->CurrentSubtrOrder = attributes.value("Substract", 0).toInt() - 1;
    plot->MakeCorrectLineByMedianaXDirIsChecked = attributes.value("gwyXFit", false).toBool();
    plot->MakeCorrectLineByMedianaYDirIsChecked = attributes.value("gwyYFit", false).toBool();
    plot->MeanValueFilterSize = attributes.value("Mean", -1).toInt();
    plot->MedianValueFilterSize = attributes.value("Median", -1).toInt();
    plot->GaussianFilterSize = attributes.value("Gaussian", -1).toInt();
    plot->HistogramEqualization = attributes.value("HistEq", false).toBool();

    plot->NOfPalette = attributes.value("Palette", 0).toInt();
    plot->MinSliderVal = attributes.value("ZMin View", 0).toDouble();
    plot->MaxSliderVal = attributes.value("ZMax View", 1).toDouble();
    plot->AutoZScaleOn = attributes.value("ZViewAuto", false).toBool();

    //apply
    int prevActive = ActivePlot;
    bool blockActive = ThreeDPlots[ActivePlot]->blockRefresh;
    ThreeDPlots[ActivePlot]->blockRefresh = true;

    RestoreEnvironmentState(ActivePlot = N);
    if (N != prevActive)
        RestoreEnvironmentState(ActivePlot = prevActive);

    ThreeDPlots[ActivePlot]->blockRefresh = blockActive; //order for N == Active
    ThreeDPlots[N]->blockRefresh = block;

    ThreeDPlots[N]->Refresh();
}
void T3dPlot::on_BSave_clicked()
{
    T3dDataOwner src;
    TMetaStorage attributes;
    if (!SavePlot(ActivePlot, &src, &attributes))
        return;

    emit AddDataToContainer(src, attributes);

    auto plot = ThreeDPlots.at(ActivePlot);
    bool filterOn = plot->isFiltering;
    if (filterOn)
    {
        plot->isFiltering = filterOn;
        plot->Replot();
        plot->ReDrawSelectedRectangle();
        plot->Refresh();
    }
}
bool T3dPlot::SavePlot(int plotId, T3dDataOwner *src, TMetaStorage *attributes)
{
    auto plot = ThreeDPlots.at(plotId);
    if (plot->SourceData.empty() || !plot->SourceData.at(0))
        return false;

    bool filterOn = plot->isFiltering;
    if (filterOn)
    {
        plot->isFiltering = false;
        plot->Replot();
    }

    TMin3dData *data = plot->SourceData.at(0).data();
    #ifdef _SCAN_SLICE
    if (plot->sliceBounds)
    {
//        auto numX = data->XLimits[1] - data->XLimits[0];
//        auto numY = data->YLimits[1] - data->YLimits[0];
//        double ammountX = plot->slicePercent*numX/100.;
//        double ammountY = plot->slicePercent*numY/100.;
//        double minX = data->XLimits[0] + ammountX;
//        double minY = data->YLimits[0] + ammountY;
//        QRectF selectedRect(minX, minY, numX - minX*2, numY - minY*2);
//        data = data->Crop(selectedRect, data->XLimits, data->YLimits);

        auto numX = data->LimsForMathX[1]-1 - data->LimsForMathX[0];
        auto numY = data->LimsForMathY[1]-1 - data->LimsForMathY[0];
        double ammountX = ThreeDPlots.at(ActivePlot)->slicePercent*numX/100.;
        double ammountY = ThreeDPlots.at(ActivePlot)->slicePercent*numY/100.;
        if (data->LimsForMathX[1]-1-ammountX - (data->LimsForMathX[0]+ammountX) < 0)
            return false;
        if (data->LimsForMathY[1]-1-ammountY - (data->LimsForMathY[0]+ammountY) < 0)
            return false;
        data = data->CropIndex(data->LimsForMathX[0]+ammountX, data->LimsForMathY[0]+ammountX,
                data->LimsForMathX[1]-1-ammountY, data->LimsForMathY[1]-1-ammountY);
    }
    else
#endif
    {
        if (data->LimsForMathX[1] == 0 && data->LimsForMathY[1] == 0)
        {
            if (filterOn)
            {
                plot->isFiltering = true;
                plot->Replot();
            }
            return false;
        }
        data = data->CropIndex(data->LimsForMathX[0],   data->LimsForMathY[0],
                               data->LimsForMathX[1]-1, data->LimsForMathY[1]-1);
    }
    src->append(QSharedPointer<TMin3dData>(data));

    double minX = data->XLimits[0];
    double minY = data->YLimits[0];
    double maxX = data->XLimits[1];
    double maxY = data->YLimits[1];

//    if (data->NYElems < 2 || data->NXElems < 2 ||
//            (maxX - minX) / (maxY - minY) > 40 ||
//            (maxY - minY) / (maxX - minX) > 40)
//    {
//        if (filterOn)
//        {
//            plot->isFiltering = true;
//            plot->Replot();
//        }
//        return false;
//    }

    attributes->insert("Preview", QVariant::fromValue(MakePreview(plotId)));

    appendAttrib(*attributes, plotId);

    attributes->insert("XPos", minX);
    attributes->insert("YPos", minY);

    attributes->insert("XMin", 0);
    attributes->insert("XMax", maxX - minX);
    attributes->insert("YMin", 0);
    attributes->insert("YMax", maxY - minY);

    attributes->insert("XYLabel", plot->XYAxisUnit.Description);
    attributes->insert("XYUnit", plot->XYAxisUnit.SIUnit);
    attributes->insert("ZLabel", plot->ZAxisUnit.Description);
    attributes->insert("ZUnit", plot->ZAxisUnit.SIUnit);

    if (filterOn)
    {
        plot->isFiltering = true;
        plot->Replot();
    }
    return true;
}
void T3dPlot::AutoSave(int Ind)
{
    int Curr = ActivePlot;

    SetActivePlot(Ind);
    on_BSave_clicked();

    SetActivePlot(Curr);
}
void T3dPlot::on_BExpandSelectRect_clicked()
{
    ThreeDPlots[ActivePlot]->ExpandSelectionRect();
    ThreeDPlots[ActivePlot]->ReDrawSelectedRectangle(TSelectEvent::SRO_Mouse);
}
void T3dPlot::on_FontSize_changed()
{
    int size = PlotControl->AGFontSizes->checkedAction()->text().toInt();
    for (auto &ThreeDPlot:ThreeDPlots)
        ThreeDPlot->SetFontSize(size);

    QFont font = qApp->font(); font.setPointSize(size);
    Slider->Title->setFont(font);
    Slider->Sliders->QwtBar->setFont(font);
}
void T3dPlot::on_BCrossection_clicked()
{
    ThreeDPlots[ActivePlot]->BCrossectionIsChecked = PlotControl->BCrossection->isChecked();
    if (PlotControl->BCrossection->isChecked())
    {
        TwoDPlot->SetTitle(QObject::tr("section"));
        TwoDPlotVertical->SetTitle(QObject::tr("section"));
        Set2dPlotVisible(true);
        AdjustTwoDPlot();
        if (TwoDPlot->overlayButtons->constantShowingMenu)
        {
            TwoDPlot->overlayButtons->setVisibleGUI2D(false);
            TwoDPlot->overlayButtons->setVisibleGUI2D(true);
        }

        PlotControl->BCrossection->setIcon(makeHugeIcon(":/plot/cross_sectionOn.png"));
    }
    else
    {
        curveData.clear();
        //Set2dPlotVisible(false);
        PlotControl->BCrossection->setIcon(makeHugeIcon(":/plot/cross_section.png"));
    }
}


void T3dPlot::on_GwyXFit_triggered(bool Checked)
{
    ThreeDPlots[ActivePlot]->MakeCorrectLineByMedianaXDir(Checked);
}
void T3dPlot::on_GwyYFit_triggered(bool Checked)
{
    ThreeDPlots[ActivePlot]->MakeCorrectLineByMedianaYDir(Checked);
}


void T3dPlot::SetFilterSizeTextInAction(QAction *Act, const int Size, const QString &FilterName)
{
    if (Size > 0) Act->setText(FilterName + " (" + QString::number(Size) + "px.)");
    else Act->setText(FilterName);
}
void T3dPlot::SetAllFilterSizesTextInAction()
{
    SetFilterSizeTextInAction(PlotControl->MMath2->actions()[PlotControl->GetNumOfAction(PlotControl->NameOfMeanValueFilter())],
                              ThreeDPlots.at(ActivePlot)->MeanValueFilterSize, PlotControl->NameOfMeanValueFilter());
    SetFilterSizeTextInAction(PlotControl->MMath2->actions()[PlotControl->GetNumOfAction(PlotControl->NameOfMedianValueFilter())],
                              ThreeDPlots.at(ActivePlot)->MedianValueFilterSize, PlotControl->NameOfMedianValueFilter());
    SetFilterSizeTextInAction(PlotControl->MMath2->actions()[PlotControl->GetNumOfAction(PlotControl->NameOfGaussianFilter())],
                              ThreeDPlots.at(ActivePlot)->GaussianFilterSize, PlotControl->NameOfGaussianFilter());
    SetFilterSizeTextInAction(PlotControl->MMath2->actions()[PlotControl->GetNumOfAction(PlotControl->NameOfHistogramFilter())],
                              0, PlotControl->NameOfHistogramFilter());
}
void T3dPlot::MakeMeanValue(int Size)
{
    ThreeDPlots[ActivePlot]->MakeMeanValue(Size);
    SetFilterSizeTextInAction(PlotControl->MMath2->actions()[PlotControl->GetNumOfAction(PlotControl->NameOfMeanValueFilter())], Size,
                              PlotControl->NameOfMeanValueFilter());
}
void T3dPlot::MakeMedianValue(int Size)
{
    ThreeDPlots[ActivePlot]->MakeMedianValue(Size);
    SetFilterSizeTextInAction(PlotControl->MMath2->actions()[PlotControl->GetNumOfAction(PlotControl->NameOfMedianValueFilter())], Size,
                              PlotControl->NameOfMedianValueFilter());
}
void T3dPlot::MakeGaussian(int Size)
{
    ThreeDPlots[ActivePlot]->MakeGaussian(Size);
    SetFilterSizeTextInAction(PlotControl->MMath2->actions()[PlotControl->GetNumOfAction(PlotControl->NameOfGaussianFilter())], Size,
                              PlotControl->NameOfGaussianFilter());
}

void T3dPlot::on_MeanValue_triggered(bool Checked)
{
    if (Checked) {
        TFilterSizeWidget FilterSizeWidget(this);
        FilterSizeWidget.setWindowTitle(tr("Mean filter size"));
        connect(&FilterSizeWidget, SIGNAL(SliderValueChanged(int)), this, SLOT(MakeMeanValue(int)));
        FilterSizeWidget.SetFilterSize(TFilterSizeWidget::DefaultFilterSize);
        Checked = FilterSizeWidget.exec();
    }
    if (!Checked) {
        MakeMeanValue(-1);
        PlotControl->MMath2->actions()[PlotControl->GetNumOfAction(PlotControl->NameOfMeanValueFilter())]->setChecked(false);
    }
}
void T3dPlot::on_MedianValue_triggered(bool Checked)
{
    if (Checked) {
        TFilterSizeWidget FilterSizeWidget(this);
        FilterSizeWidget.setWindowTitle(tr("Median filter size"));
        connect(&FilterSizeWidget, SIGNAL(SliderValueChanged(int)), this, SLOT(MakeMedianValue(int)));
        FilterSizeWidget.SetFilterSize(TFilterSizeWidget::DefaultFilterSize);
        Checked = FilterSizeWidget.exec();
    }
    if (!Checked) {
        MakeMedianValue(-1);
        PlotControl->MMath2->actions()[PlotControl->GetNumOfAction(PlotControl->NameOfMedianValueFilter())]->setChecked(false);
    }
}
void T3dPlot::on_Gaussian_triggered(bool Checked)
{
    if (Checked) {
        TFilterSizeWidget FilterSizeWidget(this);
        FilterSizeWidget.setWindowTitle(tr("Gaussian filter size"));
        connect(&FilterSizeWidget, SIGNAL(SliderValueChanged(int)), this, SLOT(MakeGaussian(int)));
        FilterSizeWidget.SetFilterSize(TFilterSizeWidget::DefaultFilterSize);
        Checked = FilterSizeWidget.exec();
    }
    if (!Checked) {
        MakeGaussian(-1);
        PlotControl->MMath2->actions()[PlotControl->GetNumOfAction(PlotControl->NameOfGaussianFilter())]->setChecked(false);
    }
}
void T3dPlot::on_Histogram_triggered(bool Checked)
{
    ThreeDPlots[ActivePlot]->MakeHistogram(Checked);
}

void T3dPlot::on_FitXOrder_changed()
{
    ThreeDPlots[ActivePlot]->MakeXFit(FindCheckedItemIn(PlotControl->AGXFitOrders) - 1);
}
void T3dPlot::on_FitYOrder_changed()
{
    ThreeDPlots[ActivePlot]->MakeYFit(FindCheckedItemIn(PlotControl->AGYFitOrders) - 1);
}
void T3dPlot::on_SubtrOrder_changed()
{
    ThreeDPlots[ActivePlot]->MakeSubtr(FindCheckedItemIn(PlotControl->AGSubtr) - 1);
}
void T3dPlot::on_AutoZScale_changed(bool on)
{
    ThreeDPlots[ActivePlot]->AutoZScaleOn = on;
    if (on) {
        auto plot = ThreeDPlots.at(ActivePlot);
        plot->UpdateHistogram();
        Slider->Sliders->SetMinMaxRelSlidersValues(plot->MinSliderVal, plot->MaxSliderVal);
    }
}

void T3dPlot::on_BFixOXDir_clicked()
{
    if (PlotControl->BFixOYDir->isChecked()) {
        ThreeDPlots[ActivePlot]->BFixOYDirIsChecked = false;
        PlotControl->BFixOYDir->setChecked(false);
        ThreeDPlots[ActivePlot]->on_BFixOYDir_clicked();

        PlotControl->BFixOYDir->setIcon(makeHugeIcon(":/plot/markers_X.png"));
    }

    if (PlotControl->BFixOXDir->isChecked())
        PlotControl->BFixOXDir->setIcon(makeHugeIcon(":/plot/markers_Y_On.png"));
    else
        PlotControl->BFixOXDir->setIcon(makeHugeIcon(":/plot/markers_Y.png"));

    ThreeDPlots[ActivePlot]->BFixOXDirIsChecked = PlotControl->BFixOXDir->isChecked();
    ThreeDPlots[ActivePlot]->on_BFixOXDir_clicked();
}
void T3dPlot::on_BFixOYDir_clicked()
{
    if (PlotControl->BFixOXDir->isChecked()) {
        ThreeDPlots[ActivePlot]->BFixOXDirIsChecked = false;
        PlotControl->BFixOXDir->setChecked(false);
        ThreeDPlots[ActivePlot]->on_BFixOXDir_clicked();

        PlotControl->BFixOXDir->setIcon(makeHugeIcon(":/plot/markers_Y.png"));
    }

    if (PlotControl->BFixOYDir->isChecked())
        PlotControl->BFixOYDir->setIcon(makeHugeIcon(":/plot/markers_X_On.png"));
    else
        PlotControl->BFixOYDir->setIcon(makeHugeIcon(":/plot/markers_X.png"));

    ThreeDPlots[ActivePlot]->BFixOYDirIsChecked = PlotControl->BFixOYDir->isChecked();
    ThreeDPlots[ActivePlot]->on_BFixOYDir_clicked();
}
void T3dPlot::UpdateAllMarkersInfo()
{
    TPure3dPlot *Plt;
    for (int i = 0; i < ThreeDPlots.size(); i++) {
         Plt = ThreeDPlots[i];
         if (Plt->BMarkersIsChecked)
            Plt->UpdateLabels(Plt->BDetailedMarkersInfoIsChecked && i == ActivePlot);
    }

    Plt = ThreeDPlots[ActivePlot];
    if (!Plt->BMarkersIsChecked || !Plt->BDetailedMarkersInfoIsChecked)
        return;

    MarkersInfo->LEdXA->setText  (Plt->AMarkerInfo.XInfo.Value);
    MarkersInfo->LXAUnit->setText(Plt->AMarkerInfo.XInfo.Unit);
    MarkersInfo->LEdYA->setText  (Plt->AMarkerInfo.YInfo.Value);
    MarkersInfo->LYAUnit->setText(Plt->AMarkerInfo.YInfo.Unit);
    MarkersInfo->LEdZA->setText  (Plt->AMarkerInfo.ZInfo.Value);
    MarkersInfo->LZAUnit->setText(Plt->AMarkerInfo.ZInfo.Unit);

    MarkersInfo->LEdXB->setText  (Plt->BMarkerInfo.XInfo.Value);
    MarkersInfo->LXBUnit->setText(Plt->BMarkerInfo.XInfo.Unit);
    MarkersInfo->LEdYB->setText  (Plt->BMarkerInfo.YInfo.Value);
    MarkersInfo->LYBUnit->setText(Plt->BMarkerInfo.YInfo.Unit);
    MarkersInfo->LEdZB->setText  (Plt->BMarkerInfo.ZInfo.Value);
    MarkersInfo->LZBUnit->setText(Plt->BMarkerInfo.ZInfo.Unit);

    double xdist = Plt->BMarkerInfo.XInfo.SrcValue - Plt->AMarkerInfo.XInfo.SrcValue;
    double ydist = Plt->BMarkerInfo.YInfo.SrcValue - Plt->AMarkerInfo.YInfo.SrcValue;
    double zdist = Plt->BMarkerInfo.ZInfo.SrcValue - Plt->AMarkerInfo.ZInfo.SrcValue;

    TValueInfoCreator ValueInfo;
    ValueInfo(Plt->AutoLegendEnabled, Plt->XYAxisUnit, xdist);
    MarkersInfo->LEdDeltaX->setText(ValueInfo.Value); MarkersInfo->LDeltaXUnit->setText(ValueInfo.Unit);

    ValueInfo(Plt->AutoLegendEnabled, Plt->XYAxisUnit, ydist);
    MarkersInfo->LEdDeltaY->setText(ValueInfo.Value); MarkersInfo->LDeltaYUnit->setText(ValueInfo.Unit);

    ValueInfo(Plt->AutoLegendEnabled, Plt->ZAxisUnit, zdist);
    MarkersInfo->LEdDeltaZ->setText(ValueInfo.Value); MarkersInfo->LDeltaZUnit->setText(ValueInfo.Unit);

    double Distance = sqrt(xdist * xdist + ydist * ydist);
    if (Plt->XYAxisUnit.SIUnit == Plt->ZAxisUnit.SIUnit)
        Distance = sqrt(xdist * xdist + ydist * ydist + zdist * zdist);

    ValueInfo(Plt->AutoLegendEnabled, Plt->XYAxisUnit, Distance);
    MarkersInfo->LEdDistance->setText(ValueInfo.Value); MarkersInfo->LDistanceUnit->setText(ValueInfo.Unit);

    MarkersInfo->LEdAngle->setText(QString::number(Plt->CalcAngle(), 'f', 1));
}
void T3dPlot::MakeCrossection()
{
	auto activePlot = ThreeDPlots.at(ActivePlot);
    double xA = activePlot->MarkerA->xValue();
    double xB = activePlot->MarkerB->xValue();
    double yA = activePlot->MarkerA->yValue();
    double yB = activePlot->MarkerB->yValue();
    double DeltaX = xB - xA, DeltaY = yB - yA;
    double NumberOfSteps = 1000;//TODO activePlot->GetMaxNumberOfPointsXY(xA, yA, xB, yB);
    double x = xA, y = yA, stepx, stepy;

    if (xA > xB)
        qSwap(xA, xB);
    if (yA > yB)
        qSwap(yA, yB);

    ResizeOwner(curveData, 1);
    curveData[0]->YSeq.clear();

    stepx = DeltaX / (NumberOfSteps - 1);
    stepy = DeltaY / (NumberOfSteps - 1);
    curveData[0]->setXCalib(0, QLineF(0, 0, stepx, stepy).length());
    for (int i = 0; i < NumberOfSteps; i++, x += stepx, y += stepy)
        curveData[0]->AddY(activePlot->value(qBound(xA, x, xB), qBound(yA, y, yB)));

    TwoDPlot->Pure2dPlot->SourceData = getUnLock(curveData);
    int lines = TwoDPlot->Pure2dPlot->SourceData.count();
    QPen **pen = new QPen*[lines];
    for (int i=0; i<lines; i++)
    {
        if (TwoDPlot->isVisible())
        {
            pen[i] = new QPen(TwoDPlot->lineColors[i]);
            pen[i]->setWidth(TwoDPlot->lineWidth);
        }
        else
        {
            pen[i] = new QPen(TwoDPlotVertical->lineColors[i]);
            pen[i]->setWidth(TwoDPlotVertical->lineWidth);
        }
        //qDebug()<<"=="<<pen[i]->color();
    }

    TwoDPlot->Pure2dPlot->SourceData = *TwoDPlot->Pure2dPlot->cloneSourceData(pen);
    TwoDPlotVertical->Pure2dPlot->SourceData = *TwoDPlot->Pure2dPlot->cloneSourceData(pen);

    AdjustAxisLegends();
    TwoDPlot->Refresh();
    TwoDPlotVertical->Refresh();
    TwoDPlotVertical->Pure2dPlot->SetAutoScaleEnabled(true, false);
    TwoDPlot->Pure2dPlot->SetAutoScaleEnabled(true, false);
}
void T3dPlot::UpdateMinMaxInSlider()
{
    double ZMult = 1.;
    QString Unit;

	auto activePlot = ThreeDPlots.at(ActivePlot);
    if (activePlot->AutoLegendEnabled) {
        ZMult = activePlot->ZAxisUnit.Multiplier;
        Unit = activePlot->ZAxisUnit.StrMultiplier + activePlot->ZAxisUnit.SIUnit;
    }

    Slider->Sliders->SetMinMaxValues(activePlot->PlotsZLimits[0] / ZMult, activePlot->PlotsZLimits[1] / ZMult);
    Slider->Title->setText(Unit);
}

template <int N>
void ColorMapHistogramEqualization(const THistogram<N> &zhistogram, QwtLinearColorMap *map, double from, double to)
{
    from *= N; //from [0..1]
    to *= N;

    //Piecewise Affine Histogram Equalization, Jose Luis Lisani, Ana Belen Petro, Catalina Sbert 2012
    //http://dx.doi.org/10.5201/ipol.2012.lps-pae
#define HIST_EQUAL_PIECES 20
    int hist_part = (int)(to) - (int)from + 1;
    int pieces = qMin(hist_part, HIST_EQUAL_PIECES);
    if (pieces < 2)
        return;

    QVector<double> cumm_hist_normal(hist_part);
    int acc = 0;
    for (int i = (int)from; i <= (int)(to); i++) {
        double active_part = qMin(1., qMin(to - i, i+1 - from));
        acc += zhistogram.bins[i] * active_part;
        cumm_hist_normal[i - (int)from] = acc;
    }
    if (cumm_hist_normal.last() < 3)
        return;
    //normalize
    for (int i = 0; i < hist_part; i++)
        cumm_hist_normal[i] /= cumm_hist_normal.last();

    QVector<double> newposx(pieces); //partition histogram in equal-volume pieces
    int j = 1; //j: histogram index
    for (int i = 0; i < pieces; i++) {
        double piece_threshold = double(i) / (pieces - 1);
        if (i == 0)
            piece_threshold = 0;
        while (j < hist_part && cumm_hist_normal.at(j) < piece_threshold)
            j++;

        double part = (piece_threshold - cumm_hist_normal.at(j-1)) /
                (cumm_hist_normal.at(j) - cumm_hist_normal.at(j-1));
        double pos = (((j-1) + part) - (from-(int)from)) / (to - from);
        newposx[i] = qBound(0., pos, 1.);
    }
    //normalize range
    for (int i = 0; i < pieces; i++)
        ;//newposx[i] /= newposx.last(); //adds some flicker

    QVector<double> newposy(pieces); //palette partition
    newposy[0] = 0;
    //limit stretch
    for (int i = 1; i < pieces; i++) {
        newposy[i] = i / double(pieces - 1); //regular initially
        double k = newposy[i] - newposy[i-1];
        double scale = newposx[i] - newposx[i-1];
        k /= scale;
        //qDebug() << i << k;
        if (k < 0.3 || k > 3.) {
            k = qBound(0.3, k, 3.); //limit
            newposy[i] = k * scale + newposy[i-1];
        }
    }
    //normalize range
    for (int i = 0; i < pieces; i++) {
        newposy[i] /= newposy.last();
        //qDebug() << newposx[i] << newposy[i] << i / double(pieces - 1);
    }

    QVector<double> stops;
    //split at palette partition borders
    stops = map->colorStops();
    stops = stops.mid(1, stops.size() - 2); //skip 0,1
    for (int i = 1; i < pieces-1; i++) { //skip first,last
        double pos = stops.first() + newposy.at(i) * double(stops.last() - stops.first());
        bool found = false;
        for (int i = 1; i < stops.size()-1; i++) //skip first,last
            if (qFuzzyCompare(pos + 1, stops.at(i) + 1)) {
                found = true;
                break;
            }
        if (!found)
            map->addColorStop(pos, map->rgb(QwtInterval(0, 1), pos));
    }

    //stretch palette from palette partition to histogram partition
    stops = map->colorStops();
    QVector<QRgb> colors(stops.size());
    for (int i = 0; i < stops.size(); i++)
        colors[i] = map->rgb(QwtInterval(0, 1), stops.at(i));

    const double len = stops.at(stops.size()-2) - stops.at(1);
    j = 1; //j: target palette piece index
    for (int i = 2; i < stops.size()-2; i++) { //skip 0,first,last,1
        stops[i] -= stops.at(1); //normalize
        stops[i] /= len;

        while (j < pieces-1 && newposy.at(j) < stops[i])
            j++;
        double cpos = (stops[i] - newposy.at(j-1)) /
                (newposy.at(j) - newposy.at(j-1));
        stops[i] = stops.at(1) + len *
                (newposx.at(j-1) + cpos *
                 (newposx.at(j) - newposx.at(j-1)));
    }

    //apply
    map->setColorInterval(colors.first(), colors.last());
    for (int i = 1; i < stops.size()-1; i++) //skip 0,1
        map->addColorStop(stops.at(i), colors.at(i));
}

template <int Size>
void T3dPlot::UpdateSliderHistogram(const THistogram<Size> &zhistogram)
{
    int max = 0;
    int sum = 0;
    for (int i = 0; i < Size; i++) {
        sum += zhistogram.bins[i];
        max = qMax(max, zhistogram.bins[i]);
    }

#define HIST_PREC_SIZE 200
    QPixmap bg(HIST_PREC_SIZE, Size);
    bg.fill(palette().color(QPalette::Background));
    if (sum) {
        QPainter p(&bg);
        p.setPen(Qt::lightGray);
        for (int i = 0; i < Size; i++)
            p.drawLine(HIST_PREC_SIZE-2, Size-1 - i,
                       HIST_PREC_SIZE-2 - HIST_PREC_SIZE*.5 * zhistogram.bins[i] / max, Size-1 - i);
        p.end();
    }

    Slider->Sliders->bgItem->setPixmap(bg);


    if (sum && ThreeDPlots.at(ActivePlot)->AutoZScaleOn) {
        ThreeDPlots[ActivePlot]->blockRefresh = true;
        Slider->Sliders->SetMinMaxRelSlidersValues(ThreeDPlots.at(ActivePlot)->MinSliderVal, ThreeDPlots.at(ActivePlot)->MaxSliderVal);
        ThreeDPlots[ActivePlot]->blockRefresh = false;
    }
}

void T3dPlot::Refresh()
{
    for (auto &ThreeDPlot:ThreeDPlots)
        ThreeDPlot->Refresh();
    TwoDPlot->Refresh();
    TwoDPlotVertical->Refresh();
}
void T3dPlot::SetPlotsXScales(double Min, double Max)
{
    for (auto &ThreeDPlot:ThreeDPlots)
        ThreeDPlot->SetPlotsXScales(Min, Max);

    if (Parent)
        QCoreApplication::postEvent(Parent,
                                    new TScalesChangedEvent(ThreeDPlots.at(0)->PlotsXLimits,
                                                            ThreeDPlots.at(0)->PlotsYLimits,
                                                            TScalesChangedEvent::R_SetPlotsXScales_Called));
}
void T3dPlot::SetPlotsYScales(double Min, double Max)
{
    for (auto &ThreeDPlot:ThreeDPlots)
        ThreeDPlot->SetPlotsYScales(Min, Max);

    if (Parent)
        QCoreApplication::postEvent(Parent,
                                    new TScalesChangedEvent(ThreeDPlots.at(0)->PlotsXLimits,
                                                            ThreeDPlots.at(0)->PlotsYLimits,
                                                            TScalesChangedEvent::R_SetPlotsYScales_Called));
}
void T3dPlot::SetTitle(const QString &Title, int N)
{
    if (N < 0)
        for (auto &ThreeDPlot:ThreeDPlots)
            ThreeDPlot->SetTitle(Title);
    else
        ThreeDPlots[N]->SetTitle(Title);
}
void T3dPlot::SetSelectWidthRelCenter(qreal W)
{
    ThreeDPlots[ActivePlot]->SetSelectWidthRelCenter(W);
}
void T3dPlot::SetSelectHeightRelCenter(qreal H)
{
    ThreeDPlots[ActivePlot]->SetSelectHeightRelCenter(H);
}
void T3dPlot::SetSelectWidth(qreal W)
{
    ThreeDPlots[ActivePlot]->SetSelectWidth(W);
}
void T3dPlot::SetSelectHeight(qreal H)
{
    ThreeDPlots[ActivePlot]->SetSelectHeight(H);
}
void T3dPlot::SetAllYScales(double Min, double Max)
{
    SetDefaultYLimits(Min, Max);
    SetPlotsYScales(Min, Max);
}
void T3dPlot::SetAllXScales(double Min, double Max)
{
    SetDefaultXLimits(Min, Max);
    SetPlotsXScales(Min, Max);
}
const QImage T3dPlot::MakePreview(int N)
{
    return ThreeDPlots[N]->MakePreview(PlotControl);
}
void T3dPlot::SetZoomBySelection(bool State)
{
    for (auto &ThreeDPlot:ThreeDPlots)
        ThreeDPlot->SetZoomBySelection(State);
}
void T3dPlot::SyncZoom(int N, const QPointF &Pos, int Increment, bool /*BFixX_isChecked*/, bool /*BFixY_isChecked*/)
{
    for (int i = 0; i < ThreeDPlots.size(); i++)
    {
        if (i == N) continue;
        ThreeDPlots[i]->ZoomPlot(Pos, Increment);
    }
}
void T3dPlot::SyncPan(int N, const QPointF &Delta)
{
    for (int i = 0; i < ThreeDPlots.size(); i++)
    {
        if (i == N) continue;
        ThreeDPlots[i]->PanPlot(Delta);
    }
}
void T3dPlot::SyncDefineClickInfo(int N, const QPoint &Pos)
{
    for (int i = 0; i < ThreeDPlots.size(); i++)
    {
        if (i == N) continue;
        ThreeDPlots[i]->DefineClickInfo(Pos);
    }
}
void T3dPlot::SetCrossCoords(double x, double y)
{
    for (auto &ThreeDPlot:ThreeDPlots)
        ThreeDPlot->SetCrossCoords(x, y);
}
void T3dPlot::HideCross()
{
    for (auto &ThreeDPlot:ThreeDPlots)
        ThreeDPlot->HideCross();
}
void T3dPlot::SetDefaultXLimits(double Min, double Max)
{
    for (auto &ThreeDPlot:ThreeDPlots)
        ThreeDPlot->SetDefaultXLimits(Min, Max);
}
void T3dPlot::SetDefaultYLimits(double Min, double Max)
{
    for (auto &ThreeDPlot:ThreeDPlots)
        ThreeDPlot->SetDefaultYLimits(Min, Max);
}
void T3dPlot::SetXYDescAndSIUnit(const QString &Desc, const QString &Unit, int N)
{
    if (N >= 0)
        ThreeDPlots[N]->XYAxisUnit.SetDescAndSIUnit(Desc, Unit);
    else
        for (auto &ThreeDPlot:ThreeDPlots)
            ThreeDPlot->XYAxisUnit.SetDescAndSIUnit(Desc, Unit);

    AdjustTwoDPlot();
}
void T3dPlot::SetZDescAndSIUnit(const QString &Desc, const QString &Unit, int N)
{
    if (N >= 0)
        ThreeDPlots[N]->ZAxisUnit.SetDescAndSIUnit(Desc, Unit);
    else
        for (auto &ThreeDPlot:ThreeDPlots)
            ThreeDPlot->ZAxisUnit.SetDescAndSIUnit(Desc, Unit);

    AdjustTwoDPlot();
}
void T3dPlot::HideSelectionIfAny()
{
    if (ThreeDPlots.at(ActivePlot)->SelectionModeEnabled)
        on_BCancel_clicked();
}
void T3dPlot::Show2dPlotControl()
{
#ifndef _ADROID
    PlotControl->BCross->change_state_man_with_sig(false);
    PlotControl->hide();
    if ((TwoDPlot->isVisible() || TwoDPlotVertical->isVisible()) && TwoDPlot->showPlotControl)
        TwoDPlot->PlotControl->show();
    else if (TwoDPlotSPM && TwoDPlotSPM->isVisible())
        TwoDPlotSPM->PlotControl->show();
#endif
}
void T3dPlot::Set2dPlotVisible(bool Visible)
{
    if (Visible && !blockVisibility2DPlot)
    {
        if (SplitPlotTab->sizes().last() == 0)
        {
            SplitPlotTab->setSizes(QList<int>() << lastSplitPlotSizes[0] << lastSplitPlotSizes[1]);
        }
    }
    else
    {
        if (SplitPlotTab->sizes().count() >= 2)
        {
            if (SplitPlotTab->sizes()[0] != 0)
                lastSplitPlotSizes[0] = SplitPlotTab->sizes()[0];
            if (SplitPlotTab->sizes()[1] != 0)
                lastSplitPlotSizes[1] = SplitPlotTab->sizes()[1];
            SplitPlotTab->setSizes(QList<int>() << 1 << 0);
        }
    }
}

void T3dPlot::ResetAllFilters()
{
    overlayButtons->resetOldFilters();
    PlotControl->ResetAllFilters();

    for (auto &ThreeDPlot : ThreeDPlots)
        ThreeDPlot->ResetAllFilters();
}

}
