#ifndef PLOT_3DPURE_H
#define PLOT_3DPURE_H

#include "2d/plot_2dplot.h"
#include "plot_3dtypes.h"
#include "widgets/plot_DoubleSlider.h"
#include "plot_3dWidgets.h"
#include <QLayout>
#include "widgets/fixedsplitter.h"
#include "widgets/plot_colorchooser.h"
#include <QSettings>

namespace plot
{

class T3dMarkerInfoHandler: public TGeneralMarkerInfoHandler
{
public:
    TValueInfoCreator ZInfo;
    QString FormUniteString();
    void operator()(bool AutoLegendEnabled, const TSIUnit &XYUnitInfo, const TSIUnit &ZUnitInfo, double XVal, double YVal, double ZVal);
};

namespace math { class TMathProcessing; }
class T3dPlot;

class TPure3dPlot : public TCommonPlot, public TCompleteInfoFor3dPlot, public ControlPanelHistory
{
    Q_OBJECT
public slots:
    void s_Refresh();
    void PaletteChoosed(QColor color, int id);
    void setFiltering(bool on);

signals:
    void MathUpdated();
    void doubleClicked();
    void onRestoredScales();
    void MouseEventSignal(QEvent*, TPure3dPlot*, int);
    void onResized(int n, QSize);

public:
    using TCommonPlot::UpdateLabels;
    void UpdateLabels(bool IsDetailedInfoVisible);
    void HandleAxisMultiplier() override;
    void UpdateAxisLegends() override;
    inline void MakeCrossection();

    void dragEnterEvent(QDragEnterEvent *event) override;
    void dropEvent(QDropEvent *event) override;

    void UpdateHistogram();
    template <int Size>
    void UpdateHistogram(const THistogram<Size> &zhistogram);

    bool stopMouseEvent = false;
    int N;
    T3dPlot *Child;
    QwtPlotMarker *Cross;
    bool AreaZzoom;
    int zoom_active;
    QPointF areazA;
    QPointF areazAo;
    QPointF areazB;
    QRubberBand *areaRB;
    bool ZoomBySelection;
    QWidget *secondParent = nullptr; //Using if it in full mode
    plot_colorChooser *colorChooser;


    int last_resize_x;

    void AreaZzoomST(double x, double y, double xo, double yo, int st);

    int Height() const;

    bool eventFilter(QObject *Object, QEvent *Event) override;
    void HandleMoveMarkers(const QPointF &APos, const QPointF &BPos);
    void ResizeQPlot(bool move = false) override;
    QSize FindAvailableSpaceForQPlot() const;
    void mouseDoubleClickEvent(QMouseEvent * Event) override;
    bool gestureEvent(QGestureEvent *event);

    void on_BFixOXDir_clicked();
    void on_BFixOYDir_clicked();
    void SelectRectChanged(const QRectF &Rect) override;
    void BRestoreScale_clicked();
    void SetZoomBySelection(bool State);
    void SelectDone();

    void SetCrossCoords(double x, double y);
    void HideCross();

    math::TMathProcessing *MathProcessing;
    bool isMathReady();
    bool IsMathEnabled() const;
    void MakeXFit(const int Order);
    void MakeYFit(const int Order);
    void MakeSubtr(const int Order);
    void MakeCorrectLineByMedianaXDir(const bool Make);
    void MakeCorrectLineByMedianaYDir(const bool Make);
    void MakeMeanValue(const int Size = 10);
    void MakeMedianValue(const int Size = 10);
    void MakeGaussian(const int Size = 10);
    void MakeHistogram(bool Make);
    void ShowHistogram(bool show);
    void ResetAllFilters();

    T3dMarkerInfoHandler AMarkerInfo;
    T3dMarkerInfoHandler BMarkerInfo;

    Q_INVOKABLE void Refresh() override;
    bool wait_for_math;
    bool math_delayed_refresh;
    bool singleInTabPage;
    int fixedSize = -1;
    bool isResizing;

    TPure3dPlot(int N, T3dPlot *Chld, QObject *Parent = nullptr, QWidget *parent = nullptr);
    ~TPure3dPlot() override;
    QObject *Parent; //for postEvent(Parent, blablabla);

    bool isFiltering = true;
    int xScanning = -1;
};

}

#endif // PLOT_3DPURE_H
