#include "plot_MathFilters.h"
#include <QDebug>

namespace plot {
namespace math {

template <class T>
void TCorrectLinesByMedian::AddValueTo(const T F, const T L, double Val) const
{
    for (T It = F; It != L; *(It++) += Val);
}
void TCorrectLinesByMedian::AddValueTo(T2dArray &Arr, const int X, const int StartY, const int EndY, const double Val) const
{
    for (int Y = StartY; Y < EndY; Arr[Y++][X] += Val);
}
void TCorrectLinesByMedian::FetchLine(QVector<double> &Line, const int X, const int StartY, const int EndY, const T2dArray &Arr) const
{
    Line.clear();
    for (int Y = StartY; Y < EndY; Line.push_back(Arr[Y++][X]));
}

TCorrectLinesByMedian::TCorrectLinesByMedian
    (T2dArray *Arr, TPlotEnums::TAxis Dir, int StartX, int StartY, int EndX, int EndY)
{
    if (Arr)
        Dir == TPlotEnums::A_XAxis ?
                    CorrectLinesByMedianXDir(*Arr, StartX, StartY, EndX, EndY) :
                    CorrectLinesByMedianYDir(*Arr, StartX, StartY, EndX, EndY);
}


void TCorrectLinesByMedian::CorrectLinesByMedianXDir(T2dArray &Arr, int StartX, int StartY, int EndX, int EndY) const
{
    QVector<double> Line, Modi;
    for (int YInd = (int)StartY; YInd < (int)EndY; YInd++)
    {
        if (EndX > (int)Arr[YInd].size()) { qDebug() << "CorrectLinesByMedian: EndX > Arr[YInd].size()!"; EndX = Arr[YInd].size(); };
        Line = Arr[YInd].mid(StartX, EndX - StartX);//Line.clear(); Line.insert(Line.begin(), Arr[YInd].begin() + StartX, Arr[YInd].begin() + EndX);
        Modi.push_back(CalcMedian(Line));
    }
    Line = Modi; double Median = CalcMedian(Line);
    for (int YInd = (int)StartY; YInd < (int)EndY; YInd++)
        AddValueTo(Arr[YInd].begin() + StartX, Arr[YInd].begin() + EndX, Median - Modi[YInd - StartY]);
}
void TCorrectLinesByMedian::CorrectLinesByMedianYDir(T2dArray &Arr, int StartX, int StartY, int EndX, int EndY) const
{
    QVector<double> Line, Modi;
    for (int XInd = (int)StartX; XInd < (int)EndX; XInd++)
    {
        FetchLine(Line, XInd, StartY, EndY, Arr);
        Modi.push_back(CalcMedian(Line));
    }
    Line = Modi; double Median = CalcMedian(Line);
    for (int XInd = (int)StartX; XInd < (int)EndX; XInd++)
        AddValueTo(Arr, XInd, StartY, EndY, Median - Modi[XInd - StartX]);
}

//======================================================================================================================

void TMeanValue::MeanValue(const T2dArray &Arr, bool Average)
{
    if (Size >= qMin(width, height))
        Size = qMin(width, height)-1;
    if (Size < 2)
        return;

    const double *srow, *trow;
    double *drow;

    if (Arr.size() == 0)
        return;

    int xres = Arr[0].size();
    int i, j, m;
    int hs2p, hs2m, vs2p, vs2m;
    double v;

    double *result = data;
    double *buffer = new double [Arr.size() * Arr[0].size()];

    //Extension to the left and to the right (for asymmetric sizes extend to the right more)
    hs2m = (Size - 1) / 2;      hs2p = Size / 2;
    vs2m = (Size - 1) / 2;      vs2p = Size / 2;

    //Row-wise sums
    for (i = 0; i < height; i++)
    {
        srow = data + (i + row) * xres + col;
        drow = buffer + i * width;
        //Left half
        drow[0] = .0;
        m = qMin(hs2p, width - 1);
        for (j = 0; j <= m; j++) drow[0] += srow[j];
        for (j = 1; j < width/2; j++)
        {
            v = ((j + hs2p < width ? srow[j + hs2p] : 0.0) - (j-1 - hs2m >= 0 ? srow[j-1 - hs2m] : .0));
            drow[j] = drow[j-1] + v;
        }
        //Right half
        drow[width-1] = 0.0;
        m = width - 1 - qMin(hs2m, width - 1);
        for (j = width-1; j >= m; j--) drow[width-1] += srow[j];
        for (j = width-2; j >= width/2; j--)
        {
            v = ((j - hs2m >= 0 ? srow[j - hs2m] : 0.0) - (j+1 + hs2p < width ? srow[j+1 + hs2p] : .0));
            drow[j] = drow[j+1] + v;
        }
    }

    //Column-wise sums (but iterate row-wise to access memory linearly)
    //Top half
    drow = result + row * xres + col;
    for (j = 0; j < width; j++) drow[j] = .0;
    m = qMin(vs2p, height - 1);
    for (i = 0; i <= m; i++)
    {
        srow = buffer + i*width;
        for (j = 0; j < width; j++) drow[j] += srow[j];
    }
    for (i = 1; i < height/2; i++)
    {
        drow = result + (i + row) * xres + col;
        if (i + vs2p < height)
        {
            srow = buffer + (i + vs2p)*width;
            if (i-1 - vs2m >= 0)
            {
                trow = buffer + (i-1 - vs2m)*width;
                for (j = 0; j < width; j++) drow[j] = *(drow + j - xres) + (srow[j] - trow[j]);
            }
            else for (j = 0; j < width; j++) drow[j] = *(drow + j - xres) + srow[j];
        }
        else
        {
            if (i - 1 - vs2m >= 0)
            {
                qDebug() << "Me thinks pure subtraction cannot occur.";
                trow = buffer + (i-1 - vs2m)*width;
                for (j = 0; j < width; j++) drow[j] = *(drow + j - xres) - trow[j];
            }
            else for (j = 0; j < width; j++) drow[j] = *(drow + j - xres);
        }
    }

    //Bottom half
    drow = result + (height - 1 + row) * xres + col;
    for (j = 0; j < width; j++) drow[j] = 0;
    m = height - 1 - qMin(vs2m, height - 1);
    for (i = height - 1; i >= m; i--)
    {
        srow = buffer + i*width;
        for (j = 0; j < width; j++) drow[j] += srow[j];
    }
    for (i = height-2; i >= height/2; i--)
    {
        drow = result + (i + row) * xres + col;
        if (i + 1 + vs2p < height)
        {
            srow = buffer + (i+1 + vs2p)*width;
            if (i - vs2m >= 0)
            {
                trow = buffer + (i - vs2m) * width;
                for (j = 0; j < width; j++) drow[j] = drow[j + xres] + (trow[j] - srow[j]);
            }
            else
            {
                qDebug() << "Me thinks pure subtraction cannot occur.";
                for (j = 0; j < width; j++) drow[j] = drow[j + xres] - srow[j];
            }
        }
        else
        {
            if (i - vs2m >= 0)
            {
                trow = buffer + (i - vs2m) * width;
                for (j = 0; j < width; j++) drow[j] = drow[j + xres] + trow[j];
            }
            else for (j = 0; j < width; j++) drow[j] = drow[j + xres];
        }
    }

    delete [] buffer;
    buffer = nullptr;
    if (!Average) return;

    //Divide sums by the numbers of pixels that entered them
    for (i = 0; i < height; i++)
    {
        int iw;
        if (i <= vs2m)                 iw = vs2p + 1 + i;
        else if (i >= height-1 - vs2p) iw = vs2m + height - i;
        else                          iw = Size;
        iw = qMin(iw, height);
        for (j = 0; j < width; j++)
        {
            int jw;
            if (j <= hs2m)                jw = hs2p + 1 + j;
            else if (j >= width-1 - hs2p) jw = hs2m + width - j;
            else                          jw = Size;
            jw = qMin(jw, width);
            result[(i + row)*xres + j + col] /= iw*jw;
        }
    }
}

TMeanValue::TMeanValue(T2dArray *Arr, int Size, int StartX, int StartY, int EndX, int EndY)
{
    if (Arr)
        MeanValue(*Arr, Size, StartX, StartY, EndX, EndY);
}
void TMeanValue::MeanValue(T2dArray &Arr, int Sz, int StartX, int StartY, int EndX, int EndY)
{
    EXEC_FILTER_WITH_SIZE(MeanValue)
}

//======================================================================================================================

void TMedianValue::MedianValue(const T2dArray &Arr) const
{
    int rowstride;
    int i, j, k, len;
    int xfrom, xto, yfrom, yto;
    double *buffer, *dt, *kernel;

    buffer = new double [width * height];
    kernel = new double [Size * Size];

    if (Arr.size() == 0)
    {
        delete [] kernel;
        delete [] buffer;
        kernel = nullptr;
        buffer = nullptr;
        return;
    }

    rowstride = Arr[0].size();
    dt = data + rowstride * row + col;

    for (i = 0; i < height; i++)
    {
        yfrom = qMax(0, i - (Size - 1) / 2);
        yto = qMin(height - 1, i + Size / 2);
        for (j = 0; j < width; j++)
        {
            xfrom = qMax(0, j - (Size - 1) / 2);
            xto = qMin(width - 1, j + Size / 2);
            len = xto - xfrom + 1;
            for (k = yfrom; k <= yto; k++) memcpy(kernel + len * (k - yfrom), dt + k*rowstride + xfrom, len*sizeof(double));
            buffer[i * width + j] = CalcMedian(kernel, kernel + len*(yto - yfrom + 1));//gwy_math_median(len*(yto - yfrom + 1), kernel);
        }
    }
    delete [] kernel;
    for (i = 0; i < height; i++) memcpy(dt + i * rowstride, buffer + i * width, width * sizeof(double));
    delete [] buffer;
    kernel = nullptr;
    buffer = nullptr;
}

TMedianValue::TMedianValue(T2dArray *Arr, int Size, int StartX, int StartY, int EndX, int EndY)
{
    if (Arr)
        MedianValue(*Arr, Size, StartX, StartY, EndX, EndY);
}
void TMedianValue::MedianValue(T2dArray &Arr, int Sz, int StartX, int StartY, int EndX, int EndY)
{
    EXEC_FILTER_WITH_SIZE(MedianValue)
}

//======================================================================================================================

TGaussian::TGaussian(T2dArray *Arr, int Size, int StartX, int StartY, int EndX, int EndY)
{
    if (Arr)
        Gaussian(*Arr, Size, StartX, StartY, EndX, EndY);
}

void TGaussian::Gaussian(T2dArray &Arr, int Sz, int StartX, int StartY, int EndX, int EndY)
{ EXEC_FILTER_WITH_SIZE(Gaussian); }
void TGaussian::Gaussian(const T2dArray &Arr) const
{
    QVector<double> kernel;
    double x;
    int res, i;
    double sigma = Size / (2. * qSqrt(2.* qLn(2)));

    if (sigma == 0.0) return;
    if (Arr.size() == 0) return;
    int dfXRes = Arr[0].size();
    int dfYRes = Arr.size();

    res = (int)ceil(5. * sigma);
    res = 2 * res + 1;
    i = qMin(dfXRes, dfYRes);
    if (res > i)
    {
        res = i;
        if (res % 2 == 0) res--;
    }

    kernel.resize(res);
    for (i = 0; i < res; i++)
    {
        x = i - (res - 1) / 2.0;
        x /= sigma;
        kernel[i] = exp(-x * x / 2.0);
    }
    MultiplyBy(kernel.begin(), kernel.end(), 1.0/CalcSum(kernel.begin(), kernel.end()));
    Convolve1d(kernel, plot::TPlotEnums::A_XAxis, Arr); // plot::TPlotEnums::A_XAxis = GWY_ORIENTATION_HORIZONTAL
    Convolve1d(kernel, plot::TPlotEnums::A_YAxis, Arr); // plot::TPlotEnums::A_XAxis = GWY_ORIENTATION_VERTICAL
}
template <class TIter>
double TGaussian::CalcSum(const TIter B, const TIter E) const
{
    double R = 0;
    for (TIter It = B; It != E; R += *(It++));
    return R;
}
template <class TIter>
void TGaussian::MultiplyBy(const TIter B, const TIter E, const double Val) const
{
    for (TIter It = B; It != E; *(It++) *= Val);
}
void TGaussian::MultiplyBy(double *Area, const int StartX, const int StartY, const int EndX, const int EndY, const double Val, const int W) const
{
    for (int Y = StartY; Y < EndY; Y++)
        for (int X = StartX; X < EndX; Area[Y*W + X++] *= Val);
}

void TGaussian::Convolve1d(const QVector<double> &kernel, const plot::TPlotEnums::TAxis orientation, const T2dArray &Arr) const
{
    int kres = kernel.size();
    if (Arr.size() == 0)
        return;
    if (kres == 1)
    {
        MultiplyBy(data, col, row, col+width, row+height, kernel[0], Arr[0].size());
        return;
    }

    if (orientation == plot::TPlotEnums::A_XAxis)
    {
        if (kres >= width) { return; }
        HConvolved(kernel, Arr);
    }
    else
    {
        if (kres >= height) { return; }
        VConvolved(kernel, Arr);
    }
}

void TGaussian::HConvolved(const QVector<double> &kernel, const T2dArray &Arr) const
{
    int kres, i, j, k, pos;
    double *buf, *drow;
    double d;
    if (Arr.size() == 0)
        return;
    int dfXRes = Arr[0].size();

    kres = kernel.size();
    buf = new double [kres];

    for (i = 0; i < height; i++)
    {
        drow = data + (row + i) * dfXRes + col;
        memset(buf, 0, kres * sizeof(buf[0]));
        for (j = 0; j < kres; j++)
        {
            k = j - kres / 2;
            d = drow[k >= 0 ? k : -k-1];
            for (k = 0; k <= j; k++) buf[k] += kernel[j - k] * d;
        }
        pos = 0;
        for (j = 0; j < width; j++)
        {
            drow[j] = buf[pos];
            buf[pos] = .0;
            pos = (pos + 1) % kres;
            k = j + kres - kres / 2;
            d = drow[k < width ? k : 2 * width - 1 - k];
            for (k = pos; k < kres; k++) buf[k] += kernel[kres-1 - (k - pos)] * d;
            for (k = 0; k < pos; k++) buf[k] += kernel[pos-1 - k] * d;
        }
    }
    delete [] buf;
}
void TGaussian::VConvolved(const QVector<double> &kernel, const T2dArray &Arr) const
{
    int kres, xres, i, j, k, pos;
    double *buf, *dcol;
    double d;

    if (Arr.size() == 0)
        return;

    kres = kernel.size();
    xres = Arr[0].size();
    buf = new double [kres];

    for (j = 0; j < width; j++)
    {
        dcol = data + row * xres + (col + j);
        memset(buf, 0, kres * sizeof(buf[0]));
        for (i = 0; i < kres; i++)
        {
            k = i - kres/2;
            d = dcol[k >= 0 ? k*xres : (-k-1)*xres];
            for (k = 0; k <= i; k++) buf[k] += kernel[i - k] * d;
        }
        pos = 0;
        for (i = 0; i < height; i++)
        {
            dcol[i*xres] = buf[pos];
            buf[pos] = 0.0;
            pos = (pos + 1) % kres;
            k = i + kres - kres/2;
            d = dcol[k < height ? k*xres : (2*height-1 - k)*xres];
            for (k = pos; k < kres; k++) buf[k] += kernel[kres-1 - (k - pos)]*d;
            for (k = 0; k < pos; k++)    buf[k] += kernel[pos-1 - k]*d;
        }
    }
    delete [] buf;
}

}
}
