#ifndef MATHPROCESSING_H_
#define MATHPROCESSING_H_

#include <2d/plot_2dtypes.h>
#include <3d/plot_3dtypes.h>
#include <QThread>
#include <QVariant>

namespace plot {
namespace math {

struct TTask
{
    TTask() {
        Action = 0; //nothing
    }
    int Action;
    QVariant arg;
};

class TMathProcessing: public QThread
{
    Q_OBJECT
public:
    enum WhatShallIDo {Nothing, Fit2d, Scale2d, Fit3dX, Fit3dY, Subtr, LineMedianX, LineMedianY, MeanValue, MedianValue, Gaussian, FiltersSize};

    void DoManualFitting(int N, double *YArray, long Size);
    double mi, mx;

    void _Fit2d();
    void _Scale2d();
    void _Fit2d(int N);
    void _Scale2d(int N);
    void _Fit3d_XDir();
    void _Fit3d_XDir(TMin3dData &Min3dData);
    void _Fit3d_YDir();
    void _Fit3d_YDir(TMin3dData &Min3dData);
    void _Subtr();
    void _Subtr(TMin3dData &Min3dData);
    void _LineCorrectByMediana();
    void _DoMeanValue();
    void _DoMedianValue();
    void _DoGaussian();

    void SmartAddTask(const TTask &Task);

    T3dDataOwner *Data;
    TCurvesDataOwner *Curves;
    QwtPlotCurve *MCCurve;

    TTask Work[FiltersSize];
    TTask ActiveTask;

protected:
    void run() override;

public:
    TMathProcessing(TCurvesDataOwner *Src, bool vertical);
    TMathProcessing(T3dDataOwner *Src, bool vertical);

    void DoFit(int N);
    void DoScale(int N = -1);
    void DoFitX(int N);
    void DoFitY(int N);
    void DoSubtr(int N);
    void DoLineCorrectByMedianaX();
    void DoLineCorrectByMedianaY();
    void DoMeanValue(int Sz = 5);
    void DoMedianValue(int Sz = 5);
    void DoGaussian(int Sz = 5);

    static void SetupMinsMaxs(unsigned int &MinX, unsigned int &MaxX, unsigned int &MinY, unsigned int &MaxY, const TMin3dData &Min3dData);

    bool vertical;
};

}
}

#endif /* MATHPROCESSING_H_ */
