#include "plot.h"

#include <QApplication>
#include "widgets/twodoverlaybuttons.h"

QDataStream &operator<<(QDataStream &st, const Range &obj)
{
    st << obj.arr[0] << obj.arr[1];
    return st;
}
QDataStream &operator>>(QDataStream &st, Range &obj)
{
    st >> obj.arr[0] >> obj.arr[1];
    return st;
}
QDataStream &operator<<(QDataStream &st, const plot::TInfoForAllPlots &obj)
{
    st << obj.AutoLegendEnabled << obj.Title
       << obj.XLegend << obj.YLegend
       << obj.CurrentXLimits << obj.CurrentYLimits
       << obj.PlotsXLimits << obj.PlotsYLimits
       << obj.DefaultXLimits << obj.DefaultYLimits;
    return st;
}
QDataStream &operator>>(QDataStream &st, plot::TInfoForAllPlots &obj)
{
    st >> obj.AutoLegendEnabled >> obj.Title
       >> obj.XLegend >> obj.YLegend
       >> obj.CurrentXLimits >> obj.CurrentYLimits
       >> obj.PlotsXLimits >> obj.PlotsYLimits
       >> obj.DefaultXLimits >> obj.DefaultYLimits;
    obj.FontSize = qApp->font().pointSize() + 2; //12
    return st;
}
QDataStream &operator<<(QDataStream &st, const plot::TSIUnit &obj)
{
    st << obj.Multiplier << obj.StrMultiplier
       << obj.Description << obj.SIUnit
       << obj.Legend;
    return st;
}
QDataStream &operator>>(QDataStream &st, plot::TSIUnit &obj)
{
    st >> obj.Multiplier >> obj.StrMultiplier
       >> obj.Description >> obj.SIUnit
       >> obj.Legend;
    return st;
}

QDataStream &operator<<(QDataStream &st, const plot::TInfoFor2dPlot &obj)
{
    st << (plot::TInfoForAllPlots)obj
       << obj.XAxisUnit << obj.YAxisUnit;
    return st;
}
QDataStream &operator>>(QDataStream &st, plot::TInfoFor2dPlot &obj)
{
    st >> (plot::TInfoForAllPlots &)obj
       >> obj.XAxisUnit >> obj.YAxisUnit;
    return st;
}
QDataStream &operator<<(QDataStream &st, const plot::TCurveData &obj)
{
    st << obj.XSeq << obj.YSeq;
    return st;
}
QDataStream &operator>>(QDataStream &st, plot::TCurveData &obj)
{
    st >> obj.XSeq >> obj.YSeq;
    return st;
}

QDataStream &operator<<(QDataStream &st, const plot::TInfoFor3dPlot &obj)
{
    st << (plot::TInfoForAllPlots)obj
       << obj.XYAxisUnit << obj.ZAxisUnit << obj.ZLegend;
    return st;
}
QDataStream &operator>>(QDataStream &st, plot::TInfoFor3dPlot &obj)
{
    st >> (plot::TInfoForAllPlots &)obj
       >> obj.XYAxisUnit >> obj.ZAxisUnit >> obj.ZLegend;
    return st;
}
QDataStream &operator<<(QDataStream &st, const plot::TMin3dData &obj)
{
    st << obj.Array
       << obj.dx << obj.dy
       << obj.NXElems << obj.NYElems
       << obj.XLimits << obj.YLimits
       << obj.LimsForMathX << obj.LimsForMathY;
    return st;
}
QDataStream &operator>>(QDataStream &st, plot::TMin3dData &obj)
{
    st >> obj.Array
       >> obj.dx >> obj.dy
       >> obj.NXElems >> obj.NYElems
       >> obj.XLimits >> obj.YLimits
       >> obj.LimsForMathX >> obj.LimsForMathY;
    return st;
}
void InitPlotIO()
{
    qRegisterMetaType<plot::TCurveData>();
    qRegisterMetaType<plot::TMin3dData>();
    qRegisterMetaType<plot::TInfoForAllPlots>();
    qRegisterMetaType<plot::TInfoFor2dPlot>();
    qRegisterMetaType<plot::TInfoFor3dPlot>();

    qRegisterMetaType<plot::TMetaStorage>("plot::TMetaStorage");
    qRegisterMetaType<QVector<double> >();
    qRegisterMetaType<plot::TCurvesData>();
    qRegisterMetaType<plot::TCurvesDataOwner>();
    qRegisterMetaType<plot::T3dData>();
    qRegisterMetaType<plot::T3dDataOwner>();

    qRegisterMetaTypeStreamOperators<plot::TCurveData>();
    qRegisterMetaTypeStreamOperators<plot::TMin3dData>();
    qRegisterMetaTypeStreamOperators<plot::TInfoForAllPlots>();
    qRegisterMetaTypeStreamOperators<plot::TInfoFor2dPlot>();
    qRegisterMetaTypeStreamOperators<plot::TInfoFor3dPlot>();
}

namespace plot {

TPlot::TPlot() : QStackedWidget(nullptr)
{
    TwoDPlot = nullptr;
    #ifdef _ADROID
    TwoDPlotVertical = nullptr;
#endif
    ThreeDPlot = nullptr;

    setCurrentIndex(0);
    CurrentPlotType = currentIndex();
}

TPlot::TPlot(QObject *Parent, QWidget *wparent) : QStackedWidget(wparent)
{
#ifdef _ADROID
    TwoDOverlayButtons *overlayBtns = new TwoDOverlayButtons(false);
#else
    TwoDOverlayButtons *overlayBtns = new TwoDOverlayButtons();
#endif
    addWidget(TwoDPlot = new T2dPlot(Parent, false, this, false, overlayBtns));
    addWidget(ThreeDPlot = new T3dPlot(1, Parent, this));

    overlayBtns->set2DPlot(TwoDPlot);

    bool filtetingByDef = false;
    {
        QSettings settings("Param/scan.conf", QSettings::IniFormat);
        settings.beginGroup("all_plot_filter");
        filtetingByDef = settings.value("enable_by_def", false).toBool();
        settings.endGroup();
    }
    TwoDPlot->Pure2dPlot->isFiltering = filtetingByDef;
    ThreeDPlot->ThreeDPlots.first()->isFiltering = filtetingByDef;
    ThreeDPlot->TwoDPlot->Pure2dPlot->isFiltering = filtetingByDef;
    ThreeDPlot->TwoDPlotVertical->Pure2dPlot->isFiltering = filtetingByDef;
#ifndef _ADROID
    ThreeDPlot->mainTab->setStyleSheet(""
                                       "QTabBar::tab { background: #F0F0F0; color: white; padding: 10px; } "
                                       "QTabBar::tab:selected { background: lightgray; } "
                                       "QTabWidget::pane { border: 0; } "
                );

#endif

	connect(TwoDPlot,   SIGNAL(AddDataToContainer(plot::TCurvesDataOwner,plot::TMetaStorage)), this, SIGNAL(AddDataToContainer(plot::TCurvesDataOwner,plot::TMetaStorage)));
    connect(ThreeDPlot, SIGNAL(AddDataToContainer(plot::T3dDataOwner, plot::TMetaStorage)), this, SIGNAL(AddDataToContainer(plot::T3dDataOwner, plot::TMetaStorage)));
    connect(ThreeDPlot, SIGNAL(AddDataToContainer(plot::TCurvesDataOwner,plot::TMetaStorage)), this, SIGNAL(AddDataToContainer(plot::TCurvesDataOwner,plot::TMetaStorage)));

    setCurrentIndex(0);
    CurrentPlotType = currentIndex();
}

}
