#pragma once

#include <QGraphicsView>
#include <QGraphicsItem>
#include <QLabel>
class QGestureEvent;

#include <qwt_scale_widget.h>
#include <qwt_scale_div.h>
#include <qwt_color_map.h>

namespace plot {

class TQwtScaleBar: public QwtScaleWidget
{
public:
    TQwtScaleBar();
    double inline GetMin() const { return ScaleDiv.lowerBound(); }
    double inline GetMax() const { return ScaleDiv.upperBound(); }
    void inline SetColourMap(QwtLinearColorMap *Map)  { setColorMap(Map ? QwtInterval(GetMin(), GetMax()) : QwtInterval(), map = Map); }
    void SetMinMaxValues(double Min, double Max);

private:
    void resizeEvent(QResizeEvent *e) override;
    QwtScaleDiv ScaleDiv;
    QwtLinearColorMap *map;
};

class TSlidersBar;
class TSlider: public QGraphicsItem
{
    friend class TSlidersBar;
public:
    TSlider(TSlidersBar *Parent, const QPixmap &Pm = QPixmap());
    void SetCPos(double Pos);
    void SetRelPos(double RP);
    double GetValue() const;

    double RelPos;

protected:
    void paint(QPainter *Painter, const QStyleOptionGraphicsItem *Option, QWidget *Widget) override;
    void mousePressEvent(QGraphicsSceneMouseEvent *Event) override;
    void mouseReleaseEvent(QGraphicsSceneMouseEvent *Event) override;
    void mouseMoveEvent(QGraphicsSceneMouseEvent *Event) override;
    QRectF boundingRect() const override { return imageRect; }

    bool active;
    QPixmap SliderImage;
    TSlidersBar *Parent;
    QRectF imageRect;
    QPointF MouseDownPos;
};

//same, but movable with right button also, and not with middle
class TRightSlider : public TSlider
{
public:
    TRightSlider(TSlidersBar *Parent, const QPixmap &Pm = QPixmap());

protected:
    void mousePressEvent(QGraphicsSceneMouseEvent *Event) override;
};

class TSlidersBar: public QGraphicsView
{
    Q_OBJECT

signals:
    void ValuesChanged(double Min, double Max);
    void AutoChanged(bool auto_mode);

public:
    TSlidersBar(QWidget *Parent = nullptr);
    void SetColourMap(QwtLinearColorMap *Map);
    void SetMinMaxValues(double Min, double Max);
    void SetMinMaxRelSlidersValues(double Min, double Max);
    void SetMinMaxSlidersValues(double Min, double Max);

    QPointF BackbonePos;
    unsigned int BackboneLength;

    TQwtScaleBar *QwtBar;
    TSlider *LowSlider, *HighSlider;
    QGraphicsProxyWidget *QColourBarProxy;
    QGraphicsPixmapItem *bgItem;

    void setAutoMode(bool on);

    bool event(QEvent *) override;
    void MouseMoveEvent(const QPointF &Pos, TSlider *Sender);
    void resizeEvent(QResizeEvent *Event) override;
    void mouseDoubleClickEvent(QMouseEvent *) override;
    bool gestureEvent(QGestureEvent *event);
    QSize sizeHint() const override;
    QSize minimumSizeHint() const override;
};

class TDoubleSlider: public QWidget
{
public:
    TSlidersBar *Sliders;
    QLabel *Title;
    TDoubleSlider();
    bool eventFilter(QObject *Object, QEvent *Event) override;
    QSize sizeHint() const override;
};

}
