#include "plot_colorchooser.h"
#include <QPainterPath>

plot_colorChooser::plot_colorChooser(QWidget *parent) : QWidget(parent)
{
    font.setFamily("arial");
    colors[0] = QColor(Qt::red);
    colors[1] = QColor(Qt::green);
    colors[2] = QColor(Qt::blue);
    colors[3] = QColor(Qt::magenta);
    colors[4] = QColor(252, 102, 0);
    colors[5] = QColor();
    colors[6] = QColor(249, 166, 2);
    colors[7] = QColor(Qt::yellow);
    colors[8] = QColor(Qt::gray);

    alpha = 170;
    for(int i=0; i<9; i++)
        colors[i].setAlpha(alpha);

    updateRectSizes();
}

void plot_colorChooser::updateRectSizes()
{
    hMini = ((width()-1)/3)*0.95;
    hMini3 = hMini/3;

    int x = 1, y = 1;
    for (int i=0; i<9; i++)
    {
        rects[i] = QRect(x, y, hMini, hMini);
        x += hMini;
        if (i == 2 || i == 5)
        {
            x = 1;
            y += hMini;
        }
    }

    int fntSize = 10*hMini3;
    if (fntSize > 99)
        fntSize = 99;
    font.setWeight(fntSize);

    update();
}

bool plot_colorChooser::event(QEvent *e)
{
    if (e->type() == QEvent::GraphicsSceneMove ||
        e->type() == QEvent::GraphicsSceneResize ||
        e->type() == QEvent::Resize)
        updateRectSizes();

    if (e->type() == QEvent::MouseButtonRelease)
    {
        auto me = dynamic_cast<QMouseEvent *>(e);
        for (int i=0; i<9; i++)
        {
            if (rects[i].contains(me->pos(), Qt::OddEvenFill))
            {
                emit colorChoosed(colors[i], i);
                break;
            }
        }
    }

    return QWidget::event(e);
}

void plot_colorChooser::paintEvent(QPaintEvent * /* event */)
{
    QPainter painter(this);

    painter.setRenderHint(QPainter::Antialiasing);
    QPen pen(Qt::black, 2);
    painter.setPen(pen);
    painter.setBrush(QBrush());

    for (int i=0; i<9; i++)
    {
        QPainterPath path;
        path.addRoundedRect(rects[i], 10, 10);
        painter.fillPath(path, QBrush(colors[i]));
        painter.drawPath(path);
    }
    {//5th
        QPainterPath path;
        path.addRoundedRect(rects[5], 10, 10);
        QLinearGradient gradient;
        gradient.setSpread(QGradient::ReflectSpread);
        gradient.setStart(rects[5].x(), rects[5].y());
        gradient.setFinalStop(rects[5].x() + hMini, rects[5].y() + hMini);
        gradient.setColorAt(0, Qt::red);
        gradient.setColorAt(0.25, Qt::red);
        gradient.setColorAt(0.5, Qt::green);
        gradient.setColorAt(0.75, Qt::blue);
        gradient.setColorAt(1, Qt::blue);
        QBrush b(gradient);
        painter.fillPath(path, b);
        painter.drawPath(path);
    }
    {//8th
        painter.setBrush(QBrush(QColor(Qt::black)));
        QString str = tr("4D");
        int w = QFontMetrics(font).width(str);
        int h = QFontMetrics(font).height();
        painter.drawText(QPoint(rects[8].x() + rects[8].width()*0.5 - w*0.5, rects[8].y() + rects[8].height()*0.5 - h*0.5 + rects[8].height()*0.25), str);
        //qDebug()<<hMini;
    }
}
