#ifndef THREEDOVERLAYBUTTONS2_H
#define THREEDOVERLAYBUTTONS2_H

#include <QObject>
#include <QPushButton>
#include <QLabel>
#include <QComboBox>
#include <QDoubleSpinBox>
#include <QActionGroup>
#include "core/plot_core.h"

class TwoStateButton;

namespace plot {

class T3dPlot;
class TPure3dPlot;

class ThreeDOverlayButtons : public QWidget
{
    Q_OBJECT
public:
    ThreeDOverlayButtons();

    enum SpecialVisibilityMode
    {
        svm_none,
        svm_nothing_show,
        svm_point_selection,
        svm_markers,
        svm_area_selection,
        svm_palettes,
        svm_range_selection,
#ifndef _OLD_EXTRA_CONTROL
        svm_extra_control,
#endif
        svm_image_crop,
    };

    void ActivateRatisMode();
    void setParentPlot(T3dPlot *plot);
    void setVisibleFilterButton(bool visible);
    void setVisibleOldFilters(bool visible);
    void resetOldFilters();
    void setVisiblePointSelection(bool visible);
    void addUpButton(QPushButton *btn, int n = -1);
    void removeButton(QPushButton *btn);
    void resetBackBtnWainting();
    static void setButtonStyle(QPushButton *btn, bool activated = false);
    static void setComboBoxStyle(QComboBox *cb);
    static void setDoubleSpinBoxStyle(QDoubleSpinBox *dsp);

    bool event(QEvent *Event) override;
    bool eventFilter(QObject *Object, QEvent *Event) override;
    void setButtonActive(QPushButton *btn, bool active);

    void setAttributes(plot::TMetaStorage *attributes);
    void setVisibleMenu(int n, plot::TPure3dPlot* plotPure, bool visible);

    bool isPlotContainMouse(int n);
    void setVisibleIfUnderMouse(int n);

    bool constantShowingMenu = false;
    float btnSizeMultip = 1.0;

    plot::TMetaStorage *attributes = nullptr;

    T3dPlot *parentPlot;
    bool enabled = true;
    int blockResizeTimes = 0;
    bool useBlockResize = false;

    bool showingBackButtonPlot[4] = {false};
    bool showingApplyButtonPlot[4] = {false};
    bool replacedGUIButtons[4] = {false};
    SpecialVisibilityMode visibilityMode[4] = {svm_none};

    TwoStateButton *BPointSelection;
    TwoStateButton *BMarkers;
    TwoStateButton *BFilters;
    TwoStateButton *BAreaZzoom;
    QPushButton *BPalette;
    QPushButton *BSaveImage;

    int minBtnsSize = 25;

    QPushButton *BOldFilter;
    QPushButton *BOldFilterAddit;

    QLabel *LFont;
    QComboBox *CBFont;

    QPushButton *BBackButton;
    QPushButton *BApplyButton;

    QMenu *MMath2;

    bool setActivePlotByMouseMove = true;

    bool blockShowing3DPlotButtons = false;
    bool hideFilterButton = false;


    QList<QPushButton*> allBtnsList;

    QList<QPushButton*> upBtnsList;
    QList<QPushButton*> rightBtnsList;

signals:
    void replacedGUIPlotResized(int n, QSize newSize);
    void replacedGUIPlotSetVisible(int n, bool visible);
    void GUIPlotResized(int n, QSize newSize);
    void GUIPlotSetVisible(int n, bool visible, SpecialVisibilityMode visibilityMode);
    void BackCancelClicked(SpecialVisibilityMode visibilityMode);
    void ApplyEnterClicked(SpecialVisibilityMode visibilityMode);

public slots:
    void onPointSelection_changed(bool pressed);
    void on_Filter_changed(bool pressed);
    void onMarkers_changed(bool pressed);
    void colorPickClicked();
    void on3DBAreaZoom_changed(bool pressed);
    void onSaveImage_clicked();
    void onBack_clicked();
    void bApply_clicked();
    void ChannelResized(int n, QSize newSize);
    void scanButtonsSetVisibleGUI(bool visible, TPure3dPlot* plotPure = nullptr, bool reSetButtonsParent = true);
    void MouseEvent(QEvent* event, TPure3dPlot* plotPure, int n);

    void fontChanged(int);

    void on_GwyXFit_triggered(bool Checked);
    void on_GwyYFit_triggered(bool Checked);
    void on_MeanValue_triggered(bool Checked);
    void on_MedianValue_triggered(bool Checked);
    void on_Gaussian_triggered(bool Checked);
    void on_Histogram_triggered(bool Checked);
    void on_FitXOrder_changed();
    void on_FitYOrder_changed();
    void on_SubtrOrder_changed();
private:


    void CreateActionsAndMenus();
    bool isOldFiltersVisible = false;

    QActionGroup *AGXFitOrders;
    QActionGroup *AGYFitOrders;
    QActionGroup *AGSubtr;
    QMenu *MMathProcessing;
    bool blockLeaveEvent = false;
};

}

#endif // THREEDOVERLAYBUTTONS2_H
