#include "2d/plot_2dplot.h"

#include "qwt_plot_layout.h"
#include <QMouseEvent>
#include "widgets/twodoverlaybuttons.h"
#include "widgets/colorrect2.h"
#include "2d/plot_2dpure.h"

namespace plot
{

#ifdef _ADROID
TwoDOverlayButtons::TwoDOverlayButtons(bool vertical) : QWidget(nullptr)
#else
TwoDOverlayButtons::TwoDOverlayButtons() : QWidget(nullptr)
#endif
{
    style = "color: #8C8C8C;";
    CurveQColours[0] = Qt::black;
    CurveQColours[1] = Qt::red;
    CurveQColours[2] = Qt::green;
    CurveQColours[3] = Qt::blue;
    CurveQColours[4] = Qt::magenta;
    CurveQColours[5] = Qt::cyan;
    CurveQColours[6] = Qt::black;

    //---------------4D buttons
    B4DSameXYPoints = new TwoStateButton(":/plot/Pall_Zoom_Of.png", ":/plot/Pall_Zoom_On.png");
    B4DSameXYPoints->setToolTip(QObject::tr("Same X\\Y points as scan"));
    B4DSameXYPoints->setFlat(true);
    setButtonStyle(B4DSameXYPoints);
    B4DSameXYPoints->hide();

    B4DSendToData = new QPushButton();
    B4DSendToData->hide();
    B4DSendToData->setFlat(true);
    setButtonStyle(B4DSendToData);
    B4DSendToData->setIcon(makeHugeIcon(":/ScanPlot/attach4DPlot2D.png"));
    B4DSendToData->setToolTip(QObject::tr("Send to data"));


    B4DClearSpectrum = new QPushButton();
    B4DClearSpectrum->hide();
    B4DClearSpectrum->setFlat(true);
    setButtonStyle(B4DClearSpectrum);
    B4DClearSpectrum->setIcon(makeHugeIcon(":/CCD/mclear.png"));
    B4DClearSpectrum->setToolTip(QObject::tr("Clear spectrum"));

    B4DSectionPointSpectr = new TwoStateButton(":/CCD/point_spect_off.png", ":/plot/point.png");
    B4DSectionPointSpectr->setToolTip(QObject::tr("Switch between section and point spectra"));
    B4DSectionPointSpectr->setFlat(true);
    setButtonStyle(B4DSectionPointSpectr);
    B4DSectionPointSpectr->hide();
    connect(B4DSectionPointSpectr, SIGNAL(state_changed(bool)), this, SLOT(on4DSwitchSpectra_clicked(bool)));

    B4DSelectCapturedSpectr = new QPushButton();
    B4DSelectCapturedSpectr->hide();
    B4DSelectCapturedSpectr->setCheckable(true);
    B4DSelectCapturedSpectr->setFlat(true);
    setButtonStyle(B4DSelectCapturedSpectr);
    B4DSelectCapturedSpectr->setIcon(makeHugeIcon(":/CCD/Sp_part.png"));
    B4DSelectCapturedSpectr->setToolTip(QObject::tr("Select captured specter part"));
    connect(B4DSelectCapturedSpectr, SIGNAL(toggled(bool)), this, SLOT(on4DSelectCapturedSpectr_clicked(bool)));

    B4DSelectConvolutionFunc = new QPushButton();
    B4DSelectConvolutionFunc->hide();
    B4DSelectConvolutionFunc->setCheckable(true);
    B4DSelectConvolutionFunc->setFlat(true);
    setButtonStyle(B4DSelectConvolutionFunc);
    B4DSelectConvolutionFunc->setIcon(makeHugeIcon(":/CCD/Func_range.png"));
    B4DSelectConvolutionFunc->setToolTip(QObject::tr("Select convolution function range"));
    connect(B4DSelectConvolutionFunc, SIGNAL(toggled(bool)), this, SLOT(on4DSelectConvolutionFunc_clicked(bool)));
    //-----------------

    //----------------- 2D buttons

    B2DMarkers = new QPushButton();
    B2DMarkers->hide();
    B2DMarkers->setCheckable(true);
    B2DMarkers->setFlat(true);
    setButtonStyle(B2DMarkers);
    B2DMarkers->setIcon(makeHugeIcon(":/ScanPlot/markersPlotOff2D.png"));
    B2DMarkers->setToolTip(QObject::tr("Markers"));
    connect(B2DMarkers, SIGNAL(toggled(bool)), this, SLOT(on2DMarkers_clicked(bool)));

    B2DGrid = new TwoStateButton(":/ScanPlot/gridPlot2D.png", ":/ScanPlot/gridPlot2D.png");
    B2DGrid->setToolTip(QObject::tr("Show grid"));
    B2DGrid->setFlat(true);
    setButtonStyle(B2DGrid);
    B2DGrid->hide();
    connect(B2DGrid, SIGNAL(state_changed(bool)), this, SLOT(on2DGrid_changed(bool)));

    B2DRestoreScale = new QPushButton();
    B2DRestoreScale->hide();
    B2DRestoreScale->setFlat(true);
    setButtonStyle(B2DRestoreScale);
    B2DRestoreScale->setIcon(makeHugeIcon(":/plot/restore_scale.png"));
    B2DRestoreScale->setToolTip(QObject::tr("Restore scale"));
    connect(B2DRestoreScale, SIGNAL(released()), this, SLOT(on2DRestoreScale_clicked()));

    B2DSaveImage = new QPushButton();
    B2DSaveImage->hide();
    B2DSaveImage->setFlat(true);
    setButtonStyle(B2DSaveImage);
    B2DSaveImage->setIcon(makeHugeIcon(":/plot/picturePlot2D.png"));
    B2DSaveImage->setToolTip(QObject::tr("Save image"));
    connect(B2DSaveImage, SIGNAL(released()), this, SLOT(on2DSaveImage_clicked()));

    B2DSendToData = new QPushButton();
    B2DSendToData->hide();
    B2DSendToData->setFlat(true);
    setButtonStyle(B2DSendToData);
    B2DSendToData->setIcon(makeHugeIcon(":/plot/attachPlot2D.png"));
    B2DSendToData->setToolTip(QObject::tr("Send to data"));
    connect(B2DSendToData, SIGNAL(released()), this, SLOT(on2DSendToData_clicked()));

    B2DBack = new QPushButton();
    B2DBack->hide();
    B2DBack->setFlat(true);
    setButtonStyle(B2DBack);
    B2DBack->setIcon(makeHugeIcon(":/ScanPlot/backPlot2D.png"));
    B2DBack->setToolTip(QObject::tr("Back"));
    B2DBack->setShortcut(QKeySequence(Qt::Key_Escape));
    connect(B2DBack, SIGNAL(released()), this, SLOT(on2DBack()));

    B2DFilter = new TwoStateButton(":/ScanPlot/filterPlot2D.png", ":/ScanPlot/filterPlot2D.png");
    B2DFilter->setToolTip(QObject::tr("enable filtering"));
    B2DFilter->setFlat(true);
    setButtonStyle(B2DFilter);
    B2DFilter->hide();
    connect(B2DFilter, SIGNAL(state_changed(bool)), this, SLOT(on2DFilter_clicked(bool)));

    BOldFilter = new QPushButton();
    BOldFilter->hide();
    BOldFilter->setFlat(true);
    setButtonStyle(BOldFilter);
    BOldFilter->setIcon(makeHugeIcon(":/plot/math.png"));
    BOldFilter->setToolTip(QObject::tr("Filters"));
    MMathProcessing = new QMenu;
    BOldFilter->setMenu(MMathProcessing);
    AGFitOrders = new QActionGroup(this);
    const QString FitOrders[] = {tr("Fit. None"), tr("Fit. 0 order"), tr("Fit. 1 order"), tr("Fit. 2 order"),
                               tr("Fit. 3 order"), tr("Fit. 4 order"), tr("Fit. 5 order"), tr("Manual fit")};
    int i = 0;
    for (auto &fitOrder:FitOrders)
    {
        AGFitOrders->addAction(MMathProcessing->addAction(fitOrder));
        AGFitOrders->actions()[i]->setCheckable(true);
        i++;
    }
    AGFitOrders->actions()[0]->setChecked(true);
    connect(AGFitOrders, SIGNAL(triggered(QAction*)), this, SLOT(on_FitOrder_changed()));

    BFitData = new QPushButton();
    BFitData->hide();
    BFitData->setFlat(true);
    setButtonStyle(BFitData);
    BFitData->setIcon(makeHugeIcon(":/ScanPlot/restore1x12D.png"));
    BFitData->setToolTip(QObject::tr("Fit data size"));
    connect(BFitData, SIGNAL(clicked(bool)), this, SLOT(onFitDataClicked()));

    //--
    BShowLineMenu = new QPushButton();
    BShowLineMenu->hide();
    BShowLineMenu->setCheckable(true);
    BShowLineMenu->setFlat(true);
    setButtonStyle(BShowLineMenu);
    BShowLineMenu->setIcon(makeHugeIcon(":/plot/line2DPlotSettings.png"));
    BShowLineMenu->setToolTip(QObject::tr("Show line settings"));
    connect(BShowLineMenu, SIGNAL(clicked(bool)), this, SLOT(onBShowLineMenuClicked()));

    LLineSelector = new QLabel(tr("Current line:"));
    LLineSelector->hide();

    LLineColor = new QLabel(tr("Line color:"));
    LLineColor->hide();

    LLineWidth = new QLabel(tr("Line width:"));
    LLineWidth->hide();

    LFont = new QLabel(tr("Font size:"));
    LFont->hide();

    QString cbStyle = "QComboBox {border-width:0; background-color:rgba(255,255,255,0);}\
            QComboBox:!on {" + style + "}\
            QComboBox:pressed { background-color:rgba(255,255,255,100); border-width:1;}";

    CBLineSelector = new QComboBox;
    CBLineSelector->hide();
    CBLineSelector->setStyleSheet(cbStyle);
    CBLineSelector->addItem("All");

    CBLineColor = new QComboBox;
    CBLineColor->hide();
    CBLineColor->setStyleSheet(cbStyle);
    const char *CurveColours[] = {"Black", "Red", "Green", "Blue", "Magenta", "Cyan", "Other..."};
    for (int i=0; i<7; i++)
    {
        CBLineColor->addItem(CurveColours[i]);
        CBLineColor->setItemData(i, QBrush(CurveQColours[i]), Qt::TextColorRole);
    }
    CBLineColor->setCurrentIndex(0);
    connect(CBLineColor, SIGNAL(currentIndexChanged(int)), this, SLOT(onCBLineColor_changed(int)));

    CBLineWidth = new QComboBox;
    CBLineWidth->hide();
    CBLineWidth->setStyleSheet(cbStyle);
    const int CurveThickness[] = {1, 2, 3, 4, 5, 6, 8, 10, 15, 20};
    for (int i=0; i<10; i++)
        CBLineWidth->addItem(QString::number(CurveThickness[i]), CurveThickness[i]);
    CBLineWidth->setCurrentIndex(0);
    connect(CBLineWidth, SIGNAL(currentIndexChanged(int)), this, SLOT(onCbLineWidth_changed(int)));

    CBFont = new QComboBox;
    CBFont->hide();
    CBFont->setStyleSheet(cbStyle);
    CBFont->addItem("6", 6);
    CBFont->addItem("8", 8);
    CBFont->addItem("10", 10);
    for (int i = 0; i < 11; i++) {
        CBFont->addItem(QString::number(12 + 4*i), 12 + 4*i);
    }
    CBFont->setCurrentIndex(3);
    connect(CBFont, SIGNAL(currentIndexChanged(int)), this, SLOT(fontChanged(int)));

    background = new colorRect2(QColor(240, 240, 240));
    background->setStyleSheet("QSplitter {border: 1px solid #B4B4B4; border-style: outset; border-width: 2px; border-radius: 10px; border-color: #B4B4B4; background-color: #E3E3EC;}");

    rightBtnsList << BShowLineMenu << B2DSendToData << B2DSaveImage << B2DRestoreScale << B2DGrid << B2DMarkers << B2DFilter;
    allBtnsList << rightBtnsList << B2DBack;

#ifdef _ADROID
    this->vertical = vertical;
#endif
    lineChangersListLabels << LLineSelector << LLineColor << LLineWidth << LFont;
    lineChangersListComboBox << CBLineSelector << CBLineColor << CBLineWidth << CBFont;
    
    for (auto lab:lineChangersListLabels)
    {
        lab->setStyleSheet(style);
    }
    //-----------------
}
void TwoDOverlayButtons::fontChanged(int n)
{
    plot->Pure2dPlot->SetAutoScaleEnabled(false, false);
    plot->PlotControl->AGFontSizes->actions().at(n)->trigger();
    if (!plot->Pure2dPlot->singleAutoScale)
        plot->Pure2dPlot->SetAutoScaleEnabled(true, false);
}
void TwoDOverlayButtons::onCBLineColor_changed(int n)
{
    plot->Pure2dPlot->SetAutoScaleEnabled(false, false);
    plot->PlotControl->AGCurveColours->actions()[n]->setChecked(true);
    plot->PlotControl->AGCurveColours->actions()[n]->trigger();
    if (!plot->Pure2dPlot->singleAutoScale)
        plot->Pure2dPlot->SetAutoScaleEnabled(true, false);
    for (int i=0; i<7; i++)
        CBLineColor->setItemData(i, QBrush(CurveQColours[i]), Qt::TextColorRole);
}
void TwoDOverlayButtons::onCbLineWidth_changed(int n)
{
    plot->Pure2dPlot->SetAutoScaleEnabled(false, false);
    plot->PlotControl->AGCurveThickness->actions()[n]->setChecked(true);
    plot->PlotControl->AGCurveThickness->actions()[n]->trigger();
    if (!plot->Pure2dPlot->singleAutoScale)
        plot->Pure2dPlot->SetAutoScaleEnabled(true, false);
}
void TwoDOverlayButtons::on_FitOrder_changed()
{
    int id = FindCheckedItemIn(AGFitOrders);
    plot->PlotControl->AGFitOrders->actions()[id]->setChecked(true);
    plot->PlotControl->AGFitOrders->actions()[id]->trigger();
}
void TwoDOverlayButtons::onBShowLineMenuClicked()
{
    plot->on_CBChooseActiveCurve_activated(0);
    setVisibleGUI2D(false);
    setVisibleGUI2D(true);
    B2DBack->show();
    showBack = true;
}
void TwoDOverlayButtons::onFitDataClicked()
{
    plot->PlotControl->BAutoScale->toggle();
}
void TwoDOverlayButtons::setVisibleGUI2D(bool visible)
{
    if (!enabled)
        visible = false;

    if (visible && blockShowing2DPlotButtons)
        return;

    for (int i=0; i<allBtnsList.size(); i++)
        allBtnsList.at(i)->setParent(plot);

    background->setParent(plot);

    for (int i=0; i<lineChangersListLabels.size(); i++)
    {
        lineChangersListLabels.at(i)->setParent(plot);
        lineChangersListComboBox.at(i)->setParent(plot);
    }

    if (plot == nullptr)
        visible = false;
    else
        plot2DResized();

    background->setVisible(visible && BShowLineMenu->isChecked());
    for (int i=0; i<lineChangersListLabels.size(); i++)
    {
        lineChangersListLabels.at(i)->setVisible(visible && BShowLineMenu->isChecked());
        lineChangersListComboBox.at(i)->setVisible(visible && BShowLineMenu->isChecked());
    }

    B2DBack->setVisible(visible && showBack);

    if (visible &&
            (plot->PlotControl->BMarkers->isChecked() ||
             B4DSelectCapturedSpectr->isChecked() ||
             B4DSelectConvolutionFunc->isChecked() ||
             BShowLineMenu->isChecked() ||
             showBack/* ||
             B2DBack->isVisible()*/))
    {
        visible = false;
    }

    for (int i=0; i<leftBtnsList.size(); i++)
        leftBtnsList.at(i)->setVisible(visible);

    for (int i=0; i<rightBtnsList.size(); i++)
        rightBtnsList.at(i)->setVisible(visible);

    //B2DFilter->setVisible(!hideFilterButton && visible);
    B2DSendToData->setVisible(!hideToDataButton && visible);

    emit replacedGUIPlotSetVisible(visible);
}
void TwoDOverlayButtons::plot2DResized()
{
    plot2DResized(plot->Pure2dPlot->size(), plot->Pure2dPlot);
}
void TwoDOverlayButtons::plot2DResized(QSize newSize, plot::TPure2dPlot* plotPure)
{
    bool horiz = !plotPure->vertical;
    int leftX = plotPure->QPlot->plotLayout()->scaleRect(QwtPlot::yLeft).width()*1.15;
    int rightX = plotPure->QPlot->width() - plotPure->QPlot->plotLayout()->scaleRect(QwtPlot::yRight).width()*1.15;
    int upY = plotPure->QPlot->plotLayout()->titleRect().height()*1.5;
    if (upY == 0)
        upY = 13*3;
    int downY = newSize.height() - plotPure->QPlot->plotLayout()->scaleRect(QwtPlot::xBottom).height();
    int lessSz = newSize.width() < newSize.height() ? newSize.width():newSize.height();
    int BtnsSize = (lessSz - leftX)*0.2;
    int UpButtonSize = BtnsSize*0.8;
    if (UpButtonSize < 55) UpButtonSize = 55;
    if (UpButtonSize > 100) UpButtonSize = 100;
    UpButtonSize *= btnSizeMultip;
    lastUpButtonSize = UpButtonSize;

    for (int i=0; i<allBtnsList.size(); i++)
    {
        allBtnsList.at(i)->setFixedSize(UpButtonSize, UpButtonSize);
        allBtnsList.at(i)->setIconSize(QSize(UpButtonSize, UpButtonSize));
    }

    double mulBtn = 0.05;

    if (horiz)
    {
        int x = leftX;
        int y = upY;
        int leftMaxX = x;
        for (int i=0; i<leftBtnsList.size(); i++)
        {
            leftBtnsList[i]->move(x, y);
            if (isLeftButtonsVertical)
            {
                y += UpButtonSize + UpButtonSize*mulBtn;
                if (y + UpButtonSize > downY - UpButtonSize*mulBtn)
                {
                    x += UpButtonSize + UpButtonSize*mulBtn;
                    y = upY;
                }
            }
            else
                x += UpButtonSize + UpButtonSize*mulBtn;

            if (x > leftMaxX)
                leftMaxX = x;
        }

        if (verticalPlacingRightButtons)
        {
            int yStart = newSize.height() - plotPure->QPlot->plotLayout()->scaleRect(QwtPlot::xBottom).height();
            y = upY - (UpButtonSize + UpButtonSize*mulBtn);
            x = newSize.width() - UpButtonSize - UpButtonSize*mulBtn;
            for (int i=0; i<rightBtnsList.size(); i++)
            {
                y += UpButtonSize + UpButtonSize*mulBtn;
                if (y > yStart - (UpButtonSize + UpButtonSize*mulBtn))
                {
                    y = upY;
                    x -= UpButtonSize + UpButtonSize*mulBtn;
                }
                rightBtnsList[i]->move(x, y);
            }
        }
        else
        {
            leftMaxX += UpButtonSize + UpButtonSize*mulBtn;
            if (leftBtnsList.size() == 0)
                leftMaxX = leftX;
            x = newSize.width();
            y = upY;
            for (int i=0; i<rightBtnsList.size(); i++)
            {
                x -= UpButtonSize + UpButtonSize*mulBtn;
                if (x <= leftMaxX)
                {
                    x = newSize.width() - UpButtonSize - UpButtonSize*mulBtn;
                    y += UpButtonSize + UpButtonSize*mulBtn;
                }
                rightBtnsList[i]->move(x, y);
            }
        }
    }
    else
    {
        int x = newSize.width() - UpButtonSize - UpButtonSize*mulBtn;

        int y = upY;
        for (int i=0; i<leftBtnsList.size(); i++)
        {
            leftBtnsList[i]->move(x, y);
            y += UpButtonSize + UpButtonSize*mulBtn;
        }
        int lastUpbtnY = y;

        int yStart = newSize.height() - plotPure->QPlot->plotLayout()->scaleRect(QwtPlot::xBottom).height();
        y = yStart;
        for (int i=0; i<rightBtnsList.size(); i++)
        {
            y -= UpButtonSize + UpButtonSize*mulBtn;
            if (lastUpbtnY > y)
            {
                y = yStart - (UpButtonSize + UpButtonSize*mulBtn);
                x -= UpButtonSize + UpButtonSize*mulBtn;
            }
            rightBtnsList[i]->move(x, y);
        }
    }

    B2DBack->setFixedSize(UpButtonSize, UpButtonSize);
    B2DBack->setIconSize(QSize(UpButtonSize, UpButtonSize));
    B2DBack->move(leftX, upY);

    #ifndef _ADROID
    B2DFilter->change_state_man(plotPure->isFiltering);
    setButtonActive(B2DFilter, plotPure->isFiltering);
    #endif

    int cbSize = (newSize.height() - upY)*0.05*1.14;
    cbSize = UpButtonSize;
    if (cbSize < 0) cbSize = 0;
    if (cbSize > 50) cbSize = 50;
    auto font = QFont(LFont->font().family(), cbSize*0.45);
    font.setBold(true);

    
    #ifdef _ADROID
    if (vertical)
    {
        vFilters->setIconSize(QSize(UpButtonSize, UpButtonSize));
        int maxHeightCB = 0;
        for (int i=0; i<lineChangersListComboBox.size(); i++)
        {
            int t = lineChangersListComboBox.at(i)->minimumSizeHint().height();
            if (t > maxHeightCB)
                maxHeightCB = t;
        }
    
        for (int i=0; i<lineChangersListComboBox.size(); i++)
        {
            auto item = lineChangersListComboBox.at(i);
            item->setFixedSize(cbSize, maxHeightCB);
        }
        
        int maxRightY = 0;
        int cbsWidth = 0;
        for (int i=0; i<lineChangersListLabels.size(); i++)
        {
            int t = lineChangersListLabels.at(i)->height() + lineChangersListComboBox.at(i)->height();
            //qDebug()<<lineChangersListLabels.at(i)->height()<<lineChangersListComboBox.at(i)->height()<<t;
            if (t > maxRightY)
                maxRightY = t;
            cbsWidth += lineChangersListComboBox.at(i)->width();
        }
    
        background->setFixedSize(cbsWidth + 20*2, maxRightY + 20*2);
        int x = rightX - background->width() - 20;
        int y = downY - background->height() - 200;
        background->move(x, y - 20);
        x += cbsWidth;
        for (int i=0; i<lineChangersListLabels.size(); i++)
        {
            lineChangersListLabels.at(i)->move(x, y);
            lineChangersListComboBox.at(i)->move(x, y + maxRightY - lineChangersListComboBox.at(i)->height());
            x -= lineChangersListComboBox.at(i)->width();
        }
    }
    else
#endif
   {     
        for (int i=0; i<lineChangersListLabels.size(); i++)
        {
            lineChangersListLabels.at(i)->setScaledContents(true);
            lineChangersListLabels.at(i)->setFont(font);
            lineChangersListLabels.at(i)->adjustSize();
        }
        int maxWidthCB = 0;
        for (int i=0; i<lineChangersListComboBox.size(); i++)
        {
            int t = lineChangersListComboBox.at(i)->minimumSizeHint().width();
            if (t > maxWidthCB)
                maxWidthCB = t;
        }
        
        for (int i=0; i<lineChangersListComboBox.size(); i++)
        {
            auto item = lineChangersListComboBox.at(i);
            auto view = item;
            view->setFont(font);
            item->setFixedSize(maxWidthCB, cbSize);
            item->setFont(font);
        }
        
        int maxRightX = 0;
        int cbsHeight = 0;
        for (int i=0; i<lineChangersListLabels.size(); i++)
        {
            int t = lineChangersListLabels.at(i)->width() + lineChangersListComboBox.at(i)->width();
            if (t > maxRightX)
                maxRightX = t;
            cbsHeight += lineChangersListComboBox.at(i)->height();
        }
        
        int x = newSize.width()*0.5 - maxRightX*0.5;
        int y = newSize.height()*0.5 - cbsHeight*0.5;
        
        background->setFixedSize(maxRightX + 20*2, cbsHeight + 20*2);
        
        //x = leftX*1.3;
        x = rightX - background->width();
        y = upY*1.9;
        
        
        background->move(x - 20, y - 20);
        for (int i=0; i<lineChangersListLabels.size(); i++)
        {
            lineChangersListLabels.at(i)->move(x, y + lineChangersListLabels.at(i)->height()*0.2);
            lineChangersListComboBox.at(i)->move(x + maxRightX - lineChangersListComboBox.at(i)->width(), y);
            y += lineChangersListComboBox.at(i)->height();
        }
    }

    emit replacedGUIPlotResized(newSize);
}
void TwoDOverlayButtons::plot2DMouseEvent(QMouseEvent* mouse, plot::TPure2dPlot* plotPure)
{
    if (constantShowingMenu)
        return;

    if (mouse->type() == QEvent::Enter || mouse->type() == QEvent::MouseMove)
    {
        setVisibleGUI2D(true);
    }
    else if (mouse->type() == QEvent::Leave)
    {
		auto rect = plotPure->rect();
        int boundH = rect.height()*0.01;
        int boundW = rect.width()*0.01;
        QRect smallerRect(rect.left()+boundW, rect.top()+boundH,
			rect.width()-boundW*2, rect.height()-boundH*2);
        auto pos = plotPure->mapFromGlobal(QCursor::pos());
        setVisibleGUI2D(smallerRect.contains(pos));
    }
    else if (mouse->type() == QEvent::Resize && plotPure->vertical)
    {
        setVisibleGUI2D(true);
        setVisibleGUI2D(false);
    }
}
void TwoDOverlayButtons::setShowingSendToDataButton(bool showing)
{
    if (showing)
    {
        if (!rightBtnsList.contains(B2DSendToData))
            rightBtnsList.insert(0, B2DSendToData);
        if (!allBtnsList.contains(B2DSendToData))
            allBtnsList << B2DSendToData;
        if (constantShowingMenu)
        {
            B2DSendToData->setParent(plot);
            B2DSendToData->setVisible(true);
        }
    }
    else
    {
        B2DSendToData->setVisible(false);
        rightBtnsList.removeAll(B2DSendToData);
        allBtnsList.removeAll(B2DSendToData);
    }
    hideToDataButton = !showing;
    plot2DResized();
}
void TwoDOverlayButtons::on4DSelectCapturedSpectr_clicked(bool pressed)
{
    if (FourDScan == "")
        return;

    waitingEnterButton = pressed;
    if (pressed)
        showBack = true;
    setVisibleGUI2D(false);
    setVisibleGUI2D(true);
}
void TwoDOverlayButtons::on4DSelectConvolutionFunc_clicked(bool pressed)
{
    if (FourDScan == "")
        return;

    waitingEnterButton = pressed;
    if (pressed)
        showBack = true;
    setVisibleGUI2D(false);
    setVisibleGUI2D(true);
}
void TwoDOverlayButtons::on4DSwitchSpectra_clicked(bool pressed)
{
    if (FourDScan == "")
    {
        B4DSectionPointSpectr->change_state_man(!pressed);
        return;
    }
}
void TwoDOverlayButtons::on2DMarkers_clicked(bool pressed)
{
    plot->PlotControl->BMarkers->click();
    if (pressed)
        showBack = true;
    setVisibleGUI2D(false);
    setVisibleGUI2D(true);
}
void TwoDOverlayButtons::on2DGrid_changed(bool pressed)
{
    setButtonActive(B2DGrid, pressed);
    plot->PlotControl->BGrid->setChecked(pressed);
}
void TwoDOverlayButtons::on2DRestoreScale_clicked()
{
    plot->Refresh();
    bool autoScaleEnabled = plot->PlotControl->BAutoScale->isChecked();
    #ifdef _ADROID
    autoScaleEnabled = autoScaleEnabled || vertical;
#endif
    plot->Pure2dPlot->SetAutoScaleEnabled(autoScaleEnabled, true);
    plot->PlotControl->BRestoreScale->click();
}
void TwoDOverlayButtons::on2DSaveImage_clicked()
{
    blockShowing2DPlotButtons = true;
    setVisibleGUI2D(false);
    plot->PlotControl->BSavePlotPixmap->click();
    blockShowing2DPlotButtons = false;
    setVisibleGUI2D(true);
}
void TwoDOverlayButtons::on2DSendToData_clicked()
{
    plot->PlotControl->BSave->click();
}

void TwoDOverlayButtons::on2DBack()
{
    showBack = false;
    B2DBack->hide();
    setVisibleGUI2D(false);
    if (B2DMarkers->isChecked())
        B2DMarkers->setChecked(false);
    else if (B4DSelectCapturedSpectr->isChecked() || B4DSelectConvolutionFunc->isChecked())
    {
        B4DSelectCapturedSpectr->setChecked(false);
        B4DSelectConvolutionFunc->setChecked(false);
        emit on_BCancelApply_clickedSignal();
    }
    else if (BShowLineMenu->isChecked())
    {
        BShowLineMenu->setChecked(false);
        if (!plot->Pure2dPlot->singleAutoScale)
            plot->Pure2dPlot->SetAutoScaleEnabled(true, false);
    }
    B2DBack->hide();
    emit onBackClicked();
    setVisibleGUI2D(true);
}
void TwoDOverlayButtons::on2DFilter_clicked(bool pressed)
{
    setButtonActive(B2DFilter, pressed);
    plot->setFiltering(pressed);
    if (plot->depent2dPlot)
        plot->depent2dPlot->setFiltering(pressed);
    else if (plot->master2dPlot)
        plot->master2dPlot->setFiltering(pressed);
    #ifdef _ADROID
    if (vertical)
        plot->Refresh();
#endif
}

void TwoDOverlayButtons::set2DPlot(plot::T2dPlot *pl)
{
    plot = pl;
    connect(CBLineSelector, SIGNAL(highlighted(int)), plot, SLOT(on_CBChooseActiveCurve_highlighted(int)));
    connect(CBLineSelector, SIGNAL(activated(int)), plot, SLOT(on_CBChooseActiveCurve_activated(int)));
    setButtonActive(B2DFilter, plot->Pure2dPlot->isFiltering);
    B2DFilter->change_state_man(plot->Pure2dPlot->isFiltering);
    
#ifdef _ADROID
    if (vertical)
    {
        const QString FitOrders[] = {tr("Fit. None"), tr("Fit. 0 order"), tr("Fit. 1 order"), tr("Fit. 2 order"),
                                   tr("Fit. 3 order"), tr("Fit. 4 order"), tr("Fit. 5 order"), tr("Manual fit")};
        const char *CurveColours[] = {"Black", "Red", "Green", "Blue", "Magenta", "Cyan", "Other..."};
        
        vMenuFilters = new verticalQMenu(pl);
        vFilters = new verticalQPushButton(vMenuFilters, pl);
        vFilters->hide();
        vFilters->setFlat(true);
        vFilters->setIcon(makeHugeIcon(":/plot/math.png"));
        vFilters->setIcon(rotareIcon(vFilters->icon(), vFilters->size(), 90));
        vFilters->setIconSize(vFilters->size());
        int i = 0;
        for (auto &fitOrder:FitOrders)
        {
            auto item = vMenuFilters->addMenuItem(fitOrder, false);
            item->checkable = true;
            if (i == 0)
                item->checked = true;
            connect(item, SIGNAL(clicked(int)), this, SLOT(setFilterVertical(int)));
            i++;
        }
        setButtonStyle(vFilters, false);
        
        vLLineSelector = new VerticalLabel(tr("Current line:"));
        vLLineSelector->hide();
    
        vLLineColor = new VerticalLabel(tr("Line color:"));
        vLLineColor->hide();
    
        vLLineWidth = new VerticalLabel(tr("Line width:"));
        vLLineWidth->hide();
    
        vLFont = new VerticalLabel(tr("Font size:"));
        vLFont->hide();
    
        vCBLineSelector = new verticalQComboBox();
        vCBLineSelector->hide();
        vCBLineSelector->addItem("All", QVariant());
        vCBLineSelector->setCurrentIndex(0);
    
        vCBLineColor = new verticalQComboBox();
        vCBLineColor->hide();
        for (int i=0; i<7; i++)
        {
            vCBLineColor->addItem(CurveColours[i], QBrush(CurveQColours[i]));
        }
        vCBLineColor->setCurrentIndex(0);
        connect(vCBLineColor, SIGNAL(currentIndexChanged(int)), this, SLOT(onCBLineColor_changed(int)));
    
        vCBLineWidth = new verticalQComboBox();
        vCBLineWidth->hide();
        const int CurveThickness[] = {1, 2, 3, 4, 5, 6, 8, 10, 15, 20};
        for (int i=0; i<10; i++)
            vCBLineWidth->addItem(QString::number(CurveThickness[i]), CurveThickness[i]);
        vCBLineWidth->setCurrentIndex(0);
        connect(vCBLineWidth, SIGNAL(currentIndexChanged(int)), this, SLOT(onCbLineWidth_changed(int)));
    
        vCBFont = new verticalQComboBox();
        vCBFont->hide();
        vCBFont->addItem("6", 6);
        vCBFont->addItem("8", 8);
        vCBFont->addItem("10", 10);
        for (int i = 0; i < 11; i++) {
            vCBFont->addItem(QString::number(12 + 4*i), 12 + 4*i);
        }
        vCBFont->setCurrentIndex(3);
        connect(vCBFont, SIGNAL(currentIndexChanged(int)), this, SLOT(fontChanged(int)));
        lineChangersListLabels.clear();
        lineChangersListComboBox.clear();
        lineChangersListLabels << vLLineSelector << vLLineColor << vLLineWidth << vLFont;
        lineChangersListComboBox << vCBLineSelector << vCBLineColor << vCBLineWidth << vCBFont;
    }
#endif
}

void TwoDOverlayButtons::set4DScan(const QString &scan)
{
    FourDScan = scan;

    for (int i=0; i<leftBtnsList.size(); i++)
        allBtnsList.removeOne(leftBtnsList.at(i));
    leftBtnsList.clear();

    if (FourDScan != "")
    {
        leftBtnsList.clear();
        leftBtnsList.push_back(B4DSendToData);
        leftBtnsList.push_back(B4DClearSpectrum);
        leftBtnsList.push_back(B4DSectionPointSpectr);
    }

    if (FourDScan == "jmscan")
    {
        allBtnsList.removeOne(B4DSelectCapturedSpectr);
        allBtnsList.removeOne(B4DSelectConvolutionFunc);
        leftBtnsList.push_back(B4DSameXYPoints);
    }
    if (FourDScan == "ccdscan")
    {
        allBtnsList.removeOne(B4DSameXYPoints);
        leftBtnsList.push_back(B4DSelectCapturedSpectr);
        leftBtnsList.push_back(B4DSelectConvolutionFunc);
    }

    for (int i=0; i<leftBtnsList.size(); i++)
        allBtnsList.push_back(leftBtnsList.at(i));
}

bool TwoDOverlayButtons::event(QEvent *Event)
{
    if (Event->type() == QEvent::KeyPress)
    {
        if (((QKeyEvent *)Event)->key() == Qt::Key_Return ||
            ((QKeyEvent *)Event)->key() == Qt::Key_Enter)
        {
            if (waitingEnterButton)
            {
                waitingEnterButton = false;
                plot->PlotControl->BApply->click();
                plot2DResized();
            }
        }
    }
    
    #ifdef _ADROID
    if (Event->type() == QEvent::MouseButtonPress)
    {
        if (vCBLineSelector->vMenu->isVisible())
            vCBLineSelector->event(Event);
        if (vCBLineColor->vMenu->isVisible())
            vCBLineColor->event(Event);
        if (vCBLineWidth->vMenu->isVisible())
            vCBLineWidth->event(Event);
        if (vCBFont->vMenu->isVisible())
            vCBFont->event(Event);
    }
#endif

    return QWidget::event(Event);
}

bool TwoDOverlayButtons::eventFilter(QObject *Obj, QEvent *Event)
{
    if (Event->type() == QEvent::KeyPress)
    {
        if (((QKeyEvent *)Event)->key() == Qt::Key_Return ||
            ((QKeyEvent *)Event)->key() == Qt::Key_Enter)
        {
            if (waitingEnterButton)
            {
                waitingEnterButton = false;
                plot->PlotControl->BApply->click();
                plot2DResized();
            }
        }
    }

    return QObject::eventFilter(Obj, Event);
}

void TwoDOverlayButtons::addLeftButton(QPushButton *btn)
{
    if (!leftBtnsList.contains(btn))
        leftBtnsList.push_back(btn);
    if (!allBtnsList.contains(btn))
        allBtnsList.push_back(btn);
}

void TwoDOverlayButtons::addRightButton(QPushButton *btn, int i)
{
    if (!leftBtnsList.contains(btn))
    {
        if (i == -1)
            rightBtnsList.push_back(btn);
        else
            rightBtnsList.insert(i, btn);
    }
    if (!allBtnsList.contains(btn))
    {
        allBtnsList.push_back(btn);
    }
}

void TwoDOverlayButtons::removeButton(QPushButton *btn)
{
    btn->setVisible(false);
    allBtnsList.removeOne(btn);
    leftBtnsList.removeOne(btn);
    rightBtnsList.removeOne(btn);
}

void TwoDOverlayButtons::setVisibleDataFitSizeButton(bool visible)
{
    dataFitSizeVisible = visible;
    if (visible)
        addRightButton(BFitData, 3);
    else
        removeButton(BFitData);
}

void TwoDOverlayButtons::removeFilterButton()
{
    hideFilterButton = true;
    isOldFiltersVisible = false;
    B2DFilter->setVisible(false);
    BOldFilter->setVisible(false);
    removeButton(B2DFilter);
    removeButton(BOldFilter);
    plot2DResized();
}

void TwoDOverlayButtons::setVisibleOldFilters(bool visible)
{
    if (isOldFiltersVisible == visible)
        return;

    hideFilterButton = !visible;
    isOldFiltersVisible = visible;
    if (visible)
    {
        plot->Pure2dPlot->isFiltering = true;
        addRightButton(BOldFilter);
        removeButton(B2DFilter);
        B2DFilter->setVisible(false);
    }
    else
    {
        removeButton(BOldFilter);
        addRightButton(B2DFilter);
    }
    plot2DResized();
}
void TwoDOverlayButtons::setButtonActive(QPushButton *btn, bool active)
{
    auto sz = btn->iconSize();
    setButtonStyle(btn, active);
    btn->setIconSize(sz);
}
void TwoDOverlayButtons::setButtonStyle(QPushButton *btn, bool activated)
{
    QString borderColor = activated ? "red" : "#9F9F9F";

    QString style = "";

    style += "QPushButton {background-color: rgba(255,255,255,10);";
    style += "border: 2px solid " + borderColor +";}";
    style += "QToolTip { color: black; background-color: #FFFFE1; border: 1px solid black; }";
    style += "QPushButton:pressed{ background-color: lightgray; }";

    btn->setStyleSheet( style );
}

void TwoDOverlayButtons::resetOldFilters()
{
    AGFitOrders->actions()[0]->setChecked(true);
    AGFitOrders->actions()[0]->trigger();

}

#ifdef _ADROID
void TwoDOverlayButtons::setFilterVertical(int id)
{
    AGFitOrders->actions()[id]->setChecked(true);
    AGFitOrders->actions()[id]->trigger();
}
#endif

}



