#ifndef VIDEOPLAYER_H
#define VIDEOPLAYER_H

#include <QDialog>
#include <QWidget>
#include <QElapsedTimer>
#include <QLabel>
#include <QGridLayout>
#include <QtEvents>
#include <QPointer>
#include <QFileDialog>
#include <QFutureWatcher>
class QCheckBox;
class QComboBox;
class QRubberBand;
#include "controls.h"
#include "edit.hpp"
#include "qwt_plot.h"
#include <qwt_plot_grid.h>
#include <QTableWidget>
#include <QHeaderView>
#include "status.hpp"
#include "videosettings.h"
#include "i_video.hpp"

class StatusWidget1;
namespace plot {
    class ImageItem;
    class TLineSegment;
}
class QwtPlotMarker;
class VideoPlayer : public QWidget
{
    Q_OBJECT
public:
    VideoPlayer(QWidget *parent = nullptr);
    ~VideoPlayer() override;
    static VideoPlayer* GetIt() noexcept;

    QList<QString> *getVideoProfiles();
    inline double getContrast();
    double getContrast(QImage *img);
    double autoFocusGetAverageContrast(int frames, int maxTimerWait);

    static int times_constructed;
    int inst_idx;

    StatusWidget1 *deviceStatus;
    TwoStateButton *m_BStart;

    QPointF posx;
    bool active;
    bool playing;
    QwtPlotMarker *marker;

    DoubleChanger *zPos;
    bool isInterrupted;

    QString configPath;

    QWidget *mainWidget;

    QFutureWatcher<void> resetWatch;
    void resetCamerasImpl(); //private

    QCheckBox *mafPaf;
    QComboBox *autoFocusMode;
    Edit *autoFocusExitDistanceEdit;
    Edit *tipThinkbnessEdit;

    int tipThinkbness = 0;
    QCheckBox *mafPrcsEnabled;
    QCheckBox *mafFastEnabled;
    QCheckBox *pafEnabled;

    Edit *motorZFocusSpeedEdit;
    Edit *motorXYStageSpeedEdit;
    int motorZFocusSpeed = 1;
    int motorXYStageSpeed = 1;

    QPushButton *BAutoFocus;
    // ///////////////////-
    int autoFocusScanVariant = 1;

    bool autoFocusStopped = true;
    QPointer<StatusBar> autoFocusBar;
    int autoFocusV1 = 30;
    int autoFocusV2 = 3;
    int autoFocusVLeftStPr = 5;
    int autoFocusDist1 = 5000;
    int autoFocusDist2 = 50000;
    int autoFocusDist3 = 50000;
    double autoFocusMagicMultiply[2] = {5, 10};
    int autoFocusMagicMultiplyCurrentId = 0;

    int autoFocusExitDistance = 100;

    int autoFocusStage = 0;
    bool autoFocusDirection = 0; //0-down, 1 - up

public slots:
    void resetCameras();
    void resetCamerasCommit(); //private

    void autoFocusStages(int motorVirt, bool atEnd, bool active);
    void autoFocusSettingsChanged();

    void cancelOperation();
    void on_Camera_ImgReceived(QImage Img);
    void calcOscContrast(QByteArray data);
    void on_BAutoFocus_clicked();
    void on_BAutoFocusOsc_clicked();
    void on_BToData_clicked();
    bool StartStop(bool value);
    void sl_saveToFile();
    void on_cwh_ch();
    void sl_center_set(bool value);
    void sl_center_set2(bool value);
    void sl_axes(bool value);
    void sl_grid(bool value);
    void sl_line(bool value);
    void sl_add_config();
    void sl_del_config();
    void sl_select_config(int val);
    void sl_save();
    void sl_line_measure(bool value);

    void sl_set_XYZ_range(double x, double y, double z);
    void sl_set_XYZ_range2(double x, double y, double z);
    void sl_range();
    double sl_get_rangeX();
    double sl_get_rangeY();

    void SetCurrentVideoProfile();

    void goZjoy(int speed);
    void goXYjoy(QPointF speedxy);
    void getXYMidx(int set, int *x, int *y);
    void BAutoFocus_clicked(bool turnOn);

    void videoPlotEvent(QObject*, QEvent*);

private:
    void setDeviceStatus(int);
    bool eventFilter(QObject *, QEvent *) override;
    void closeEvent(QCloseEvent *) override;
    void addNewProfileInTable(int index);
    void setVisiblePlotButtons(bool visible);
    void loadVideoConfiguration();

    int plot_mode;
    QPointF prev_xy_joy_output;
    QRubberBand *head_cant;
    QRubberBand *stage_cant;
    QRubberBand *stage_laser;
    QwtPlotMarker *markerA;
    QwtPlotMarker *markerB;
    QwtPlotMarker *markerC;
    plot::TLineSegment *lineAB;

    videoSettings vSettings;
    int prev_idx;

    double Xmax, Ymax;
    double Xmax2, Ymax2;
    qreal mkm_px_x;
    qreal mkm_px_y;
    Edit *ewsz;
    TwoStateButton *BCenterAndSet;
    TwoStateButton *BCenterAndSet2;
    TwoStateButton *BLine;
    TwoStateButton *BLineMeasure;
    QPushButton *BAddConfig;
    QPushButton *BDelConfig;
    QCheckBox *flipx;
    QCheckBox *flipy;
    QCheckBox *rotxy;
    QComboBox *xyMsel;

    double cw;
    double ch;

    QVector<IVideo *> CameraList;
    IVideo *activeCamera;

    int frames;
    double osc_contrast;
    bool new_osc_contrast;

    QPushButton *saveB;

    QPushButton *BAutoFocusOsc;
    QPushButton *BToData;
    TwoStateButton *BCantileverRange;
    TwoStateButton *BLaserRange;
    TwoStateButton *BGrid;
    TwoStateButton *BAxes;

    QwtPlot *QPlot;
    QwtPlotGrid *QGrid;
    plot::ImageItem *ritem;
    QElapsedTimer refresh_limit_timer;
    qint64 refresh_limit_last_elapsed;

    QTableWidget *TableVideoProfiles;

public: signals:
    void ImgReceivedAfter(QImage Img);
    void VideoProfileChanged(int profileId);
    void AutoFocusEnded(bool sucsessfull);
    void AutoFocus2Ended(bool sucsessfull);
};

#endif
