#pragma once

#include <QWidget>
#include "plot.h"
#include <defs.h>
#include "tools.hpp"
#include "controls.h"
#include "gui/status.hpp"
#include "oscilloscope.h"
#include "scanplot.h"
#include "gui/status.hpp"
#include "modules.hpp"
#include <QApplication>
#include <QSettings>
#include <QCheckBox>
#include <QMessageBox>
#include <QDebug>
#include "options_window.h"
class QCheckBox;
class QTableWidget;

class DelayedInitWidget : public QWidget
{
public:
    DelayedInitWidget(QWidget *parent = nullptr) : QWidget(parent) { }

    bool needGuiInit = true;
    virtual void GuiCreate() = 0;
    void showEvent(QShowEvent *) override {
        if (needGuiInit) {
            needGuiInit = false;
            GuiCreate();
        }
    }
};

class Oscilloscope;

class AutoCalibPlot : public DelayedInitWidget
{
    Q_OBJECT

public slots:
    void BGetClicked();
    void BSendClicked();
    void ReCalcFromCalibCurve();
    void ReCalcFromSourceMan();
    void ReCalcFromSource();
    void ReCalcCoefficent();
    void ValuesUpdate();

public:
    AutoCalibPlot(QWidget *parent = nullptr);
    void GuiCreate() override;
    bool eventFilter(QObject *, QEvent *) override;

    Oscilloscope *osc;

    double calib_coeff;
    double calib_min;
    QVector<double> adcvalues;
    QwtPlotMarker *levelMarker;
    QVector<QwtPlotMarker *> markers;
    QwtPlotMarker *activemarker;
    int activepoint;

    plot::T2dPlot *Source;
    plot::T2dPlot *CalibCurve;
    plot::TCurvesDataOwner curveData;
    plot::TCurvesDataOwner curveDataCalib;

    int CBScanner;
    int CBValue;
    int CBAxis;

    QCheckBox *CBLevel;
    Edit *LEPeriod;
    Edit *LELevel;
    QPushButton *coeffOk;

    double lineC[6];
};

class AutoCalib : public QObject, public IProcessPacket
{
    Q_OBJECT

public:
    bool sensorNoZ;
    bool isInterrupted;


    double read_direct(int ch);

protected:
    void SetOscChannel(int ch);
    double ReadOscFirst(int timeout = 1000);
    double ReadOscAverage(int timeout = 1000);
    double ReadOscLine(WORD *data_line, int timeout = 1000);
    double LineCalcAverage(WORD *data_line);
    double LineCalcRms(WORD *data_line);
    long LineCalcMaximums(WORD *data_line);

    void go_osc_xyz(double xpos, double ypos, double zpos);
    void read_osc(double *v, int ch);
    void read_osc_xyz(double *x, double *y, double *z);
    void read_osc_xyz_v(double *x, double *y, double *z);
    bool read_osc_xyz_no_outer_drift(double *x, double *y, double *z, int x_sign, int y_sign, int z_sign);
    bool read_osc_xyz_v_no_outer_drift(double *x, double *y, double *z, int x_sign, int y_sign, int z_sign);
    void read_osc_xyz(double xpos, double ypos, double zpos,
                      double *x, double *y, double *z);
    void read_osc_xyz_no_outer_drift(double xpos, double ypos, double zpos,
                                     double *x, double *y, double *z, int x_sign, int y_sign, int z_sign);

    bool processPacket(const BYTE addr, const BYTE sub_addr, const void *data, const DWORD size) override;
    bool use_direct_get_data;
    int capbits_updated;
    bool new_osc_data;
    WORD osc_data[OSC_LINE_SIZE];
    int want_osc_ch;
    bool is_optical_scanner[3];
    BYTE scanner;

public:
    AutoCalib();
    Oscilloscope *osc;
    QPointer<StatusBar> barRecalc;

signals:
    void calibDone(bool);
    void sensorsRecalibDone(bool);
    void calibInProgress(bool);
    void calibCanceled(); //Canceled by User or ended, but not done

public slots:
    void s_pidError(int);
    void doXYcalib(int);

    void recvOscData(QByteArray data);
    void capBits(int, long);
    void reset();
    void cancelOperation();

    bool SensorReCalc(BYTE scanner, bool silent = false);
    void PICalcAuto(BYTE scanner, bool silent = false);

private:
    bool restoreSensorReCalc(double x_range, double y_range, double z_range,
                             bool old_fb, bool old_fbz, bool old_nl,
                             DWORD Scanner_data_scale[], long Scanner_data_Shift[],
                             int old_ch[], QWidgetList cwl);
    bool fixScaleAtBounds(double x, double y, double z, double *scale);
    void restorePICalc(double x_range, double y_range, double z_range,
                             bool old_fb, bool old_fbz, bool old_nl,
                             int old_ch[], QWidgetList cwl);

};
