#pragma once

#define ACQMODE_SINGLE 0
#define ACQMODE_ACCUM 1
#define ACQMODE_KINETIC 2

#define RESMODE_FVB 0
#define RESMODE_SINGLETRACK 1
#define RESMODE_IMAGE 2

#define TRGMODE_AUTO 0
#define TRGMODE_EXT 1
#define TRGMODE_EXT2 2

class QWidget;

class I_CCD
{
public:
    I_CCD() { }
    virtual ~I_CCD() {}

    virtual bool initialize(int num = 0) = 0;
    virtual bool shutdown() = 0;

    virtual bool setAcquisitionMode(int mode) = 0;
    virtual bool setResultsMode(int mode) = 0;
    virtual bool setTriggerMode(int mode) = 0;

    virtual bool setTemperature(int temp) { temperature = temp; return false; }
    virtual int getTemperature() { return temperature; }

    virtual bool startAcquisition() = 0;
    virtual bool stopAcquisition() = 0;
    virtual bool isReady() = 0;
    virtual bool getEffectiveTimings(double *exposure, double *accumulate, double *kinetic) = 0; //ms
    virtual bool getData() { return false; }

    virtual QWidget *createInterfaceWidget() { return nullptr; }

    bool initialized = false;
    long *p_data = nullptr;

    bool hasTemperature = false;
    int temperature = 20; //C
    int minTemperature = -20;
    int maxTemperature = 20;

    int xPixelsCount = 1;
    int yPixelsCount = 1;
    double xPixelSize = 0.024; //mm
    double yPixelSize = 0.024;

    int acqMode = 0;
    int resMode = 2;
    int triggerMode = 0;

    int centralRow = 0;
    int trackHeight = 1;

    double exposureTime = 0.01; //s
    double accumulateTime = 0.1;
    double kineticCycleTime = 0.1;

    int accumulateCount = 1;
    int kineticCyclesCount = 1;
};
