#include "ascifile.h"
#include <QFile>
#include <QTextStream>

ascifile::ascifile(QString filename_new)
{
    data_x = nullptr;
    data_y = nullptr;
    xpts = ypts = 0;
    filename=filename_new;
}
void ascifile::set_instrument(QString instrument_new)
{
    instrument=instrument_new;
}
void ascifile::set_data_name(QString dataname_new)
{
    dataname=dataname_new;
}
void ascifile::set_x_type(QString x_name_new)
{
    X_name=x_name_new;
}
void ascifile::set_y_type(QString y_name_new)
{
    Y_name=y_name_new;
}
void ascifile::set_xpts(int n)
{
    xpts=n;
}
void ascifile::set_ypts(int n)
{
    ypts=n;
}
void ascifile::set_x_data(const double *x)
{
    data_x=x;
}
void ascifile::set_y_data(const double *y)
{
    data_y=y;
}
void ascifile::set_array(plot::T2dArray _data)
{
    data=_data;
}
void ascifile::set_xsz(Range &n)
{
    xsz=n;
}
void ascifile::set_ysz(Range &n)
{
    ysz=n;
}
QString ascifile::realToString(const double d)
{
    auto str = QString::number(d);
    str = str.replace('.', ',');
    return str;
}
void ascifile::write_file()
{
    QFile dataFile(filename);
    dataFile.open(QIODevice::WriteOnly | QIODevice::Text);
    QTextStream outData(&dataFile);

    double xMax = 0.;
    double yMax = 0.;
    double xMin = 99999999;
    double yMin = 99999999;

    for (int i=0; i < xpts; i++)
    {
        if (data_x[i] > xMax)
            xMax = data_x[i];
        if (data_y[i] > yMax)
            yMax = data_y[i];
        if (data_x[i] < xMin)
            xMin = data_x[i];
        if (data_y[i] < yMin)
            yMin = data_y[i];
    }

    outData << "# " << instrument << endl;
    outData << "# " << dataname << endl;
    outData << "# X units = " << X_name << endl;
    outData << "# Y units = " << Y_name << endl;
    outData << "# Points = " << xpts << endl;
    outData << "# X min = " << xMin << endl;
    outData << "# Y min = " << yMin << endl;
    outData << "# X max = " << xMax << endl;
    outData << "# Y max = " << yMax << endl << endl;

    for (int i=0; i < xpts; i++)
        outData << realToString(data_x[i]) << ';' << realToString(data_y[i]) << endl;

    dataFile.close();
}
void ascifile::write_file_3d()
{
    QFile dataFile(filename);
    dataFile.open(QIODevice::WriteOnly | QIODevice::Text);
    QTextStream outData(&dataFile);

    outData << "# " << instrument << endl;
    outData << "# " << dataname << endl;
    outData << "# XY units = " << X_name << endl;
    outData << "# Z units = " << Y_name << endl;
    outData << "# X min max = " << realToString(xsz[0]) << ' ' << realToString(xsz[1]) << endl;
    outData << "# Y min max = " << realToString(ysz[0]) << ' ' << realToString(ysz[1]) << endl;
    outData << "# Points X = " << xpts << endl;
    outData << "# Points Y = " << ypts << endl << endl;

    for (int i=0; i < ypts; i++) {
        for (int j=0; j < xpts; j++)
            outData << realToString(data[i][j]) << ';';
        outData << endl;
    }

    dataFile.close();
}
