#include "container.h"
#include "datalist.hpp"
#include "gui/status.hpp"
#include "ccd/ccd.h"
#include "flowlayout.hpp"
#include "gwynetio.hpp"
#include "cont_filesproccessing.h"

#ifndef _ADROID
#include "mainwindow.h"
#include "modules.hpp"
#include "spmtunning.h"
#endif

#include <qwt_plot_marker.h>
#include <QApplication>
#include <QTimer>
#include <QSettings>
#include <QDragMoveEvent>
#include <QDragEnterEvent>
#include <QProgressBar>
#include <QShortcut>
#include <QSplitter>
#include <QTreeWidget>
#include <QFileDialog>
#include <QHeaderView>
#include <QTimer>
#include <QDateTime>
#include <QUrl>
#include "gui/datalistoverlaybuttons.h"
#include "widgets/threedoverlaybuttons.h"
#include "widgets/twodoverlaybuttons.h"

#ifdef _ADROID
    #include <QStandardPaths>
    void doBeep(){}
    class SleepExport : public QThread
    {
    public:
        static void msleep2(unsigned long ms) { msleep(ms); }
    };
#endif

TContainer::~TContainer()
{
    delete FileProccessing;
}
void TContainer::thread_close()
{
    while (FileProccessing->isRunning()) {
        qApp->processEvents();
        SleepExport::msleep2(10);
    }

    #ifndef _ADROID
    if (gwi) {
        gwi->quit();
        gwi->wait();
        delete gwi;
    }
#endif
}
TContainer::TContainer(plot::TPlot *PlotFromDataPlot, QSplitter *parent) : QWidget(parent)
{
    ParentSplitter = parent;
    updateVisibilityOverlayTimer = new QTimer(this);
    connect(updateVisibilityOverlayTimer, SIGNAL(timeout()), this, SLOT(updateOverlayVisibility()));

    AutoSaveTime = 0;
    autosave_idx = 0;
    Plot = PlotFromDataPlot;

    #ifndef _ADROID
    pl3 = nullptr;
    prevSizes[0] = prevSizes[1] = QSize(0, 0);
#endif

    new3d = true;
    ActiveHBL = false;
    lastXind = lastYind = 0;

    isAutosave = false;
    preview_update = false;
    copypaste = false;
    func_max = func_min = 0;

    imageitem = new plot::ImageItem;
    imageitem->hide();

    GUICreate();

    FileProccessing = new TFileProccessing(this);
    FileProccessing->setObjectName(QString::fromUtf8("FileProccessing"));
    connect(FileProccessing, SIGNAL(DoAddBlanks(int )), SLOT(AddBlanks(int )));
    connect(FileProccessing, SIGNAL(Error(QString,QString)), SLOT(Error(QString,QString)));

    SetDownButtonsEnabled(false);

    AutosaveTimer = new QTimer(this);
    connect(AutosaveTimer, SIGNAL(timeout()), this, SLOT(AutoSave()));

    QMetaObject::connectSlotsByName(this);

    t3dench = new T3dEnchancement(this);

#ifndef _ADROID
    gwi = nullptr;
    QSettings settings("Param/gui.conf", QSettings::IniFormat);
    if (settings.value("net/gwy_import_srv", true).toBool()) {
        gwi = new GwyddionImport();//start gwyddion receive thread
        connect(gwi, SIGNAL(AddDataToContainer(plot::TCurvesDataOwner,plot::TMetaStorage)), this, SLOT(AddDataToContainer(plot::TCurvesDataOwner,plot::TMetaStorage)), Qt::QueuedConnection);
        connect(gwi, SIGNAL(AddDataToContainer(plot::T3dDataOwner,plot::TMetaStorage)), this, SLOT(AddDataToContainer(plot::T3dDataOwner,plot::TMetaStorage)), Qt::QueuedConnection);
        connect(gwi, SIGNAL(Refresh()), this, SLOT(on_FileProccessing_UpdatePreview()));
    }
#endif

    setAcceptDrops(true);

    initPlots();
}

void TContainer::initPlots()
{
    if (!Plot->ThreeDPlot || !Plot->TwoDPlot)
        return;

    imageitem->attach(Plot->ThreeDPlot->ThreeDPlots.at(0)->QPlot);
    connect(Plot->TwoDPlot->Pure2dPlot, SIGNAL(MathUpdated()), this, SLOT(Send2dPlotData()));
    connect(Plot->ThreeDPlot->ThreeDPlots.at(0), SIGNAL(MathUpdated()), this, SLOT(Send3dPlotData()));
    connect(Plot->ThreeDPlot->ThreeDPlots.at(0), SIGNAL(MathUpdated()), this, SLOT(on3dUpdate()));
    connect(Plot->ThreeDPlot->ThreeDPlots.at(0), SIGNAL(MathUpdated()), this, SLOT(DelayedPreviewCreation()));
    t3dench->sourceplot = Plot->ThreeDPlot;
    connect(Plot->ThreeDPlot->Slider->Sliders, SIGNAL(ValuesChanged(double,double)), t3dench, SLOT(queueUpdate()));
    connect(Plot->ThreeDPlot->PlotControl->AGPalettes, SIGNAL(triggered(QAction*)), t3dench, SLOT(queueUpdate()));

    QSettings settingsGui("Param/gui.conf", QSettings::IniFormat);
    showRightMenu = settingsGui.value("gui/show_data_right_menu", true).toBool();
    Plot->TwoDPlot->showPlotControl = showRightMenu;
    Plot->ThreeDPlot->TwoDPlot->showPlotControl = showRightMenu;
    Plot->ThreeDPlot->TwoDPlotVertical->showPlotControl = showRightMenu;
    Plot->ThreeDPlot->showPlotControl = showRightMenu;
    if (!showRightMenu)
    {
        Plot->TwoDPlot->PlotControl->hide();
        Plot->ThreeDPlot->TwoDPlot->PlotControl->hide();
        Plot->ThreeDPlot->TwoDPlotVertical->PlotControl->hide();
        Plot->ThreeDPlot->PlotControl->hide();
    }
}

bool TContainer::eventFilter(QObject *obj, QEvent *event)
{
    #ifndef _ADROID
    if (obj == CBDataLists && event->type() == QEvent::DragEnter)
        event->accept();
    if (obj == CBDataLists && event->type() == QEvent::DragMove) {
        event->ignore();
        QTabBar *tb = nullptr;
#if (QT_VERSION >= QT_VERSION_CHECK(5, 0, 0))
        tb = CBDataLists->tabBar();
#else
        tb = CBDataLists->findChild<QTabBar *>(QString());//QLatin1String("qt_tabwidget_tabbar"));
#endif
        int idx = -1;
        if (tb) idx = tb->tabAt(((QDragMoveEvent *)event)->pos() - tb->pos());
        if (idx != -1) CBDataLists->setCurrentIndex(idx);
    }
    if (obj == CBDataLists && event->type() == QEvent::ShortcutOverride)
    {
        QKeyEvent *keyEvent = static_cast<QKeyEvent*>(event);

        bool eventCaptured = DataListShortCuts(keyEvent);
        if (eventCaptured)
        {
            event->ignore();
            return true;
        }
    }
    if (obj == CBDataLists && event->type() == QEvent::Resize)
    {
        if (CBDataLists->size() == prevSizes[0] ||
            CBDataLists->size() == prevSizes[1])
        {
            event->ignore();
            return true;
        }
        int index = (prevSizeIndex == 0) ? 1:0;
        prevSizeIndex = index;
        prevSizes[index] = CBDataLists->size();
        overlayButtons->resized(CBDataLists->size());
    }
#endif

    return false;
}
bool TContainer::DataListShortCuts(QKeyEvent *keyEvent)
{
    bool ctrl = keyEvent->modifiers().testFlag(Qt::ControlModifier);
    bool shift = keyEvent->modifiers().testFlag(Qt::ShiftModifier);

    if (ctrl && shift && keyEvent->key() == Qt::Key_C)
    {
        CopyGroup();
    }
    else if ((ctrl && keyEvent->key() == Qt::Key_C)
              || keyEvent->key() == Qt::Key_Copy)
    {
        btnCopy_clicked();
    }
    else if (shift && keyEvent->key() == Qt::Key_Delete)
    {
        RemoveAll();
    }
    else if (keyEvent->key() == Qt::Key_Delete)
    {
        RemoveCurrentItem();
    }
    else if ((ctrl && keyEvent->key() == Qt::Key_V)
              || keyEvent->key() == Qt::Key_Paste)
    {
        btnPaste_clicked();
    }
    else
        return false;

    return true;
}
void TContainer::dragEnterEvent(QDragEnterEvent *event)
{
    if (event->mimeData()->hasFormat("text/uri-list"))
        event->acceptProposedAction();
}
void TContainer::dropEvent(QDropEvent *event)
{
    event->acceptProposedAction();

    QList<QString> lst;
    foreach (QUrl url, event->mimeData()->urls()) {
        QString rl = url.toLocalFile();
        if (rl.endsWith(".nstdat"))
            lst.append(rl);
    }

    if (lst.isEmpty()) {
        #ifndef _ADROID
        new Notification(tr("Drag && drop") + ": " + tr("No supported files found"), Notification::Warn, 10);
        doBeep();
#endif
        return;
    }

    if (FileProccessing->isRunning()) {
        #ifndef _ADROID
        new Notification(tr("Drag && drop") + ": " + tr("Please wait, another operation is in progress"), Notification::Warn, 10);
        doBeep();
#endif
        return;
    }

    if (lst.size() > 1)
        qDebug() << "Only one drag & drop file supported";

    QString FileName = lst.first();
    OpenFile(FileName, true);
}
void TContainer::GUICreate()
{
    PBBar = new QProgressBar;
    PBBar->hide();

    gb4DButtons = new QGroupBox(tr("4D"));
    lay4DButtons = new QVBoxLayout(gb4DButtons);
    gb4DButtons->hide();

    BShow3DView = new QPushButton;
    BShow3DView->setIcon(makeHugeIcon(":/container/3dimage.png"));
    BShow3DView->setToolTip(tr("Show 3D View"));
    BShow3DView->setFlat(true);
    BShow3DView->setCheckable(true);
    Plot->ThreeDPlot->overlayButtons->setButtonStyle(BShow3DView);
    connect(BShow3DView, SIGNAL(toggled(bool)), this, SLOT(btn3d_toggled(bool)));

    Convol_func = new QToolButton;
    Convol_func->setIcon(makeHugeIcon(":/CCD/Func.png"));
    Convol_func->setToolTip(tr("Convolution function"));
    Convol_func->setPopupMode(QToolButton::InstantPopup);
    Convol_func->hide();

    QMenu *M_Convol_func = new QMenu;
    Convol_func->setMenu(M_Convol_func);

    AG_Convol_func = new QActionGroup(this);
    AG_Convol_func->addAction(M_Convol_func->addAction(tr("Integrate")));
    AG_Convol_func->addAction(M_Convol_func->addAction(tr("Maximum")));
    AG_Convol_func->addAction(M_Convol_func->addAction(tr("Maximum Position")));
    AG_Convol_func->addAction(M_Convol_func->addAction(tr("MassCenter Position")));
    AG_Convol_func->addAction(M_Convol_func->addAction(tr("Peak Width")));
    AG_Convol_func->addAction(M_Convol_func->addAction(tr("Stiffness")));
    AG_Convol_func->addAction(M_Convol_func->addAction(tr("Adhesion")));
    AG_Convol_func->addAction(M_Convol_func->addAction(tr("Energy Dissipation")));
    AG_Convol_func->addAction(M_Convol_func->addAction(tr("Baseline")));
    AG_Convol_func->actions()[0]->setCheckable(true);
    AG_Convol_func->actions()[1]->setCheckable(true);
    AG_Convol_func->actions()[2]->setCheckable(true);
    AG_Convol_func->actions()[3]->setCheckable(true);
    AG_Convol_func->actions()[4]->setCheckable(true);
    AG_Convol_func->actions()[5]->setCheckable(true);
    AG_Convol_func->actions()[6]->setCheckable(true);
    AG_Convol_func->actions()[7]->setCheckable(true);
    AG_Convol_func->actions()[8]->setCheckable(true);
    AG_Convol_func->actions()[0]->setChecked(true);

    for (int i = 0; i < 9; i++)
        connect(AG_Convol_func->actions()[i], SIGNAL(triggered()), this, SLOT(onAG_ConvFunc_trigered()));

    //-----------------------
    BConvolFuncSelect = new QPushButton;
    BConvolFuncSelect->setIcon(makeHugeIcon(":/CCD/Func.png"));
    BConvolFuncSelect->setToolTip(QObject::tr("Convolution function"));
    BConvolFuncSelect->setFlat(true);
    Plot->TwoDPlot->overlayButtons->setButtonStyle(BConvolFuncSelect);

    QMenu *M_Convol_func2 = new QMenu;
    BConvolFuncSelect->setMenu(M_Convol_func2);

    AGConvolFuncSelect = new QActionGroup(this);
    AGConvolFuncSelect->addAction(M_Convol_func2->addAction(tr("Integrate")));
    AGConvolFuncSelect->addAction(M_Convol_func2->addAction(tr("Maximum")));
    AGConvolFuncSelect->addAction(M_Convol_func2->addAction(tr("Maximum Position")));
    AGConvolFuncSelect->addAction(M_Convol_func2->addAction(tr("MassCenter Position")));
    AGConvolFuncSelect->addAction(M_Convol_func2->addAction(tr("Peak Width")));
    AGConvolFuncSelect->addAction(M_Convol_func2->addAction(tr("Stiffness")));
    AGConvolFuncSelect->addAction(M_Convol_func2->addAction(tr("Adhesion")));
    AGConvolFuncSelect->addAction(M_Convol_func2->addAction(tr("Energy Dissipation")));
    AGConvolFuncSelect->addAction(M_Convol_func2->addAction(tr("Baseline")));
    AGConvolFuncSelect->actions()[0]->setCheckable(true);
    AGConvolFuncSelect->actions()[1]->setCheckable(true);
    AGConvolFuncSelect->actions()[2]->setCheckable(true);
    AGConvolFuncSelect->actions()[3]->setCheckable(true);
    AGConvolFuncSelect->actions()[4]->setCheckable(true);
    AGConvolFuncSelect->actions()[5]->setCheckable(true);
    AGConvolFuncSelect->actions()[6]->setCheckable(true);
    AGConvolFuncSelect->actions()[7]->setCheckable(true);
    AGConvolFuncSelect->actions()[8]->setCheckable(true);
    AGConvolFuncSelect->actions()[0]->setChecked(true);

    for (int i = 0; i < 9; i++)
        connect(AGConvolFuncSelect->actions()[i], SIGNAL(triggered()), this, SLOT(onAGConvolFuncSelect_trigered()));
    //-----------------------

    Func_interval = new TwoStateButton(":/CCD/Func_range.png",":/CCD/Func_range_on.png");
    Func_interval->setToolTip(tr("Function convolution range selection"));
    Func_interval->setCheckable(true);
    Func_interval->hide();
    connect(Func_interval, SIGNAL(state_changed(bool)), SLOT(sl_Func_range_select(bool)));

    BFuncInterval = new TwoStateButton(":/CCD/Func_range.png",":/CCD/Func_range_on.png");
    BFuncInterval->setToolTip(tr("Function convolution range selection"));
    BFuncInterval->setCheckable(true);
    BFuncInterval->setFlat(true);
    Plot->TwoDPlot->overlayButtons->setButtonStyle(BFuncInterval);
    BFuncInterval->hide();
    connect(BFuncInterval, SIGNAL(state_changed(bool)), SLOT(onFuncInterval_changed(bool)));
    #ifndef _ADROID
    connect(Plot->ThreeDPlot->TwoDPlot->overlayButtons->B2DBack, SIGNAL(released()), this, SLOT(onBackButton_clicked()));
    connect(Plot->ThreeDPlot->TwoDPlotVertical->overlayButtons->B2DBack, SIGNAL(released()), this, SLOT(onBackButton_clicked()));
    connect(Plot->TwoDPlot->overlayButtons->B2DBack, SIGNAL(released()), this, SLOT(onBackButton_clicked()));
#endif

    Scale_select = new QToolButton;
    Scale_select->setIcon(makeHugeIcon(":/container/Lunits.png"));
    Scale_select->setToolTip(tr("Display units"));
    Scale_select->setPopupMode(QToolButton::InstantPopup);
    Scale_select->hide();

    QMenu *M_Scale_select = new QMenu;
    Scale_select->setMenu(M_Scale_select);

    AG_Scale_select = new QActionGroup(this);
    AG_Scale_select->addAction(M_Scale_select->addAction(tr("Pixels")))->setCheckable(true);
    AG_Scale_select->addAction(M_Scale_select->addAction(tr("Wavelength")))->setCheckable(true);
    AG_Scale_select->addAction(M_Scale_select->addAction(tr("Raman shift")))->setCheckable(true);
    AG_Scale_select->actions()[0]->setChecked(true);
    connect(AG_Scale_select, SIGNAL(triggered(QAction*)), this, SLOT(onAG_Scale_select_trigered()));

    B3DImageCrop = new QPushButton;
    B3DImageCrop->setIcon(makeHugeIcon(":/plot/Cuting.png"));
    B3DImageCrop->setToolTip(QObject::tr("Image crop"));
    B3DImageCrop->setFlat(true);
    B3DImageCrop->setCheckable(true);
    Plot->ThreeDPlot->overlayButtons->setButtonStyle(B3DImageCrop);
    connect(B3DImageCrop, SIGNAL(released()), this, SLOT(on3DImageCrop()));
    connect(Plot->ThreeDPlot->overlayButtons, &plot::ThreeDOverlayButtons::ApplyEnterClicked,
            this, &TContainer::on3DApply_clicked);
    connect(Plot->ThreeDPlot->overlayButtons, &plot::ThreeDOverlayButtons::BackCancelClicked,
            this, &TContainer::on3DCancel_clicked);

    //-----------------------
    BScaleSelect = new QPushButton;
    BScaleSelect->setIcon(makeHugeIcon(":/container/Lunits.png"));
    BScaleSelect->setToolTip(QObject::tr("Display units"));
    BScaleSelect->setFlat(true);
    Plot->TwoDPlot->overlayButtons->setButtonStyle(BScaleSelect);

    QMenu *M_Scale_select2 = new QMenu;
    BScaleSelect->setMenu(M_Scale_select2);

    AGScaleSelect = new QActionGroup(this);
    AGScaleSelect->addAction(M_Scale_select2->addAction(tr("Pixels")))->setCheckable(true);
    AGScaleSelect->addAction(M_Scale_select2->addAction(tr("Wavelength")))->setCheckable(true);
    AGScaleSelect->addAction(M_Scale_select2->addAction(tr("Raman shift")))->setCheckable(true);
    AGScaleSelect->actions()[0]->setChecked(true);
    connect(AGScaleSelect, SIGNAL(triggered(QAction*)), this, SLOT(onAGScaleSelect_trigered()));
    //-----------------------

    lay4DButtons->addWidget(Func_interval, 0, Qt::AlignLeft);
    lay4DButtons->addWidget(Convol_func, 0, Qt::AlignLeft);

    GTableSplitter = new QSplitter(this);
    GTableSplitter->installEventFilter(this);
    GTableSplitter->setStyleSheet("QSplitter {border: 1px solid #B4B4B4; border-style: outset; border-width: 2px; border-radius: 10px; border-color: #B4B4B4; background-color: white;}");

    QHBoxLayout *MainLayout = new QHBoxLayout(this);
    MainLayout->addWidget(GTableSplitter, 1);

    QWidget* GrData = new QWidget();
    GTableSplitter->addWidget(GrData);
    QVBoxLayout *LData = new QVBoxLayout(GrData);

    GTableSplitter->setSizes(QList<int>()<<1<<0);

    GTable = new QGroupBox(tr("Info"));
    GTable->hide();
    GTableSplitter->addWidget(GTable);
    QHBoxLayout *Ltable = new QHBoxLayout(GTable);

    table = new QTreeWidget(GTable);
    table->setMinimumWidth(230);
    Ltable->addWidget(table);
    connect(table, SIGNAL(collapsed(QModelIndex)), this, SLOT(attribEditorResize()));
    connect(table, SIGNAL(expanded(QModelIndex)), this, SLOT(attribEditorResize()));
    connect(table, SIGNAL(itemChanged(QTreeWidgetItem*,int)), this, SLOT(attribConfigEdited(QTreeWidgetItem*,int)));

    QList<QString> header;
    header.push_back(tr("Name"));
    header.push_back(tr("Value"));
    table->setHeaderLabels(header);

    #ifndef _ADROID
    CBDataLists = new QTabWidget();
    CBDataLists->setDocumentMode(true);
    CBDataLists->setAcceptDrops(true);
    CBDataLists->installEventFilter(this);
    CBDataLists->setTabsClosable(true);
    CBDataLists->setTabBarAutoHide(false);
    addTab(tr("Default"));
    CurrentDataList = (TListWidget *)CBDataLists->widget(0);
    connect(CBDataLists, SIGNAL(currentChanged(int)), SLOT(TabChanged(int)));
    connect(CBDataLists, SIGNAL(tabCloseRequested(int)), SLOT(TabClose(int)));

    overlayButtons = new DataListOverlayButtons(this, CBDataLists);
    updateVisibilityOverlayTimer->start(100);


    LData->addWidget(CBDataLists, 1);
#endif
    LData->addWidget(PBBar);
}

void TContainer::on3DCancel_clicked(plot::ThreeDOverlayButtons::SpecialVisibilityMode visibilityMode)
{
    Plot->ThreeDPlot->on_BCancel_clicked();
    on3DCancelApply(visibilityMode);
}

void TContainer::on3DApply_clicked(plot::ThreeDOverlayButtons::SpecialVisibilityMode visibilityMode)
{
    Plot->ThreeDPlot->on_BApply_clicked();
    on3DCancelApply(visibilityMode);
}

void TContainer::on3DCancelApply(plot::ThreeDOverlayButtons::SpecialVisibilityMode visibilityMode)
{
    if (visibilityMode == plot::ThreeDOverlayButtons::svm_image_crop || B3DImageCrop->isChecked())
    {
        B3DImageCrop->setChecked(false);
        Plot->ThreeDPlot->overlayButtons->BBackButton->setVisible(Plot->ThreeDPlot->overlayButtons->showingBackButtonPlot[0] = false);
        Plot->ThreeDPlot->overlayButtons->scanButtonsSetVisibleGUI(false);
        Plot->ThreeDPlot->overlayButtons->setActivePlotByMouseMove = false;
        Plot->ThreeDPlot->overlayButtons->setVisibleIfUnderMouse(0);
        if (Plot->ThreeDPlot->overlayButtons->constantShowingMenu)
        {
            Plot->ThreeDPlot->overlayButtons->scanButtonsSetVisibleGUI(true, Plot->ThreeDPlot->ThreeDPlots[0]);
        }
    }
}

void TContainer::on_KeyEnterPressed3D()
{
    Plot->ThreeDPlot->PlotControl->BApply->click();
}

void TContainer::on3DImageCrop()
{
    Plot->ThreeDPlot->overlayButtons->visibilityMode[Plot->ThreeDPlot->ActivePlot] = Plot->ThreeDPlot->overlayButtons->svm_image_crop;
    Plot->ThreeDPlot->overlayButtons->BBackButton->setVisible(Plot->ThreeDPlot->overlayButtons->showingBackButtonPlot[0] = true);
    Plot->ThreeDPlot->overlayButtons->BApplyButton->setVisible(Plot->ThreeDPlot->overlayButtons->showingApplyButtonPlot[0] = true);
    Plot->ThreeDPlot->overlayButtons->scanButtonsSetVisibleGUI(false);
    Plot->ThreeDPlot->overlayButtons->setVisibleIfUnderMouse(0);
    Plot->ThreeDPlot->PlotControl->BSelectByRect->click();
}

void TContainer::onBackButton_clicked()
{
    if (BFuncInterval->isOn)
        BFuncInterval->change_state_man_with_sig(false);
}

void TContainer::onFuncInterval_changed(bool pressed)
{
    if (pressed)
    {
        Plot->ThreeDPlot->TwoDPlot->overlayButtons->showBack = true;
        Plot->ThreeDPlot->TwoDPlot->overlayButtons->B2DBack->show();
    }
    Plot->ThreeDPlot->TwoDPlot->overlayButtons->setVisibleGUI2D(false);
    Plot->ThreeDPlot->TwoDPlot->overlayButtons->setVisibleGUI2D(true);
    Func_interval->change_state_man_with_sig(pressed);
}

void TContainer::onAGScaleSelect_trigered()
{
    int Scale = FindCheckedItemIn(AGScaleSelect);
    AG_Scale_select->actions()[Scale]->setChecked(true);
    AG_Scale_select->actions()[Scale]->trigger();
    onAG_Scale_select_trigered();
}

void TContainer::onAGConvolFuncSelect_trigered()
{
    int i = FindCheckedItemIn(AGConvolFuncSelect);
    AG_Convol_func->actions()[i]->setChecked(true);
    AG_Convol_func->actions()[i]->trigger();
}

void TContainer::SetDownButtonsEnabled(bool State)
{
    if (!State) {
        if (BShow3DView->isChecked()) BShow3DView->toggle();
    }

    canRemove = State;
    BShow3DView->setEnabled(State);
    Plot->TwoDPlot->overlayButtons->enabled = State;
    #ifndef _ADROID
    overlayButtons->BToAscii->setEnabled(State);
#else
    if (Plot->TwoDPlotVertical)
        Plot->TwoDPlotVertical->overlayButtons->enabled = State;
#endif
    Plot->ThreeDPlot->overlayButtons->enabled = State;
    Plot->ThreeDPlot->TwoDPlot->overlayButtons->enabled = State;
    if (!State)
    {
	if ((Plot->ThreeDPlot->ThreeDPlots[0] != NULL) && (Plot->ThreeDPlot->ThreeDPlots[0]->SourceData.isEmpty() != true) && (Plot->ThreeDPlot->ThreeDPlots[0]->SourceData.data() != NULL))
	{
            Plot->ThreeDPlot->ThreeDPlots[0]->SourceData.data()->clear();
        };
    	Plot->ThreeDPlot->Refresh();

    	if(Plot->TwoDPlot->Pure2dPlot->SourceData.data()->isNull() != true)
	    Plot->TwoDPlot->Pure2dPlot->SourceData.data()->clear();
        Plot->TwoDPlot->Refresh();

        if(Plot->ThreeDPlot->TwoDPlot->Pure2dPlot->SourceData.data()->isNull() != true)
    	    Plot->ThreeDPlot->TwoDPlot->Pure2dPlot->SourceData.data()->clear();
        Plot->ThreeDPlot->TwoDPlot->Refresh();

        if(Plot->ThreeDPlot->TwoDPlotVertical->Pure2dPlot->SourceData.data()->isNull() != true)
    	    Plot->ThreeDPlot->TwoDPlotVertical->Pure2dPlot->SourceData.data()->clear();
        Plot->ThreeDPlot->TwoDPlotVertical->Refresh();
    }
}


void TContainer::SavePreview()
{
    if (!CurrentDataList || !CurrentDataList->ClickedItem)
        return;

    QImage img;
    if (CurrentDataList->ClickedItem->InfoType == TAbstractInfo::_2d) {
        if (!Plot->TwoDPlot->Pure2dPlot->isMathReady())
            return;
        img = Plot->TwoDPlot->MakePreview();
    } else if (CurrentDataList->ClickedItem->InfoType == TAbstractInfo::_3d ||
               CurrentDataList->ClickedItem->InfoType == TAbstractInfo::_4d) {
        if (!Plot->ThreeDPlot->ThreeDPlots.at(0)->isMathReady())
            return;
        img = Plot->ThreeDPlot->MakePreview();
    } else
        return;

    CurrentDataList->ClickedItem->attributes.insert("Preview", QVariant::fromValue(img));
    CurrentDataList->blockSignals(true);
    CurrentDataList->ClickedItem->Item->setIcon(QIcon(QPixmap::fromImage(img)));
    CurrentDataList->blockSignals(false);
}

void TContainer::UpdatePreview()
{
    if (!CurrentDataList || !CurrentDataList->ClickedItem)
        return;

    preview_update = true;
    CurrentDataList->ClickedItem->UpdatePreview = true;
    CurrentDataList->setCurrentItem(CurrentDataList->ClickedItem->Item);
    preview_update = false;
}

void TContainer::SaveFilters()
{
    if (!CurrentDataList || !CurrentDataList->ClickedItem)
        return;

    TListItem *ClickedItem = CurrentDataList->ClickedItem;
    if (ClickedItem->InfoType == TAbstractInfo::_2d) {
        ClickedItem->attributes.insert("Fit mode", FindCheckedItemIn(Plot->TwoDPlot->PlotControl->AGFitOrders));

        QStringList clrs;
        for (int i = 0; i < ((plot::TCurvesDataOwner *)ClickedItem->Info)->size(); i++)
            clrs.append((*(plot::TCurvesDataOwner *)ClickedItem->Info)[i]->Pen.color().name());

        ClickedItem->attributes.insert("Curve colour", clrs.join(","));
        ClickedItem->attributes.insert("Curve thickness", FindCheckedItemIn(Plot->TwoDPlot->PlotControl->AGCurveThickness));
    } else if (ClickedItem->InfoType == TAbstractInfo::_3d ||
               ClickedItem->InfoType == TAbstractInfo::_4d) {
        Plot->ThreeDPlot->appendAttrib(ClickedItem->attributes);
    }
}

void TContainer::updateDataListWidth()
{
#ifdef _WIN32
    int w = 0;
    for (int i=0; i<CBDataLists->tabBar()->count(); i++)
    {
        w += CBDataLists->tabBar()->tabRect(i).width();
    }
    w += 40;
    ParentSplitter->setSizes(QList<int>()<<w<<ParentSplitter->sizes().last());

    if (overlayButtons)
        overlayButtons->resized(QSize(0,0));
#endif
}

void TContainer::TabChanged(int index)
{
    #ifndef _ADROID
    SaveFilters();
    SavePreview();

    if (index == -1) {
        if (CBDataLists->count() == 0)
            addTab(tr("Default"));

        CBDataLists->setCurrentIndex(0);
        return;
    }

    CurrentDataList = (TListWidget *)CBDataLists->widget(index);

    SetDownButtonsEnabled(CurrentDataList->DataList.count() != 0);

    if (CurrentDataList->ClickedItem) {
        preview_update = true;
        CurrentDataList->on_itemPressed(CurrentDataList->ClickedItem->Item);
        preview_update = false;
    } else {
        Func_interval->hide();
        Convol_func->hide();
        Scale_select->hide();
        Plot->ThreeDPlot->TwoDPlot->overlayButtons->removeButton(BScaleSelect);
        Plot->TwoDPlot->overlayButtons->removeButton(BScaleSelect);
        BScaleSelect->hide();

        Plot->ThreeDPlot->TwoDPlot->overlayButtons->removeButton(BFuncInterval);
        Plot->TwoDPlot->overlayButtons->removeButton(BFuncInterval);
        BFuncInterval->hide();

        Plot->ThreeDPlot->TwoDPlotVertical->overlayButtons->removeButton(BScaleSelect);
        Plot->ThreeDPlot->TwoDPlotVertical->overlayButtons->removeButton(BFuncInterval);

        updateAttributes();
    }
    //updateDataListWidth();
    CBDataLists->tabBar()->setVisible(true);
#endif
}

void TContainer::TabClose(int index)
{
    #ifndef _ADROID
    TListWidget *Current = (TListWidget *)CBDataLists->widget(index);
    Current->ClickedItem = nullptr;
    
    if (Plot->ThreeDPlot->overlayButtons->visibilityMode[0] != plot::ThreeDOverlayButtons::SpecialVisibilityMode::svm_none)
    {
        Plot->ThreeDPlot->overlayButtons->resetBackBtnWainting();
    }

    if (index == 0) {
        Current->ClearAll();
        Current->filename.clear();

        if (!CurrentDataList->DataList.count())
            SetDownButtonsEnabled(false);

        if (Current == CurrentDataList) {
            Func_interval->hide();
            Convol_func->hide();
            Scale_select->hide();
            Plot->ThreeDPlot->TwoDPlot->overlayButtons->removeButton(BScaleSelect);
            Plot->ThreeDPlot->TwoDPlotVertical->overlayButtons->removeButton(BScaleSelect);
            Plot->TwoDPlot->overlayButtons->removeButton(BScaleSelect);
            BScaleSelect->hide();

            Plot->ThreeDPlot->TwoDPlot->overlayButtons->removeButton(BFuncInterval);
            Plot->ThreeDPlot->TwoDPlotVertical->overlayButtons->removeButton(BFuncInterval);
            Plot->TwoDPlot->overlayButtons->removeButton(BFuncInterval);
            BFuncInterval->hide();

            updateAttributes();
        }

        return;
    }

    CBDataLists->removeTab(index);

    if (Current == CurrentDataList)
        CurrentDataList = nullptr;

    delete Current;

    canPaste = false;

    updateDataListWidth();
#endif
}

TListWidget* TContainer::addTab(QString name)
{
    if (name.isEmpty())
        name = tr("New tab");

    TListWidget *newList = new TListWidget(this);
    newList->setMinimumWidth(200);
    connect(newList, SIGNAL(ItemClicked(const plot::T3dDataOwner*)), SLOT(onItemClicked(const plot::T3dDataOwner*)));
    connect(newList, SIGNAL(ItemClicked(const plot::TCurvesDataOwner*)), SLOT(onItemClicked(const plot::TCurvesDataOwner*)));
    connect(newList, SIGNAL(ItemClicked(const ccd::TSetOfSpectrum *)), SLOT(onItemClicked(const ccd::TSetOfSpectrum *)));
    connect(newList, SIGNAL(ItemClicked(const QImage)), SLOT(onItemClicked(const QImage)));

    #ifndef _ADROID
    int index = CBDataLists->addTab(newList, name);

    CBDataLists->setCurrentIndex(index);
    CBDataLists->setTabBarAutoHide(false);
    CBDataLists->tabBar()->setVisible(true);
    CBDataLists->tabBar()->setDocumentMode(false);
#endif
    updateDataListWidth();

    return newList;
}


void TContainer::CopyGroup()
{
    if (!CurrentDataList || !CurrentDataList->ClickedItem)
        return;

    info_copypaste.clear();
    copypaste_type.clear();
    copypaste_attrib.clear();

    QString group = CurrentDataList->ClickedItem->attributes.value("GroupId").toString();
	for (int i = 0; i < CurrentDataList->DataList.size(); i++)
	{
		auto data = CurrentDataList->DataList[i];
		if (data->attributes.value("GroupId").toString() == group) {
			info_copypaste.push_back(data->Info);
			copypaste_type.push_back(data->InfoType);
			copypaste_attrib.push_back(data->attributes);
		}
	}

    canPaste = true;
}

void TContainer::DoDrag(TListWidget *from, TListWidget *to)
{
    copypaste = true;
    for (int i = 0; i < from->selectedItems().size(); i++) {
        auto item = from->FindItem(from->selectedItems()[i]);
        try {
            if (item->InfoType == TAbstractInfo::_3d) {
                plot::T3dDataOwner *old = ((plot::T3dDataOwner *)item->Info);
                plot::T3dDataOwner newOne;
                plot::ResizeOwner(newOne, old->size());
                for (int j = 0; j < newOne.size(); j++)
                    *newOne.at(j) = *old->at(j);

                AddDataToContainer(newOne, item->attributes);
            } else if (item->InfoType == TAbstractInfo::_2d) {
                plot::TCurvesDataOwner *old = ((plot::TCurvesDataOwner *)item->Info);
                plot::TCurvesDataOwner newOne;
                plot::ResizeOwner(newOne, old->size());
                for (int j = 0; j < newOne.size(); j++)
                    *newOne.at(j) = *old->at(j);

                AddDataToContainer(newOne, item->attributes);
            } else if (item->InfoType == TAbstractInfo::_4d) {
                ccd::TSetOfSpectrum *spectr = new ccd::TSetOfSpectrum;
                ccd::TSetOfSpectrum *old = ((ccd::TSetOfSpectrum *)item->Info);
                *(ccd::TInfoForSetOfSpectrums *)spectr = *old;
                spectr->CalibInfo = old->CalibInfo;
                *spectr->Results[0] = *old->Results[0];

                AddDataToContainer(spectr, item->attributes);
            } else if (item->InfoType == TAbstractInfo::_Image) {
                AddDataToContainer(item->attributes.value("Preview").value<QImage>(), item->attributes);
            }
        } catch (...) {
            //Mem fail
        }
    }
    UpdateHierarchy();

    copypaste = false;
}

void TContainer::btnCopy_clicked()
{
    if (!CurrentDataList || !CurrentDataList->ClickedItem)
        return;

    info_copypaste.clear();
    copypaste_type.clear();
    copypaste_attrib.clear();

    for (int i=0; i < CurrentDataList->selectedItems().count(); i++)
    {
        auto item = CurrentDataList->FindItem(CurrentDataList->selectedItems()[i]);
        info_copypaste.push_back(item->Info);
        copypaste_type.push_back(item->InfoType);
        copypaste_attrib.push_back(item->attributes);
    }

    canPaste = true;
}
void TContainer::btnPaste_clicked()
{
    if (!canPaste || !info_copypaste.size())
        return;

    copypaste = true;
    for (int i = 0; i < info_copypaste.size(); i++) {
        try {
            if (copypaste_type[i] == TAbstractInfo::_3d) {
                plot::T3dDataOwner *old = ((plot::T3dDataOwner *)info_copypaste[i]);
                plot::T3dDataOwner newOne;
                plot::ResizeOwner(newOne, old->size());
                for (int j = 0; j < newOne.size(); j++)
                    *newOne.at(j) = *old->at(j);

                AddDataToContainer(newOne, copypaste_attrib[i]);
            } else if (copypaste_type[i] == TAbstractInfo::_2d) {
                plot::TCurvesDataOwner *old = ((plot::TCurvesDataOwner *)info_copypaste[i]);
                plot::TCurvesDataOwner newOne;
                plot::ResizeOwner(newOne, old->size());
                for (int j = 0; j < newOne.size(); j++)
                    *newOne.at(j) = *old->at(j);

                AddDataToContainer(newOne, copypaste_attrib[i]);
            } else if (copypaste_type[i] == TAbstractInfo::_4d) {
                ccd::TSetOfSpectrum *spectr = new ccd::TSetOfSpectrum;
                ccd::TSetOfSpectrum *old = ((ccd::TSetOfSpectrum *)info_copypaste[i]);
                *(ccd::TInfoForSetOfSpectrums *)spectr = *old;
                spectr->CalibInfo = old->CalibInfo;
                *spectr->Results[0] = *old->Results[0];

                AddDataToContainer(spectr, copypaste_attrib[i]);
            } else if (copypaste_type[i] == TAbstractInfo::_Image) {
                AddDataToContainer(copypaste_attrib[i].value("Preview").value<QImage>(), copypaste_attrib[i]);
            }
        } catch (...) {
            //Mem fail
        }
    }

    UpdateHierarchy();

    copypaste = false;
    
    #ifndef _ADROID
    CBDataLists->setFocus();
#endif
}

void TContainer::btnMerge_clicked()
{
    #ifndef _ADROID
    addTab(tr("Merge tab"));
    int idx_result = CBDataLists->currentIndex();

    setEnabled(false);
    qApp->processEvents();

    info_copypaste.clear();
    copypaste_type.clear();
    copypaste_attrib.clear();

    for (int idx = 0; idx < CBDataLists->count(); idx++) {
        if (idx == idx_result)
            continue;

        TListWidget *w = (TListWidget *)CBDataLists->widget(idx);

        for (int i = 0; i < w->DataList.count(); i++) {
			auto data = w->DataList[i];
            info_copypaste.push_back(data->Info);
            copypaste_type.push_back(data->InfoType);
            copypaste_attrib.push_back(data->attributes);
        }
    }

    CBDataLists->setCurrentIndex(idx_result);
    bool lastCanPaste = canPaste;
    canPaste = true;
    btnPaste_clicked();
    canPaste = lastCanPaste;

    setEnabled(true);
#endif
}


void TContainer::OpenFile(QString FileName, bool createNewTab)
{
    if (FileName.isEmpty())
        return;

    QString name = FileName;
    if (name.lastIndexOf("/") != -1)
        name.remove(0, name.lastIndexOf("/") + 1);
    if (name.lastIndexOf("\\") != -1)
        name.remove(0, name.lastIndexOf("\\") + 1);

    updateOverlayVisibility();
    if (createNewTab)
    {
        addTab(name);
        updateOverlayVisibility();
    }

    CurrentDataList->filename = FileName;

    Load(FileName);
    updateOverlayVisibility();

    SetDownButtonsEnabled(true);
}

QString TContainer::getDataDir()
{
    QString datadir = "NST data files";
#ifndef _WIN32
#ifdef _ADROID
    //datadir = QStandardPaths::writableLocation(QStandardPaths::DocumentsLocation);// + "/" + datadir;
    datadir = "/sdcard";
#else
    datadir = QDir::homePath() + "/" + datadir;
#endif
#endif
    QSettings settings("Param/gui.conf", QSettings::IniFormat);
    datadir = settings.value("data_dir", datadir).toString();
    return datadir;
}

void TContainer::btnOpen_clicked()
{
    QString FileName;
    FileName = FileDir.isEmpty() ? TContainer::getDataDir() + "/" : FileDir;

    FileName = QFileDialog::getOpenFileName(this, QString(), FileName,
                                            QString("NST files(*.nstdat)"),
                                            nullptr, QFileDialog::DontUseNativeDialog);

    if (FileName.isEmpty())
    {
        updateOverlayVisibility();
        return;
    }

    FileDir = FileName.left(FileName.lastIndexOf("/") + 1);

    OpenFile(FileName, true);
    updateOverlayVisibility();
}

void TContainer::btnSave_clicked()
{
    btnSave_clicked(this);
}

void TContainer::btnSave_clicked(QWidget *parentDialog)
{
    SaveFilters();
    SavePreview();

    if (!CurrentDataList->DataList.count())
        return;
    if (!CurrentDataList->isUpdated && isAutosave)
        return;


    QString FileName;
    if (!CurrentDataList->filename.isEmpty())
        FileName = CurrentDataList->filename;
    else if (isAutosave) {
        FileName = TContainer::getDataDir() + QString("/AutoSave%1_").arg(++autosave_idx) + QDateTime::currentDateTime().toString("yyyy-MM-dd_hhap") + ".nstdat";
    } else {
        FileName = (FileDir.isEmpty() ? TContainer::getDataDir() + "/" : FileDir) + QDateTime::currentDateTime().toString("yyyy-MM-dd_hhap") + ".nstdat";

        if (parentDialog == nullptr)
            parentDialog = this;
        FileName = QFileDialog::getSaveFileName(parentDialog, tr("Save"), FileName,
                                                QString("NST files(*.nstdat)"),
                                                nullptr, QFileDialog::DontUseNativeDialog);

        if (FileName.isEmpty())
            return;

        FileDir = FileName.left(FileName.lastIndexOf("/") + 1);
    }


    Save(FileName);

    while (FileProccessing->isRunning()) {
        qApp->processEvents();
        SleepExport::msleep2(10);
    }

    if (isAutosave)
        return;

    CurrentDataList->filename = FileName;

    if (FileName.lastIndexOf("/") != -1)
        FileName.remove(0, FileName.lastIndexOf("/") + 1);
    if (FileName.lastIndexOf("\\") != -1)
        FileName.remove(0, FileName.lastIndexOf("\\") + 1);

    #ifndef _ADROID
    CBDataLists->setTabText(CBDataLists->indexOf(CurrentDataList), FileName);

    if (CBDataLists->indexOf(CurrentDataList) == 0) //for "Default"
    {
        CBDataLists->blockSignals(true);
        addTab(tr("Default"));
        CBDataLists->findChild<QTabBar *>()->moveTab(CBDataLists->count()-1, 0);
        CBDataLists->setCurrentIndex(1);
        CBDataLists->blockSignals(false);
    }
#endif
}

void TContainer::btnSaveAs_clicked()
{
    SaveAs(false);
}
void TContainer::SaveAs(bool oldFormat, QWidget *parentDialog)
{
    SaveFilters();
    SavePreview();

    if (!CurrentDataList->DataList.count())
        return;

    QString FileName;
    if (CurrentDataList->filename.isEmpty())
        FileName = (FileDir.isEmpty() ? TContainer::getDataDir() + "/" : FileDir) + QDateTime::currentDateTime().toString("yyyy-MM-dd_hhap") + ".nstdat";
    else
        FileName = CurrentDataList->filename; //TODO add "_copy"

    QString fileType = "";
    if (oldFormat)
    {
        FileName = FileName.remove(".nstdat");
        fileType = "Gwyddion files(*.gwy)";
    }
    else
        fileType = "NST files(*.nstdat)";
    if (parentDialog == nullptr)
        parentDialog = this;
    FileName = QFileDialog::getSaveFileName(parentDialog, tr("Save As"), FileName, fileType,
                                            nullptr, QFileDialog::DontUseNativeDialog);

    if (FileName.isEmpty())
        return;

    FileDir = FileName.left(FileName.lastIndexOf("/") + 1);

    Save(FileName, oldFormat);
}
void TContainer::SaveAll()
{
    SaveFilters();
    SavePreview();

    #ifndef _ADROID
    int old_idx = CBDataLists->currentIndex();
    int count = CBDataLists->count();
    bool def_tab = false;

    for (int idx = 0; idx < count; idx++) {
        CBDataLists->setCurrentIndex(idx);
        btnSave_clicked();

        //if default tab was saved into another tab
        if (count != CBDataLists->count()) {
            def_tab = true;
            count = CBDataLists->count();
            idx++;
        }
    }

    if (def_tab && old_idx == 0)
        old_idx++;

    CBDataLists->setCurrentIndex(old_idx);
#endif
}

void TContainer::RemoveAll()
{
    if (!canRemove)
        return;

    CurrentDataList->ClearAll();
    CurrentDataList->filename.clear();

    SetDownButtonsEnabled(false);

    CurrentDataList->ClickedItem = nullptr;
    canPaste = false;
    updateAttributes();
}
void TContainer::RemoveCurrentItem()
{
    if (!canRemove || !CurrentDataList->ClickedItem)
        return;

    if (qApp->queryKeyboardModifiers() & Qt::ControlModifier) {
        //remove group instead
        QString group = CurrentDataList->ClickedItem->attributes.value("GroupId").toString();
        QVector<int> lst;
        for (int i = 0; i < CurrentDataList->DataList.size(); i++)
            if (CurrentDataList->DataList[i]->attributes.value("GroupId").toString() == group)
                lst.append(i);
        for (int i = lst.size()-1; i >= 0; i--)
            CurrentDataList->Delete(lst.at(i));
    } else
        CurrentDataList->Delete();

    CurrentDataList->ClickedItem = nullptr;
    CurrentDataList->setCurrentItem(nullptr);
    SetDownButtonsEnabled(false);
    canPaste = false;
    updateAttributes();
    CurrentDataList->isUpdated = true;
}

void TContainer::showAttributes_toggled(bool checked)
{
    #ifndef _ADROID
    auto data = DataPlot::GetIt();
    QSplitter *spl = data->Splitter;
    QSplitter *spl2 = ((QSplitter *)GTable->parentWidget());
    int sz1 = spl->sizes().at(0);
    int sz2 = spl->sizes().at(1);
    int sz3 = -GTable->sizeHint().width() - spl->handleWidth();

    overlayButtons->BToDevice->setVisible(checked && SPMTunning::GetIt());
    GTable->setVisible(checked);

    setUpdatesEnabled(false);
    qApp->processEvents();
    setUpdatesEnabled(true);

    if (checked)
        sz3 = GTable->sizeHint().width() + spl->handleWidth();
    spl2->setSizes(QList<int>() << 100000 << 1);
    spl->setSizes(QList<int>() << sz1+sz3 << sz2-sz3);

    if (!checked) {
        //table->clear();
        return;
    }

    updateAttributes();
#endif
}

void TContainer::attribEditorResize()
{
    table->header()->resizeSections(QHeaderView::ResizeToContents);
}

void TContainer::attribConfigEdited(QTreeWidgetItem *, int)
{
    SaveAttrib();
    CurrentDataList->on_itemPressed(CurrentDataList->ClickedItem->Item);
}

void TContainer::addAttribute(QString groupname, QString name, QString val)
{
    QTreeWidgetItem *root = nullptr;
    if (!groupname.isEmpty()) {
        if (!attribItemsRoots.contains(groupname)) {
            root = new QTreeWidgetItem(table, QStringList() << groupname);

            attribItemsRoots.insert(groupname, root);
        } else
            root = attribItemsRoots.value(groupname);
    }

    QTreeWidgetItem *item;
    if (root) {
        item = new QTreeWidgetItem(root, QStringList() << name << val);
        root->setExpanded(true);
    } else {
        item = new QTreeWidgetItem(table, QStringList() << name << val);
        item->setData(0, Qt::UserRole, true);
    }

    #ifndef _ADROID
    item->setFlags(item->flags() | Qt::ItemIsEditable);
    #endif
}

QString attribGroup(QString name)
{
    if (name == "Palette" ||
            name == "Curve colour" ||
            name == "Curve thickness")
        return "Plot";

    if (name == "XLabel" ||
            name == "XUnit" ||
            name == "YLabel" ||
            name == "YUnit" ||
            name == "ZLabel" ||
            name == "ZUnit" ||
            name == "XYUnit" ||
            name == "XYLabel")
        return "Labels";

    if (name == "Name" ||
            name == "Date" ||
            name == "Id" ||
            name == "GroupId" ||
            name == "Parent" ||
            name == "Source")
        return "Common";

    if (name == "CCD NX" ||
            name == "CCD NY" ||
            name == "CCD Central Wavelength, nm" ||
            name == "CCD Central Pixel" ||
            name == "CCD Dispersion")
        return "CCD";

    if (name == "CCD X idx" ||
            name == "CCD Y idx" ||
            name == "CCD X screen pos, m" ||
            name == "CCD Y screen pos, m" ||
            name == "CCD X global pos, m" ||
            name == "CCD Y global pos, m")
        return "CCD Scan Pos";

    if (name == "XPos" ||
            name == "YPos" ||
            name .startsWith("Head X") ||
            name .startsWith("Head Y") ||
            name .startsWith("Stage X") ||
            name .startsWith("Stage Y"))
        return "Position";

    if (name == "Nx" ||
            name == "pixelsize" ||
            name == "Laser Wavelength, nm" ||
            name == "Central Wavelength, nm" ||
            name == "Central Pixel" ||
            name == "Dispersion")
        return "Calibration";

    if (name == "XMin" ||
            name == "XMax" ||
            name == "YMin" ||
            name == "YMax" ||
            name == "ZMin View" ||
            name == "ZMax View" ||
            name == "ZViewAuto")
        return "XYZ Limits";

    if (name == "Fit mode" ||
            name == "XFit" ||
            name == "YFit" ||
            name == "Substract" ||
            name == "gwyXFit" ||
            name == "gwyYFit" ||
            name == "Mean" ||
            name == "Median" ||
            name == "Gaussian" ||
            name == "HistEq")
        return "Filters";

    if (name == "Mode" ||
            name == "Accum num" ||
            name == "Accum cycle time" ||
            name == "Trigger mode" ||
            name == "Kinetic Series num" ||
            name == "Kinetic Cycle num" ||
            name == "Read mode" ||
            name == "Single track center row" ||
            name == "Single track height" ||
            name == "Exposure time" ||
            name == "Grating" ||
            name == "ND filter" ||
            name == "CCD Temperature" ||
            name == "Objective Z" ||
            name == "X points" ||
            name == "Y points")
        return "CCD settings";

    if (name == "FB P" ||
            name == "FB I" ||
            name == "FB D" ||
            name == "FB JM Precision" ||
            name == "FB JM Height" ||
            name == "Set point" ||
            name == "From, kHz" ||
            name == "To, kHz" ||
            name == "Current frequency, kHz" ||
            name == "Drive intensity, V" ||
            name == "Low pass filter, kHz" ||
            name == "Phase shift, degree" ||
            name == "ChannelIdx" ||
            name == "DFLIdx" ||
            name == "LFLIdx" ||
            name == "INTIdx" ||
            name == "SPM FB PID" ||
            name == "SPM FB scanner" ||
            name == "SPM FB mode")
        return "SPM settings";

    if (name == "Ch A" ||
            name == "Ch B" ||
            name == "Ch C" ||
            name == "Ch D" ||
            name.startsWith("Lift ") ||
            name == "Velocity, Hz" ||
            name == "XY PID" ||
            name.startsWith("Scan "))
        return "Scan settings";

    return "";
}

void TContainer::updateAttributes()
{
    table->blockSignals(true);
    table->clear();
    if (!CurrentDataList || !CurrentDataList->DataList.count() || !CurrentDataList->ClickedItem) {
        table->setEnabled(false);

        table->blockSignals(false);
        return;
    }
    table->setEnabled(true);

    table->sortItems(-1, Qt::AscendingOrder);

    attribItemsRoots.clear();

    QHash<QString, QVariant>::const_iterator it = CurrentDataList->ClickedItem->attributes.constBegin();
    for (; it != CurrentDataList->ClickedItem->attributes.constEnd(); ++it)
        if (it.value().canConvert<QString>())
            addAttribute(attribGroup(it.key()), it.key(), it.value().toString());

    table->sortItems(0, Qt::AscendingOrder);

    table->blockSignals(false);
}

void TContainer::addAttrib_clicked()
{
    table->blockSignals(true);

    addAttribute(QString(), tr("<unset>"), tr("<unset>"));

    table->blockSignals(false);
}

void TContainer::SaveAttrib()
{
    if (!CurrentDataList || !CurrentDataList->ClickedItem)
        return;

    if (CurrentDataList->ClickedItem->InfoType == TAbstractInfo::_Image) {
        QImage data = CurrentDataList->ClickedItem->attributes.value("Preview").value<QImage>();
        CurrentDataList->ClickedItem->attributes.clear();
        CurrentDataList->ClickedItem->attributes.insert("Preview", QVariant::fromValue(data));
    } else
        CurrentDataList->ClickedItem->attributes.clear();

    for (int i = 0; i < table->topLevelItemCount(); i++) {
        QTreeWidgetItem *item = table->topLevelItem(i);
        if (item->data(0, Qt::UserRole).toBool()) {
            if (!item->text(0).isEmpty())
                CurrentDataList->ClickedItem->attributes.insert(item->text(0), item->text(1));
        }

        for (int j = 0; j < item->childCount(); j++) {
            QTreeWidgetItem *it = item->child(j);
            if (!it->text(0).isEmpty())
                CurrentDataList->ClickedItem->attributes.insert(it->text(0), it->text(1));
        }
    }

    CurrentDataList->blockSignals(true);
    CurrentDataList->ClickedItem->Item->setText(CurrentDataList->ClickedItem->attributes.value("Name").toString());
    CurrentDataList->blockSignals(false);

    if (CurrentDataList->ClickedItem->InfoType == TAbstractInfo::_4d) {
        ccd::TSetOfSpectrum *info = (ccd::TSetOfSpectrum *)CurrentDataList->ClickedItem->Info;

        info->XLimits[0] = CurrentDataList->ClickedItem->attributes.value("XMin").toDouble();
        info->XLimits[1] = CurrentDataList->ClickedItem->attributes.value("XMax").toDouble();
        info->YLimits[0] = CurrentDataList->ClickedItem->attributes.value("YMin").toDouble();
        info->YLimits[1] = CurrentDataList->ClickedItem->attributes.value("YMax").toDouble();
        info->UpdateDxDy();
    } else if (CurrentDataList->ClickedItem->InfoType == TAbstractInfo::_3d) {
        plot::T3dDataOwner *info = (plot::T3dDataOwner *)CurrentDataList->ClickedItem->Info;
		auto firstInfo = info->at(0);
		firstInfo->XLimits[0] = CurrentDataList->ClickedItem->attributes.value("XMin").toDouble();
		firstInfo->XLimits[1] = CurrentDataList->ClickedItem->attributes.value("XMax").toDouble();
		firstInfo->YLimits[0] = CurrentDataList->ClickedItem->attributes.value("YMin").toDouble();
		firstInfo->YLimits[1] = CurrentDataList->ClickedItem->attributes.value("YMax").toDouble();
		firstInfo->UpdateDxDy();
    } else if (CurrentDataList->ClickedItem->InfoType == TAbstractInfo::_2d) {
        //plot::TCurvesDataOwner *info = (plot::TCurvesDataOwner *)CurrentDataList->ClickedItem->Info;

    }
}


QDateTime itemDate(const TListItem *s, QDateTime default_val = QDateTime::currentDateTime())
{
    QString id = s->attributes.value("Date").toString();
    QDateTime tt = QDateTime::fromString(id, "yyyy-MM-dd hh:mm:ss:zzz");
    if (tt.isValid())
        return tt;

    return default_val;
}
bool dateCompareItem(const TListItem *s1, const TListItem *s2)
{
    QDateTime t1 = itemDate(s1, QDateTime());
    QDateTime t2 = itemDate(s2, QDateTime());
    return t1 > t2;
}
bool dateCompareItemList(const TListItems &s1, const TListItems &s2)
{
    if (s1.size() == 0 || s2.size() == 0) { qDebug() << "fail"; return false; }
    return dateCompareItem(s1.first(), s2.first());
}

void TContainer::UpdateHierarchy()
{
    setEnabled(false);

    //fill groups
    TListItem *citem;
    QVector<TListItems> igroups;
    QVector<QString> groups;

    for (int j = 0; j < CurrentDataList->DataList.count(); j++) {
        citem = CurrentDataList->DataList[j];

        QString id = citem->attributes.value("GroupId", "empty").toString();

        int idx = 0;
        for (; idx < groups.size(); idx++)
            if (groups[idx] == id) {
                igroups[idx].append(citem);
                break;
            }

        if (idx == groups.size()) {
            groups.append(id);
            igroups.append(TListItems() << citem);
        }
    }
    groups.clear();

    //sort items
    for (int i = 0; i < igroups.size(); i++)
        qStableSort(igroups[i].begin(), igroups[i].end(), dateCompareItem);

    //sort groups
    qStableSort(igroups.begin(), igroups.end(), dateCompareItemList);

    CurrentDataList->blockSignals(true);

    //color & name
    static QColor colours[5] = { QColor(234, 99, 30),
                                 QColor(157, 57, 26),
                                 QColor(111, 159, 36),
                                 QColor(106, 197, 220),
                                 QColor(101, 58, 140) };
    int in = 0;
    for (int idx = igroups.size()-1; idx >= 0 ; idx--) {
        bool grouped = igroups[idx].size() > 1;
        for (int i = 0; i < igroups[idx].size(); i++) {
            igroups[idx][i]->Item->setBackground(grouped ? colours[(in % 5)] : Qt::white);

            QString name = igroups[idx][i]->attributes.value("Name", "-").toString();
            igroups[idx][i]->Item->setText(name);
        }
        if (grouped) in++;
    }

    //parents
    //NYI

    //clear list
    while (CurrentDataList->count())
        CurrentDataList->takeItem(0);

    //fill list
    for (int idx = 0; idx < igroups.size(); idx++)
        for (int i = 0; i < igroups[idx].size(); i++)
            CurrentDataList->addItem(igroups[idx][i]->Item);

    if (CurrentDataList->ClickedItem)
        CurrentDataList->setCurrentItem(CurrentDataList->ClickedItem->Item);

    CurrentDataList->blockSignals(false);

    setEnabled(true);
}

void TContainer::updateOverlayVisibility()
{
    #ifndef _ADROID
    //overlayButtons->setVisibleButtons(true);
#endif
}

void TContainer::on_FileProccessing_started()
{
    setEnabled(false);
    PBBar->setVisible(true);
    updateOverlayVisibility();
}
void TContainer::on_FileProccessing_finished()
{
    setEnabled(true);
    PBBar->setVisible(false);
    updateVisibilityOverlayTimer->start(100);
    updateOverlayVisibility();
}
void TContainer::on_FileProccessing_UpdatePreview()
{
	if (CurrentDataList == nullptr)
		return;

    parentWidget()->setEnabled(false);
    preview_update = true;

    CurrentDataList->ClickedItem = nullptr;
    CurrentDataList->setCurrentItem(nullptr); //reset current to always get update even for single data

    for (int i = 0; i < CurrentDataList->DataList.count(); i++) {
        CurrentDataList->ClickedItem = CurrentDataList->DataList[i];
		if (CurrentDataList->ClickedItem == nullptr)
			continue;
        CurrentDataList->ClickedItem->UpdatePreview = true;
        CurrentDataList->setCurrentItem(CurrentDataList->ClickedItem->Item);

        qApp->processEvents();
        while (CurrentDataList->ClickedItem &&
               CurrentDataList->ClickedItem->UpdatePreview) {
            qApp->processEvents();
            SleepExport::msleep2(3);
        }
    }

    preview_update = false;
    parentWidget()->setEnabled(true);
    updateOverlayVisibility();
}
void TContainer::on_FileProccessing_FileProccessed(double TotalNumber, double Current)
{
    PBBar->setUpdatesEnabled(false);
    if (TotalNumber < 0 || Current < 0)
        PBBar->reset();
    else {
        PBBar->setMaximum(int(TotalNumber * 100));
        PBBar->setValue(int(Current * 100));
    }
    PBBar->setUpdatesEnabled(true);
    updateOverlayVisibility();
}
void TContainer::on_FileProccessing_AllDataLoaded(const TAbstractInfos *DataList)
{
    for (int i = 0; i < DataList->size(); i++) {
        if ((*DataList)[i].InfoType == TAbstractInfo::_2d)
            CurrentDataList->Add(DataList->at(i).attributes, (plot::TCurvesDataOwner *)DataList->at(i).Info, DataList->at(i).UpdatePreview);
        else if ((*DataList)[i].InfoType == TAbstractInfo::_3d)
            CurrentDataList->Add(DataList->at(i).attributes, (plot::T3dDataOwner *)DataList->at(i).Info, DataList->at(i).UpdatePreview);
        else if ((*DataList)[i].InfoType == TAbstractInfo::_4d) {
            ((ccd::TSetOfSpectrum *)DataList->at(i).Info)->CalcResults();
            CurrentDataList->Add(DataList->at(i).attributes, (ccd::TSetOfSpectrum *)DataList->at(i).Info, DataList->at(i).UpdatePreview);
        }
        else if ((*DataList)[i].InfoType == TAbstractInfo::_Image)
            CurrentDataList->Add(DataList->at(i).attributes, DataList->at(i).attributes.value("Preview").value<QImage>(), false);
    }
    CurrentDataList->isUpdated = false;

    delete DataList;

    UpdateHierarchy();
    setEnabled(true);

    SendContainerData();

    if (CurrentDataList->DataList.count())
        CurrentDataList->on_itemPressed(CurrentDataList->DataList.first()->Item);
}

void TContainer::AddBlanks(int num)
{
    CurrentDataList->AddBlanks(num);
}
void TContainer::Error(QString com, QString msg)
{
    //todo mobile version make there mobile notification
    #ifndef _ADROID
    new Notification(com + ": " + msg, Notification::Bad, 20);
    doBeep();
#endif
}

void TContainer::Save(QString &FileName, bool oldFormat)
{
    if (FileName.isEmpty())
        return;

    if (!oldFormat && !FileName.endsWith(".nstdat"))
        FileName += ".nstdat";
    else if (oldFormat && !FileName.endsWith(".gwy"))
        FileName += ".gwy";

    CurrentDataList->isUpdated = false;
    FileProccessing->Save(FileName, GenerateDataForSave());
}
void TContainer::AutoSave()
{
    isAutosave = true;
    autosave_idx = 0;
    SaveAll();
    isAutosave = false;
}
void TContainer::SetAutoSaveTime(int t)
{
    AutosaveTimer->stop();

    AutoSaveTime = t;
    AutosaveTimer->setInterval(t);

    if (AutoSaveTime != 0)
        AutosaveTimer->start();
}

TAbstractInfos *TContainer::GenerateDataForSave()
{
    TAbstractInfos *DataForSave = new TAbstractInfos;
    DataForSave->resize(CurrentDataList->DataList.size());

    for (int i = 0; i < CurrentDataList->DataList.size(); i++)
    {
        (*DataForSave)[i].Info     = CurrentDataList->DataList[i]->Info;
        (*DataForSave)[i].InfoType = CurrentDataList->DataList[i]->InfoType;
        (*DataForSave)[i].attributes = CurrentDataList->DataList[i]->attributes;
    }
    return DataForSave;
}
void TContainer::Load(const QString &FileName)
{
    if (FileName.isEmpty())
        return;

    TAbstractInfos *LoadedData = new TAbstractInfos;
    FileProccessing->Load(FileName, LoadedData);
}


void TContainer::AddDataToContainer_slot(plot::TCurvesDataOwner src, plot::TMetaStorage attributes)
{
    if (!CurrentDataList || !CurrentDataList->ClickedItem)
        return;

    QString date = getDateHash();
    QString uniqid = date + "Data";
    quint16 id = qChecksum(uniqid.toStdString().data(), uniqid.length());
    attributes.insert("Id", QString::number(id, 16).toUpper());
    attributes.insert("GroupId", CurrentDataList->ClickedItem->attributes.value("GroupId"));
    attributes.insert("Date", CurrentDataList->ClickedItem->attributes.value("Date"));
    attributes.insert("Parent", CurrentDataList->ClickedItem->attributes.value("Id"));
    attributes.insert("Name", CurrentDataList->ClickedItem->attributes.value("Name"));
    attributes.insert("Source", "Data");

    if (CurrentDataList->ClickedItem->InfoType == TAbstractInfo::_2d) {

    } else if (CurrentDataList->ClickedItem->InfoType == TAbstractInfo::_3d) {
		auto plot = Plot->ThreeDPlot->ThreeDPlots.at(0);
        attributes.insert("Marker A x", plot->MarkerA->xValue()*1e6);
        attributes.insert("Marker A y", plot->MarkerA->yValue()*1e6);

        attributes.insert("Marker B x", plot->MarkerB->xValue()*1e6);
        attributes.insert("Marker B y", plot->MarkerB->yValue()*1e6);

    } else if (CurrentDataList->ClickedItem->InfoType == TAbstractInfo::_4d) {
        attributes.insert("Name", Plot->TwoDPlot->Pure2dPlot->Title);

        int _x = lastXind;
        int _y = lastYind;
        attributes.insert("CCD X idx", _x);
        attributes.insert("CCD Y idx", _y);

        ccd::TSetOfSpectrum *spec = (ccd::TSetOfSpectrum *)CurrentDataList->ClickedItem->Info;
        spec->UpdateDxDy();
        double XInd = 0., YInd = 0.;
        if ((bool)spec->Direction) {
            switch (spec->StartPoint) {
            case ccd::TSetOfSpectrum::TopLeft:     XInd = spec->XLimits[0]+_x*spec->dx; YInd = spec->YLimits[1]-_y*spec->dy; break;
            case ccd::TSetOfSpectrum::TopRight:    XInd = spec->XLimits[1]-_x*spec->dx; YInd = spec->YLimits[1]-_y*spec->dy; break;
            case ccd::TSetOfSpectrum::BottomRight: XInd = spec->XLimits[1]-_x*spec->dx; YInd = spec->YLimits[0]+_y*spec->dy; break;
            case ccd::TSetOfSpectrum::BottomLeft:  XInd = spec->XLimits[0]+_x*spec->dx; YInd = spec->YLimits[0]+_y*spec->dy; break;
            }
        } else {
            switch (spec->StartPoint) {
            case ccd::TSetOfSpectrum::TopLeft:    XInd = spec->YLimits[1]-_y*spec->dy; YInd = spec->XLimits[0]+_x*spec->dx;break;
            case ccd::TSetOfSpectrum::TopRight:   XInd = spec->YLimits[1]-_y*spec->dy; YInd = spec->XLimits[1]-_x*spec->dx;break;
            case ccd::TSetOfSpectrum::BottomRight:XInd = spec->YLimits[0]+_y*spec->dy; YInd = spec->XLimits[1]-_x*spec->dx;break;
            case ccd::TSetOfSpectrum::BottomLeft: XInd = spec->YLimits[0]+_y*spec->dy; YInd = spec->XLimits[0]+_x*spec->dx;break;
            }
        }

        attributes.insert("CCD X screen pos, m", XInd);
        attributes.insert("CCD Y screen pos, m", YInd);

        if (CurrentDataList->ClickedItem->attributes.contains("XMin"))
            attributes.insert("CCD X global pos, m", XInd + CurrentDataList->ClickedItem->attributes.value("XMin").toDouble());

        if (CurrentDataList->ClickedItem->attributes.contains("YMin"))
            attributes.insert("CCD Y global pos, m", YInd + CurrentDataList->ClickedItem->attributes.value("YMin").toDouble());

/*
        attributes.insert("Marker A x", Plot->ThreeDPlot->ThreeDPlots.at(0)->MarkerA->xValue()*1e6);
        attributes.insert("Marker A y", Plot->ThreeDPlot->ThreeDPlots.at(0)->MarkerA->yValue()*1e6);

        attributes.insert("Marker B x", Plot->ThreeDPlot->ThreeDPlots.at(0)->MarkerB->xValue()*1e6);
        attributes.insert("Marker B y", Plot->ThreeDPlot->ThreeDPlots.at(0)->MarkerB->yValue()*1e6);
*/

        attributes.insert("Laser Wavelength, nm", CurrentDataList->ClickedItem->attributes.value("Laser Wavelength, nm"));
        attributes.insert("Central Wavelength, nm", CurrentDataList->ClickedItem->attributes.value("Central Wavelength, nm"));
        attributes.insert("Dispersion", CurrentDataList->ClickedItem->attributes.value("Dispersion"));
        attributes.insert("pixelsize", CurrentDataList->ClickedItem->attributes.value("pixelsize"));
        attributes.insert("Nx", CurrentDataList->ClickedItem->attributes.value("Nx"));
        attributes.insert("Central Pixel", CurrentDataList->ClickedItem->attributes.value("Central Pixel").toDouble() - spec->MinForRecord);
    }

    AddDataToContainer(src, attributes);
}
void TContainer::AddDataToContainer_slot(plot::T3dDataOwner src, plot::TMetaStorage attributes)
{
    if (!CurrentDataList || !CurrentDataList->ClickedItem)
        return;

    QString date = getDateHash();

    if (CurrentDataList->ClickedItem->InfoType == TAbstractInfo::_3d) {
        attributes = CurrentDataList->ClickedItem->attributes;

		auto data = src.at(0);
        attributes.insert("XMin", 0);
        attributes.insert("YMin", 0);
        attributes.insert("XMax", data->XLimits[1] - data->XLimits[0]);
        attributes.insert("YMax", data->YLimits[1] - data->YLimits[0]);

        QString name = Plot->ThreeDPlot->ThreeDPlots.at(0)->Title;
        attributes.insert("Name", name + tr(" - Crop"));

    } else if (CurrentDataList->ClickedItem->InfoType == TAbstractInfo::_4d) {
        attributes.insert("Name", Plot->ThreeDPlot->ThreeDPlots.at(0)->Title);

        attributes.insert("Date", date);
        attributes.insert("Source", "Data");
    }

    QString uniqid = date + "Data";
    quint16 id = qChecksum(uniqid.toStdString().data(), uniqid.length());
    attributes.insert("Id", QString::number(id, 16).toUpper());
    attributes.insert("GroupId", CurrentDataList->ClickedItem->attributes.value("GroupId"));
    attributes.insert("Parent", CurrentDataList->ClickedItem->attributes.value("Id"));

    AddDataToContainer(src, attributes);
}


QString TContainer::getDateHash()
{
    static QDateTime stamp(QDate(1, 1, 1), QTime(1, 1));

    if (stamp.msecsTo(QDateTime::currentDateTime()) >= 1000)
        stamp = QDateTime::currentDateTime();

    return stamp.toString("yyyy-MM-dd hh:mm:ss");
}
void TContainer::addDateIdsToAttributes(plot::TMetaStorage &attributes)
{ //requires unique "Source" for unique "Id"
    if (attributes.contains("Date"))
        return;

    QString date = getDateHash();
    attributes.insert("Date", date);

    quint16 gid = qChecksum(date.toStdString().data(), date.length());
    attributes.insert("GroupId", QString::number(gid, 16).toUpper());

    QString uniqid = date + attributes.value("Source").toString();
    quint16 id = qChecksum(uniqid.toStdString().data(), uniqid.length());
    attributes.insert("Id", QString::number(id, 16).toUpper());
}
void TContainer::AddDataToContainer(ccd::TSetOfSpectrum *src, plot::TMetaStorage attributes)
{
    if (!src->Array.size()) {
        delete src;
        return;
    }

    #ifndef _ADROID
    if (!copypaste) {
        CurrentDataList = (TListWidget *)CBDataLists->widget(0);
        CBDataLists->setCurrentIndex(0);
    }
#endif

    addDateIdsToAttributes(attributes);

    CurrentDataList->AddBlanks(1);
    CurrentDataList->Add(attributes, src);
    if (!copypaste)
        UpdateHierarchy();

    SetDownButtonsEnabled(true);
}
void TContainer::AddDataToContainer(plot::TCurvesDataOwner src, plot::TMetaStorage attributes)
{
    if (!src.size())
        return;

    #ifndef _ADROID
    if (!copypaste) {
        CBDataLists->setCurrentIndex(0);
        TabChanged(0);
    }
#endif

    addDateIdsToAttributes(attributes);

    plot::TCurvesDataOwner *data = new plot::TCurvesDataOwner;
    *data = src;
    CurrentDataList->AddBlanks(1);
    CurrentDataList->Add(attributes, data);
    if (!copypaste) {
        UpdateHierarchy();

        CurrentDataList->ClickedItem = CurrentDataList->DataList.last();
        UpdatePreview();
    }

    SetDownButtonsEnabled(true);
}
void TContainer::AddDataToContainer(plot::T3dDataOwner src, plot::TMetaStorage attributes)
{
    #ifndef _ADROID
    if (!copypaste) {
        CurrentDataList = (TListWidget *)CBDataLists->widget(0);
        CBDataLists->setCurrentIndex(0);
    }
#endif

    addDateIdsToAttributes(attributes);

    plot::T3dDataOwner *data = new plot::T3dDataOwner;
    *data = src;
    CurrentDataList->AddBlanks(1);
    CurrentDataList->Add(attributes, data, true);
    if (!copypaste) {
        UpdateHierarchy();

        CurrentDataList->ClickedItem = CurrentDataList->DataList.last();
        UpdatePreview();
    }

    SetDownButtonsEnabled(true);
}
void TContainer::AddDataToContainer(QImage src, plot::TMetaStorage attributes)
{
    //qDebug()<<"GOT2---------------------------";
    #ifndef _ADROID
    if (!copypaste) {
        CurrentDataList = (TListWidget *)CBDataLists->widget(0);
        CBDataLists->setCurrentIndex(0);
    }
#endif

    addDateIdsToAttributes(attributes);

    CurrentDataList->AddBlanks(1);
    CurrentDataList->Add(attributes, src, true);
    if (!copypaste) {
        UpdateHierarchy();

        CurrentDataList->ClickedItem = CurrentDataList->DataList.last();
        //UpdatePreview();
    }

    SetDownButtonsEnabled(true);
}


void TContainer::Send2dPlotData()
{
    #ifndef _ADROID
    send2dPlot(PlotID::Data, Plot->TwoDPlot);
#endif
}
void TContainer::Send3dPlotData()
{
    #ifndef _ADROID
    if (!CurrentDataList || !CurrentDataList->ClickedItem)
        return;

    send3dPlot(PlotID::Data, Plot->ThreeDPlot, 0, CurrentDataList->ClickedItem->InfoType == TAbstractInfo::_4d);
#endif
}

void TContainer::SendContainerData()
{
    #ifndef _ADROID
    QStringList lst;
    QVariantList previews;
    foreach (QListWidgetItem *wit, CurrentDataList->findItems("*", Qt::MatchWildcard)) {
        TListItem *it = CurrentDataList->FindItem(wit);
        if (!it) continue;
        lst.append(((it->InfoType == TAbstractInfo::_2d) ? "2D" : (it->InfoType == TAbstractInfo::_3d) ? "3D" : "4D") +
                   QString(": ") + (it->Item->text()));
        previews.append(it->Item->icon().pixmap(100, 100).toImage());
    }
    MobileServer::sendData(DataID::DataDataList, lst);
    MobileServer::sendData(DataID::DataPreviewList, previews);
#endif
}

