#include "gwynetio.hpp"

#include "cont_basetypes.h"
//#include "ccd/ccd.h"
#include <QDir>
#include <QProcess>
#include <QTcpSocket>
#include <QTcpServer>
#include <QTimer>


GwyddionImport::GwyddionImport()
{
    connect(this, SIGNAL(finished()), SLOT(deleteLater()));
    start();
}

static
void do_cleanup(QTcpSocket *client)
{
    client->disconnectFromHost();
    //if (client->state() == QAbstractSocket::UnconnectedState ||
    //        client->waitForDisconnected())
        delete client;
}

static
bool read_bytes(QTcpSocket *client, QByteArray &buffer, unsigned int size)
{
    try {
        //if (buffer.size() < (int)size) //breaks code dependent on .size()
            buffer.resize(size);
    } catch (...) {
        qDebug() << "no mem";
        do_cleanup(client);
        return false;
    }

    unsigned int total = 0;
    while (total < size) {
        client->waitForReadyRead();
        unsigned int read = client->read(buffer.data() + total, size - total);
        if (read <= 0) {
            do_cleanup(client);
            return false;
        }

        total += read;
    }

    return true;
}

void GwyddionImport::accept_and_recv()
{
    QTcpSocket *client = server->nextPendingConnection();

    if (client == nullptr)
        return;

    plot::TCurvesDataOwner info2;
    plot::T3dDataOwner info3;
    //ccd::TSetOfSpectrum *info4;
    plot::TMetaStorage attributes;

    enum TypeOfInfo_old {_2d, _3d, _4d, Blank};
    QByteArray buffer;

    while (true) {
        if (!read_bytes(client, buffer, 3))
            return;

        if (buffer[0] != (char)0xDE ||
                buffer[1] != (char)0xAD) {

            do_cleanup(client);
            return;
        }

        signed char type = buffer[2];
        if (type == _2d) {
            if (!read_bytes(client, buffer, sizeof(short)))
                return;

            short ny = *(short *)(buffer.constData());
            plot::ResizeOwner(info2, ny);
            for (int j = 0; j < ny; j++) {
                if (!read_bytes(client, buffer, sizeof(short)))
                    return;

                short nx = *(short *)(buffer.constData());

                if (!read_bytes(client, buffer, nx*2*sizeof(double)))
                    return;

                for (int k = 0; k < nx; k++)
                    info2[j]->AddPoint(
                                *(double *)(buffer.constData()+k*sizeof(double)),
                                *(double *)(buffer.constData()+(nx+k)*sizeof(double)));
            }

            if (!read_bytes(client, buffer, 1))
                return;
            if (!read_bytes(client, buffer, buffer[0]))
                return;

            attributes.insert("XLabel", QString::fromUtf8(buffer.data(), buffer.size()));

            if (!read_bytes(client, buffer, 1))
                return;
            if (!read_bytes(client, buffer, buffer[0]))
                return;

            attributes.insert("YLabel", QString::fromUtf8(buffer.data(), buffer.size()));

            if (!read_bytes(client, buffer, 1))
                return;
            if (!read_bytes(client, buffer, buffer[0]))
                return;

            attributes.insert("XUnit", QString::fromUtf8(buffer.data(), buffer.size()));

            if (!read_bytes(client, buffer, 1))
                return;
            if (!read_bytes(client, buffer, buffer[0]))
                return;

            attributes.insert("YUnit", QString::fromUtf8(buffer.data(), buffer.size()));

            if (!read_bytes(client, buffer, 1))
                return;
            if (!read_bytes(client, buffer, buffer[0]))
                return;

            attributes.insert("Name", QString::fromUtf8(buffer.data(), buffer.size()));

            emit AddDataToContainer(info2, attributes);

            info2.clear();
            attributes.clear();

        } else if (type == _3d) {
            plot::ResizeOwner(info3, 1);

            if (!read_bytes(client, buffer, 2*sizeof(short) + 2*sizeof(double)))
                return;

            short ny = *(short *)(buffer.constData());
            short nx = *(short *)(buffer.constData()+2);
            double sizey = *(double *)(buffer.constData()+4);
            double sizex = *(double *)(buffer.constData()+4+sizeof(double));

            if (!read_bytes(client, buffer, nx*ny*sizeof(double)))
                return;

			auto firstInfo = info3.at(0);
			firstInfo->setSize(nx, ny);
            for (int i = 0; i < ny; i++) {
                for (int j = 0; j < nx; j++) {
					firstInfo->Array[ny-1-i][j] = *(double *)(buffer.constData()+(i*nx + j)*sizeof(double));
                }
            }

            if (!read_bytes(client, buffer, 1))
                return;

            QString Tmp;
            switch (buffer[0]) {
            case 0:
                Tmp = QObject::tr("m");
                break;
            case 1:
                Tmp = QObject::tr("V");
                break;
            case 2:
                Tmp = QObject::tr("Counts");
                break;
            }
            attributes.insert("XYUnit", Tmp);

            if (!read_bytes(client, buffer, 1))
                return;
            switch (buffer[0]) {
            case 0:
                Tmp = QObject::tr("m");
                break;
            case 1:
                Tmp = QObject::tr("V");
                break;
            case 2:
                Tmp = QObject::tr("Counts");
                break;
            }
            attributes.insert("ZUnit", Tmp);

            attributes.insert("XMin", info3.at(0)->XLimits[0] = 0);
            attributes.insert("YMin", info3.at(0)->YLimits[0] = 0);
            attributes.insert("XMax", info3.at(0)->XLimits[1] = sizex);
            attributes.insert("YMax", info3.at(0)->YLimits[1] = sizey);

            if (!read_bytes(client, buffer, 1))
                return;
            if (!read_bytes(client, buffer, buffer[0]))
                return;

            attributes.insert("Name", QString::fromUtf8(buffer.data(), buffer.size()));

            emit AddDataToContainer(info3, attributes);

            info3.clear();
            attributes.clear();

        } else if (type == _4d) {
            //info4 = 0;
            //NYI
        } else if (type == -1)
            break;
    }

    emit Refresh();

    do_cleanup(client);
}
void GwyddionImport::run()
{
    server = new QTcpServer();
    connect(server, SIGNAL(newConnection()), SLOT(accept_and_recv()), Qt::DirectConnection);
    server->listen(QHostAddress::Any, 1667);

    exec();
    delete server;
}

GwyddionExport::GwyddionExport()
{
    count = 0;
    buffer.clear();
    client = nullptr;
    connect(this, SIGNAL(finished()), SLOT(deleteLater()));
}
GwyddionExport::~GwyddionExport()
{
    QFile::remove("dummy.ngi");
}

void GwyddionExport::AddData(void *data, int type, plot::TMetaStorage &meta)
{
    char tp = type;
    short size = 0;

    if (data == nullptr)
        return;

    int a0;
    double val;
    QString str;

    enum TypeOfInfo_old {_2d, _3d, _4d, Blank};

try {
    switch (type) {
    case TAbstractInfo::_2d: {
        plot::TCurvesDataOwner *info2 = ((plot::TCurvesDataOwner *)data);

        if (info2->size() < 1)
            return;

        tp = _2d;
        buffer.append((char)0xDE);//magic low
        buffer.append((char)0xAD);//magic high
        buffer.append((char *)&tp, sizeof(tp));

        size = info2->size();
        buffer.append((char *)&size, sizeof(size));

        for (int i = 0; i < info2->size(); i++) {
            size = qMin(32767, info2->at(i)->size());
            buffer.append((char *)&size, sizeof(size));

            for (int j = 0; j < size; j++) {
                val = info2->at(i)->getXat(j);
                buffer.append((char *)&val, sizeof(val));
            }

            for (int j = 0; j < size; j++) {
                val = info2->at(i)->YSeq.at(j);
                buffer.append((char *)&val, sizeof(val));
            }
        }

        str = meta.value("XLabel").toString();
        tp = str.toUtf8().length();
        buffer.append((char *)&tp, sizeof(tp));
        buffer.append((char *)str.toUtf8().data(), str.toUtf8().length());

        str = meta.value("YLabel").toString();
        tp = str.toUtf8().length();
        buffer.append((char *)&tp, sizeof(tp));
        buffer.append((char *)str.toUtf8().data(), str.toUtf8().length());

        str = meta.value("XUnit").toString();
        tp = str.toUtf8().length();
        buffer.append((char *)&tp, sizeof(tp));
        buffer.append((char *)str.toUtf8().data(), str.toUtf8().length());

        str = meta.value("YUnit").toString();
        tp = str.toUtf8().length();
        buffer.append((char *)&tp, sizeof(tp));
        buffer.append((char *)str.toUtf8().data(), str.toUtf8().length());

        str = meta.value("Name").toString();
        tp = str.toUtf8().length();
        buffer.append((char *)&tp, sizeof(tp));
        buffer.append((char *)str.toUtf8().data(), str.toUtf8().length());

        count++;
    }
        break;
    case TAbstractInfo::_3d: {
        plot::T3dDataOwner *info3 = ((plot::T3dDataOwner *)data);

        tp = _3d;
        buffer.append((char)0xDE);//magic low
        buffer.append((char)0xAD);//magic high
        buffer.append((char *)&tp, sizeof(tp));

		auto firstInfo = info3->at(0);
		firstInfo->UpdateNxNy();
        size = firstInfo->NYElems;
        buffer.append((char *)&size, sizeof(size));
        size = firstInfo->NXElems;
        buffer.append((char *)&size, sizeof(size));

        val = meta.value("YMax").toDouble() - meta.value("YMin").toDouble();
        buffer.append((char *)&val, sizeof(val));
        val = meta.value("XMax").toDouble() - meta.value("XMin").toDouble();
        buffer.append((char *)&val, sizeof(val));

        a0 = qMin<int>(firstInfo->Array.size(), firstInfo->NYElems);
        //for (int i = 0; i < a0; i++) {
        for (int i = a0-1; i >= 0; i--) { //invert y
            int a1 = qMin<int>(firstInfo->Array[i].size(), firstInfo->NXElems);
            for (int j = 0; j < a1; j++) {
                val = firstInfo->Array[i][j];
                buffer.append((char *)&val, sizeof(val));
            }
            for (int j = a1; j < firstInfo->NXElems; j++) {
                val = 0;
                buffer.append((char *)&val, sizeof(val));
            }
        }
        for (int i = a0; i < firstInfo->NYElems; i++) {
            for (int j = 0; j < firstInfo->NXElems; j++) {
                val = 0;
                buffer.append((char *)&val, sizeof(val));
            }
        }

        str = meta.value("XYUnit").toString();
        if (str == QObject::tr("m"))
            tp = 0;
        else if (str == QObject::tr("V"))
            tp = 1;
        else if (str == QObject::tr("Counts"))
            tp = 2;
        buffer.append((char *)&tp, sizeof(tp));

        str = meta.value("ZUnit").toString();
        if (str == QObject::tr("m"))
            tp = 0;
        else if (str == QObject::tr("V"))
            tp = 1;
        else if (str == QObject::tr("Counts"))
            tp = 2;
        buffer.append((char *)&tp, sizeof(tp));

        str = meta.value("Name").toString();
        tp = str.toUtf8().length();
        buffer.append((char *)&tp, sizeof(tp));
        buffer.append((char *)str.toUtf8().data(), str.toUtf8().length());

        count++;
    }
        break;
    case TAbstractInfo::_4d: {
        //ccd::TSetOfSpectrum *info4 = ((ccd::TSetOfSpectrum *)data);
        //NYI
        qDebug() << "4d send to gwyddion NYI";
    }
        break;
    };
} catch (...) {
    buffer.clear();
    qDebug() << "gwyexp add mem? fail";
}
}

void GwyddionExport::accept_and_send()
{
    client = server->nextPendingConnection();
    server->close();

    if (client == nullptr) {
        buffer.clear();
        quit();
        return;
    }

try {
    client->write(buffer);
} catch (...) {
    qDebug() << "gwyexp send mem? fail";
}
    buffer.clear();

    client->disconnectFromHost();
    if (client->state() == QAbstractSocket::UnconnectedState ||
            client->waitForDisconnected())
        delete client;

    quit();
}
void GwyddionExport::run()
{
#ifdef _WIN32
    QString program = QDir::currentPath() + "/gwyddion/gwyddion";
    if (!QFileInfo(program+".exe").exists())
        program = QDir::currentPath() + "/gwyddion/bin/gwyddion";
    program = "C:/Program Files/Gwyddion/bin/gwyddion.exe";
#else
    QString program = "gwyddion";
#endif

    if (buffer.size() <= 4) {
        buffer.clear();

        QStringList arguments;
        arguments << "--no-splash" << "--remote-new";

        QProcess::startDetached(program, arguments);
        return;
    }

    //inverse push order
try {
    buffer.prepend(count);
    buffer.prepend((char)0xAD);//magic high
    buffer.prepend((char)0xDE);//magic low
} catch (...) {
    buffer.clear();
    qDebug() << "gwyexp thread mem? fail";
    return;
}

    server = new QTcpServer();
    QTimer *timeout = new QTimer();

    QStringList arguments;
    if (!QDir().exists("dummy.ngi")) {
        QFile lockfile("dummy.ngi");
        lockfile.open(QIODevice::WriteOnly);
        lockfile.write("#traash");
    }
    arguments << "--no-splash" << "--remote-new" << QDir::currentPath() + "/dummy.ngi";

    connect(server, SIGNAL(newConnection()), SLOT(accept_and_send()), Qt::DirectConnection);
    connect(timeout, SIGNAL(timeout()), SLOT(quit()), Qt::DirectConnection);

    server->listen(QHostAddress::Any, 1666);
    timeout->start(10*1000);

    if (!QProcess::startDetached(program, arguments)) {
        emit no_gwyddion();

        buffer.clear();
        delete server;
        delete timeout;

        return;
    }

    exec();
    buffer.clear();
    delete server;
    delete timeout;
}
