#include "deflect_widget.hpp"
#include <QtEvents>
#include <QGestureEvent>
#include <QPainter>
#include <QPainterPath>
#include <QDebug>
#include <QtGlobal>
#include <QApplication>

DeflectWid::DeflectWid()
{
    qValue = tr("none");
    probeCheckScanBtn = new QPushButton(QIcon(":/start.png"), "", this);
    probeCheckScanBtn->setFlat(true);
    probeCheckScanBtn->setStyleSheet( "background-color: rgba(255,255,255,10);" );
    freqCurve = QPolygonF(1000);
    freqCurveDraw = QPolygonF(1000);
    oscMagCurve = QVector<double>(1024);
    for (int i=0; i<freqCurve.size(); i++)
    {
        freqCurve[i].setX(i);
    }
    setBackgroundRole(QPalette::Base);
    setAutoFillBackground(true);

    pixmap.load(":/SPMTuning_window/laser_spot3.png");
    TuneFork.load(":/SPMTuning_window/approach/TunFork.png");
    DflPix.load(":/SPMTuning_window/dflVert.png");
    LflPix.load(":/SPMTuning_window/lfl.png");

    setInt(0);
    setXY(0, 0);
    Zpos = 0.5;
    FBscanner = 0;
    bSTM = false;
    bSnotra = false;

    font.setFamily("arial");

    setMinimumSize(200,200);
    setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding);

    grabGesture(Qt::TapAndHoldGesture);
}

bool DeflectWid::event(QEvent *event)
{
    if (event->type() == QEvent::Gesture) {
        return gestureEvent(static_cast<QGestureEvent*>(event));
    }
    return QWidget::event(event);
}
void DeflectWid::mouseDoubleClickEvent(QMouseEvent * /*Event*/)
{
    emit doubleClicked();
}
bool DeflectWid::gestureEvent(QGestureEvent *event)
{
    if (event->gesture(Qt::TapAndHoldGesture))
        mouseDoubleClickEvent(nullptr);

    return true;
}
void DeflectWid::setTuningFork(bool id)
{
    bSnotra=id;
    update();
}
void DeflectWid::setSTM(bool id)
{
    bSTM=id;
    update();
}

void DeflectWid::SetFreqCurveDataMaximum(float yMax)
{
    fresX = yMax;
}
void DeflectWid::AddFreqCurveData(int x, qreal y)
{
    freqCurve[x].setY(y);
}
void DeflectWid::newOscHeightRMS(double rms, QString units)
{
    oscHeightRMS = tr("Z noise") + ": " + QString::number(rms, 'f', 3) + " " + units;
}
void DeflectWid::newOscMAGData(QList<double> dt, int secs, int oscLineLength)
{
    if (oscTime != secs)
    {
        oscTime = secs;
        oscMagCurve.clear();
        oscMagCurve = QVector<double>(oscTime*1024);
    }

    if (lastIdOscMagCurve >= oscTime*1024)
    {
        lastIdOscMagCurve = 0;
    }

    for (int i=lastIdOscMagCurve, b = 0; b<oscLineLength; i++, b++)
    {
        oscMagCurve[i] = qBound(0.0, dt[b], 3.0);
    }
    lastIdOscMagCurve += oscLineLength;



    frameRater++;
    if (frameRater % 10 == 0)
    {
        this->repaint();
        frameRater=0;
    }
    dt.clear();
}
void DeflectWid::paintEvent(QPaintEvent * /* event */)
{
    QPainter painter(this);

    QString s;
    painter.setRenderHint(QPainter::SmoothPixmapTransform, true);
    painter.setPen(QColor(0, 191, 255));
    painter.setBrush(QBrush(QColor(32, 32, 128)));
    painter.drawRect(QRect(0, 0, width() - 1, height() - 1));
    painter.setBrush(QBrush());

    int minsz = qMin(width(), height());
    int size = qMax<int>(10, minsz * intens / SPOT_SCALE);
    int fontsize = minsz / (FONT_SCALE);
    font.setPixelSize(fontsize);
    painter.setFont(font);

    if (!bSnotra && !bSTM) {
        double Cx = 0.5 + qRound(width()/2.);
        double Cy = 0.5 + qRound(height()/2.);
        int x = Cx + width()*X - size/2;
        int y = Cy - height()*Y - size/2;
        int R = qMax(Cx, Cy) / 6;

        painter.setRenderHint(QPainter::Antialiasing, true);
        for (int i = 6; i > 1; i--)
            painter.drawEllipse(QRectF(Cx-R*i,Cy-R*i, 2*R*i, 2*R*i));
        painter.setRenderHint(QPainter::Antialiasing, false);

        painter.setPen(QColor(0, 191, 255));
        painter.drawLine(Cx, 0, Cx, height());
        painter.drawLine(0, Cy, width(), Cy);

        painter.drawPixmap(x, y, size, size, pixmap);
    }
    painter.setPen(Qt::white);
    if (bSTM) {
        painter.drawText(width()/2.2, height()/2 - 0.8*fontsize, tr("STM MODE"));
    } else
    if (bSnotra) {
        int TF_size = minsz / 4;
        painter.drawText(width()/2.2, height()/2 - 0.8*fontsize, tr("TF MODE"));
        painter.drawPixmap(0.1*width(), 0.15*height(), 1.5*TF_size, 3*TF_size, TuneFork);
    }

    painter.setPen(QPen(Qt::white, height() / 150));
    painter.drawLine(0.60*width(), 0.13*height(), 0.95*width(), 0.13*height());
    painter.drawLine(0.60*width(), 0.13*height(), 0.60*width(), 0.15*height());
    painter.drawLine(0.69*width(), 0.13*height(), 0.69*width(), 0.15*height());
    painter.drawLine(0.78*width(), 0.13*height(), 0.78*width(), 0.15*height());
    painter.drawLine(0.86*width(), 0.13*height(), 0.86*width(), 0.15*height());
    painter.drawLine(0.95*width(), 0.13*height(), 0.95*width(), 0.15*height());


    QColor color = (Zpos > 0.5) ? Qt::green : ((Zpos > 0.25) ? Qt::yellow : Qt::red);
    painter.setPen(QPen(color, height() / 40));
    painter.drawLine((0.95-0.35*Zpos)*width(), 0.1*height(), 0.95*width(), 0.1*height());

    /*if (FBscanner >= 0) {
        s = tr("FB Z: ") + (FBscanner==0 ? tr("Tip") : tr("Sample"));
        painter.setPen(FBscanner==0 ? Qt::green : Qt::red);
        painter.drawText(5, 0.84*height(), s);
    }*/
    painter.setPen(Qt::white);

    painter.drawText(0.95*width() - QFontMetrics(font).width(oscHeightRMS),
                      5 + fontsize, oscHeightRMS);

    int TF_size = minsz / 4;
    if (!bSnotra && !bSTM)
    {
        s = tr("%1").arg(LFL);
        painter.drawPixmap(5, 5 + 2*fontsize, 0.4*TF_size, 0.2*TF_size, LflPix);
        painter.drawText(5 + 0.4*TF_size, 5 + 2.75*fontsize, s);

        s = tr("%1").arg(DFL);
        painter.drawPixmap(5, 5, 0.2*TF_size, 0.4*TF_size, DflPix);
        painter.drawText(5 + 0.4*TF_size, 5 + 1.3*fontsize, s);

        s = tr("INT, %: %1").arg(intens*100, 6, 'f', 1);
        painter.drawText(5, height() + 5 - fontsize, s);
    }
    if (!bSTM) {
        if (MAG.length() <= 7)
        {
            s = QString(tr("MAG: %1")).arg(QString());
            painter.drawText(width()- 10 - QFontMetrics(font).width(s) - fontsize*3.5, height() + 5 - fontsize, s);
            s = MAG;
        }
        else
            s = QString(tr("MAG: %1")).arg(QString()) + MAG;
        //painter.drawText(width()- 10 - QFontMetrics(font).width(s), height() - 5 - 2*fontsize, s);
        //s = MAG;
        painter.drawText(width()- 10 - QFontMetrics(font).width(s), height() + 5 - fontsize, s);
    } else {
        s = QString(tr("ITIP: %1")).arg(DFL);
        painter.drawText(width()- 10 - QFontMetrics(font).width(s), height() + 5 - fontsize, s);
    }

    drawOscMagCurve(&painter, fontsize);

    painter.setPen(Qt::white);

    fontsize /= 2;
    font.setPixelSize(fontsize);
    painter.setFont(font);

    s = QString(tr("Z Range(%):%1")).arg(Zpos*100, 6, 'f', 1);
    painter.drawText(0.62*width(), 0.2*height(), s);

    drawFreqCurveGraph(&painter, fontsize);
}
void DeflectWid::drawOscMagCurve(QPainter *painter, int fontsize)
{
    QPolygonF oscMagCurveDraw = QPolygonF(lastIdOscMagCurve);

    int yDown = 0.90*height() - 2*fontsize;
    int x = 0.95*width();
    int yUp = 0.2*height() + 10;
    int heightOsc = yDown - yUp;
    int widthOsc = 0.2*width();

    for(int i=0; i<lastIdOscMagCurve; i++)
    {
        oscMagCurveDraw[i].setX(x - (widthOsc/3.0)*oscMagCurve[i]);
        oscMagCurveDraw[i].setY(yDown - i*(heightOsc/(oscTime*1024.)));
    }

    painter->drawPolyline(oscMagCurveDraw);

    painter->setPen(QPen(Qt::white, height() / 150));

    painter->drawLine(x, yDown, x-widthOsc, yDown);
    painter->drawLine(x, yDown, x, yDown+10);
    painter->drawLine(x-widthOsc, yDown, x-widthOsc, yDown+10);
    painter->drawText(x-5, yDown+15+fontsize, "0");
    painter->drawText(x-widthOsc-5, yDown+15+fontsize, "3 "+tr("V"));
}
void DeflectWid::drawFreqCurveGraph(QPainter *painter, int fontsize)
{
    int yUpFC = 5 + 3*2*fontsize + 5 + 0.1*height();
    int yDownFC = height() - yUpFC - 0.1*height();
    int xRightFC = 0.33*width() - 5;
    int xLeftFC = 5;
    int heightFC = yDownFC - yUpFC;
    int widthFC = xRightFC - xLeftFC;

    if (height() > width() + 30)
    {
        heightFC = 150;
        yDownFC = 0.5*height() + 0.5*heightFC;
        yUpFC = yDownFC - heightFC;
    }

    double yMax = 0;
    for (int i=0; i<freqCurve.size(); i++)
        if (yMax < freqCurve[i].y())
            yMax = freqCurve[i].y();

    for (int i=0; i<freqCurve.size(); i++)
    {
        freqCurveDraw[i].setX(xLeftFC + i*(widthFC/1000.));
        freqCurveDraw[i].setY(yDownFC - (heightFC/yMax)*freqCurve[i].y());
    }
    painter->drawPolyline(freqCurveDraw);

    fontsize *= 1.5;
    font.setPixelSize(fontsize);
    painter->setFont(font);

    int y = yDownFC + fontsize;
    painter->drawText(xLeftFC, y,
                      tr("Fres")+": "+QString::number((double)fresX) + " " + tr("kHz"));

    y += fontsize*1.1;
    painter->drawText(xLeftFC, y, tr("Q = ") + qValue);
    y += fontsize*1.1;
    QString probeText = tr("Probe check");
    painter->drawText(xLeftFC, y, probeText);
    probeCheckScanBtn->move(xLeftFC + QFontMetrics(font).width(probeText), y - height()*0.1*0.6);
    probeCheckScanBtn->setFixedSize(height()*0.1, height()*0.1);
    probeCheckScanBtn->setIconSize(QSize(height()*0.1, height()*0.1));
}
void DeflectWid::setQ(bool reset, double min, double max)
{
    int q = 0;
    qValue = tr("none");
    if (reset)
        return;
    double yMax = -1;
    int xMax = -1;
    for (int i=0; i<freqCurve.size(); i++)
        if (yMax < freqCurve[i].y())
            yMax = freqCurve[xMax = i].y();

    if (yMax > 0 && xMax > 0)
    {
        double F1 = -1., F2 = -1.;
        double halfMaxMag = yMax*0.5;
        for (int i=xMax-1; i>=0; i--)
        {
            if (freqCurve.at(i).y() < halfMaxMag)
            {
                F1 = freqCurve.at(i).x();
                break;
            }
        }
        for (int i=xMax+1; i<freqCurve.count(); i++)
        {
            if (freqCurve.at(i).y() < halfMaxMag)
            {
                F2 = freqCurve.at(i).x();
                break;
            }
        }
        if (F1 < 0 || F2 < 0)
            return;

        double df = max - min;

        double step = df/999.;
        F1 = min + F1*step;
        F2 = min + F2*step;
        double diffF = qAbs(F2 - F1);
        if (diffF < 0.1 || diffF > 100.0)
            return;

        int q = fresX / diffF;
        qValue = QString::number(q);
    }
}
void DeflectWid::setFbscanner(int value)
{
    if (FBscanner == value)
        return;

    FBscanner = value;
    update();
}
void DeflectWid::setZpos(float Zpos_new)
{
    if (Zpos == Zpos_new)
        return;

    Zpos = Zpos_new;
    update();
}
void DeflectWid::setMag(QString Mag_new)
{
    if (MAG == Mag_new)
        return;

    MAG = Mag_new;
    update();
}
void DeflectWid::setDfl(QString Mag_new)
{
    if (DFL == Mag_new)
        return;

    DFL = Mag_new;
    update();
}
void DeflectWid::setLfl(QString Mag_new)
{
    if (LFL == Mag_new)
        return;

    LFL = Mag_new;
    update();
}
void DeflectWid::setXY(double a, double b)
{
    if (X == a && Y == b)
        return;

    X = a;
    Y = b;
    update();
}
void DeflectWid::setInt(float i)
{
    if (intens == i)
        return;

    intens = i;
    update();
}
