#pragma once

#include <QDoubleSpinBox>

class Edit : public QDoubleSpinBox
{
    Q_OBJECT
public:
    Edit(double min = 0, double max = 100, int decimals = 0);

    double fixedValue(double value) const;
    double value() const;

    void setImmediateMode(bool on = true);

signals:
    void editingStarted();
    void newValue();
    void newValue(int);
    void newValue(double);

public slots:
    void setNewValue(bool emitNewValue = true);
    void setValue(int value) { setValue(double(value)); }
    void setValue(double value);
    void resetValue();
    void updated();

    void setMinimum(double min) { QDoubleSpinBox::setMinimum(min); }
    void setMaximum(double max) { QDoubleSpinBox::setMaximum(max); }
    void showButtons(bool show) { setButtonSymbols(show ? QAbstractSpinBox::UpDownArrows : QAbstractSpinBox::NoButtons); }

protected:
    QString textFromValue(double value) const override;
    void focusInEvent(QFocusEvent *event) override;
    void focusOutEvent(QFocusEvent *event) override;

public:
    void stepBy(int steps) override;
    QSize sizeHint() const override;

private:
    double prevValue;
    bool immediateMode = false;
};
