#include "hoverbutton.h"

#include <QApplication>
#include <QDesktopWidget>
#include <QtEvents>
#include <QPainter>
#include <QPainterPath>

HoverButton::HoverButton(Direction dir, QColor boundColor, QWidget *parent) : QWidget(parent)
{
    setAttribute(Qt::WA_AcceptTouchEvents);
    direction = dir;
    this->boundColor = boundColor;
    font = QFont("Arial", 10);
}

void HoverButton::paintEvent(QPaintEvent *)
{
    QPainter painter(this);

    painter.setPen(boundColor);
    QColor backColor(Qt::gray);
    int w = width();
    int h = height();
    int offset = 1;
    if (direction == TopArrow)
    {
        auto ptsTria = new QPoint[3];
        ptsTria[0].setX(offset);
        ptsTria[0].setY(h - offset);
        ptsTria[1].setX(w*0.5);
        ptsTria[1].setY(offset);
        ptsTria[2].setX(w - offset);
        ptsTria[2].setY(h - offset);
        if (pressed)
            painter.setBrush(QBrush(backColor));
        painter.drawPolygon(ptsTria, 3);
    }
    else if (direction == BottomArrow)
    {
        auto ptsTria = new QPoint[3];
        ptsTria[0].setX(offset);
        ptsTria[0].setY(offset);
        ptsTria[1].setX(w*0.5);
        ptsTria[1].setY(h - offset);
        ptsTria[2].setX(w - offset);
        ptsTria[2].setY(offset);
        if (pressed)
            painter.setBrush(QBrush(backColor));
        painter.drawPolygon(ptsTria, 3);
    }
    else if (direction == LeftArrow)
    {
        auto ptsTria = new QPoint[3];
        ptsTria[0].setX(w - offset);
        ptsTria[0].setY(offset);
        ptsTria[1].setX(offset);
        ptsTria[1].setY(h*0.5);
        ptsTria[2].setX(w - offset);
        ptsTria[2].setY(h - offset);
        if (pressed)
            painter.setBrush(QBrush(backColor));
        painter.drawPolygon(ptsTria, 3);
    }
    else if (direction == RightArrow)
    {
        auto ptsTria = new QPoint[3];
        ptsTria[0].setX(offset);
        ptsTria[0].setY(offset);
        ptsTria[1].setX(w - offset);
        ptsTria[1].setY(h*0.5);
        ptsTria[2].setX(offset);
        ptsTria[2].setY(h - offset);
        if (pressed)
            painter.setBrush(QBrush(backColor));
        painter.drawPolygon(ptsTria, 3);
    }
    else if (direction == Circle)
    {
        painter.drawEllipse(offset, offset, w - offset*2, h - offset*2);
    }

    painter.setPen(QColor(fontColor));
    painter.setFont(font);
    painter.drawText(w*0.5 - QFontMetrics(font).width(text)*0.5, h*0.5 + QFontMetrics(font).height()*0.4, text);
}

void HoverButton::mousePressEvent(QMouseEvent *)
{
    pressed = true;
    update();
    emit holdingPressed(pressed);
}
void HoverButton::mouseReleaseEvent(QMouseEvent *)
{
    pressed = false;
    update();
    emit holdingPressed(pressed);
}
