#pragma once

#include <qwt_slider.h>

class QGestureEvent;
class NSTSlider : public QwtSlider
{
    Q_OBJECT
public:
    NSTSlider(double min, double max, int decimals = 0, QWidget *parent = nullptr, bool vertical = false);

    double fixedValue(double value) const;

    double minV, maxV;
    double stepV;

    QSize minimumSizeHint() const override;

signals:
    void newValue();
    void newValue(int);
    void newValue(double);

public slots:
    void setNewValue();

public slots:
    void setValue(double new_value) override;
    void stepBy(int steps) { setValue(value + steps * stepV); setNewValue(); }
    void stepUp() { stepBy(1); }
    void stepDown() { stepBy(-1); }
    void pageUp() { stepBy(10); }
    void pageDown() { stepBy(-10); }

public slots:
    void slPressed() { pressed = 1; }
    void slUnPressed() { pressed = 0; }
    void setMainRangeAndStep(double min, double max, double step);

signals:
    void rangeStepChanged(double min, double max, double step);

protected:
    bool event(QEvent *) override;
    void mousePressEvent(QMouseEvent *) override;
    void mouseMoveEvent(QMouseEvent *) override;
    void wheelEvent(QWheelEvent *) override;
    void mouseDoubleClickEvent(QMouseEvent *) override;
    bool gestureEvent(QGestureEvent *event);
    void resizeEvent(QResizeEvent *) override;

    //void drawSlider ( QPainter *, const QRect & ) const;
    void drawHandle( QPainter *, const QRect &, int pos ) const override;
    
    QPoint pos;
    int prevZoom;
    int pressed;
    QVector<QPair<double, QColor> > marks;
};
