#include "ratiolayout.hpp"

RatioLayout::RatioLayout(QWidget *parent) : QLayout(parent) {}
RatioLayout::~RatioLayout()
{
    delete item;
}

void RatioLayout::addItem(QLayoutItem *_item)
{
    delete item;
    item = _item;
}

int RatioLayout::count() const
{
    return (item == nullptr) ? 0 : 1;
}

QLayoutItem *RatioLayout::itemAt(int index) const
{
    return (index == 0) ? item : nullptr;
}

QLayoutItem *RatioLayout::takeAt(int index)
{
    return (index == 0) ? item : nullptr;
}

static
QRect alignRect(QRect need, QRect target, Qt::Alignment flag)
{
    need.moveTopLeft(target.topLeft());
    if (flag & Qt::AlignRight)
        need.moveRight(target.right());
    if (flag & Qt::AlignBottom)
        need.moveBottom(target.bottom());
    if (flag & Qt::AlignHCenter)
        need.moveCenter(QPoint(target.center().x(), need.center().y()));
    if (flag & Qt::AlignVCenter)
        need.moveCenter(QPoint(need.center().x(), target.center().y()));
    return need;
}

void RatioLayout::setGeometry(const QRect &rect)
{
    QLayout::setGeometry(rect);

    if (item != nullptr) {
        qreal ratio = qreal(item->minimumSize().height()) / item->minimumSize().width();
        if ( qreal(rect.height()) / rect.width() > ratio)
            item->setGeometry(alignRect(QRect(rect.topLeft(), QSize(rect.width(), rect.width()*ratio)), rect, alignment()));
        else
            item->setGeometry(alignRect(QRect(rect.topLeft(), QSize(rect.height()/ratio, rect.height())), rect, alignment()));
    }
}

QSize RatioLayout::minimumSize() const
{
    return item == nullptr ? QSize() : item->minimumSize();
}

QSize RatioLayout::maximumSize() const
{
    return item == nullptr ? QSize() : item->maximumSize();
}

QSize RatioLayout::sizeHint() const
{
    return item == nullptr ? QSize() : item->sizeHint();
}

Qt::Orientations RatioLayout::expandingDirections() const
{
    return item == nullptr ? QFlags<Qt::Orientation>() : item->expandingDirections();
}
