#include "rectslider.h"
#include <QtEvents>
#include <QGestureEvent>
#include <QPainter>
#include <QPainterPath>
#include <QDebug>
#include <QtGlobal>
#include <QTimer>
#include <QPaintEngine>
#include <QTapAndHoldGesture>
#include <qmath.h>

double RectSlider::mouseWheelForceMultiply = 1.0;

RectSlider::RectSlider()
{
    centralWidget = nullptr;
    centralWidgetPos = nullptr;

    holdTimer = new QTimer();
    //holdTimer->setInterval(timerHoldInterval);
    connect(holdTimer, SIGNAL(timeout()), this, SLOT(mouseHolded()));
    font.setFamily("arial");
    setMinimumSize(90,90);
    setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding);

    QTapAndHoldGesture::setTimeout(80);
    grabGesture(Qt::TapAndHoldGesture);
}

bool RectSlider::gestureEvent(QGestureEvent *event)
{
    if (event && event->gesture(Qt::TapAndHoldGesture) &&
        !holdingMouseRect && !holding && enableControl)
    {
        if (enableMouseRectMoveChange)
        {
            QPoint mid(width()*0.5, height()*0.5);
            QPoint mouse = mapFromGlobal(QCursor::pos());
            QPoint point = mid - mouse;
            int radius = width()*(0.5 - 0.15 - 0.0525)*1.9;
            double trueLength = std::sqrt(std::pow(point.x(), 2) + std::pow(point.y(), 2));
            //if (trueLength < radius)
            {
                holdingMouseRect = true;
                event->ignore();
            }
        } else if (!holdingMouseRect)
        {
            holding = true;

            holdTimer->start(timerHoldInterval);
        }
    }

    return true;
}

void RectSlider::paintEvent(QPaintEvent * /* event */)
{
    QPainter painter(this);

    painter.setRenderHint(QPainter::Antialiasing, true);
    //painter.setRenderHint(QPainter::SmoothPixmapTransform, true);
    painter.setPen(QColor(Qt::black));
    painter.setBrush(QBrush(Qt::white));
    painter.drawRect(QRect(0, 0, width() - 1, height() - 1));
    painter.setBrush(QBrush());

    int minsz = qMin(width(), width());
    double fontsize = FONT_SCALE*0.2;
    //font.setPixelSize(fontsize*10);
    font.setPointSizeF(FONT_SCALE);
    painter.setFont(font);

    double xBotRect = 0;
    double yBotRect = 0;
    double xMid = width()*0.5;
    double yMid = yBotRect + 0.5*(height() - yBotRect);


    double circleBound = 0.15*width();
    QRectF rectangle(circleBound, yBotRect + circleBound, width() - 2*circleBound, height() - 2*circleBound - yBotRect);
    double offsetRect = circleBound*0.7;
    QRectF rectangleMini(circleBound + offsetRect*0.5, yBotRect + circleBound + offsetRect*0.5,
                         width() - 2*circleBound - offsetRect, height() - 2*circleBound - yBotRect - offsetRect);

    int startAngle = arcRightAngle * 16;
    int spanAngle = -(arcLeftAngle*(value/maxValue)) * 16;

    QPen pen = QPen(QColor(Qt::black));
    pen.setWidthF(1.3);
    painter.setPen(pen);
    painter.drawArc(rectangleMini, startAngle, arcLeftAngle*16 - startAngle);

    painter.setBrush(QBrush(QColor(Qt::red)));
    painter.drawLine(xBotRect, yBotRect, width(), yBotRect);
    pen = QPen(QColor(Qt::red));
    pen.setWidthF(4.);
    painter.setPen(pen);

    int spaceRectAngle = 13 * 16;
    int rectWidhAngle = 0.5 * 16;

    for (int angle = startAngle - rectWidhAngle*1.5; spanAngle <= 0; angle += -rectWidhAngle - spaceRectAngle, spanAngle += rectWidhAngle + spaceRectAngle)
    {
        painter.drawArc(rectangle, angle, -rectWidhAngle);
    }



    painter.setPen(QColor(27, 29, 194));
    if (dimensionNewLine)
    {
        QString str;
        if (numbersAfterPoint == 0)
            str = QString::number(value);
        else
            str = QString::number(value, 'f', numbersAfterPoint);
        painter.drawText(xMid - 0.5*QFontMetrics(font).width(str),
                         yMid + 1*fontsize - 4., str);
        painter.drawText(xMid - 0.5*QFontMetrics(font).width(dimension),
                         yMid + 8*fontsize - 4., dimension);
    }
    else
    {
        QString str;
        if (numbersAfterPoint == 0)
            str = QString::number((int)value) + dimension;
        else
            str = QString::number(value, 'f', numbersAfterPoint) + dimension;
        painter.drawText(xMid - 0.5*QFontMetrics(font).width(str),
                         yMid + 3*fontsize, str);
    }

    if (!enableMouseRectMoveChange)
    {
        //+
        pen = QPen(QColor(enableControl ? Qt::darkGreen : Qt::gray));
        pen.setWidthF(1.);
        painter.setPen(pen);
        painter.drawLine(0.1*width(), 0.85*height(), 0.2*width(), 0.85*height());
        painter.drawLine(0.15*width(), 0.8*height(), 0.15*width(), 0.9*height());

        //-
        pen = QPen(QColor(enableControl ? Qt::red : Qt::gray));
        pen.setWidthF(2.);
        painter.setPen(pen);
        painter.drawLine(0.9*width(), 0.85*height(), 0.8*width(), 0.85*height());
    }

    if (holding && enableControl)
    {
        painter.setOpacity(0.5);
        QLinearGradient gradient(0, 0, width(), height()); // diagonal gradient from top-left to bottom-right

        gradient.setStart(0, 0.5*height());
        gradient.setFinalStop(width(), 0.5*height());
        QPoint p = mapFromGlobal(QCursor::pos());
        if (p.x() < width()/2) //+
        {
            gradient.setColorAt(0, QColor(76, 222, 81));
            gradient.setColorAt(1, Qt::white);
        }
        else //-
        {
            gradient.setColorAt(1, QColor(222, 76, 81));
            gradient.setColorAt(0, Qt::white);
        }
        painter.fillRect(0, 0, width(), height(), gradient);
    }

    if (centralWidget && centralWidget->isVisible())
        centralWidget->move(width()*centralWidgetPos->x(), height()*centralWidgetPos->y());
}

void RectSlider::setWheelPower(double pow)
{
    mouseWheelForceMultiply = pow;
}

void RectSlider::wheelEvent(QWheelEvent *event)
{
    if (!enableControl)
    {
        QWidget::wheelEvent(event);
        return;
    }
    bool ctrlBtn = event->modifiers().testFlag(Qt::ControlModifier);

#if (QT_VERSION >= QT_VERSION_CHECK(5, 0, 0))
    int numDegrees = event->angleDelta().y() / 8;
#else
    int numDegrees = event->delta() / 8;
#endif
    if (ctrlBtn) numDegrees *= 2;
    if (numDegrees != 0)
    {
        double force = numDegrees / 15.0;
        double f = mouseWheelForceMultiply * force;
        if (reduceWheelPowerBy)
            f /= reduceWheelPowerBy;
        addValue(-f);
    }

    event->accept();
    //QWidget::wheelEvent(event);
}

bool RectSlider::event(QEvent *event)
{
    if (event->type() == QEvent::Gesture)
    {
        auto mPos = this->mapFromGlobal(QCursor::pos());

        if (centralWidget && centralWidget->isVisible() &&
                mPos.x() >= centralWidget->x() && mPos.x() <= centralWidget->x() + centralWidget->width() &&
                mPos.y() >= centralWidget->y() && mPos.y() <= centralWidget->y() + centralWidget->height())
            return QWidget::event(event);

        lastMousePos = mPos;
        return gestureEvent(dynamic_cast<QGestureEvent*>(event));
    }
    if (holdingMouseRect && event->type() == QEvent::MouseButtonPress)
    {
        double diapValue = maxValue - minValue;
        double valuePart = diapValue/holdingMouseRectParts;
        prevNumParts = (int)(value/valuePart);
        holdingMouseRectwaitingZero = (valuePart == 0);
    }
    if (holdingMouseRect && event->type() == QEvent::MouseMove)
    {
        auto mouse = dynamic_cast<const QMouseEvent*>( event );
        QPoint mid(width()*0.5, height()*0.5);
        QPoint mPos = mouse->pos();

        double diapValue = maxValue - minValue;
        double valuePart = diapValue/holdingMouseRectParts;
        //double numPartsDouble = value/valuePart;
        //int numParts = (int)(numPartsDouble);
        double x1 = mid.x(),
               y1 = mid.y();
        double x2 = mPos.x(),
               y2 = mPos.y();
        double A = qAtan2(y1 - y2, x1 - x2) / M_PI * 180;
        A = (A < 0) ? A + 360 : A;

        A = A/360.0;
        A = A - 0.25; //rotating to right for 90.0
        if (A < 0)
            A = 1.0 + A;

        double leftAngle = ((360.0 - arcLeftAngle)/360.0)*0.5;
        double rightAngle = (-arcRightAngle - 270.0)/360.0;
        if (A < 90.0/360.0)
            A = A - leftAngle/2;
        else if (A > 270.0/360.0)
            A = A + rightAngle;

        if (A < 0.0)
        {
            A = 0.0;
        }
        if (A > 1.0)
        {
            A = 1.0;
        }

        double ADiff = A - holdingMouseRectPrevA;
        if (ADiff > 0.8)
            ADiff = -(1.0 - A + holdingMouseRectPrevA);
        if (ADiff < -0.8)
            ADiff = 1.0 - holdingMouseRectPrevA + A;
        double valueDiff = ADiff*valuePart;

        //qDebug()<<"****?"<<"("<<rightAngle<<")"<<holdingMouseRectPrevA<<A<<"diff"<<ADiff;

//        if ((A >= 0.01 && A <= 0.3) &&
//            (holdingMouseRectPrevA <= 1.0 && holdingMouseRectPrevA > 0.8))
//        {
//            if (!holdingMouseRectwaitingZero)
//                numParts++;
//            qDebug()<<"****?"<<numParts;
//            A = 0.01;
//            holdingMouseRectwaitingZero = false;
//        }
//        else if ((A <= 1.0 && A > 0.4) &&
//                 (holdingMouseRectPrevA >= 0.01 && holdingMouseRectPrevA <= 0.3))
//        {
//            numParts--;
//            qDebug()<<"****?"<<numParts;
//            A = 0.9;
//            if (numParts < 0)
//                holdingMouseRectwaitingZero = true;
//        }

        //value = numParts*valuePart + valuePart*A;
        //value = minValue + diapValue*A;
        value += valueDiff;
        if (value > maxValue)
            value = maxValue;
        if (value < minValue/* || holdingMouseRectwaitingZero*/)
            value = minValue;

        holdingMouseRectPrevA = A;

        repaint();
        event->ignore();
        return true;
    }
    if (event->type() == QEvent::MouseButtonRelease)
    {
        auto mouse = dynamic_cast<const QMouseEvent*>( event );
        lastMousePos = mouse->pos();
        if (holding)
            mouseHoldingEnd();
        else if (holdingMouseRect)
            mouseHoldingRectMoveEnd();
        else
            mouseClicked(lastMousePos);
        event->ignore();
        return true;
    }
    return QWidget::event(event);
}

void RectSlider::addValue(double v)
{
    setValue(value + v);
}

void RectSlider::setValue(double val)
{
    prevValue = value = qBound(minValue, val, maxValue);
    emit valueChanged(value);
    emit valueChanged((int)value);
    emit valueChanged();
    repaint();
}
void RectSlider::setValue_man(double val)
{
    //value = qBound(minValue, val, maxValue);
    value = val;
    repaint();
}
double RectSlider::Value()
{
    return value;
}

void RectSlider::mouseHoldingEnd()
{
    holdTimer->stop();
    secondSpeedActive = holding = false;
    repaint();
    prevValue = value;
    emit valueChanged(value);
    emit valueChanged((int)value);
    emit valueChanged();
}

void RectSlider::mouseHoldingRectMoveEnd()
{
    holdingMouseRect = false;
    repaint();
    prevValue = value;
    emit valueChanged(value);
    emit valueChanged((int)value);
    emit valueChanged();
}

void RectSlider::mouseClicked(QPoint p)
{
    if (!enableControl || enableMouseRectMoveChange)
        return;

    setValue(value + ((p.x() < width()/2) ? clickInc : -clickInc));
}
void RectSlider::mouseHolded()
{
    if (!enableControl)
        return;

    QPoint p = mapFromGlobal(QCursor::pos());
    value += ((p.x() < width()/2) ? holdInc : -holdInc);
    value = qBound(minValue, value, maxValue);
    repaint();

    if (secondSpeedActive || !enableSecondSpeed)
        return;

    if (value > prevValue)
    {
        if (value - prevValue >= holdIncSecondSpeedlimit)
            secondSpeedActive = true;
    }
    else if (value < prevValue)
    {
        if (prevValue - value >= holdIncSecondSpeedlimit)
            secondSpeedActive = true;
    }

    if (secondSpeedActive)
    {
        holdTimer->stop();
        holdTimer->start(timerHoldInterval/holdTimerMultiply);
    }
}

RectSlider::~RectSlider()
{
    if (holdTimer->isActive())
        holdTimer->stop();
    delete holdTimer;
}

void RectSlider::SetMaxValue(double max)
{
    maxValue = max;
    if (value > max)
    {
        value = max;
        emit valueChanged(value);
        emit valueChanged((int)value);
        emit valueChanged();
    }
    repaint();
}
void RectSlider::SetDimension(QString dim, bool newLine)
{
    dimension = dim;
    dimensionNewLine = newLine;
}
void RectSlider::SetHoldIncreaseSpeed(double speed)
{
    holdInc = speed;
}
void RectSlider::SetClickIncrease(double add)
{
    clickInc = add;
}
void RectSlider::SetNumberAfterPoint(int num)
{
    numbersAfterPoint = num;
}
void RectSlider::SetMinValue(double min)
{
    minValue = min;
}
void RectSlider::SetTickNSeconds(int nsec)
{
    timerHoldInterval = nsec;
}
void RectSlider::EnableSecondSpeed(bool enable)
{
    enableSecondSpeed = enable;
}
double RectSlider::GetMaximum()
{
    return maxValue;
}

bool RectSlider::hasHeightForWidth() const {
    return true;
}

int RectSlider::minRectWidth() const
{
    return 90;
}
