#ifndef RECTSLIDER_H
#define RECTSLIDER_H

#include <QObject>
#include <QWidget>
#include <QtEvents>
#include <QGestureEvent>
#include <QPainter>
#include <QDebug>
#include <QtGlobal>
#include <QComboBox>
#include <QPushButton>
#include <QPointF>

class RectSlider : public QWidget
{
    Q_OBJECT
public:
    explicit RectSlider();
    ~RectSlider() override;
    double Value();
    void SetDimension(QString dim, bool newLine = false);
    void SetHoldIncreaseSpeed(double speed);
    void SetClickIncrease(double add);
    void SetNumberAfterPoint(int num = 0);
    void SetTickNSeconds(int nsec);
    void EnableSecondSpeed(bool enable);
    double GetMaximum();
    int minRectWidth() const;
    QString label = "";
    bool enableControl = true;
    bool enableMouseRectMoveChange = false;

    QPointF *centralWidgetPos;
    QWidget *centralWidget;
    int arcLeftAngle = 360;
    int arcRightAngle = -270;
    int reduceWheelPowerBy = 0;
    static void setWheelPower(double pow);
    int FONT_SCALE = 10*1.5;

protected:
    bool event(QEvent *) override;
    void paintEvent(QPaintEvent *event) override;
    void wheelEvent(QWheelEvent *event) override;
    bool gestureEvent(QGestureEvent *event);

    bool hasHeightForWidth() const override;

private slots:
    void mouseHolded();

private:
    static double mouseWheelForceMultiply;
    void mouseClicked(QPoint p);
    void mouseHoldingEnd();
    void mouseHoldingRectMoveEnd();
    double value = 0.0;
    bool holding = false;
    bool holdingMouseRect = false;
    int holdingMouseRectParts = 10;
    int prevNumParts = 0;
    bool holdingMouseRectwaitingZero = false;
    double holdingMouseRectPrevA = 0.0;
    QPoint lastMousePos;
    QFont font;
    QTimer *holdTimer;
    QString dimension = "%";
    double maxValue = 100.0;
    double minValue = 0.0;
    int numbersAfterPoint = 0;
    bool dimensionNewLine = false;

    bool enableSecondSpeed = false;
    double holdInc = 0.1;
    int holdTimerMultiply = 2;
    double holdIncSecondSpeedlimit = 1.2;
    bool secondSpeedActive = false;
    int timerHoldInterval = 25;
    double clickInc = 1;



    double prevValue = 0.0;

signals:
    void valueChanged(double);
    void valueChanged(int);
    void valueChanged();
    void CalcMinimumSizeSignal(QSize&);

public slots:
    void addValue(double);
    void setValue(double);
    void setValue_man(double);
    void SetMinValue(double min = 0.0);
    void SetMaxValue(double max = 100.0);
};

#endif // RECTSLIDER_H
