#include "abstractinoutlines.h"

AbstractInOutLines::AbstractInOutLines(uint numOut, uint numIn) : countOut(numOut), countIn(numIn)
{
    backgroundColor = QColor(226, 226, 245);//212

    absPos = new QPoint();
    if (countOut > 0)
    {
        connectOutId = new uint[countOut];
        for (uint i=0; i<countOut; i++)
        {
            ptsOut.push_back(new QPoint());
            absOut.push_back(nullptr);
            connectOutId[i] = 0;
            cornersOut.push_back(new QVector<QPoint*>());
        }
    }
    else
    {
        connectOutId = nullptr;
    }

    for (uint i=0; i<countIn; i++)
    {
        ptsIn.push_back(new QPoint());
        absIn.push_back(nullptr);
    }
}

void AbstractInOutLines::connectAbs(int mineOutId, AbstractInOutLines *other, int otherInId)
{
    absOut[mineOutId] = other;
    connectOutId[mineOutId] = otherInId;
    other->absIn[otherInId] = this;
}

void AbstractInOutLines::setPtsInXY(int id, int x, int y)
{
    ptsIn[id]->setX(x);
    ptsIn[id]->setY(y);
}

void AbstractInOutLines::setPtsOutXY(int id, int x, int y)
{
    ptsOut[id]->setX(x);
    ptsOut[id]->setY(y);
}

void AbstractInOutLines::setPosXY(int x, int y)
{
    absPos->setX(x);
    absPos->setY(y);
    recalcPoints();
}

void AbstractInOutLines::addCorner(int outId, int x, int y)
{
    cornersOut[outId]->append(new QPoint(x, y));
}

void AbstractInOutLines::setShadow(QWidget *wid, int w, const QColor &color)
{
    auto *effect = new QGraphicsDropShadowEffect;
    effect->setBlurRadius(w);
    effect->setXOffset(w);
    effect->setYOffset(w);
    effect->setColor(color);
    wid->setGraphicsEffect(effect);
}

AbstractInOutLines::~AbstractInOutLines()
{
    for (auto &pts : ptsOut) {
            delete pts;
    }
    ptsOut.clear();
    for (auto &pts : ptsIn) {
            delete pts;
    }
    ptsIn.clear();

    for (auto &vec : cornersOut) {
        for (auto &point : *vec) {
            delete point;
        }
        delete vec;
    }
    cornersOut.clear();

    absOut.clear();
    absIn.clear();
    delete absPos;
    delete[] connectOutId;
}
