#include "schinputgains.h"

#include <QGraphicsDropShadowEffect>

schInputGains::schInputGains(QWidget *parent) : QWidget(parent), AbstractInOutLines(3, 3)
{
    triangleW = 0;
    triangleH = 0;
    headText = new QLabel(tr("Inp. Gain"), this);
    cbDFL = new QComboBox(this);
    cbLF = new QComboBox(this);
    cbINT = new QComboBox(this);
    initPositions(0, 0);

    auto strList = new QStringList();
    strList->append("x1");
    strList->append("x2");
    strList->append("x4");
    strList->append("x5");
    strList->append("x10");
    strList->append("x20");
    strList->append("x25");
    strList->append("x50");
    strList->append("x100");
    cbDFL->addItems(*strList);
    cbLF->addItems(*strList);
    cbINT->addItem("x1");
    cbINT->addItem("x2");
    cbINT->addItem("x5");
    cbINT->addItem("x10");

    QFont font;
    font.setPointSizeF(6);
    font.setBold(true);
    font.setFamily(cbDFL->font().family());
    cbDFL->setFont(font);
    cbLF->setFont(font);
    cbINT->setFont(font);

    cbDFL->show();
    cbLF->show();
    cbINT->show();

    cbINT->setFixedSize(cbDFL->size());
}

void schInputGains::recalcPoints()
{
    initPositions(width(), height());

    setPtsInXY(0, boxX, triangleY[0] + triangleCenterY);
    setPtsInXY(1, boxX, triangleY[1] + triangleCenterY);
    setPtsInXY(2, boxX, triangleY[2] + triangleCenterY);

    setPtsOutXY(0, boxX + boxW, triangleY[0] + triangleCenterY);
    setPtsOutXY(1, boxX + boxW, triangleY[1] + triangleCenterY);
    setPtsOutXY(2, boxX + boxW, triangleY[2] + triangleCenterY);
}

void schInputGains::initPositions(int w, int h)
{
    boxX = 0;
    boxY = h*0.15;
    boxW = w;
    boxH = h - boxY;
    triangleCenterY = triangleH*0.5;
    triangleY[0] = boxY + boxH*0.05;
    triangleY[1] = boxY + boxH*0.35;
    triangleY[2] = boxY + boxH*0.65;

    double scale = width()/100.;
    if (scale > maxFontScale)
        scale = maxFontScale;
    headText->setFont(QFont(headText->font().family(), 10*scale));
    headText->setScaledContents(true);
    headText->adjustSize();

    QFont font;
    font.setPointSizeF((6 > 8*scale) ? 6:8*scale);
    font.setBold(true);
    font.setFamily(cbDFL->font().family());
    cbDFL->setFont(font); cbDFL->adjustSize();
    cbLF->setFont(font); cbLF->adjustSize();
    cbINT->setFont(font); cbINT->adjustSize();
    cbINT->setFixedSize(cbDFL->size());
}

bool schInputGains::event(QEvent *e)
{
    if (QEventResizedMoved(e))
        recalcPoints();

    return QWidget::event(e);
}

void schInputGains::paintEvent(QPaintEvent *)
{
    QPainter painter(this);

    int w = width();
    int h = height();

    initPositions(w, h);
    painter.setRenderHint(QPainter::Antialiasing, true);

    //Box
    painter.setPen(QPen(QBrush(outterLineColor), outterLineWidth));
    painter.setBrush(QBrush(backgroundColor));
    painter.drawRect(boxX, boxY, boxW, boxH);

    painter.setPen(QColor(Qt::black));

    //Triangles
    paint_triangle(cbDFL, &painter, boxX + boxW*0.5 - triangleW*0.5, triangleY[0], triangleW);
    paint_triangle(cbLF,  &painter, boxX + boxW*0.5 - triangleW*0.5, triangleY[1], triangleW);
    paint_triangle(cbINT, &painter, boxX + boxW*0.5 - triangleW*0.5, triangleY[2], triangleW);

    //Inp Gain text
    headText->move(boxX, boxY - headText->height());
}

inline void schInputGains::paint_triangle(QComboBox *cb, QPainter *painter, int x, int y, int w)
{
    //connect lines
    painter->drawLine(boxX, y + triangleCenterY, x, y + triangleCenterY);
    painter->drawLine(x + w, y + triangleCenterY, boxX + boxW, y + triangleCenterY);

    painter->drawLine(x, y, x + w, y + triangleCenterY);
    painter->drawLine(x, y + triangleH, x + w, y + triangleCenterY);
    painter->drawLine(x, y, x, y + triangleH);

    cb->move(x, y + triangleCenterY - cb->height()*0.5);
}
