#include "schoscilator.h"

#include <QGraphicsDropShadowEffect>

schOscilator::schOscilator(QWidget *parent) : QWidget(parent), AbstractInOutLines(1, 1)
{
    freqText = new QLabel(tr("Freq"), this);
    freqText->setScaledContents(true);
    amplText = new QLabel(tr("Amp"), this);
    amplText->setScaledContents(true);
    freqCurve = QPolygonF(1000);
    freqCurveDraw = QPolygonF(1000);
    for (int i=0; i<freqCurve.size(); i++)
    {
        freqCurve[i].setX(i);
    }

    QSettings settings("Param/analog.conf", QSettings::IniFormat);
    double a1 = settings_double(settings, "dac/vdrive_min", 0.);
    double b1 = settings_double(settings, "dac/vdrive_max", 10.);

    font.setFamily("arial");
    edFreq = new Edit(0, 1000, 3);
    edAmp = new Edit(a1, b1, 3);
    edFreq->setParent(this);
    edAmp->setParent(this);
    edFreq->show();
    edAmp->show();
    edFreq->setFixedWidth(80);
    edAmp->setFixedWidth(80);
    btnFreqStartScan = new TwoStateButton(":start.png",":stop.png");
    btnFreqStartScan->setToolTip(tr("Frequency scan start"));
    btnAutoFreqScan = new QPushButton(makeHugeIcon(":AutoStart.png"), "");
    btnAutoFreqScan->setToolTip(tr("Auto freq scan"));
    cbChannel = new QComboBox();
    edFreqScanMin = new Edit(0, 20000, 1);
    edFreqScanMin->setImmediateMode();
    edFreqScanMin->setValue(50);
    edFreqScanMin->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Fixed);

    edFreqScanMax = new Edit(0, 20000, 1);
    edFreqScanMax->setImmediateMode();
    edFreqScanMax->setValue(500);
    edFreqScanMax->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Fixed);
    chbSetOn = new QCheckBox(tr("set on"));
    chbSetOn->setToolTip(tr("Set excitation frequency on maximum in scan"));
    chbSetOn->setChecked(true);

    chbSecHarmonic = new QCheckBox(tr("2nd Harmonic"));
    chbSecHarmonic->setToolTip(tr("Detects 2nd harmonic of DFL and LFL signals"));

    mainBLay = new QVBoxLayout(this);
    mainBLay->addWidget(chbSecHarmonic);
    mainBLay->addWidget(chbSetOn);
    mainHLay = new QHBoxLayout();
    mainBLay->addLayout(mainHLay);
    subVLays = new QVBoxLayout*[4];
    for (int i=0; i<4; i++)
    {
        subVLays[i] = new QVBoxLayout();
        mainHLay->addLayout(subVLays[i]);
    }
    auto *layFreqBtns = new QHBoxLayout();
    subVLays[0]->addLayout(layFreqBtns);
    layFreqBtns->addWidget(btnAutoFreqScan);
    layFreqBtns->addWidget(btnFreqStartScan);

    auto *layChnl = new QHBoxLayout();
    subVLays[0]->addLayout(layChnl);
    channelLabel = new QLabel(tr("Chnl"));
    channelLabel->setScaledContents(true);
    layChnl->addWidget(channelLabel);
    layChnl->addWidget(cbChannel);

    fromLabel = new QLabel(tr("From"));
    fromLabel->setScaledContents(true);
    auto fromLay = new QHBoxLayout();
    fromLay->addWidget(fromLabel);
    fromLay->addWidget(edFreqScanMin);
    subVLays[1]->addLayout(fromLay);

    toLabel = new QLabel(tr("To"));
    toLabel->setScaledContents(true);
    auto toLay = new QHBoxLayout();
    toLay->addWidget(toLabel);
    toLay->addWidget(edFreqScanMax);
    subVLays[1]->addLayout(toLay);

    khzLabel = new QLabel(tr("kHz"));
    khzLabel->setScaledContents(true);
    subVLays[2]->addWidget(khzLabel, 0, Qt::AlignCenter);

    headText = new QLabel(tr("Lock In Ampl."), this);
    headText->setScaledContents(true);
    initPositions(0, 0);
}

void schOscilator::recalcPoints()
{
    initPositions(width(), height());

    setPtsInXY(0, boxX, boxY + 20);
    setPtsOutXY(0, width()*0.5, boxY);
}

void schOscilator::initPositions(int w, int h)
{
    boxX = 0;
    boxY = h*0.1;
    boxW = w;
    boxH = h - boxY;
    mainBLay->setGeometry(QRect(0, boxY + boxH*0.4, boxW, h - (boxY + boxH*0.4)));

    double scale = w/300.;
    if (scale > maxFontScale)
        scale = maxFontScale;
    QFont font = QFont(headText->font().family(), 10*scale);
    headText->setFont(font);
    headText->adjustSize();
    freqText->setFont(font);
    freqText->adjustSize();
    amplText->setFont(font);
    amplText->adjustSize();
    channelLabel->setFont(font);
    channelLabel->adjustSize();
    fromLabel->setFont(font);
    fromLabel->adjustSize();
    toLabel->setFont(font);
    toLabel->adjustSize();
    khzLabel->setFont(font);
    khzLabel->adjustSize();
    chbSetOn->setFont(font);
    chbSetOn->adjustSize();
    chbSecHarmonic->setFont(font);
    chbSecHarmonic->adjustSize();

    fromLabel->update();

    mainBLay->update();
}

bool schOscilator::event(QEvent *e)
{
    if (QEventResizedMoved(e))
        recalcPoints();

    return QWidget::event(e);
}

void schOscilator::paintEvent(QPaintEvent *)
{
    QPainter painter(this);

    int w = width();
    int h = height();

    initPositions(w, h);
    painter.setRenderHint(QPainter::Antialiasing, true);

    //Box
    painter.setPen(QPen(QBrush(outterLineColor), outterLineWidth));
    painter.setBrush(QBrush(backgroundColor));
    painter.drawRect(boxX, boxY, boxW, boxH);
    painter.setBrush(QBrush());

    painter.setPen(QColor(Qt::black));

    //Oscelator text
    headText->move(boxX, boxY - headText->height());

    //Freq and Amp
    int maxLabelWidth = freqText->width() < amplText->width() ? amplText->width():freqText->width();
    edFreq->move(5 + maxLabelWidth + 1, boxY + boxH*0.1);
    edAmp->move(5 + maxLabelWidth + 1, boxY + boxH*0.3);
    freqText->move(5, boxY + boxH*0.1);
    amplText->move(5, boxY + boxH*0.3);

    //Fres Graph
    int xLeftFC = 5 + maxLabelWidth + 5 + edFreq->width();
    if (xLeftFC < 5 + chbSecHarmonic->width() + 1)
        xLeftFC = 5 + chbSecHarmonic->width() + 1;
    drawFreqCurveGraph(&painter, xLeftFC);
}
void schOscilator::SetFreqCurveDataMaximum(float yMax)
{
    fresX = yMax;
}
void schOscilator::AddFreqCurveData(int x, qreal y)
{
    freqCurve[x].setY(y);
}
void schOscilator::drawFreqCurveGraph(QPainter *painter, int xLeftFC)
{
    int heightFC = btnAutoFreqScan->pos().y() - boxY - 20 - 3;
    int yUpFC = boxY + 15 + 3;
    int yDownFC = yUpFC + heightFC - 3;
    int xRightFC = width() - 15;
    int widthFC = xRightFC - xLeftFC;

    painter->setPen(QColor(Qt::black));
    painter->drawRect(xLeftFC, yUpFC - 3, widthFC, heightFC + 3);

    double yMax = 0.;
    for (int i=0; i<freqCurve.size(); i++)
        if (yMax < freqCurve[i].y())
            yMax = freqCurve[i].y();

    if (yMax == 0.)
        return;

    for (int i=0; i<freqCurve.size(); i++)
    {
        freqCurveDraw[i].setX(xLeftFC + i*(widthFC/1000.));
        freqCurveDraw[i].setY(yDownFC - (heightFC/yMax)*freqCurve[i].y());
    }
    painter->setPen(QColor(Qt::red));
    painter->drawPolyline(freqCurveDraw);

    //On first pass the curve drawings with artefacts
    static bool secondPass = false;
    if (secondPass)
    {
        update();
    }
    secondPass = !secondPass;
}
