#include "schpsd.h"

schPSD::schPSD(QWidget *parent) : QWidget(parent), AbstractInOutLines(4, 0)
{
    laserImgOn = new QImage(":/SPMTuning_window/laser_onPsd.png");
    laserImgOff = new QImage(":/SPMTuning_window/laser_off.png");
    laserImgHovered = new QImage(":/SPMTuning_window/laser_onPsd.png");
    hovered = false;
    spmLaserOff = false;
    headText = new QLabel(tr("PSD"), this);
    setMouseTracking(true);
    initPositions(0, 0);
}

bool schPSD::event(QEvent *e)
{
    if (QEventResizedMoved(e))
        recalcPoints();
    if (e->type() == QEvent::Leave)
    {
        hovered = false;
        update();
    }
    else if (e->type() == QEvent::MouseMove)
    {
        auto me = dynamic_cast<QMouseEvent *>(e);

        QPoint point = me->pos() - ellLCenter;
        if (point.manhattanLength() <= ellBRadius + 2)
        {
            hovered = true;
        }
        else
            hovered = false;
        update();
    }
    if (e->type() == QEvent::MouseButtonRelease && hovered)
    {
        spmLaserOff = !spmLaserOff;
        update();

        emit ChangedValue(spmLaserOff);
    }

    return QWidget::event(e);
}
void schPSD::recalcPoints()
{
    initPositions(width(), height());

    int boxQuarter = boxW*0.25;
    setPtsOutXY(0, boxX + boxQuarter, boxY);
    setPtsOutXY(1, boxX + boxW, boxY + boxQuarter);
    setPtsOutXY(2, boxX + boxW, ellBCenter.y() + boxQuarter);
    setPtsOutXY(3, boxX + boxQuarter, boxY + boxW);
}

void schPSD::initPositions(int w, int h)
{
    boxX = w*0.5;
    boxY = h*0.2;
    boxW = w*0.4;
    ellBCenter.setX(boxX + boxW*0.5);
    ellBCenter.setY(boxY + boxW*0.5);
    ellLCenter.setX(boxX*0.5);
    ellLCenter.setY(ellBCenter.y());
    ellBRadius = boxW*0.25;

    double scale = width()/150.;
    if (scale > maxFontScale)
        scale = maxFontScale;
    headText->setFont(QFont(headText->font().family(), 10*scale));
    headText->setScaledContents(true);
    headText->adjustSize();
}

void schPSD::paintEvent(QPaintEvent *)
{
    QPainter painter(this);

    int w = width();
    int h = height();

    painter.setRenderHint(QPainter::Antialiasing, true);
    //painter.setRenderHint(QPainter::SmoothPixmapTransform, true);

    initPositions(w, h);

    //Box
    painter.setPen(QPen(QBrush(outterLineColor), outterLineWidth));
    painter.setBrush(QBrush(backgroundColor));
    painter.drawRect(boxX, boxY, boxW, boxW);
    painter.setBrush(QBrush());

    painter.setPen(QColor(Qt::black));

    //ellipse in box
    if (!spmLaserOff)
        painter.setBrush(QBrush(Qt::red));
    painter.drawEllipse(ellBCenter, ellBRadius, ellBRadius);
    painter.setBrush(QBrush());

    //lines in box
    painter.drawLine(ellBCenter.x(), boxY, ellBCenter.x(), boxY + boxW);
    painter.drawLine(boxX, ellBCenter.y(), boxX + boxW, ellBCenter.y());

    //ellipse left
    if (hovered)
    {
        painter.setOpacity(0.5);
        painter.setBrush(QBrush(Qt::red));
    }
    else
        painter.setBrush(QBrush(backgroundColor));
    painter.drawEllipse(ellLCenter, ellBRadius, ellBRadius);
    painter.setBrush(QBrush());

    painter.setOpacity(1);

    QRect imgRect(ellLCenter - QPoint(ellBRadius, ellBRadius), QSize(ellBRadius*2, ellBRadius*2));
    if (hovered)
        painter.drawImage(imgRect, *laserImgHovered);
    else if (!spmLaserOff)
        painter.drawImage(imgRect, *laserImgOn);
    else
        painter.drawImage(imgRect, *laserImgOff);

    //line from ellipse left and line to center box
    int centerCantelX = boxX*0.8;
    int centerCantelY = boxY + boxW + h*0.2;
    painter.drawLine(ellLCenter.x(), ellLCenter.y(), centerCantelX, centerCantelY);
    painter.drawLine(centerCantelX, centerCantelY, ellBCenter.x(), ellBCenter.y());

    //canteliver
    int leftCantelX = centerCantelX - w*0.1;
    int downCantelX = leftCantelX + w*0.05;
    int downCantelY = centerCantelY + h*0.05;
    painter.drawLine(leftCantelX, centerCantelY, leftCantelX + w*0.3, centerCantelY);
    painter.drawLine(centerCantelX, centerCantelY, downCantelX, downCantelY);
    painter.drawLine(downCantelX, downCantelY, leftCantelX, centerCantelY);

    //PSO text
    headText->move(boxX - headText->width()*0.5, boxY - headText->height());
}
