#include "schzpiezo.h"

schZPiezo::schZPiezo(QWidget *parent) : QWidget(parent), AbstractInOutLines (4, 1)
{
    //piezoText = new QLabel(tr("   Z\nPiezo"), this);
    XText = new QLabel(tr("X"), this);
    XText->setScaledContents(true);
    YText = new QLabel(tr("Y"), this);
    YText->setScaledContents(true);
    ZText = new QLabel(tr("Z"), this);
    ZText->setScaledContents(true);

    xColor = QColor(244, 122, 122, 75);
    yColor = QColor(112, 192, 62, 75);
    zColor = QColor(55, 64, 154, 75);
    tableColor = QColor(25, 47, 130, 75);

    initPositions(0, 0);
}

void schZPiezo::initPositions(int w, int h)
{
    box1X = 0;
    box1Y = 0;
    box1W = w;
    box1H = h*0.1;
    box2X = w*0.1;
    box2Y = box1H;
    box2W = w - 2*box2X;
    box2H = h - box2Y;

    double scale = w/100.;
    if (scale > maxFontScale)
        scale = maxFontScale;
    QFont font = QFont(XText->font().family(), 10*scale);
    XText->setFont(font);
    XText->adjustSize();
    YText->setFont(font);
    YText->adjustSize();
    ZText->setFont(font);
    ZText->adjustSize();
}

void schZPiezo::recalcPoints()
{
    initPositions(width(), height());
    setPtsInXY(0, box2X + box2W,
                  box2Y + box2H*0.8);
    setPtsOutXY(0, box1X, box1H*0.5);
    setPtsOutXY(1, box2X, box2Y + box2H*0.25);
    setPtsOutXY(2, box2X, box2Y + box2H*0.55);
    setPtsOutXY(3, box2X, box2Y + box2H*0.85);
}

bool schZPiezo::event(QEvent *e)
{
    if (QEventResizedMoved(e))
        recalcPoints();

    return QWidget::event(e);
}

void schZPiezo::paintEvent(QPaintEvent *)
{
    QPainter painter(this);

    int w = width();
    int h = height();

    initPositions(w, h);
    painter.setRenderHint(QPainter::Antialiasing, true);

    //Box up
    painter.setPen(QColor(Qt::black));
    painter.setBrush(QBrush(tableColor));
    painter.drawRect(box1X, box1Y, box1W, box1H);

    //box bottom
    painter.setPen(QColor(Qt::black));
    painter.setBrush(QBrush(backgroundColor));
    painter.drawRect(box2X, box2Y, box2W, box2H);
    painter.setBrush(QBrush());

    //Z Piezo text
//    piezoText->move(box2X - piezoText->width()*0.5 + box2W*0.5,
//                    box2Y + box2H*0.5 - piezoText->height());

    //Lines in box2
    int xCenter2 = box2X + box2W*0.5;
    int yCenter2 = box2Y + box2H - box2H*0.4;
    painter.drawLine(xCenter2, box2Y,
                     xCenter2, box2Y + box2H - box2H*0.4);
    painter.drawLine(box2X, yCenter2,
                     box2X + box2W, yCenter2);

    //Fill x/y/z boxs
    painter.setPen(QColor());
    painter.setBrush(xColor);
    painter.drawRect(box2X, box2Y, box2W*0.5, yCenter2 - box2Y);
    painter.setBrush(yColor);
    painter.drawRect(box2X + box2W*0.5, box2Y, box2W*0.5, yCenter2 - box2Y);
    painter.setBrush(zColor);
    painter.drawRect(box2X, yCenter2, box2W, box2Y + box2H - yCenter2);
    painter.setBrush(QBrush());

    //X,Y,Z text
    XText->move(box2X*0.5 + box2W*0.5*0.5 - XText->widthMM()*0.5,
                box2Y + box2H*0.6*0.5 - XText->height()*0.5);
    YText->move(box2X*0.5 + box2W - box2W*0.5*0.5 - YText->widthMM()*0.5,
                box2Y + box2H*0.6*0.5 - YText->height()*0.5);
    ZText->move(box2X*0.5 + box2W*0.5 - ZText->widthMM()*0.5,
                box2Y + box2H - box2H*0.4*0.5 - ZText->height()*0.5);

    //Y point-box
    painter.setPen(QColor(Qt::black));
    painter.setBrush(QBrush(Qt::black));
    const int pointRadius = 2;
    int pointCenterX = box2X*0.5 + box2W - box2W*0.5*0.5 + YText->widthMM();
    painter.drawRect(pointCenterX - pointRadius, box2Y + box2H*0.55 - pointRadius,
                     pointRadius*2, pointRadius*2);
    painter.setBrush(QBrush());

    //Line to Y point-box
    painter.drawLine(ptsOut[2]->x(), ptsOut[2]->y(),
                     pointCenterX, ptsOut[2]->y());
}
