#include "swaptwostatebutton.h"
#if (QT_VERSION >= QT_VERSION_CHECK(5, 0, 0))
#include <QtMath>
#else
#include <qmath.h>
#endif

swapTwoStateButton::swapTwoStateButton(QString offRight, QString offDown, QString on)
{
    offstring[swapDir::dir_right] = offRight;
    offstring[swapDir::dir_down] = offDown;
    onstring = on;
    on_off = false;

    setIcon(makeHugeIcon(offstring[dir]));
    connect(this, SIGNAL(clicked()), SLOT(change_state()));
    setMouseTracking(true);
}
swapTwoStateButton::swapTwoStateButton(const QString & text, QString offRight, QString offDown, QString on) : QPushButton (QIcon(offRight), text)
{
    offstring[swapDir::dir_right] = offRight;
    offstring[swapDir::dir_down] = offDown;
    onstring = on;
    on_off = false;

    setIcon(makeHugeIcon(offstring[dir]));

    connect(this, SIGNAL(clicked()), SLOT(change_state()));
    setMouseTracking(true);
}
bool swapTwoStateButton::event(QEvent *event)
{
    auto me = dynamic_cast<QMouseEvent *>(event);
    if (!on_off && event->type() == QEvent::MouseButtonPress)
    {
        mousePressedPoint = me->pos();
    }
    if (!on_off && event->type() == QEvent::MouseMove)
    {
        if (mousePressedPoint.x() == 0 && mousePressedPoint.y() == 0)
            return QPushButton::event(event);
        int dist = 0;
        int quaX = mousePressedPoint.x() - me->pos().x();
        quaX *= quaX;
        int quaY = mousePressedPoint.y() - me->pos().y();
        quaY *= quaY;
        dist = quaX + quaY;
        dist = qSqrt(dist);
        if (dist > width()*0.25)
        {
            blockClick = true;
            auto r = me->pos() - mousePressedPoint;
            if (r.x() < r.y())
                changeDirection(swapDir::dir_down, true);
            else
                changeDirection(swapDir::dir_right, true);
        }
    }
    if (!on_off && event->type() == QEvent::MouseButtonRelease)
    {
        mousePressedPoint = QPoint(0, 0);
        if (blockClick)
        {
            blockClick = false;
            event->ignore();
            hide();
            show();
            return false;
        }
    }

    return QPushButton::event(event);
}

void swapTwoStateButton::changeDirection(swapDir::Direction dir, bool emitChange)
{
    if (on_off)
        return;

    if (this->dir == dir)
        return;
    this->dir = dir;
    setIcon(makeHugeIcon(offstring[dir]));
    if (emitChange)
        emit swapped(dir);
}

void swapTwoStateButton::change_state()
{
    change_state_man_with_sig(!on_off);
}
void swapTwoStateButton::change_state_noEmit()
{
    change_state_man(!on_off);
}
void swapTwoStateButton::change_state_man(bool st)
{
    setChecked(st);
    setIcon(makeHugeIcon(st ? onstring : offstring[dir]));
    on_off = st;
}
void swapTwoStateButton::change_state_man_with_sig(bool st)
{
    change_state_man(st);
    emit state_changed(st);
}
