#include "threestatebutton.h"

ThreeStateButton::ThreeStateButton(const QString &st1, const QString &st3, double scale)
    : iconSt1(st1), iconSt3(st3)
{
    aminationState2Period = 500;
    currentAnimIcon = state = 0;
    iconSizeWidth = iconSize().width() * scale;
    changeIcon(st1);
    state2Timer = new QTimer();

    animateClick(0); //that need for apply big icon size

    connect(state2Timer, SIGNAL(timeout()), this, SLOT(timerState2Ended()));
    connect(this, SIGNAL(clicked()), SLOT(onClickedThreeStateButton()));
}

void ThreeStateButton::onClickedThreeStateButton()
{
    change_state(state+1);
}

void ThreeStateButton::change_state_man(int st)
{
    if (state == st)
        return;
    if (state2Timer->isActive())
        state2Timer->stop();

    if (st > 3 || st < 1)
        st = 1;

    int oldSt = state;
    state = st;
    if (st == 1 || st == 3)
    {
        changeIcon(st == 1 ? iconSt1 : iconSt3);
        currentAnimIcon = st;
    }
    else
    {
        currentAnimIcon = (currentAnimIcon == 1) ? 3 : 1;
        changeIcon(currentAnimIcon == 1 ? iconSt1 : iconSt3);
        state2Timer->start(aminationState2Period);
    }
    if (oldSt == 0)
        return;
    emit state_changed_man(oldSt, state);
}

void ThreeStateButton::change_state(int st)
{
    if (state == st)
        return;
    int oldSt = state;
    change_state_man(st);
    if (oldSt == 0 && state == 1)
        return;
    emit state_changed(oldSt, state);
}

void ThreeStateButton::timerState2Ended()
{
    currentAnimIcon = (currentAnimIcon == 1) ? 3 : 1;
    changeIcon(currentAnimIcon == 1 ? iconSt1 : iconSt3);
}

void ThreeStateButton::changeIcon(const QString &icon)
{
    if (iconSizeWidth > 0)
    {
        setIcon(makeHugeIcon(icon));
        setIconSize(QSize(iconSizeWidth, iconSizeWidth));
    }
    else
        setIcon(makeHugeIcon(icon));
}

ThreeStateButton::~ThreeStateButton()
{
    if (state2Timer->isActive())
        state2Timer->stop();
}

int ThreeStateButton::getState() const
{
    return state;
}

int ThreeStateButton::minRectWidth() const
{
    return 90;
}

