#include "motic1080.hpp"

#include <QCamera>
#include <QMediaService>
#include <QMediaRecorder>
#include <QCameraViewfinder>
#include <QCameraInfo>
#include <QMediaMetaData>
#include <QMessageBox>
#include <QPalette>
#include <QAbstractVideoSurface>
#include <QVideoSurfaceFormat>
#include <QSettings>

motic1080::motic1080()
{
    cameraInited = false;
    camera = nullptr;
    videoSurf = nullptr;
}

motic1080::~motic1080()
{
    this->closeCamera();
}

void motic1080::Start()
{
    if (!cameraInited)
    {
        foreach (const QCameraInfo &cameraInfo, QCameraInfo::availableCameras()) {
            //if (cameraInfo.description().contains("Moticam1080"))
                camera = new QCamera(cameraInfo);
        }
        cameraInited = true;
        camera = new QCamera(QCameraInfo::defaultCamera());
        connect(camera, SIGNAL(error(QCamera::Error)), this, SLOT(displayCameraError(QCamera::Error)));
        videoSurf = new CamVideoSurceface();
        camera->setViewfinder(videoSurf);
        connect(videoSurf, SIGNAL(ImgReceived(QImage)), this, SLOT(processCapturedImage(QImage)));
    }
    camera->start();
}

void motic1080::Stop()
{
    if (cameraInited)
        camera->stop();
}

bool motic1080::openCamera()
{
    this->closeCamera();

    foreach (const QCameraInfo &cameraInfo, QCameraInfo::availableCameras()) {
        //if (cameraInfo.description().contains("Moticam1080"))
            return true;
    }

    return false;
}

void motic1080::processCapturedImage(QImage img)
{
    emit ImgReceived(img);
}

void motic1080::displayCameraError(QCamera::Error /*error*/)
{
    QString str = camera->errorString();
    QMessageBox msgBox;
    msgBox.setText(str);
    msgBox.setWindowTitle("Camera error");
    msgBox.exec();
}

void motic1080::closeCamera()
{
    if (cameraInited)
    {
        delete camera;
        delete videoSurf;
        cameraInited = false;
    }
}

void motic1080::setExposure(double value)
{
    camera->exposure()->setExposureCompensation(value);
}

void motic1080::SetDoAWB(bool bAWB)
{

}

void motic1080::SetDoAE(bool bAE)
{

}
