#include "nspectr.hpp"

#include "nspectr_proxy.h"
#include <QSettings>

INspectr::INspectr(void *_hwnd) : hwnd(_hwnd)
{
    cp = 0;
    pxsize = 0.;
    cwl = 0.;
}

bool INspectr::initialize(int)
{
    if (initialized)
        shutdown();

    int MonochModel = 1; //m833
    QSettings settings("Param/ccd.conf", QSettings::IniFormat);
    if (settings.value("use_m266", false).toBool())
        MonochModel = 0; //m266

    DWORD Icon = 0;
#ifdef _WIN32
#ifndef GCL_HICON
#define GCL_HICON (-14)
#endif
    if (hwnd) Icon = GetClassLongA(*((HWND*)(&hwnd)), GCL_HICON); //fuck gcc
#endif

    return initialized = Proxy_Init(*((HWND*)(&hwnd)), Icon, MonochModel); //fuck gcc
}
bool INspectr::shutdown()
{
    if (!initialized)
        return false;

    QMutexLocker lock(&guiThreadLock);
    Proxy_Final();
    initialized = false;
    return true;
}


QString INspectr::getName()
{
    QSettings settings("Param/ccd.conf", QSettings::IniFormat);
    if (settings.value("use_m266", false).toBool())
        return "M266";
    else
        return "M833";
}
bool INspectr::isReady()
{
    QMutexLocker lock(&guiThreadLock);
    if (!initialized)
        return true;

    return Proxy_GetStatus() == STATUS_READY;
}


bool INspectr::preparePixelWl(double pixelsize, int /*nx*/, int central_pixel)
{
    QMutexLocker lock(&guiThreadLock);
    if (!initialized)
        return false;

    cp = central_pixel;
    pxsize = 1000 * pixelsize;
    cwl = Proxy_GetWL();

    double pxlWL = 0;
    return Proxy_GetPixelClbr(cwl, cp, pxsize, 0, &pxlWL);
}
double INspectr::getPixelWl(int pixel_idx)
{
    QMutexLocker lock(&guiThreadLock);
    if (!initialized)
        return 0;

    double pxlWL = 0;
    Proxy_GetPixelClbr(cwl, cp, pxsize, pixel_idx, &pxlWL);
    return pxlWL;
}


bool INspectr::setWl(double wl, bool reset)
{
    QMutexLocker lock(&guiThreadLock);
    if (!initialized)
        return false;

    if (!Proxy_IsValidWL(wl))
        return false;

    return Proxy_SetWL(wl, reset);
}
double INspectr::getWl()
{
    QMutexLocker lock(&guiThreadLock);
    if (!initialized)
        return 0;

    return Proxy_GetWL();
}
double INspectr::getDispersion()
{
    QMutexLocker lock(&guiThreadLock);
    if (!initialized)
        return 1;

    return Proxy_GetDispersion();
}


int INspectr::getGratingCount()
{
    QMutexLocker lock(&guiThreadLock);
    if (!initialized)
        return 0;

    return Proxy_GetGratingCount();
}
QString INspectr::getGratingName(int idx)
{
    QMutexLocker lock(&guiThreadLock);
    if (!initialized)
        return QString();

    double gr, min, max;
    if (Proxy_GetGratingPrm(idx, &gr, &min, &max)) //fixme is it right order od Min and Max ???
        return QString::number(gr) + tr(" lines/mm");

    return QString::number(idx + 1);
}
bool INspectr::setActiveGrating(int idx, bool)
{
    QMutexLocker lock(&guiThreadLock);
    if (!initialized)
        return false;

    return Proxy_SetActiveGrating(idx);
}
int INspectr::getActiveGrating()
{
    QMutexLocker lock(&guiThreadLock);
    if (!initialized)
        return 0;

    return Proxy_GetActiveGrating();
}


bool INspectr::setSlitWidth(int port, double width)
{
    QMutexLocker lock(&guiThreadLock);
    if (!initialized)
        return false;

    return Proxy_SetSlitWidth(port == SPECTR_PORT_IN ? ID_ENTRANCE_SLIT_1 : ID_EXIT_SLIT_1,
                              width, TRUE);
}
double INspectr::getSlitWidth(int port)
{
    QMutexLocker lock(&guiThreadLock);
    if (!initialized)
        return 0;

    return Proxy_GetSlitWidth(port == SPECTR_PORT_IN ? ID_ENTRANCE_SLIT_1 : ID_EXIT_SLIT_1);
}


bool INspectr::setPortPosition(int port, int idx)
{
    QMutexLocker lock(&guiThreadLock);
    if (!initialized)
        return false;

    if (port == SPECTR_PORT_OUT) {
        DWORD pos = idx == 0 ? MRP_AXIAL : MRP_LATERAL;
        return Proxy_SetMirror(pos);
    }
    return false;
}
int INspectr::getPortPosition(int port)
{
    QMutexLocker lock(&guiThreadLock);
    if (!initialized)
        return 0;

    if (port == SPECTR_PORT_OUT)
        return Proxy_GetMirror() == MRP_AXIAL ? 0 : 1;

    return 0;
}
