#pragma once

#include <QObject>
#include <QQueue>
#include <QPointer>
#include <QVariant>
#include "modules.hpp"

class QTimer;
class MobileServer;
class BaseNetworkClient;
class MobileInstance : public QObject
{
    Q_OBJECT
public:

    ~MobileInstance();

public slots:
    void newData();
    void keepalive();

public:
    bool auth = false;
    BaseNetworkClient *socket;
    MobileServer *serv;

    QQueue<int> dropped_list;
    int dropped_count = 0;

    QTimer *keepalive_timer = nullptr;
    int keepalive_times = 0;

};

class I_NET_MODULE_BASE
{
public:
    virtual void initialSend(MobileInstance * /*client*/) { }
    virtual void setValue(int /*id*/, QVariant /*val*/ = QVariant(), QObject * /*sender*/ = nullptr) { }
};

class QUdpSocket;
class QHostAddress;
class INetworkClient;
class StatusWidget1;
class MobileServer : public QObject, public I_MODULE
{
    Q_OBJECT
public:
    MobileServer();
    static MobileServer* GetIt() noexcept;

    bool need_send() { return clients.size(); }
    bool any_need_data(const char *name);
    bool any_need_data(const QString &name);

    QMap<QString, int> ban_map;
    QTimer *ban_timer;
    QList<MobileInstance *> clients;
    int max_dropped_count; //need to set to [1.2-5]*max_drop_channels
    QPointer<StatusWidget1> networkStatus;

    QUdpSocket *discovery_socket;

public slots:
    void ban(const QHostAddress &ip);
    void banStep();
    void newDiscovery();
    void newConnection(INetworkClient *);
    void newDisConnection();
    void setPropIdVar(int id, QVariant val = QVariant(), int can_drop_id = 0);
    void setPropIdVarS(BaseNetworkClient *socket, int id, QVariant val = QVariant());

public:
    static void sendData(int id, QVariant val = QVariant());

    //I_MODULE
    static const QLatin1String name() { return QLatin1String("net"); }
    void *basecast(const QLatin1String name) override { return name.latin1() != nullptr ? nullptr : static_cast<MobileServer *>(this); }

};
