#pragma once

#include <QWidget>
#include "edit.hpp"
//#include "3d/plot_3dplot.h"
#include "options/options.h"

namespace plot {
    class T3dPlot;
    class T3dPlotControl;
}

class QCheckBox;
class QTreeWidget;
class QComboBox;
class QTreeWidgetItem;
class QSettings;

class TOptionsWindow: public QWidget
{
    Q_OBJECT
public:
    TOptionsWindow(QWidget *parent);

    QCheckBox *chShowScanRightMenu;
    QCheckBox *chShowDataRightMenu;
    QCheckBox *chShowSpectrRightMenu;
    QCheckBox *chRatisXYZStageMode; //mode when everything is hidded except oscilloscope
    QCheckBox *chHideDidntFindedSensorsErrorMessages;
    QCheckBox *chIgnoreScanSetPointError;

    #ifdef _SCAN_SLICE
    QCheckBox *chEnableScanSlice;
    Edit *edScanZoomSlice;
#endif

    QCheckBox *chMoveXYZToZeroOff;
    Edit *edZeroWaitPerChannel;

public slots:
    void selectCustomTR();
    void Show();
    void Apply(bool definitlyRestart = false);
    void BFont();
    void BData();
    void configEdited(QTreeWidgetItem *item, int column);
    void editorResize();
    void saveScanFilterSettings();
    void saveScan2DFilterSettings();
    void saveRectSliderMouseWheelPower(int);
    void applyFilter2DSettings();
    void applyFilterSettings();
    void applyFilterSettings(plot::T3dPlot *t3plot, plot::T3dPlotControl *PlotControl);
    void saveScanAllFilterSettings();

private:
    void addConfig(QSettings &settings, const QString &file, const QString &path, const QVariant &default_val, const QString &comment = "");
    void addConfig(QSettings &settings, const QString &file, const QString &path, const QString &comment = "");
    void addConfig(const QString &file, const QString &path, const QVariant &default_val, const QString &comment = "");
    void addConfig(const QString &file, const QString &path, const QString &comment = "");

    void initGeneralGUI();

    options *optionsParent;

    QTreeWidget *editor;
    QMap<QString, QTreeWidgetItem *> items;

    QComboBox *Ctranslation;
    Edit *szbox;
    Edit *as_time;

    QFont font;
    QString datadir;

    QCheckBox *CBgwy_imp;
    QCheckBox *CBremote_net;
    QCheckBox *CBremote_pwd;
    QCheckBox *CBremote_log;
    QCheckBox *CBremote_guest;

    Edit *remote_port;
    QLineEdit *remote_pwd;

    QCheckBox *chDataModule;
    QCheckBox *chsSpectroscopyModule;
    QCheckBox *chScanModule;
    QCheckBox *chSpmModule;
    QCheckBox *chOscilloscopeModule;
    QCheckBox *chLithographyModule;
    QCheckBox *chVideoModule1;
    QCheckBox *chVideoModule2;

    QCheckBox *chHeadScannerModule;
    QCheckBox *chStageScannerModule;
    QCheckBox *chZ3ScannerModule;
    QCheckBox *chAboutModule;
    QCheckBox *chShuttleStatusModule;

    QCheckBox *chAutoSendToData;
    QCheckBox *chOpticInitOnStart;
    QCheckBox *chUseSound;
    QCheckBox *chUnlockToolbars;


    QCheckBox *chShowBwChan;
    QCheckBox *chShowFz;
    QCheckBox *chShow4D;
    QCheckBox *chShowLiftMod;
    QCheckBox *chShowContScan;
    QCheckBox *chShowSecondChannelTab;
    #ifdef _OLD_EXTRA_CONTROL
    QCheckBox *chShowExtraControl;
#endif
    QCheckBox *chShowDeviceControl;
    QCheckBox *chShowScanSlider;


    //------------3d filter
    QCheckBox *chFitLines;
    QCheckBox *chMeanValue;
    QCheckBox *chMedianValue;
    QCheckBox *chGaussian;
    QCheckBox *chHistogramEqualization;
    QComboBox *cbSubstract;
    QComboBox *cbFitting;
    Edit *edGaussian;
    Edit *edMeanValue;
    Edit *edMedianValue;
    //--------------

    //-------------- common filter
    QCheckBox *cbByDefEnableFilters;
    //--------------

    //--------------2d filtger
    QComboBox *cb2DFilter;
    //--------------

    Edit *edRectSliderMouseWheelPower;
};
