#pragma once

#include <plot.h>
#include "controls.h"
#include "edit.hpp"
#include <defs.h>
#include "tools.hpp"
#include <QPointer>
#include <QElapsedTimer>
#include "USBVideo/videoplayer.h"
#include "deflect_widget.hpp"
#include "gui/rectslider.h"
#include <QPointer>
#include "status.hpp"
#include "gui/t3dchooser.h"
#include "gui/threestatebutton.h"
#include "gui/devicescheme.h"
#include "gui/hoverbutton.h"
#include "gui/manystatebutton.h"
#include "options/options.h"
#include "gui/swaptwostatebutton.h"
#include "plot/3d/plot_3dplot.h"
#include "plot/widgets/threedoverlaybuttons.h"
class QCheckBox;

class IExternalScan : public QObject
{
    Q_OBJECT

public:
    IExternalScan() {
        displayPlot = nullptr;
        nx = ny = dir = start_point = dir_stpoint = 0;
        started = false;
        plot::ResizeOwner(Output3d, 1);
        plot::ResizeOwner(Section, 1);
        Section[0]->Pen.setColor(Qt::red);
    }
    ~IExternalScan() override {}
    void SetDirStPoint(int _dir, int _stpoint) { dir = _dir; start_point = _stpoint; }
    void SetStPointDirection(int value) {
        switch(value) {
        case spTopLeftAxisX: SetDirStPoint(Horiz, TopLeft); break;
        case spTopRightAxisX: SetDirStPoint(Horiz, TopRight); break;
        case spBottomRightAxisX: SetDirStPoint(Horiz, BottomRight); break;
        case spBottomLeftAxisX: SetDirStPoint(Horiz, BottomLeft); break;
        case spTopLeftAxisY: SetDirStPoint(Vert, TopLeft); break;
        case spTopRightAxisY: SetDirStPoint(Vert, TopRight); break;
        case spBottomRightAxisY: SetDirStPoint(Vert, BottomRight); break;
        case spBottomLeftAxisY: SetDirStPoint(Vert, BottomLeft); break;
        }
    }
    enum TDirections { Vert, Horiz };
    enum TStartPoints { BottomLeft, BottomRight, TopLeft, TopRight };

    virtual bool init(int _nx, int _ny, int _dir_stpoint, Range _sx, Range _sy) {
        nx = _nx;
        ny = _ny;
        dir_stpoint = _dir_stpoint;
        SetStPointDirection(dir_stpoint);
        sx = _sx;
        sy = _sy;
        started = true;

        clear();

        Output3d.at(0)->XLimits = sx;
        Output3d.at(0)->YLimits = sy;
        Output3d.at(0)->setSize(nx, ny);

        foreach (QWidget *wid, disableWidgets)
            wid->setEnabled(false);

        return true;
    }
    virtual void finish() {
        started = false;
        foreach (QWidget *wid, disableWidgets)
            wid->setEnabled(true);
    }
    virtual void save() { }
    virtual void clear() {
        lastError.clear();
        Output3d.at(0)->clear();
        Section[0]->clear();
    }
    virtual int getDataChannel() { return srcCCD; }
    virtual void updateData() { emit updated(); }
    virtual void setPointSpection(bool pressed) {}
    virtual void sl_Spectrum_range_select(bool pressed) {} //todo remove it from this
    virtual void sl_Func_range_select(bool pressed) {} //todo remove it from this
    virtual void setSelectionRange(double Min, double Max) {}
    virtual void delayInit() {} //that function executing when showing GUI
    virtual int get2DPlotFilter() {return 0;}
    
    virtual QVector<QWidget *> *createInterfaceWidget(QVBoxLayout**, QVBoxLayout*) { return nullptr; }

    virtual bool acquireNextPoint() = 0;
signals:
    void acquired();
    void updated();
    void setSectionEnabled(bool);

public:
    int nx, ny, dir, start_point, dir_stpoint;
    Range sx, sy;
    bool started;
    plot::T3dPlot *displayPlot;

    QString name = "";

    //data interface
    QString lastError;
    plot::TCurvesDataOwner Section;
    plot::T3dDataOwner Output3d;
    plot::TMetaStorage attributes;
    QVector<QWidget *> disableWidgets;
};

class SPMTunning;
IExternalScan *getNewJMIExtScan(SPMTunning *);

class QwtThermo;
class StatusBar;
class TTabWidget;
class ScanPlot: public QWidget, public IProcessPacket, public IWriter
{
    Q_OBJECT

signals:
    void sig_XYPosition(double x, double y);
    void sig_XYPosition2(double x, double y);

    void AddDataToContainer(plot::T3dDataOwner, plot::TMetaStorage);
    void AddDataToContainer(plot::TCurvesDataOwner, plot::TMetaStorage);

    //litho
    void SendBackground(const QImage &Img);
    void SendRect(const QRectF &Rect);

    void recalibrateSensors(int);

private:
    QString getLegendCurrPoint();
    QString getLegendNameChannel(int i);
    const double velocityMulfactorValues[6] = { 0.1, 1., 5., 10., 50., 100. };

public:
    void resizeEvent();
    static const int numScanChannel = 4;

    BYTE scan_on_off;
    float scan_lm_h;
    double Step_X;
    double Step_Y;
    BYTE scan_dir_st_point;
    BYTE scan_ch[numScanChannel];

    bool LmEn, LmEn_ch[numScanChannel];
    double LmHeight;

    bool scan_precise_mode;
    int scanner_scan;
    int ccdNX,ccdNY;

    QComboBox *comboVideoProfile;
    QComboBox *comboMaterialThin;

public slots:
    void reset_scan_test();
    void injectTestData();

    void scan_first_delay(double time);
    void scan_line_delay(double time);
    void scan_point_delay(double time);
    void spm_pid_precision(double delta);
    void scan_fast_return(bool);
    void scan_sync_mode(int);
    void scan_lm_height(double);
    void autoFocusEnded(bool);
    void autoFocus2Ended(bool);
    void autoFocus2_ChangedState(bool);
    void materialThinChangedSave(int);

public:
    Q_SIGNAL void sync_scan_x(double);
    Q_SIGNAL void sync_scan_y(double);
    Q_SIGNAL void sync_scan_x2(double);
    Q_SIGNAL void sync_scan_y2(double);
    void sendXYsync(double x, double y, int sc);
    void updateVideoProfiles(VideoPlayer *v, int currProfId = 0);

public slots:
    void currentLineFontSettingsChanged(int n);
    void currentLineColorChanged(QColor, int);

    void startMAFPrcs();
    void startPAFFast();
    void startPAF();

    #ifndef _OLD_EXTRA_CONTROL
    void showExtraControlOnPlot();
#endif

    void lowVoltageModeChanged(bool);
    void setLowVoltageMode(bool pressed);
    void setScannerLowVoltageMode(int);
    void onFilter_changed(bool pressed);
    void onFilter2D_changed(bool pressed);
    void updateVideoChannel();
    void setAutoTipPush(bool);
    void setFBoffon(bool);
    void multSpmPidStageChanged(int);
    void SignalsTabChanged(int);
    void splitterMoved(int pos, int index);
    void FeedBackChangedUpdateMag(bool);
//    void DCfbSetPointChanged(double value);
    void rectSliderSlFbiChanged(double);
    void doubleClickedLaserSpotWindow();
    void reset();
    void DCfbPPercNewValue(double);
    void DCfbPNewValue(double);
    void DCfbPNewValue_emit(double val);
    void spm_SmartLandingButtonChangedMan(int oldSt, int newSt);
    void spm_SmartLandingButtonChanged(int oldSt, int newSt);
    void capBits(int, long);
    void s_scanError();
    void recalibDone(bool);
    void cancel_clicked(plot::ThreeDOverlayButtons::SpecialVisibilityMode visibilityMode);
    void cancelApply_clicked(plot::ThreeDOverlayButtons::SpecialVisibilityMode visibilityMode);
    void on2DBackButton_clicked();

    bool sl_fast_scan_correction(int ch);
    void sl_ScanCorrection();
    void sl_motor_level();
    void make_screen_videocam(bool start);

    void Send2dPlotData();
    void Send3dPlotData1();
    void Send3dPlotData2();
    void Send3dPlotData3();
    void Send3dPlotData4();
    void BSendToGraphEditor_clicked();

    void AddDataToContainer_slot(plot::TCurvesDataOwner, plot::TMetaStorage);
    void AddDataToContainer_slot(plot::T3dDataOwner, plot::TMetaStorage);
    void FillContainer(int plotId, plot::T3dDataOwner *src, plot::TMetaStorage *attributes);
    void FillDataAttributes();
    void FillDataAttributes2();
    void restoreAttributes(const plot::TMetaStorage&);
    void slAutosaveData();

    void sl_setCurrentX(double v);
    void sl_setCurrentY(double v);
    void sl_setCurrentX2(double v);
    void sl_setCurrentY2(double v);
    void sl_syncXY();
    void sl_Update_XYZ_pos();
    void sendXYPos();
    void onLEdDeltaX_editingFinished();
    void onLEdDeltaY_editingFinished();
    void sl_set_XYZ_range(double, double, double);
    void sl_set_XYZ_range2(double, double, double);
    void sl_set_XYZ_range_sync();
    void slUpdZscanner(double zpos);

    void updateDirection();
    void check_nx_ny();
    void check_speed();
    void onBApply_clicked();

    void scan_start_stop(bool);

    #ifdef _SCAN_SLICE
    Range* getScanSliceRangeX();
    Range* getScanSliceRangeY();
    void scan_slice();
#endif

    void s_stopScan();
    void sl_stop_scan();
    bool removeOldData(int ch);
    void sl_update_progress(QByteArray data);
    void sl_scan_refresh_ch(int n);

    void acquireNextPoint();
    void sl_ext_scan_acquired();
    void sl_ext_scan_update();
    void sl_ext_scan_refresh();
    void sl_ext_mode();
    void sl_dacscan(bool);
    void sl_dacscan_advance();
    bool is_dacscan();

    void setScanner(int sc);
    void LmOnOff(bool value);
    void sl_CB_update();
    void sl_CB_filters_update();
    void updateScan2DPlotVisible();
    void update2DPlotScanMode();

    void sl_section_legend(int);
    void sl_set_section_enabled(bool);
    void SetSectionUnits();
    void receiveCameraImage(QImage img);
    void on_BSave_clicked();
    void feedBackClickedInSPM(bool val);
    void selectByRect_clicked();

    void tipDriveNewValue_FromSlider(double);
    void tipDriveNewValue(double val);

    void plotsChanged();
    void spectrButtonStateChanged(bool);
    void Plot3DTabChanged(int tabId);

    void ChannelResized(int n, QSize newSize);
    void ChannelSetVisibleGUI(int n, bool visible, plot::ThreeDOverlayButtons::SpecialVisibilityMode visibilityMode);

    void videoChannelResized(int n, QSize newSize);
    void videoChannelSetVisibleGUI(int n, bool visible);
    void autoFocusPrec_ChangedState(bool state);
    void autoFocusFast_ChangedState(bool state);
    void autoFocus_ChangedState(bool on);

    void motorZFocusUp(bool pressed);
    void motorZFocusDown(bool pressed);

    void motorXStageLeft(bool pressed);
    void motorXStageRight(bool pressed);
    void motorYStageTop(bool pressed);
    void motorYStageBottom(bool pressed);

    void autoFocusUpDownBtnWaiting();
    void autoFocusUpDownBtn(int oldSt, int newSt);
    void autoFocusUpDownBtn_man(int oldSt, int newSt);
    void autoFocusUpDownEnded(bool sucsess);

    void zLandButtonSysOn(bool pressed);
    void zLandButtonSysOff(bool pressed);
    void zLandButtonSysOn_motorEvent(int mZLanding, bool atEnd, bool active);
    void zLandButtonSysOnHeadLanding_motorEvent(int mRealId, bool atEnd, bool active);
    void zLandButtonSysOffHeadLanding_motorEvent(int mRealId, bool atEnd, bool active);
    void zLandButtonSysOff_motorEvent(int mZLanding, bool atEnd, bool active);


    void scan_direction_changed(swapDir::Direction);
    void fontSize_changed();
    void plotFontSize_changed();

    void onSaveImage_clicked();
    void onSendToDataVideo_clicked();
    void onSendToData_clicked();

    void ShowRightMenu(bool show);

    void on4DSendToData_clicked();
    void on4DClear_clicked();
    void on4DSwitchSpectra_clicked(bool pressed);
    void on4DSelectCapturedSpectr_clicked(bool pressed);
    void on4DSelectConvolutionFunc_clicked(bool pressed);

    void onVideoCBChannelChanged(int index);

    void cancelZLandingSysOn();
public:
    ScanPlot(QWidget *parent = nullptr);
    ~ScanPlot() override;
    static ScanPlot* GetIt() noexcept;

    void GUICreate();
    bool eventFilter(QObject *Object, QEvent *Event) override;

    bool sendData(const void *handle) override;
    void resetWriter() override;

    #ifdef _OLD_EXTRA_CONTROL
    void extraControlTabCreated();
#endif

    bool processPacket(const BYTE addr, const BYTE sub_addr, const void *data, const DWORD size) override;
    void Fill_Data_Array(plot::T2dArray& Array, const EG_ScanData *Scan_Data, const EG_ScanData *lsb_Scan_Data = nullptr, plot::TCurveData *Section = nullptr, bool raw = false);
    QByteArray lsbData;
    EG_ScanData *ScanDataPart[8];
    EG_ScanData *ScanDataPart_lsb[8];

    double oldMag = 5.0;

    bool Autosend;

    QComboBox *CBChannel[numScanChannel];
    QCheckBox *CBSec[numScanChannel];

    
    Edit *LEdDeltaX;

    unsigned short int Nx_points, Ny_points;
    
    double xLeft, xRight, yTop, yBottom;
    
    bool scan_direction;

    #ifdef _OLD_EXTRA_CONTROL
    QWidget *BoxDelays;
    QWidget *BoxSync;
    QGroupBox *scanTabGB;
    QLabel *LScanner;
#endif

    QWidget *BoxExternal;
    TwoStateButton *BCCDXYLock;
    
    QComboBox *CBPointsOYext;
    QComboBox *CBPointsOXext;
    
    IExternalScan *currExtScan;
    void addExternalScan(const QString &name, IExternalScan *iface);

    void zLandClearZLandingMotorsStopped();
    int zLandCountStoppedZLandingMotorsStopped();
    

    TwoStateButton *PBSpectr_start_stop;
    QComboBox *CB_Channel;
    Edit *ev_Speed_spectr;
    QComboBox *CB_Points_spectr;
    Edit *ev_Z1_spectr;
    Edit *ev_Z2_spectr;

    DoubleChanger *DCfbP;
    DoubleChanger *DCfbPPerc;
    DoubleChanger *DCfbI;
    DoubleChanger *DCfbSetPoint;

    QComboBox *CBExternalMode;
    DoubleChanger *driveInt;

    #ifdef _OLD_EXTRA_CONTROL
    DoubleChanger *firstDelay;
    DoubleChanger *lineDelay;
    DoubleChanger *pointDelay;
#else
    QLabel *LExScanFirstPDelay;
    Edit *EdExScanFirstPDelay;
    QLabel *LExLineFirstPDelay;
    Edit *EdExLineFirstPDelay;
    QLabel *LExFastBackward;
    QLabel *LExSync;
    QLabel *LExSyncPointDelay;
    Edit *EdExSyncPointDelay;
    QLabel *LExScanType;
    QLabel *LExScanner;
    QLabel *LExLowVoltage;
    QCheckBox *CBExLowVoltage;

    QList<QLabel*> extraControlsLabels;
    QList<QWidget*> extraControlsRightPart;

    colorRect2 *ExtraBackground;
#endif

    
    QComboBox *pointSync;
    
    QComboBox *CBPointsOY;
    QComboBox *CBPointsOX;
    
    void Set_CBPoints(QComboBox *box);

    plot::T3dPlot *Plot;

    swapTwoStateButton *m_BStart;
    QPushButton *BMotorLevel;
    TwoStateButton *BMakeScreen;
    TwoStateButton *BAutoFocus;
    TwoStateButton *BAutoFocus2;

    static const int videoChannelId = 3;
    
    TwoStateButton *videoChannelAutoFocusButtonPrec;
    TwoStateButton *videoChannelAutoFocusButtonFast;
    TwoStateButton *videoChannelAutoFocusButtonPAF;

    QComboBox *videoChannelMagnification;
    
    //ManyStateButton *videoChannelAutoFocusUpDownBtn;

    bool twodPlotFilterScanMode = true;

    QPushButton *smartLandingBtn1;
    QPushButton *smartLandingBtn2;
    ThreeStateButton *smartLanding;
    TwoStateButton *spmMonitorFeedback;
    QHBoxLayout *spmMonitorGBHlay;
    DeflectWid *LaserSpotWindow;
    
    RectSlider *rectSlFbp;
    RectSlider *rectSlVelocity;
    RectSlider *rectSlFbi;
    RectSlider *rectAmplitude;

    deviceScheme *deviceSchemeWid;

private:
    void GUICreate_Plot();
    void GUICreate_TipMonitor();
    void GUICreate_TipMonitor_Approach(QHBoxLayout *lay_rectSlidersBox);
    void GUICreate_TipMonitor_Speed(QHBoxLayout *lay_rectSlidersBox);
    void GUICreate_TipMonitor_Sensitivity(QHBoxLayout *lay_rectSlidersBox);
    void GUICreate_TipMonitor_TipPush(QHBoxLayout *lay_rectSlidersBox);
    void GUICreate_TipMonitor_TipDrive(QHBoxLayout *lay_rectSlidersBox);
    void GUICreate_Signals();
    void GUICreate_Signals_Channels();
    void GUICreate_Signals_Fz();
    void GUICreate_Signals_4D();
    void GUICreate_Signals_LiftMode();
    void GUICreate_Signals_ContScan();
    void GUICreate_Signals_ChannelChooser(QHBoxLayout *channelTabGBlay);
    void GUICreate_ExtraControl();
    void GUICreate_SPMMonitor();
    void GUICreate_DeviceControl();

    void GUISetSizeTipMonitor(int width);
    int GUIMinimumWidthTipMonitor();

    void resizeEvent(QResizeEvent *e) override;
    void GUIResizeSPM();
    bool GUIInited = false;
    QGroupBox *tipMonitorGB;
    QGroupBox *signalsGB;
    int minimumLeftPanelWidth = 100;
    int minimumRectSliderSize = 10;
    int minimumTipMonitorHeight = 10;
    int minimumSignalsHeight = 10;
    
    int ccdMode;
    
    WORD m_spm_pid_precision;
    TMessageFlag updateSPMPrecision;
    WORD m_scan_first_delay;
    TMessageFlag updateFirstDelay;
    WORD m_scan_line_delay;
    TMessageFlag updateLineDelay;
    WORD m_scan_point_delay;
    TMessageFlag updatePointDelay;
    bool m_scan_fast_return;
    TMessageFlag updateFastReturn;
    int m_scan_sync_mode;
    TMessageFlag updateSyncMode;

    TMessageFlag sendSpeed, sendLMheight, sendConfig, sendExtConfig, sendStartStop, sendExtStartStop, sendExtReady;
    
    QVector<QWidget *> scanControlWidgets; //disabled on scan
    QPointer<StatusBar> scan_bar;
    QPointer<StatusBar> ccd_scan_bar;
    bool scanFailed;

    Range *copyX = nullptr;
    Range *copyMaxX = nullptr;
    
    bool ratisMode = false;
    QTimer *AutosaveTimer;
    QTimer *RefreshTimer4d;
    QElapsedTimer refresh_3d_limit_timer[numScanChannel];
    qint64 refresh_limit_last_elapsed[numScanChannel];
    bool refresh_limit_need_math_update[numScanChannel];
    int channelsActive;
    unsigned int Ch_activated;
    
    //QGroupBox *GrBDACscan;
    QComboBox *CBDACscanCh;
    Edit *EDACcount;
    Edit *EDACstep;
    double DACinitialValue;
    int DACcurrentNum;
    bool show4dMenu = true;

    QColor lastSelectedLineColor = Qt::black;
    
    bool sectionEnabled;

    bool l2k_controller, l2k_controller_v22;
    bool is_optical_scanner[3];

    double requested_scan_speed; //to hw
    double user_scan_speed;    
    
    Edit *LEdX;
    Edit *LEdY;
    Edit *LEdDeltaY;
    QLabel *LDeltaX;
    QLabel *LDeltaY;
    double x, y, x2, y2;
    double Xmax, Ymax;
    double Xmax2, Ymax2;
    QwtThermo *Zposition;
    QLabel *ZposValue;
    
    int line_number[4];
    QwtPlotMarker *scanLineInd[4];
    bool ready[4];
    int nlcupd[4];
    double stPointX, stPointY;
    
    
    
    QVector<IExternalScan *> externalScans;
    QVector<QVector<QWidget *> *> externalScanWidgets; //List of all widgets in scan widgets
    
    QVBoxLayout **lay4D;
    QGroupBox *gb4DButtons;
    QVBoxLayout *lay4DButtons;

    BYTE fast_scan_correction;
    bool fast_scan_CLZ_disabled;

    TTabWidget *tabSignals;
    const int pos4DTab = 3;
    const int posFzTab = 2;
    QWidget *fzWidget;
    QHBoxLayout *fzHMainLayout;
    
    const int numZLandingMotors = 3;
    QList<bool> zLandingMotorsStopped;
    QPointer<StatusBar> zLandingStatusBar;
    
    DoubleChanger *spmPrecision;
    
    QCheckBox *fastReturn;
    QComboBox *CBScanner;
    QLabel *LPointsOX;
    QLabel *LPointsOY;
    
    QComboBox *CBStartPos; //todo remove that button, that replaced by button on plot
    plot::TMetaStorage scan_attributes;
    plot::T3dDataOwner *dataScan;
    plot::T3dDataOwner *dataScan_src;
    plot::T2dArray *dataScan_X;
    plot::T2dArray *dataScan_Xb;
    plot::T2dArray *dataScan_Y;
    plot::T2dArray *dataScan_Yb;
    
    plot::T3dDataOwner scanData[2][numScanChannel];
    plot::TCurvesDataOwner Section_data;

    QLabel *lb_ccd_scan;
    QLabel *lb_ccd_scan_sm;

    QCheckBox *cbLiftMode;
    QCheckBox *LmSwitch[numScanChannel];
    QLabel *LblLiftMode;
    Edit *LEd_LFmodeHeight;
    
    QCheckBox *cbContScan;

    QToolButton *FastScanCorrection;
    TwoStateButton *BPreciseScan;
    TwoStateButton *BXYLock;
    
    QPushButton *BSendToData;
    QToolButton *TBFontSize;
    QActionGroup *AGFontSizes;

    QPushButton *BVideoSaveImage;
    QPushButton *BVideoSendToData;

    QVBoxLayout *LeftLay;
    
    plot::ImageItem *ritem;
    
    bool showingVideoFrame;
    double videoChannelAxisScaleXMax[2];
    double videoChannelAxisScaleYMax[2];
    QString plotLegends[numScanChannel];
    bool plotPointChanged;
    
    bool videoChannelAutoFocusPrevCameraState = false;
    QList<QPushButton*> allVideoBtnsList;
    TwoStateButton *videoChannelPlayCamera;
    QLabel *videoChannelMagnificationLabel;
    QLabel *videoChannelMaterialThinLabel;
    QComboBox *videoChannelMaterialThin;
    int videoChannelAutoFocusPrevMode = -1;
    int videoChannelAutoFocusUpDownPrevMode = -1;
    
    TwoStateButton *lowVoltageModeButton;

    HoverButton *hbMotorZFocusUp;
    HoverButton *hbMotorZFocusDown;
    HoverButton *hbMotorZFocusEllipse;
    HoverButton *hbMotorXStageLeft;
    HoverButton *hbMotorXStageRight;
    HoverButton *hbMotorYStageTop;
    HoverButton *hbMotorYStageBottom;
    HoverButton *hbMotorStageEllipse;
    
    int zLandingOff_stage = 0;
    bool zLandingWaitsMAF = false;
    
    QPushButton *increaseLaserSpotWindow;
    QGroupBox *spmMonitorGB;
    
    QHBoxLayout *mainLay;
    bool isLaserSpotWindowIncreased;
    QPointer<Notification> selection_notification;

    t3dChooser *plotChooser;
    
    QVBoxLayout *ExtraLay;
    QHBoxLayout *lay_inExtHor;
    QScrollArea *scrollAreaSchemeWid;
};
