#include "scanplot.h"
#include "defs.h"
#include "hardware/EG3000.h"
#include "container/datalist.hpp"
#include "gui/status.hpp"
#include "spmtunning.h"
#include "spm_motors.hpp"
#include "motors.hpp"
#include "nstslider.h"
#include "remote_nst_dataid.hpp"
#include "mobile/remote_mobile.hpp"
#include "dataplot.h"
#include "oscilloscope.h"
#include "ccd/ccdplot.h"

#include "3d/plot_3dplot.h"
#include "math/plot_SubtPlane.h"
#include "math/plot_MathProcessing.h"
#include "USBVideo/videoplayer.h"
#include <qwt_plot_marker.h>
#include <functional>
#include <numeric>
#include <algorithm>
#include <limits>
#include <QApplication>
#include <QSplitter>
#include <QTimer>
#include <QMessageBox>
#include <QSettings>
#include <QCheckBox>
#include <QDebug>
#include <QThread>
#include "widgets/threedoverlaybuttons.h"
#include "widgets/twodoverlaybuttons.h"
#include "options_window.h"

bool ScanPlot::eventFilter(QObject *Object, QEvent *Event)
{
    static int times = 0;
    if (times < 3 && Event->type() == QEvent::Paint)
    {
        resizeEvent();
        times++;
    }

    if(((QKeyEvent *)Event)->key() == Qt::Key_Escape)
        if (Plot->TwoDPlot->PlotControl->BCancel->isVisible())
            Plot->TwoDPlot->PlotControl->BCancel->animateClick();

    if (Event->type() == QEvent::KeyPress)
    {
        if ((((QKeyEvent *)Event)->key() == Qt::Key_Return || ((QKeyEvent *)Event)->key() == Qt::Key_Enter) &&
                Plot->overlayButtons->BBackButton->isHidden())
        {
            if (Plot->PlotControl->BSelectByRect->isChecked() ||
                Plot->overlayButtons->visibilityMode[Plot->ActivePlot] == plot::ThreeDOverlayButtons::svm_area_selection)
            {
                Plot->overlayButtons->blockResizeTimes = 20;
                QWidget * fw = qApp->focusWidget();

                if (!fw)
                {
                    Plot->PlotControl->BApply->animateClick();
                    return true;
                }
                else if (fw->whatsThis() == "")
                {
                    Plot->PlotControl->BApply->animateClick();
                    return true;
                }
            }
            else if (Plot->TwoDPlot->PlotControl->BApply->isVisible())
            {
                Plot->TwoDPlot->PlotControl->BApply->animateClick();
            }
        }
    }
    /*if(Event->type() == QEvent::GraphicsSceneResize)
    {
        QPushButton *button = qobject_cast<QPushButton*>(Object);
        if(button)
        {
            Event->ignore();
            return false;
        }
    }*/


    if (currExtScan && currExtScan->eventFilter(Object, Event))
        return true;

    QPointF top_lef_point;
    QPointF bot_right_point;
    if (Event->type() == plot::TSelectEvent::id)
    {
        static Range x, y;
        if (((plot::TSelectEvent *)Event)->State == plot::TSelectEvent::St_SelectionStarted)
        {
            x = Plot->ThreeDPlots.at(Plot->ActivePlot)->PlotsXLimits;
            y = Plot->ThreeDPlots.at(Plot->ActivePlot)->PlotsYLimits;
//            if (Plot->ActivePlot == videoChannelId)
//            {
//                x[0] = videoChannelAxisScaleXMin;
//                x[1] = videoChannelAxisScaleXMax[0];

//                y[0] = videoChannelAxisScaleYMin;
//                y[1] = videoChannelAxisScaleYMax[0];
//            }

            Plot->on_BRestoreScale_clicked();

            double w, h;
//            if (Plot->ActivePlot == videoChannelId && showingVideoFrame)
//            {
//                Plot->ThreeDPlots.at(videoChannelId)->SetPlotsXLimits(videoChannelAxisScaleXMin, videoChannelAxisScaleXMax[0]);
//                Plot->ThreeDPlots.at(videoChannelId)->SetPlotsYLimits(videoChannelAxisScaleYMin, videoChannelAxisScaleYMax[0]);
//            }
            Plot->ThreeDPlots.at(Plot->ActivePlot)->FindSizeWithAspectRatio(x[1] - x[0], y[1] - y[0], &w, &h);
//            if (Plot->ActivePlot == videoChannelId && showingVideoFrame)
//            {
//                updateVideoChannel();
//            }
            double s = qMax((x[1] - x[0]) / w, (y[1] - y[0]) / h);
            w = (w * s - (x[1] - x[0])) / 2; h = (h * s - (y[1] - y[0])) / 2;

            double w2 = w, h2 = h;
            s = qMax(0., -(x[0] - w));
            w -= s; w2 += s;
            s = qMax(0., -(y[0] - h));
            h -= s; h2 += s;
            s = qMax(0., (x[1] + w2) - LEdX->maximum()*1e-6);
            w += s; w2 -= s;
            s = qMax(0., (y[1] + h2) - LEdY->maximum()*1e-6);
            h += s; h2 -= s;

            for (int i = 0; i < Plot->ThreeDPlots.size(); i++)
            {
                //if (i != videoChannelId || !showingVideoFrame)
                {
                    Plot->ThreeDPlots[i]->SetCurrentXScales(x[0] - w, x[1] + w2);
                    Plot->ThreeDPlots[i]->SetCurrentYScales(y[0] - h, y[1] + h2);
                }
                Plot->ThreeDPlots[i]->ResizeQPlot();
                Plot->ThreeDPlots[i]->PlotWasChanged();

//                if (i == videoChannelId)
//                {
//                    videoChannelAxisScaleXMin = x[0] - w;
//                    videoChannelAxisScaleXMax[showingVideoFrame] = x[1] + w2;
//                    videoChannelAxisScaleYMin = y[0] - h;
//                    videoChannelAxisScaleYMax[showingVideoFrame] = y[1] + h2;
//                }
            }

			auto plot = Plot->ThreeDPlots[Plot->ActivePlot];
			plot->UpdateCanvasRect();
			plot->Updatekxky();
			plot->ReInitSelectRectGeometry(2.);
			plot->SetSelectWidth((x[1] - x[0]) / 2.);
			plot->SetSelectHeight((y[1] - y[0]) / 2.);

            qApp->sendPostedEvents(Plot, plot::TScalesChangedEvent::id);
            qApp->sendPostedEvents(this, plot::TScalesChangedEvent::id);

            LEdDeltaX->setEnabled(true);
            LEdDeltaY->setEnabled(true);
            m_BStart->setEnabled(false);
        }
        else if (((plot::TSelectEvent *)Event)->State == plot::TSelectEvent::St_SelectionCancled)
        {
            LEdDeltaX->setEnabled(false);
            LEdDeltaY->setEnabled(false);
            m_BStart->setEnabled(true);

            Plot->SetPlotsXScales(x[0], x[1]);
            Plot->SetPlotsYScales(y[0], y[1]);
            for (int i = 0; i < 4; i++)
                Plot->ThreeDPlots[i]->ResizeQPlot();

            QList<QVariant> list;
            list << Plot->ThreeDPlots[0]->PlotsXLimits[0]
                 << Plot->ThreeDPlots[0]->PlotsXLimits[1]
                 << Plot->ThreeDPlots[0]->PlotsYLimits[0]
                 << Plot->ThreeDPlots[0]->PlotsYLimits[1]
                 << 1e6 * ((plot::TSelectEvent *) Event)->SelectedRect.width();
            auto mob = MobileServer::GetIt();
            if (mob && mob->need_send())
                mob->setPropIdVar(DataID::plot3DAreaSelected, list);
        }
        else if (((plot::TSelectEvent *)Event)->State == plot::TSelectEvent::St_SelectionApplied)
        {
            QList<QVariant> list;
            list << Plot->ThreeDPlots[0]->PlotsXLimits[0]
                 << Plot->ThreeDPlots[0]->PlotsXLimits[1]
                 << Plot->ThreeDPlots[0]->PlotsYLimits[0]
                 << Plot->ThreeDPlots[0]->PlotsYLimits[1]
                 << 1e6 * ((plot::TSelectEvent *) Event)->SelectedRect.width();
            auto mob = MobileServer::GetIt();
            if (mob && mob->need_send())
                mob->setPropIdVar(DataID::plot3DAreaSelected, list);
        }
        if (LEdDeltaX->isEnabled() && LEdDeltaY->isEnabled())
        {
            LEdDeltaX->setValue(1e6 * ((plot::TSelectEvent *) Event)->SelectedRect.width());
            LEdDeltaY->setValue(1e6 * ((plot::TSelectEvent *) Event)->SelectedRect.height());
            if (BXYLock->isOn && (Plot->ThreeDPlots.at(Plot->ActivePlot)->SelectAction == plot::TPlotSelectionData::MoveLeft ||
                                    Plot->ThreeDPlots.at(Plot->ActivePlot)->SelectAction == plot::TPlotSelectionData::MoveRight)) {
                LEdDeltaY->setValue(LEdDeltaX->value());
                Plot->SetSelectHeight(LEdDeltaY->value() / 1e6);
            }
            if (BXYLock->isOn && (Plot->ThreeDPlots.at(Plot->ActivePlot)->SelectAction == plot::TPlotSelectionData::MoveTop ||
                                    Plot->ThreeDPlots.at(Plot->ActivePlot)->SelectAction == plot::TPlotSelectionData::MoveBottom)) {
                LEdDeltaX->setValue(LEdDeltaY->value());
                Plot->SetSelectWidth(LEdDeltaX->value() / 1e6);
            }
        }
    }
    else if (Event->type() == plot::TCrossEvent::id)
    {
        LEdX->setValue(1e6 * ((plot::TCrossEvent *) Event)->Coord.x() );
        LEdY->setValue(1e6 * ((plot::TCrossEvent *) Event)->Coord.y() );
        sendXYPos();
    }
    else if (Event->type() == plot::TScalesChangedEvent::id)
    {
        top_lef_point   = QPointF(1e6 * ((plot::TScalesChangedEvent *) Event)->XLimits[0],
                                  1e6 * ((plot::TScalesChangedEvent *) Event)->YLimits[0]);
        bot_right_point = QPointF(1e6 * ((plot::TScalesChangedEvent *) Event)->XLimits[1],
                                  1e6 * ((plot::TScalesChangedEvent *) Event)->YLimits[1]);

        double active_range0 = !scan_direction ? xRight - xLeft : yTop - yBottom;

        xLeft   = top_lef_point.x();
        xRight  = bot_right_point.x();
        yTop    = bot_right_point.y();
        yBottom = top_lef_point.y();

        double active_range1 = !scan_direction ? xRight - xLeft : yTop - yBottom;
//        if (active_range1 > 0) {
//            user_scan_speed *= active_range0 / active_range1; //preserve tip speed
//            if (rectSlFbp->Value() != user_scan_speed)
//            {
//                rectSlFbp->setValue(user_scan_speed);
//                rectSlFbp->setValue_man(user_scan_speed);
//            }
//        }


        LEdDeltaX->setValue(1e6 * (((plot::TScalesChangedEvent *) Event)->XLimits[1] - ((plot::TScalesChangedEvent *) Event)->XLimits[0]));
        LEdDeltaY->setValue(1e6 * (((plot::TScalesChangedEvent *) Event)->YLimits[1] - ((plot::TScalesChangedEvent *) Event)->YLimits[0]));

        if (((plot::TScalesChangedEvent *)Event)->Reason == plot::TScalesChangedEvent::R_BApply_clicked ||
                ((plot::TScalesChangedEvent *)Event)->Reason == plot::TScalesChangedEvent::R_BRestoreScale_clicked)
        {
            for (int i = 0; i < 4; i++)
                if (Plot->ThreeDPlots[i]->isVisible())
                    Plot->ThreeDPlots[i]->ResizeQPlot();

            LEdDeltaX->setEnabled(false);
            LEdDeltaY->setEnabled(false);
            m_BStart->setEnabled(true);
        }
        check_speed();
    }
    if (Event->type() == QEvent::Close) {
        closeEvent((QCloseEvent *)Event);
        return true;
    }



    return QWidget::eventFilter(Object, Event);
}


//----------------------------------------------------------
#include "ccd/ccdplot.h"
int ch_frac(bool on) { return on ? 1 : 0; }
void ScanPlot::scan_start_stop(bool value)
{
    user_scan_speed = rectSlFbp->Value();

    dataScan->clear();
    plot::ResizeOwner(*dataScan, 4);
    for (int i = 0; i < 4; i++) {
        dataScan->at(i)->clear();
        dataScan_src->at(i)->clear();
    }

    dataScan_X->clear();
    dataScan_Y->clear();
    dataScan_Xb->clear();
    dataScan_Yb->clear();

    lsbData.clear();

    for (int i = 0; i < 4; i++)
        scan_ch[i] = CBChannel[i]->itemData(CBChannel[i]->currentIndex()).toInt();

    bool saveToData = true;
    QList<QVariant> list;
    list << value << saveToData;
    MobileServer::sendData(DataID::ScanOnOff, list);

    channelsActive = qMax<int>(1, 4 - std::count(scan_ch, scan_ch+4, srcNone));
    if (value && std::count(scan_ch, scan_ch+4, srcNone) != 4)
    {
        scanFailed = false;

//        if (showingVideoFrame)
//            scan_ch[videoChannelId] = 1;

        /*Plot->Split3d->setSizes(QList<int>() << ch_frac(scan_ch[0] != srcNone || scan_ch[1] != srcNone) << ch_frac(scan_ch[2] != srcNone || scan_ch[3] != srcNone));
        Plot->Containers.at(0)->setSizes(QList<int>() <<
                                         ch_frac(scan_ch[0] != srcNone || (scan_ch[1] != srcNone && (scan_ch[2] != srcNone || scan_ch[3] != srcNone))) <<
                                         ch_frac(scan_ch[1] != srcNone || (scan_ch[0] != srcNone && (scan_ch[2] != srcNone || scan_ch[3] != srcNone))));
        Plot->Containers.at(1)->setSizes(QList<int>() <<
                                         ch_frac(scan_ch[2] != srcNone || (scan_ch[3] != srcNone && (scan_ch[0] != srcNone || scan_ch[1] != srcNone))) <<
                                         ch_frac(scan_ch[3] != srcNone || (scan_ch[2] != srcNone && (scan_ch[0] != srcNone || scan_ch[1] != srcNone))));*/

        if (scan_ch[Ch_activated] == srcNone)
            for (int i = 0; i < 4; i++)
                if (scan_ch[i] != srcNone) {
                    Plot->SetActivePlot(i);
                    break;
                }

//        if (showingVideoFrame)
//            scan_ch[videoChannelId] = 0;

        onBApply_clicked();

        FillDataAttributes();

        SetSectionUnits();

        sendXYsync(stPointX, stPointY, CBScanner->currentIndex());

        for (int i = 0; i < 4; i++) {
            if (!cbContScan->isChecked()) { //continuous scan
				auto plot = Plot->ThreeDPlots[i];
				plot->MinSliderVal = 0;
				plot->MaxSliderVal = 1;
				plot->AutoZScaleOn = true;
				plot->ReSetAxisScales();
            }

            scanLineInd[i]->setXValue(stPointX);
            scanLineInd[i]->setYValue(stPointY);
            scanLineInd[i]->setLineStyle(scan_direction ? QwtPlotMarker::VLine : QwtPlotMarker::HLine);
        }
        if (!cbContScan->isChecked()) { //continuous scan
            Plot->Slider->Sliders->SetMinMaxRelSlidersValues(0, 1);
            Plot->Slider->Sliders->setAutoMode(true);
            Plot->Refresh();
        }

        try {
            int sc[4];
            for (int i = 0; i < 4; i++) {
                auto data = dataScan->at(i);
                sc[i] = (scan_ch[i] != srcNone) && (scan_ch[i] != srcCCD);
                if (sc[i] || fast_scan_correction)
                {
                    //fixme why there 'at(0)', maybe it supposed to be 'at(i)' ?
                    auto plot = Plot->ThreeDPlots.at(0);
                    data->XLimits = plot->PlotsXLimits;
                    data->YLimits = plot->PlotsYLimits;
                    data->setSize(Nx_points, Ny_points);
                    data->LimsForMathX[1] = data->LimsForMathY[1] = 0;
                }
                if (scan_ch[i] != srcNone || fast_scan_correction)
                    scanData[scanner_scan][i].prepend(data);
            }

            if (fast_scan_correction && (sc[0] || sc[1])) {
                *dataScan_X = dataScan->at(0)->Array;
                *dataScan_Y = dataScan->at(1)->Array;
            }
            if (fast_scan_correction && (sc[2] || sc[3])) {
                *dataScan_Xb = dataScan->at(2)->Array;
                *dataScan_Yb = dataScan->at(3)->Array;
            }

            if (fast_scan_correction) {
                if (dataScan_src == dataScan) {
                    dataScan_src = new plot::T3dDataOwner;
                    plot::ResizeOwner(*dataScan_src, 4);
                }
                for (int i = 0; i < 4; i++)
                    *dataScan_src->at(i) = *dataScan->at(i);
            } else if (dataScan_src != dataScan) {
                delete dataScan_src;
                dataScan_src = dataScan;
            }
        } catch (std::bad_alloc) {
            for (int i = 0; i < 4; i++) {
                dataScan->at(i)->clear();
                dataScan_src->at(i)->clear();
            }
            dataScan_X->clear();
            dataScan_Y->clear();
            dataScan_Xb->clear();
            dataScan_Yb->clear();

            new Notification(tr("Scan") + ": " + tr("Out of memory"), Notification::Bad, 60);
            doBeep();

            scanFailed = true;
            m_BStart->change_state_man_with_sig(false);
            return;
        }
        for (int i = 0; i < 8; i++) {
            if (ScanDataPart[i])
                ScanDataPart[i]->Line = 65000;
            if (ScanDataPart_lsb[i])
                ScanDataPart_lsb[i]->Line = 65000;
        }

        Plot->PlotControl->BCrossection->setChecked(false);
        Plot->PlotControl->BMarkers->setChecked(false);

        if (sectionEnabled) {
            Plot->TwoDPlot->Pure2dPlot->SourceData.clear();
            Plot->TwoDPlot->Pure2dPlot->SourceData.resize(4);
            Plot->TwoDPlotVertical->Pure2dPlot->SourceData.clear();
            Plot->TwoDPlotVertical->Pure2dPlot->SourceData.resize(4);
        }

        foreach (QWidget *wid, scanControlWidgets)
            wid->setEnabled(false); // !m_BStart->on_off

        for (int i = 0; i < 4; i++) {
            refresh_3d_limit_timer[i].invalidate();
            line_number[i] = 0;
            if (scan_ch[i] != srcNone) {
                ready[i] = false;
                nlcupd[i] = 0;
            }
        }

        if (!scan_bar) {
            scan_bar = new StatusBar(tr("Scan"), scan_direction ? Nx_points : Ny_points);
            connect(scan_bar, SIGNAL(canceled()), this, SLOT(s_stopScan()));
            scan_bar->add();
        }
        scan_bar->setValue(0);
        scan_bar->cancel->setEnabled(m_BStart->isEnabled());

        scan_on_off = 1;
        reset_scan_test();
        if (!currExtScan) {
            sendStartStop = true;
        } else {
            bool ok = currExtScan->init(CBPointsOXext->currentText().toInt(), CBPointsOYext->currentText().toInt(), scan_dir_st_point,
                                        Plot->ThreeDPlots.at(0)->PlotsXLimits, Plot->ThreeDPlots.at(0)->PlotsYLimits);

            if (ok) { //check for canceled in fast scan preparation
                if (!ccd_scan_bar) {
                    ccd_scan_bar = new StatusBar(tr("External scan"), CBPointsOXext->currentText().toInt()*CBPointsOYext->currentText().toInt());
                    connect(ccd_scan_bar, SIGNAL(canceled()), this, SLOT(s_stopScan()));
                    ccd_scan_bar->add();
                }
                ccd_scan_bar->setValue(0);

                sendExtStartStop = true;
            } else {
                //TODO silent if empty?
                new Notification(tr("External scan") + ": " + tr("Scan init error:") + " " + currExtScan->lastError, Notification::Bad, 10);
                doBeep();

                destructStatus(ccd_scan_bar);
                destructStatus(scan_bar);

                scanFailed = true;
                m_BStart->change_state_man_with_sig(false);
            }
        }

        if (Plot->showPlotControl)
        {
            Plot->PlotControl->show();
            Plot->PlotControl->hide();
        }
    }
    else
    {
        //TODO crop new data on cancel, remove crop from plot Save()
        for (int n = 0; n < 4; n++) {
            if (removeOldData(n))
                Plot->ThreeDPlots[n]->SourceData = plot::getUnLock(scanData[scanner_scan][n]);
        }
        Plot->Refresh();

        if (std::count(scan_ch, scan_ch+4, srcNone) == 4) {
            m_BStart->change_state_man(false);
        } else
        {
            scan_on_off = 0;
            reset_scan_test();
            if (CBExternalMode->currentIndex() == 0) {
                sendStartStop = !scanFailed;
            } else {
                sendExtStartStop = !scanFailed;
                currExtScan->finish();
            }

            Plot->Refresh();

            destructStatus(ccd_scan_bar);
            destructStatus(scan_bar);

            foreach (QWidget *wid, scanControlWidgets)
                wid->setEnabled(true); // !m_BStart->on_off

            sendXYsync((xLeft + xRight) / 2, (yTop + yBottom) / 2, CBScanner->currentIndex());

            if (DACcurrentNum > 0) {
                DACcurrentNum = EDACcount->value()-1;//properly reset dacscan
                sl_dacscan_advance();
            }
            //if (GrBDACscan->isChecked())
            //    GrBDACscan->setChecked(false); //do not want autosave triggering advance

            FillDataAttributes2();
            if (Autosend && !scanFailed)
                AutosaveTimer->start(100);
        }

        DACcurrentNum = 0;
        for (int i = 0; i < 4; i++) {
            ready[i] = true;
            nlcupd[i] = 0;
        }
    }
    scanFailed = false;

    if (value)
    {
        Plot->setScanSPM2DPlot(true);
    }

    Plot->TwoDPlot->overlayButtons->B4DSelectCapturedSpectr->setEnabled(!value);

    //-------------------------------
    QElapsedTimer t; t.start();
    while (t.elapsed() < 200) {
        qApp->processEvents();
        SleepExport::msleep2(10);
    }

    if (!Plot->PlotControl->isVisible())
    {
        if (Plot->showingScan2DPlot)
        {
            if (!Plot->TwoDPlot->PlotControl->isVisible() && Plot->TwoDPlot->showPlotControl)
                Plot->TwoDPlot->PlotControl->show();
        }
        else {
            if (!Plot->TwoDPlotSPM->PlotControl->isVisible())
                Plot->TwoDPlotSPM->PlotControl->show();
        }
    }


    if (value)
    {
        Plot->TwoDPlot->Pure2dPlot->SourceData[0] = (currExtScan) ? currExtScan->Section.at(0) : Section_data.at(0);

        Plot->TwoDPlot->Refresh();
        Plot->TwoDPlotVertical->Refresh();
        Plot->Set2dPlotVisible(true);
    }

    #ifdef _SCAN_SLICE
    scan_slice();
#endif
//    auto osc = Oscilloscope::GetIt();
//    if (osc)
//        osc->MyPl->SetAutoScalingEnabled(false);
}

#ifdef _SCAN_SLICE
Range* ScanPlot::getScanSliceRangeX()
{
    auto stPlot = Plot->ThreeDPlots.first();
    auto opt = options::GetIt();
    auto slice = opt->optWin->edScanZoomSlice->value();
    auto sliceHalf = slice*0.5;
    auto lim = stPlot->PlotsXLimits;
    auto numX = lim.arr[1] - lim.arr[0];
    auto sliceValue = (numX*sliceHalf)/100.0;
    double minX = lim.arr[0] + sliceValue;
    double maxX = lim.arr[1] - sliceValue;

    return new Range(minX, maxX);
}
Range* ScanPlot::getScanSliceRangeY()
{
    auto stPlot = Plot->ThreeDPlots.first();
    auto opt = options::GetIt();
    auto slice = opt->optWin->edScanZoomSlice->value();
    auto sliceHalf = slice*0.5;
    auto lim = stPlot->PlotsYLimits;
    auto numY = lim.arr[1] - lim.arr[0];
    auto sliceValue = (numY*sliceHalf)/100.0;
    double minY = lim.arr[0] + sliceValue;
    double maxY = lim.arr[1] - sliceValue;

    return new Range(minY, maxY);
}

void ScanPlot::scan_slice()
{
    auto stPlot = Plot->ThreeDPlots.first();
    auto opt = options::GetIt();
    bool enableSlice = opt->optWin->chEnableScanSlice->isChecked();
    auto slice = opt->optWin->edScanZoomSlice->value();
    auto sliceHalf = slice*0.5;

    Range *xLims = getScanSliceRangeX();
    Range *yLims = getScanSliceRangeY();

    for (int i=0; i<Plot->ThreeDPlots.count(); i++)
    {
        auto plot = Plot->ThreeDPlots.at(i);

        plot->sliceBounds = enableSlice;
        plot->sliceX.arr[0] = xLims->arr[0];
        plot->sliceX.arr[1] = xLims->arr[1];
        plot->sliceY.arr[0] = yLims->arr[0];
        plot->sliceY.arr[1] = yLims->arr[1];
        plot->slicePercent = sliceHalf;

        if (enableSlice)
        {
            plot->SetCurrentXScales(xLims->arr[0], xLims->arr[1]);
            plot->SetCurrentYScales(yLims->arr[0], yLims->arr[1]);
            plot->ReAssignPlotOptions();
            plot->Replot();
            plot->PlotWasChanged();
        }
    }

    auto mob = MobileServer::GetIt();
    if (mob && mob->need_send())
    {
        auto plot = Plot->ThreeDPlots.at(0);
        QList<QVariant> list;
        list << enableSlice;
        list << xLims->arr[0];
        list << xLims->arr[1];
        list << yLims->arr[0];
        list << yLims->arr[1];
        list << plot->XYAxisUnit.Multiplier;
        mob->setPropIdVar(DataID::plot3DRestoreScale, list);
    }
}
#endif
void ScanPlot::s_stopScan()
{
    if (m_BStart->on_off)
        m_BStart->change_state_man_with_sig(false);
}

void ScanPlot::sl_stop_scan()
{
    if (!std::accumulate(ready, ready+4, true, std::logical_and<bool>()))
        return;

    m_BStart->change_state_man(false);
    bool saveToData = true;
    QList<QVariant> list;
    list << false << saveToData;
    MobileServer::sendData(DataID::ScanOnOff, list);

    if (currExtScan)
        currExtScan->finish();

    if (cbContScan->isChecked()) { //continuous scan
        int idx = scan_dir_st_point;
        switch (idx) {
        case 0:
            idx = 4;
            break;
        case 1:
            idx = 3;
            break;
        case 2:
            idx = 6;
            break;
        case 3:
            idx = 1;
            break;
        case 4:
            idx = 0;
            break;
        case 5:
            idx = 7;
            break;
        case 6:
            idx = 2;
            break;
        case 7:
            idx = 5;
            break;
        }

        CBStartPos->setCurrentIndex(idx);
        if (!is_dacscan()) {
            m_BStart->change_state_man_with_sig(true);
            return;
        }
    }

    if (!is_dacscan())
        for (int i = 0; i < 4; i++)
            Plot->ThreeDPlots[i]->ReSetAxisScales();
    Plot->Refresh();

    //TODO deduplicate gui enable/disable code with scan_start_stop
    destructStatus(ccd_scan_bar);
    destructStatus(scan_bar);

    foreach (QWidget *wid, scanControlWidgets)
        wid->setEnabled(true); // !m_BStart->on_off

    sendXYsync((xLeft + xRight) / 2, (yTop + yBottom) / 2, CBScanner->currentIndex());

    FillDataAttributes2();
    if (Autosend) {
        int delay = 100;
        if (cbContScan->isChecked()) {
            double scans_per_sec = rectSlFbp->Value() / 2 / (scan_direction ? Nx_points : Ny_points);
            if (scans_per_sec >= 1.0)
                delay = 10 + 100 / (scans_per_sec+0.1);
        }
        AutosaveTimer->start(delay); //starts sl_dacscan_advance too
    } else
        sl_dacscan_advance();

    doBeep();
    #ifdef _SCAN_SLICE
    scan_slice();
#endif
}
void ScanPlot::slAutosaveData()
{
    auto data = DataPlot::GetIt();
    if (!Autosend || !data)
        return;

    if (currExtScan)
        currExtScan->save();

    QMessageBox mb(QMessageBox::NoIcon, tr("Autosend to data"), "0/4 " + tr("Please wait."));
    mb.setStandardButtons(nullptr);
    bool mbon = !cbContScan->isChecked() || !is_dacscan(); //this eats up to 100ms directly, and as much indirectly
    if (mbon) {
        mb.show();
        mb.repaint();
    }

    for (int i = 0; i < 4; i++) {
        if (scan_ch[i] != srcNone) {
            TContainer *c = data->Container;

            Plot->AutoSave(i);

            while (c->CurrentDataList && c->CurrentDataList->ClickedItem &&
                   c->CurrentDataList->ClickedItem->UpdatePreview) {
                qApp->processEvents();
                SleepExport::msleep2(1);
                c->UpdateHierarchy();
            }
        }

        if (mbon) {
            mb.setText(QString::number(i+1) + "/4 " + tr("Please wait."));
            mb.repaint();
        }
    }

//    auto mob = MobileServer::GetIt();
//    if (mob && mob->need_send())
//    {
//        mob->setPropIdVar(DataID::plot3DAutoSave);
//        mob->setPropIdVar(DataID::plot2DAutoSave);
//    }

    if (mbon)
        mb.hide();
}

bool ScanPlot::removeOldData(int n)
{
    if (scan_ch[n] == srcNone)
        return false;

    plot::T3dDataOwner &data = scanData[scanner_scan][n];
    if (!data.size())
        return false;

    bool removed = false;
    QRectF ThisRect = plot::ConstructRectF(Plot->ThreeDPlots.at(0)->PlotsXLimits, Plot->ThreeDPlots.at(0)->PlotsYLimits);

    double scans_per_sec = rectSlFbp->Value() / 2 / (scan_direction ? Nx_points : Ny_points);
    bool remove_incomplete_fastscan = !ready[n] && cbContScan->isChecked() && scans_per_sec >= 1.0;
    if (!data[0]->LimsForMathCorrect() ||
            data[0]->LimsForMathX[1] - data[0]->LimsForMathX[0] < 3 ||
            data[0]->LimsForMathY[1] - data[0]->LimsForMathY[0] < 3 ||
            remove_incomplete_fastscan) { //remove these empty data instead
        for (int i = 1; i < data.size(); i++) {
            //only if have same old
            if (data[i] && ThisRect == plot::ConstructRectF(data[i]->XLimits, data[i]->YLimits)) {
                data.remove(0);
                removed = true;
                break;
            }
        }
    } else
        for (int i = 1; i < data.size(); i++) {
            //retain first smaller-but-not-same-size data
            if (data[i] && (i > 1 ? ThisRect.contains(plot::ConstructRectF(data[i]->XLimits, data[i]->YLimits)) :
                            ThisRect == plot::ConstructRectF(data[i]->XLimits, data[i]->YLimits))) {
                data.remove(i);
                removed = true;
                i--;
            }
        }

    return removed;
}

void ScanPlot::check_nx_ny()
{
    unsigned short NX = CBPointsOX->currentText().toUShort();
    unsigned short NY = CBPointsOY->currentText().toUShort();
    if (Nx_points == NX && Ny_points == NY)
        return;

    if (BXYLock->isOn) {
        if (sender() == CBPointsOX) {
            CBPointsOY->setCurrentIndex(CBPointsOX->currentIndex());
            NY = NX;
        }
        if (sender() == CBPointsOY) {
            CBPointsOX->setCurrentIndex(CBPointsOY->currentIndex());
            NX = NY;
        }
    }

    Nx_points = NX;
    Ny_points = NY;
    MobileServer::sendData(DataID::ScanNxNy, qSqrt(NX*NY));

    if (CBExternalMode->currentIndex() == 0 || BCCDXYLock->isOn) {
        CBPointsOXext->setCurrentIndex(CBPointsOX->currentIndex());
        CBPointsOYext->setCurrentIndex(CBPointsOY->currentIndex());
    }

    check_speed();

    auto mob = MobileServer::GetIt();
    if (mob && mob->need_send())
    {
        auto plot = Plot->ThreeDPlots[0];
        QList<QVariant> list;
        list << CBPointsOX->currentIndex();
        list << LEdDeltaX->value();
        list << LEdDeltaX->isEnabled();
        list << plot->XYAxisUnit.Legend;
        list << plot->XYAxisUnit.Multiplier;
        mob->setPropIdVar(DataID::plot3DChangeSize, list);
    }
}
void ScanPlot::check_speed()
{
    bool mobileSend = false;
    auto mob = MobileServer::GetIt();
    if (mob && mob->need_send())
        mobileSend = true;

    QSettings settings("Param/controller.conf", QSettings::IniFormat);
    auto osc = Oscilloscope::GetIt();
    if (!osc)
        return;

    if (sender() == rectSlFbp)
        user_scan_speed = rectSlFbp->Value();

    double scan_points = !scan_direction ? Nx_points : Ny_points; //double to skip conversion for division
    double range = !scan_direction ? (CBScanner->currentIndex() == 0 ? Xmax : Xmax2) :
                                     (CBScanner->currentIndex() == 0 ? Ymax : Ymax2);
    double active_range = !scan_direction ? xRight - xLeft : yTop - yBottom;


    //hw limits
    int scan_rate = 20000;
    int jm_rate = 20000;
    int fastscan_rate = 320000; //real limit is data sampling rate, not capture rate
    int csb_rate = !osc ? 10000 : osc->CalibWidget->get_csb_speed(); //fixme !osc is always false

    int scan_cyc_min = 0; //minimum
    int scan_cyc_perpoint = 2; //latency
    int pid_cyc_perpoint = 2; //target >1
    int pid_dv_perpoint = 150; //fw-dependent
    if (l2k_controller) {
        jm_rate = 312500;
        scan_rate = 60000;
        pid_dv_perpoint = 16;
    } else
    if (settings.value("use_eg4000", true).toBool())
    {
        scan_cyc_min = 1;
        scan_cyc_perpoint = 3;
        pid_dv_perpoint = 150 / 4; //150 in 18b lsb, = 37.5
    }

    double rate_limit_scan_cycles = scan_rate / scan_points / (scan_cyc_min + scan_cyc_perpoint);
    int limit_pid_cycles = qMax(pid_cyc_perpoint, qCeil(65535 * active_range / range / scan_points / pid_dv_perpoint));
    double rate_limit_pid_dv = csb_rate / scan_points / limit_pid_cycles;
    double rate_limit_fastscan = fastscan_rate / scan_points;

    double limit = rate_limit_scan_cycles;
    bool cl_on = !osc ? false : (CBScanner->currentIndex() == 0 ? osc->CalibWidget->CL_status_xy : osc->CalibWidget->CL2_status_xy); //fixme !osc is always false
    if (cl_on)// && !fast_scan_correction) //not interested in bad scan
        limit = qMin(limit, rate_limit_pid_dv);
    bool fastscan_possible = !cl_on && l2k_controller_v22 && scan_points <= 1000 &&
            !cbLiftMode->isChecked() && !currExtScan &&
            pointSync->currentIndex() == 0 && !BPreciseScan->isOn;
    if (fastscan_possible)
        limit = qMax(limit, rate_limit_fastscan);

    double max_scan_rate = settings_double(settings, "limit_scan_rate", 10);
    max_scan_rate *= qMax(1., range / active_range); //faster for smaller scans
    {
        double new_max = qMin(limit, max_scan_rate);
        double target_val = qMax(rectSlFbp->Value(), user_scan_speed);
        if (rectSlFbp->GetMaximum() != new_max)
        {
            rectSlFbp->SetMaxValue(new_max);
            if (mobileSend)
                mob->setPropIdVar(DataID::rectMaxSpeed, new_max);
        }
        if (rectSlFbp->Value() != target_val)
        {
            rectSlFbp->setValue(target_val);
        }

        if (mobileSend)
            mob->setPropIdVar(DataID::rectSpeed, rectSlFbp->Value());
    }


    //closest available rates
    double srate = qMax(0.01, rectSlFbp->Value());
    int rate = cl_on ? csb_rate : scan_rate;
    double raw_cycles = rate / scan_points / srate;
    int cycles = qMax(cl_on ? limit_pid_cycles : scan_cyc_min + scan_cyc_perpoint, qRound(raw_cycles));
    double real_rate = rate / scan_points / cycles;
    requested_scan_speed = rate / scan_points / (0.9 + cycles - (cl_on ? 0 : scan_cyc_perpoint));

    //double rate_max_requested = rate_limit_scan_cycles * (scan_cyc_min + scan_cyc_perpoint) / (0.9 + 0);
    double rate_max_requested = scan_rate / scan_points / (0.9 + 0); //equals the one above
    if (fastscan_possible && srate > rate_max_requested
            /*srate > rate_limit_scan_cycles*/ /*requested_scan_speed > rate_max_requested*/) {
        requested_scan_speed = srate;
        real_rate = srate;
    }
    requested_scan_speed = qMax(0.01, requested_scan_speed);
    if (sender() == rectSlFbp)
        sendSpeed = true;

    //jm & mv rate limiting warning
    //bool rate_warn_on = LScanRateWarn->isVisibleTo(LScanRateWarn->parentWidget());
    //if (!rate_warn_on)
        return;

    /*double max_jm_rate = 10000;
    auto spm = SPMTunning::GetIt();
    if (spm && spm->BFeedbackOnOffMode->on_off)
        max_jm_rate = jm_rate / 2 / (spm->DCfbHeight->value() / (1e-3 * qMax(spm->DCfbPrec->value(), l2k_controller ? 0.01 : 0.3)) )
                / scan_points;


    bool jm_limited = real_rate / max_jm_rate > 1.07;
    bool mv_limited = srate / real_rate > 1.07 || srate / real_rate < 0.93;

    if (DCScanRate->slider->marks.size() || fastscan_possible) {
        DCScanRate->slider->marks.clear();
        if (fastscan_possible)
            DCScanRate->slider->marks.append(QPair<double, QColor>(rate_max_requested, Qt::red));
        DCScanRate->slider->update();
    }*/
    /*if (mv_limited && rate_warn_on) {
        Qt::GlobalColor clrs[3] = {Qt::darkGray, Qt::green, Qt::darkGray};
        for (int i = -1; i <= 1; i++)
            DCScanRate->slider->marks.append(QPair<double, QColor>(
                                                 rate / scan_points / (cycles + i)
                                                 , clrs[i+1]));
        DCScanRate->slider->update();
    }*/

    //double mins = (!scan_direction ? Ny_points : Nx_points) / qMin(real_rate, max_jm_rate) / 60;

    /*LScanRateWarn->setEnabled(mv_limited || jm_limited);
    LScanRateWarn->setToolTip(tr("Effective scan rate: ") + QString::number(qMin(real_rate, max_jm_rate)) +
                              (jm_limited ? "\n" + tr("JM speed limitation") : QString()) +
                              (mv_limited ? (jm_limited ? "\n" + tr("Movement rate: ") + QString::number(real_rate) : QString())
                                            + tr(" - green") + "\n" + tr("next/prev available - grey") : QString()) +
                              (fastscan_possible ? "\n" + tr("fastscan boundary - red") : QString()) +
                              (mins > 5 ? "\n" + tr("Scan time: ") + QString::number(mins, 'f', 1) + tr("min") : QString()));*/
}

void ScanPlot::onBApply_clicked()
{
    for (int i = 0; i < 4; i++) {
        scan_ch[i] = CBChannel[i]->itemData(CBChannel[i]->currentIndex()).toInt();
        LmEn_ch[i] = LmSwitch[i]->isChecked();
    }

    scanner_scan = CBScanner->currentIndex();
    LmEn = cbLiftMode->isChecked();
    LmHeight = LEd_LFmodeHeight->value()/1000.;
    scan_precise_mode = BPreciseScan->isChecked();

    Nx_points = CBPointsOX->currentText().toUShort();
    Ny_points = CBPointsOY->currentText().toUShort();

    Step_X = fabs((xRight - xLeft) / ((float)Nx_points - 1));
    Step_Y = fabs((yTop - yBottom) / ((float)Ny_points - 1));

    scan_dir_st_point = CBStartPos->currentIndex();
    switch(scan_dir_st_point) {
    case 0:
    case 1:
        stPointX=xLeft;
        stPointY=yTop;
        break;
    case 2:
    case 3:
        stPointX=xRight;
        stPointY=yTop;
        break;
    case 4:
    case 5:
        stPointX=xLeft;
        stPointY=yBottom;
        break;
    case 6:
    case 7:
        stPointX=xRight;
        stPointY=yBottom;
        break;
    }

    sendConfig = true;

    ccdMode = CBExternalMode->currentIndex() == 0 ? 0 : BCCDXYLock->isOn ? 1 : 2;
    ccdNX = CBPointsOXext->currentText().toInt();
    ccdNY = CBPointsOYext->currentText().toInt();
    if (CBExternalMode->currentIndex() != 0)
        sendExtConfig = true;
}

//----------------------------------------------------------

void ScanPlot::Fill_Data_Array(plot::T2dArray& Array, const EG_ScanData *Scan_Data, const EG_ScanData *lsb_Scan_Data, plot::TCurveData *Section, bool raw)
{
    bool backward_line = false; //dirFW
    switch(scan_dir_st_point) {
    case spTopLeftAxisX:
    case spTopRightAxisX:
    case spTopRightAxisY:
    case spBottomRightAxisY:
        backward_line = true; //dirBW
        break;
    case spBottomRightAxisX:
    case spBottomLeftAxisX:
    case spTopLeftAxisY:
    case spBottomLeftAxisY:
        break;
    }

    bool directionY = (Scan_Data->Axis == AxisY);

    int ny = Array.size();
    int nx = ny > 0 ? Array[ny-1].size() : 0;
    int size = Scan_Data->ScanSize;
    size = directionY ? qMin(size, nx) : qMin(size, ny);
    int size2 = Scan_Data->Size;
    size2 = directionY ? qMin(size2, ny) : qMin(size2, nx);

    int line = Scan_Data->Line;
    line = qMin(line, (size-1));
    if (backward_line) line = (size - 1) - line;

    if (Section) {
        double start = directionY ? yBottom : xLeft;
        double step = directionY ? Step_Y : Step_X;

        Section->setXCalib(start * 1e-6, step * 1e-6);
        Section->YSeq.resize(size2);
    }

    for (int i = 0; i < size2; i++) {
        double val = Scan_Data->Data[i] + (lsb_Scan_Data ? ((double)lsb_Scan_Data->Data[i]) / 0x10000 : 0);
        if (!raw)
            val = input_calib(Scan_Data->Source, val);

        if (directionY)
            Array[i][line] = val;
        else
            Array[line][i] = val;

        if (Section)
            Section->YSeq[i] = val;
    }
}
void ScanPlot::sl_update_progress(QByteArray data)
{
    if (!m_BStart->on_off)
        return;

    const EG_ScanData *sdata = (const EG_ScanData *)data.constData();
    bool dir = sdata->Direction == dirBW;
    int ch = sdata->Channel + (dir ? 2 : 0);

    if (sdata->Scanner & 0x80) {
        lsbData = data;
        return;
    }

    const EG_ScanData *sdata2 = (const EG_ScanData *)lsbData.constData();
    if (lsbData.isEmpty() || sdata2->Channel % 4 != sdata->Channel)
        sdata2 = nullptr;

    bool refresh = false;
    if (fast_scan_correction) {
        int chx, chy;
        if (fast_scan_correction == 3) {
            chx = scanner_scan ? srcDACX2 : srcDACX;
            chy = scanner_scan ? srcDACY2 : srcDACY;
        } else {
            chx = scanner_scan ? srcCX2 : srcCX;
            chy = scanner_scan ? srcCY2 : srcCY;
        }
        if (sdata->Source == chx) {
            Fill_Data_Array(dir ? *dataScan_Xb : *dataScan_X, sdata, sdata2, nullptr, true);
            refresh = true;
        }
        if (sdata->Source == chy) {
            Fill_Data_Array(dir ? *dataScan_Yb : *dataScan_Y, sdata, sdata2, nullptr, true);
            refresh = true;
        }

        if (refresh) {
            refresh = false;
            int chh = 0 + (dir ? 2 : 0);
            for (int i = 0; i < 4; i++)
                if (scan_ch[chh+i] != srcNone &&
                        (++nlcupd[chh+i]) >= 2 && //increment xy ch upd counter
                        sdata->Channel >= 2) //this data only for xy buffer, need refresh
                {
                    if (ready[chh+i])
                        refresh_3d_limit_timer[chh+i].invalidate();
                    double scans_per_sec = rectSlFbp->Value() / 2 / (scan_direction ? Nx_points : Ny_points);
                    if (!(cbContScan->isChecked() && !ready[chh+i] && scans_per_sec >= 3.0)) {
                        refresh = true;
                        sl_scan_refresh_ch(chh+i);
                    }
                }
            if (refresh)
                //if (std::accumulate(ready, ready+4, true, std::logical_and<bool>()))
                    sl_stop_scan();
        }
        refresh = false;
    }

    if (sdata->Channel >= 2 || ch >= 4) {
        lsbData.clear();
        return;
    }

    line_number[ch]++;
    if (scan_bar != nullptr)
        scan_bar->setValue(line_number[ch]);

    if (line_number[ch] >= (scan_direction ? Nx_points : Ny_points)) {
        ready[ch] = true;
        refresh_3d_limit_timer[ch].invalidate();
    }

    bool is_ext = (ch == 1 && CBExternalMode->currentIndex() != 0);
    if (!is_ext) { //discard fake data
        Fill_Data_Array(dataScan_src->at(ch)->Array, sdata, sdata2, Section_data[ch].data());
        double scans_per_sec = rectSlFbp->Value() / 2 / (scan_direction ? Nx_points : Ny_points);
        if (!(fast_scan_correction && nlcupd[ch] < 2) &&
                !(cbContScan->isChecked() && !ready[ch] && scans_per_sec >= 3.0)) {
            refresh = true;
            sl_scan_refresh_ch(ch);
        }
    }

    lsbData.clear();

    if (!fast_scan_correction || is_ext || refresh) //we need that last nlc line displayed before end
    //if (std::accumulate(ready, ready+4, true, std::logical_and<bool>()))
        sl_stop_scan();
}

bool ScanPlot::processPacket(const BYTE addr, const BYTE sub_addr, const void *data, const DWORD )
{
    if (addr != ADR_CPU)
        return false;

    if (sub_addr == CPU_STATUS) {
        EG_Status *Deflectomter = (EG_Status *)data;
        QMetaObject::invokeMethod(this, "slUpdZscanner", Q_ARG(double, Deflectomter->Zpos / 65535.));
        return false;
    }

    if (sub_addr == CPU_SCANDATA ||
        sub_addr == CPU_SCANDATA_PART) {
        EG_ScanData *Scan_Data = (EG_ScanData *)data;

        if (sub_addr == CPU_SCANDATA_PART) {
            BYTE ch = Scan_Data->Channel;
            if (ch >= 8) {
                qDebug() << "scan part unexpected channel";
                return true;
            }

            EG_ScanData **bufarr = Scan_Data->Scanner & 0x80 ? ScanDataPart_lsb : ScanDataPart;
            EG_ScanData *buf = bufarr[ch];

            if (!buf) {
                buf = bufarr[ch] = (EG_ScanData *)malloc(132000);
                if (!buf) {
                    qDebug() << "scan part out of memory";
                    return true;
                }

                buf->Line = 65000;
            }

            if (buf->Line != Scan_Data->Line ||
                    buf->Direction != Scan_Data->Direction) { //TODO we can clear size after send, always sync other fields
                buf->Scanner = Scan_Data->Scanner;
                buf->Channel = Scan_Data->Channel;
                buf->Axis = Scan_Data->Axis;
                buf->Direction = Scan_Data->Direction;
                buf->Line = Scan_Data->Line;
                buf->ScanSize = Scan_Data->ScanSize;
                buf->Source = Scan_Data->Source;
                buf->Size = 0;
            }

            int sz = buf->Axis == AxisX ? Nx_points : Ny_points;

            for (int i = 0; i < Scan_Data->Size; i++) {
                if (buf->Direction == dirFW)
                    buf->Data[buf->Size + i] = Scan_Data->Data[i];
                else
                    buf->Data[sz - buf->Size - i - 1] = Scan_Data->Data[i];
            }
            buf->Size += Scan_Data->Size;

            if (buf->Size != sz)
                return true;

            Scan_Data = buf;
        }

        int dataSize = sizeof(EG_ScanData) - (MAX_LINE_SIZE - Scan_Data->Size)*sizeof(WORD);
        QMetaObject::invokeMethod(this, "sl_update_progress", Q_ARG(QByteArray, QByteArray((char *)Scan_Data, dataSize)));
        return true;
    }

    if (sub_addr == CPU_CCD_START) {
        QMetaObject::invokeMethod(this, "acquireNextPoint");
        return true;
    }

    return false;
}

//----------------------------------------------------------

void get_scan_lim(plot::TMin3dData *data, int dir, int line, int &x0, int &x1, int &y0, int &y1)
{
    x0 = y0 = x1 = y1 = 0;
    if (dir == spBottomLeftAxisX || dir == spBottomRightAxisX)
    {
        x1 = data->NXElems;
        y1 = line;
    }
    if (dir == spTopLeftAxisX || dir == spTopRightAxisX)
    {
        x1 = data->NXElems;
        y0 = data->NYElems - line;
        y1 = data->NYElems;
    }
    if (dir == spTopLeftAxisY || dir == spBottomLeftAxisY)
    {
        x1 = line;
        y1 = data->NYElems;
    }
    if (dir == spTopRightAxisY || dir == spBottomRightAxisY)
    {
        x0 = data->NXElems - line;
        x1 = data->NXElems;
        y1 = data->NYElems;
    }
}

void ScanPlot::sl_scan_refresh_ch(int n)
{

    bool is_ext = (n == 1 && CBExternalMode->currentIndex() != 0);
    if (!m_BStart->on_off && !is_ext)
        return;

    if (!is_ext && refresh_3d_limit_timer[n].isValid() && refresh_3d_limit_timer[n].elapsed() <= 100 + refresh_limit_last_elapsed[n]) {
        if (fast_scan_correction)
            nlcupd[n] = 0; //prevent next update while incomplete
        return; //~10 refreshes per second max
    }

    if (CBChannel[n]->currentIndex() == 0)
    {
        if (!Plot->TwoDPlot->Pure2dPlot->SourceData[n].isNull())
        {
            Plot->TwoDPlotVertical->Pure2dPlot->SourceData[n].clear();
            Plot->TwoDPlot->Pure2dPlot->SourceData[n].clear();
            Plot->TwoDPlot->Refresh();
            Plot->TwoDPlotVertical->Refresh();
        }
        return;
    }

    refresh_3d_limit_timer[n].start();
    refresh_limit_need_math_update[n] = true;

    if (sectionEnabled && Plot->TwoDPlot->Pure2dPlot->SourceData.size() == 4) {
        if (CBSec[n]->isChecked()) {
            Plot->TwoDPlot->Pure2dPlot->SourceData[n] = (is_ext && currExtScan) ? currExtScan->Section.at(0) : Section_data.at(n);
            Plot->TwoDPlotVertical->Pure2dPlot->SourceData[n].clear();
            Plot->TwoDPlotVertical->Pure2dPlot->SourceData[n] = (is_ext && currExtScan) ?
                    *(new QSharedPointer<plot::TCurveData>(currExtScan->Section.at(0)->clone())) :
                    *(new QSharedPointer<plot::TCurveData>(Section_data.at(n)->clone()));



            Plot->Set2dPlotVisible(true);
            Plot->TwoDPlot->Refresh();
            Plot->TwoDPlotVertical->Refresh();
        } else if (!Plot->TwoDPlot->Pure2dPlot->SourceData[n].isNull()) {
            Plot->TwoDPlotVertical->Pure2dPlot->SourceData[n].clear();
            Plot->TwoDPlot->Pure2dPlot->SourceData[n].clear();
            Plot->TwoDPlot->Refresh();
            Plot->TwoDPlotVertical->Refresh();
        }
    }

    bool need_refresh = true;

    if (fast_scan_correction) {
        nlcupd[n] = 0;
        bool fastscan_ok = sl_fast_scan_correction(n);
        if (!fastscan_ok && !is_ext)
            need_refresh = false;
    }

    if (!is_ext) {
        int x0, x1, y0, y1;
		auto data = dataScan->at(n);
        get_scan_lim(data.data(), scan_dir_st_point, line_number[n], x0, x1, y0, y1);
		data->LimsForMathX[0] = x0;
		data->LimsForMathX[1] = x1;
		data->LimsForMathY[0] = y0;
		data->LimsForMathY[1] = y1;
        if (line_number[n] < 2)
            need_refresh = false;
    } else if (currExtScan)
        scanData[scanner_scan][n][0] = currExtScan->Output3d.at(0);

    const plot::TMin3dData *Min3dData = scanData[scanner_scan][n].at(0).data();
    if (scan_direction) {
        Range r = Min3dData->LimsForMathX;
        if (is_ext) r[1]--; //current line is still incomplete
        double d = Min3dData->XLimits[0] + (r[0] != 0 ? r[0] : r[1]-1) * Min3dData->dx;
        scanLineInd[n]->setXValue(d);
    } else {
        Range r = Min3dData->LimsForMathY;
        if (is_ext) r[1]--; //current line is still incomplete
        double d = Min3dData->YLimits[0] + (r[0] != 0 ? r[0] : r[1]-1) * Min3dData->dy;
        scanLineInd[n]->setYValue(d);
    }

    if (ready[n] && dataScan_src != dataScan)
        dataScan_src->at(n)->clear();
    if (ready[0] && ready[1] &&
            nlcupd[0] == 0 && nlcupd[1] == 0) {
        dataScan_X->clear();
        dataScan_Y->clear();
    }
    if (ready[2] && ready[3] &&
            nlcupd[2] == 0 && nlcupd[3] == 0) {
        dataScan_Xb->clear();
        dataScan_Yb->clear();
    }

    if (ready[n]) { //cleanup previous hidden
        need_refresh = true;
        removeOldData(n);
    }

    if (need_refresh) {
        Plot->ThreeDPlots[n]->SourceData = plot::getUnLock(scanData[scanner_scan][n]);
        Plot->ThreeDPlots[n]->Refresh();
    } else
        Plot->ThreeDPlots[n]->Replot();

    //also included in plot refreshed signal handler, to accounts for math\replot etc.
    refresh_limit_last_elapsed[n] = refresh_3d_limit_timer[n].elapsed() / 0.8 * channelsActive; //time budget, %
}


//----------------------------------------------------------

void
data_field_area_fit_plane(double *data,
                          int col, int row,
                          int width, int height,
                          int xres, int /*yres*/,
                          double *pa, double *pbx, double *pby)
{
    double sumx, sumy, sumz, sumxx, sumyy, sumxy, sumxz, sumyz;
    double alpha1, alpha2, alpha3, beta1, beta2, gamma1;
    double det;
    double a, b, c;
    double x, y, z;
    double n;
    double *datapos = nullptr;
    double *drow = nullptr;
    int i, j;

    n = 0;

    sumx = sumy = sumz = sumxx = sumyy = sumxy = sumxz = sumyz = 0;
    datapos = data + row*xres + col;
    for (i = 0; i < height; i++) {
        drow = datapos + i*xres;
        for (j = 0; j < width; j++) {
            x = j;
            y = i;
            z = drow[j];

            sumx += x;
            sumy += y;
            sumz += z;
            sumxx += x*x;
            sumyy += y*y;
            sumxy += x*y;
            sumxz += x*z;
            sumyz += y*z;
            n++;
        }
    }

    det = (n*sumxx*sumyy) + (2*sumx*sumxy*sumy) - (sumx*sumx*sumyy)
            -(sumy*sumy*sumxx) - (n*sumxy*sumxy);

    /* try to return something reasonable in case of singularity */
    if (det == 0.0)
        a = b = c = 0.0;
    else
    {
        det = 1.0/det;

        alpha1 = (n*sumyy) - (sumy*sumy);
        alpha2 = (n*sumxx) - (sumx*sumx);
        alpha3 = (sumxx*sumyy) - (sumxy*sumxy);
        beta1 = (sumx*sumy) - (n*sumxy);
        beta2 = (sumx*sumxy) - (sumxx*sumy);
        gamma1 = (sumxy*sumy) - (sumx*sumyy);

        a = det*(alpha1*sumxz + beta1*sumyz + gamma1*sumz);
        b = det*(beta1*sumxz + alpha2*sumyz + beta2*sumz);
        c = det*(gamma1*sumxz + beta2*sumyz + alpha3*sumz);
    }

    if (pbx)
        *pbx = a;
    if (pby)
        *pby = b;
    if (pa)
        *pa = c;
}

void ScanPlot::sl_motor_level()
{
    auto spm = SPMTunning::GetIt();
    auto motors = Motors::GetIt();
    if (!motors || !spm)
        return;

    int idx = -1;
    bool is_inverted = true;
    for (int i = 0; i < 4; i++) {
        if (scan_ch[i] == srcHeight) {
            idx = i;
            break;
        }
    }
    if (idx == -1)
    for (int i = 0; i < 4; i++) {
        if (scan_ch[i] == (spm && spm->FB_Scanner ? srcCZ2 : srcCZ)) { //fixme spm is always true
            idx = i;
            is_inverted = false;
            break;
        }
    }

    if (idx == -1 || !scanData[scanner_scan][idx].size() || !scanData[scanner_scan][idx].at(0)->Array.size()) {
        new Notification(tr("Motor level") + ": " + tr("Please acquire some height data before."), Notification::Warn, 10);
        doBeep();

        return;
    }
    if (scanner_scan != scHead) {
        new Notification(tr("Motor level") + ": " + tr("Stage axes orientation is unknown."), Notification::Warn, 10);
    }

    const plot::TMin3dData *Min3dData = scanData[scanner_scan][idx].at(0).data();

    QVector<double> Points;
    for (int i = 0; i < Min3dData->Array.size(); i++)
        for (int j = 0; j < Min3dData->Array.at(i).size(); j++)
             Points.push_back(Min3dData->Array.at(i).at(j));

    unsigned int MinX = 0, MaxX = 0, MinY = 0, MaxY = 0;
    plot::math::TMathProcessing::SetupMinsMaxs(MinX, MaxX, MinY, MaxY, *Min3dData);

    double a, b, c;
    data_field_area_fit_plane(&*Points.begin(), MinX, MinY, MaxX - MinX, MaxY - MinY, Min3dData->Array.at(0).size(), 0, &c, &a, &b);
    qDebug() << a << b;

    a /= Min3dData->dx; b /= Min3dData->dy; //tg(a) = dz/dx

    a *= 136e3; b *= 104.5e3; // * head dx = motors dz, mkm

    if (is_inverted) { //inverted, as height = inverted dz
        a *= -1;
        b *= -1;
    }

    double m1 = 0, m2 = 0, m3 = 0;
    //x
    if (a > 0)
        m1 += fabsl(a);
    else
        m2 += fabsl(a);
    m3 += fabsl(a) / 2; //preserve y angle

    //y
    //inverted in new head
    if ((scanner_scan == scHead && is_optical_scanner[scanner_scan]) ? b > 0 : b < 0) {
        m3 += fabsl(b);
    } else {
        m1 += fabsl(b);
        m2 += fabsl(b);
    }
    qDebug() << m1 << m2 << m3;

    double speed = spm ? spm->OptionsForMotor->up_speed : 500; //fixme spm is always true
    if (m1 > 0.001)
        motors->goUp(motors->mapPhysToVirt(1), speed, m1);
    if (m2 > 0.001)
        motors->goUp(motors->mapPhysToVirt(2), speed, m2);
    if (m3 > 0.001)
        motors->goUp(motors->mapPhysToVirt(3), speed, m3);

    if (spm && /*spm->BFeedbackOnOff->on_off &&*/ m1+m2+m3 > 0.001) //fixme spm is always true
        new Notification(tr("You need to perform landing again after motors stop."), Notification::Warn, 30);
}


//----------------------------------------------------------

void ScanPlot::sl_ScanCorrection()
{
    int st = FindCheckedItemIn(FastScanCorrection->menu()->actions()[0]->actionGroup());
    if (st < 0) st = 0;
    FastScanCorrection->setIcon(makeHugeIcon(st > 0 ? ":/SPMTuning_window/FastScanCorrOff.png" : ":/SPMTuning_window/FastScanCorrOn.png"));
    fast_scan_correction = st;

    {
        QSettings settings("Param/controller.conf", QSettings::IniFormat);
        settings.setValue("fast_scan_correction", st);
    }

    auto osc = Oscilloscope::GetIt();
    if (!osc) return;

    if (st > 0) {
       osc->CalibWidget->Scanner_type->setCurrentIndex(CBScanner->currentIndex());
        if (osc->CalibWidget->fb_on_off_xy->isOn) {
            osc->CalibWidget->fb_on_off_xy->change_state_man_with_sig(false);
            fast_scan_CLZ_disabled = true;
        }
    } else if (fast_scan_CLZ_disabled) {
        fast_scan_CLZ_disabled = false;
        osc->CalibWidget->Scanner_type->setCurrentIndex(CBScanner->currentIndex());
        if (!osc->CalibWidget->fb_on_off_xy->isOn)
            osc->CalibWidget->fb_on_off_xy->change_state_man_with_sig(true);
    }
}

bool ScanPlot::sl_fast_scan_correction(int ch)
{
#define SCANCORR_DISTRIBUTE_1PX //extend to unused 1px range
#define SCANCORR_FIT_POLYNOM //reduces effect from large >1px noise
    if ((scan_ch[ch] == srcNone) ||
            (scan_ch[ch] == srcCCD) ||
            !dataScan_src->at(ch) ||
            !dataScan_src->at(ch)->Array.size())
        return false;


    plot::T2dArray dataScan_new;
    dataScan_new.resize(Ny_points);

    plot::T2dArray dataScan_fill;
    dataScan_fill.resize(Ny_points);

    QVector<double> Row;
    Row.resize(Nx_points);
    for (int Xind = 0; Xind < Nx_points; Xind++)
        Row[Xind] = 0;
    for (int Yind = 0; Yind < Ny_points; Yind++) {
        dataScan_new[Yind] = Row;
        dataScan_fill[Yind] = Row;
    }


    int Xmin = 0, Xmax = 0, Ymin = 0, Ymax = 0;
    get_scan_lim(dataScan_src->at(ch).data(), scan_dir_st_point, line_number[ch], Xmin, Xmax, Ymin, Ymax);
    Ymax = qMax(0, Ymax-1);
    Xmax = qMax(0, Xmax-1);

    Ymin = qMin(Ymin, dataScan_src->at(ch)->Array.size()-1);
    Ymax = qMin(Ymax, dataScan_src->at(ch)->Array.size()-1);
    Xmin = qMin(Xmin, dataScan_src->at(ch)->Array[Ymin].size()-1);
    Xmax = qMin(Xmax, dataScan_src->at(ch)->Array[Ymax].size()-1);

    plot::T2dArray *xdataptr = ch < 2 ? dataScan_X : dataScan_Xb;
    plot::T2dArray *ydataptr = ch < 2 ? dataScan_Y : dataScan_Yb;

    Ymin = qMin(Ymin, qMin((*xdataptr).size()-1, (*ydataptr).size()-1));
    Ymax = qMin(Ymax, qMin((*xdataptr).size()-1, (*ydataptr).size()-1));
    Xmin = qMin(Xmin, qMin((*xdataptr)[Ymin].size()-1, (*ydataptr)[Ymin].size()-1));
    Xmax = qMin(Xmax, qMin((*xdataptr)[Ymax].size()-1, (*ydataptr)[Ymax].size()-1));

    Ymin = qBound(0, Ymin, (int)Ny_points-1);
    Ymax = qBound(0, Ymax, (int)Ny_points-1);
    Xmin = qBound(0, Xmin, (int)Nx_points-1);
    Xmax = qBound(0, Xmax, (int)Nx_points-1);


    double min_value = std::numeric_limits<double>::max();

    auto osc = Oscilloscope::GetIt();
    if (!osc) return false;

    BYTE sc = scanner_scan == scHead ? AxisX : AxisX2;

	auto plot = Plot->ThreeDPlots.at(0);
    double xmin = plot->PlotsXLimits[0] * 1e6;
    double ymin = plot->PlotsYLimits[0] * 1e6;
    double xdiv = plot->PlotsXLimits[1] * 1e6 - xmin;
    double ydiv = plot->PlotsYLimits[1] * 1e6 - ymin;
    if (xdiv != 0) xdiv = 1. / xdiv;
    if (ydiv != 0) ydiv = 1. / ydiv;
    xdiv *= (Nx_points-1);
    ydiv *= (Ny_points-1);

    auto calibFPtr = (fast_scan_correction == 3 ?
                          &XYZCalibrations::reverse_nlc_voltage :
                          &XYZCalibrations::reverse_nlc_sensor);

    int fitpoints = scan_direction ? Ny_points : Nx_points;
    QScopedArrayPointer<double> xline(new double[fitpoints]);
    QScopedArrayPointer<double> yline(new double[fitpoints]);

#ifdef SCANCORR_FIT_POLYNOM
    if (!scan_direction) { //x
#endif
        for (int Yind = Ymin; Yind <= Ymax; Yind++) {
            for (int Xind = Xmin, c = 0; Xind <= Xmax; Xind++, c++) {
                xline[c] = xdiv * ((osc->CalibWidget->*calibFPtr)
                                      (AxisX + sc, (*xdataptr)[Yind][Xind]) - xmin);
                yline[c] = ydiv * ((osc->CalibWidget->*calibFPtr)
                                      (AxisY + sc, (*ydataptr)[Yind][Xind]) - ymin);
            }

#ifdef SCANCORR_FIT_POLYNOM
            double coeffsX[4], coeffsY[4];
            plot::math::dataLinePartFitPolynom(xline.data(), 3, coeffsX, 0, fitpoints);
            plot::math::dataLinePartFitPolynom(yline.data(), 3, coeffsY, 0, fitpoints);
            for (int Xind = Xmin, c = 0; Xind <= Xmax; Xind++, c++) {
                xline[c] = coeffsX[0] + c*(coeffsX[1] + c*(coeffsX[2] + c*coeffsX[3]));
                yline[c] = coeffsY[0] + c*(coeffsY[1] + c*(coeffsY[2] + c*coeffsY[3]));
            }
#endif

            for (int Xind = Xmin, c = 0; Xind <= Xmax; Xind++, c++) {
                int xind = qRound(xline[c]);
                int yind = qRound(yline[c]);

                if (xind >= 0 && xind < Nx_points && yind >= 0 && yind < Ny_points) {
                    if (min_value > dataScan_src->at(ch)->Array[Yind][Xind])
                        min_value = dataScan_src->at(ch)->Array[Yind][Xind];
                    dataScan_new[yind][xind] += dataScan_src->at(ch)->Array[Yind][Xind];
                    dataScan_fill[yind][xind]++;
                }
            }
        }
#ifdef SCANCORR_FIT_POLYNOM
    } else { //same for y-dir, x/y loops are swapped
        for (int Xind = Xmin; Xind <= Xmax; Xind++) {
            for (int Yind = Ymin, c = 0; Yind <= Ymax; Yind++, c++) {
                xline[c] = xdiv * ((osc->CalibWidget->*calibFPtr)
                                      (AxisX + sc, (*xdataptr)[Yind][Xind]) - xmin);
                yline[c] = ydiv * ((osc->CalibWidget->*calibFPtr)
                                      (AxisY + sc, (*ydataptr)[Yind][Xind]) - ymin);
            }

#ifdef SCANCORR_FIT_POLYNOM
            double coeffsX[4], coeffsY[4];
            plot::math::dataLinePartFitPolynom(xline.data(), 3, coeffsX, 0, fitpoints);
            plot::math::dataLinePartFitPolynom(yline.data(), 3, coeffsY, 0, fitpoints);
            for (int Yind = Ymin, c = 0; Yind <= Ymax; Yind++, c++) {
                xline[c] = coeffsX[0] + c*(coeffsX[1] + c*(coeffsX[2] + c*coeffsX[3]));
                yline[c] = coeffsY[0] + c*(coeffsY[1] + c*(coeffsY[2] + c*coeffsY[3]));
            }
#endif

            for (int Yind = Ymin, c = 0; Yind <= Ymax; Yind++, c++) {
                int xind = qRound(xline[c]);
                int yind = qRound(yline[c]);

                if (xind >= 0 && xind < Nx_points && yind >= 0 && yind < Ny_points) {
                    if (min_value > dataScan_src->at(ch)->Array[Yind][Xind])
                        min_value = dataScan_src->at(ch)->Array[Yind][Xind];
                    dataScan_new[yind][xind] += dataScan_src->at(ch)->Array[Yind][Xind];
                    dataScan_fill[yind][xind]++;
                }
            }
        }
    }
#endif

    bool was_empty = false;
    if (min_value == std::numeric_limits<double>::max()) {
        min_value = 0;
        was_empty = true;
    }

    //nearest pixels relative coordinates
    const int xseq[8] = {0, 0, 1, -1, 1, 1, -1, -1};
    const int yseq[8] = {1, -1, 0, 0, 1, -1, 1, -1};

    //fill empty with min_value, normalize averages, [interpolate gaps]
    for (int Yind=0; Yind<Ny_points; Yind++) {
        for (int Xind=0; Xind<Nx_points; Xind++) {
            if (dataScan_fill[Yind][Xind] < 0.5) { // ==0
#ifdef SCANCORR_DISTRIBUTE_1PX
                int count = 0;

                if (!was_empty)
                for (int i=0; i<8; i++) {
                    int _x = Xind + xseq[i];
                    int _y = Yind + yseq[i];
                    if (_x < 0 || _x >= Nx_points || _y < 0 || _y >= Ny_points)
                        continue;

                    if (dataScan_fill[_y][_x] > 0.5) { // !=0
                        dataScan_new[Yind][Xind] += dataScan_new[_y][_x] / dataScan_fill[_y][_x];
                        count++;
                    }
                }
                if (count != 0)
                    dataScan_new[Yind][Xind] /= count;
                else
#endif
                    dataScan_new[Yind][Xind] = min_value;
            } else {
                dataScan_new[Yind][Xind] /= dataScan_fill[Yind][Xind];
                dataScan_fill[Yind][Xind] = 1;
            }
        }
    }

    dataScan->at(ch)->Array = dataScan_new;
    return !was_empty;
}
