#include "spmtunning.h"
#include "spm_motors.hpp"
#include "edit.hpp"

#include <QTimer>
#include <QApplication>
#include <QCheckBox>
#include <QGroupBox>
#include <QComboBox>
#include <QVBoxLayout>
#include <QSettings>
#include <QProgressBar>
#include <QFormLayout>
#include "flowlayout.hpp"
#include "autocalib.hpp"
#include "oscilloscope.h"
#include "hardware/EG3000.h"
#include "gui/status.hpp"
#include "remote_nst_dataid.hpp"
#include "mobile/remote_mobile.hpp"
#include "hardware/shuttle.h"
#include "motors.hpp"
#include "scanplot.h"


typedef QPair<int, int> FBinfo;
Q_DECLARE_METATYPE(FBinfo)


TOptionsForMotor::TOptionsForMotor()
{
    up_speed = 0.;
    up_dist = 0.;
    land_speed = 0.;
    land_speed_close = 0.;
    land_close_perc = 0.;
    landUpward = false;
    LEdUpSpeed = new Edit(0, 10000);
    LEdUpDist = new Edit(0.1*1e3, 100*1e3, 1);
    LEdDistReady = new Edit(0.1*1e3, 100*1e3, 1);
    CB_HeadLanding = new QCheckBox();
    LEdHeadLandingExitDist = new Edit(0, 100*1e3, 1);
    LEdHeadLandingOffsetDist1 = new Edit(-100*1e3, 100*1e3, 1);
    LEdHeadLandingOffsetDist3 = new Edit(-100*1e3, 100*1e3, 1);
    CB_sysONMAF = new QCheckBox();
    connect(CB_sysONMAF, SIGNAL(stateChanged(int)), this, SLOT(sl_update()));
    connect(CB_HeadLanding, SIGNAL(stateChanged(int)), this, SLOT(sl_update()));
    connect(LEdUpSpeed, SIGNAL(newValue()), this, SLOT(sl_update()));
    connect(LEdUpDist, SIGNAL(newValue()), this, SLOT(sl_update()));
    connect(LEdDistReady, SIGNAL(newValue()), this, SLOT(sl_update()));
    connect(LEdHeadLandingExitDist, SIGNAL(newValue()), this, SLOT(sl_update()));
    connect(LEdHeadLandingOffsetDist1, SIGNAL(newValue()), this, SLOT(sl_update()));
    connect(LEdHeadLandingOffsetDist3, SIGNAL(newValue()), this, SLOT(sl_update()));

    LEdLandingSpeed = new Edit(0, 1000);
    LEdLandingSpeedClose = new Edit(0, 1000);
    LEdLandingSpeedCloseC = new Edit(0, 1, 2);
    connect(LEdLandingSpeed, SIGNAL(newValue()), this, SLOT(sl_update()));
    connect(LEdLandingSpeedClose, SIGNAL(newValue()), this, SLOT(sl_update()));
    connect(LEdLandingSpeedCloseC, SIGNAL(newValue()), this, SLOT(sl_update()));


    CB_landing_up = new QCheckBox(tr("Land upward"));
    connect(CB_landing_up, SIGNAL(clicked(bool)), this, SLOT(sl_update()));

    QHBoxLayout *HBLParams = new QHBoxLayout(this);

    QGroupBox *GrBUp = new QGroupBox(tr("SPM"), this);
    QHBoxLayout *layBUp = new QHBoxLayout(GrBUp);
    QFormLayout *VBLUpSpeed = makeFormLayout();
    QFormLayout *VBLUpSpeed2 = makeFormLayout();
    layBUp->addLayout(VBLUpSpeed);
    layBUp->addLayout(VBLUpSpeed2);

    QGroupBox *GrBLanding = new QGroupBox(tr("Landing"), this);
    QFormLayout *VBLLanding = makeFormLayout(GrBLanding);

    HBLParams->addWidget(GrBLanding, 1);
    HBLParams->addWidget(GrBUp, 1);

    VBLUpSpeed->addRow(tr("Speed (")+tr("micro")+tr("m/s)"), LEdUpSpeed);
    VBLUpSpeed->addRow(tr("Distance (mkm)"), LEdUpDist);
    VBLUpSpeed->addRow(tr("Distance ready (mkm)"), LEdDistReady);
    VBLUpSpeed->addRow(tr("Head landing"), CB_HeadLanding);
    VBLUpSpeed->addRow(tr("Head landing exit distance (mkm)"), LEdHeadLandingExitDist);
    VBLUpSpeed->addRow(tr("Head landing offset distance first motor (mkm)"), LEdHeadLandingOffsetDist1);
    VBLUpSpeed->addRow(tr("Head landing offset distance third motor(mkm)"), LEdHeadLandingOffsetDist3);
    VBLUpSpeed->setContentsMargins(6, 3, 6, 6);

    VBLUpSpeed2->addRow(tr("System ON + autofocus"), CB_sysONMAF);

    VBLLanding->addRow(CB_landing_up);
    VBLLanding->addRow(tr("Speed (")+tr("micro")+tr("m/s)"), LEdLandingSpeed);
    VBLLanding->addRow(tr("Slow Speed (")+tr("micro")+tr("m/s)"), LEdLandingSpeedClose);
    VBLLanding->addRow(tr("Slow SP %[0..1]"), LEdLandingSpeedCloseC);
    VBLLanding->setContentsMargins(6, 3, 6, 6);

    setWindowTitle(tr("Motor Options - NSpec"));
    sl_load_from_file();
}
void TOptionsForMotor::sl_save_to_file()
{
    QSettings settings("Param/motor.conf", QSettings::IniFormat);
    settings.setValue("up_speed", up_speed);
    settings.setValue("up_dist", up_dist);
    settings.setValue("dist_ready", dist_ready);

    settings.setValue("land_speed", land_speed);
    settings.setValue("land_speed_close", land_speed_close);
    settings.setValue("land_close_perc", land_close_perc);
    settings.setValue("landing_upward", landUpward);
    settings.setValue("head_landing", headLanding);
    settings.setValue("head_landing_exit_distance", headn_land_exit_dist);
    settings.setValue("head_landing_offset_distance1", head_land_offset_dist_1);
    settings.setValue("head_landing_offset_distance3", head_land_offset_dist_3);
    settings.setValue("systemOn_MAF", sysOnMAF);
}
void TOptionsForMotor::sl_load_from_file()
{
    QSettings settings("Param/motor.conf", QSettings::IniFormat);

    up_speed = settings.value("up_speed", 500).toDouble();
    up_dist = settings.value("up_dist", 1*1e3).toDouble();
    dist_ready = settings.value("dist_ready", 1*1e3).toDouble();

    land_speed = settings.value("land_speed", 50).toDouble();
    land_speed_close = settings.value("land_speed_close", 5).toDouble();
    land_close_perc = settings.value("land_close_perc", 0.7).toDouble();
    landUpward = settings.value("landing_upward", false).toBool();
    headLanding = settings.value("head_landing", true).toBool();
    sysOnMAF = settings.value("systemOn_MAF", false).toBool();
    headn_land_exit_dist = settings.value("head_landing_exit_distance", 0.7*1e3).toDouble();
    head_land_offset_dist_1 = settings.value("head_landing_offset_distance1", 0.7*1e3).toDouble();
    head_land_offset_dist_3 = settings.value("head_landing_offset_distance3", 0.7*1e3).toDouble();

    LEdUpSpeed->setValue(up_speed);
    LEdUpDist->setValue(up_dist);
    LEdDistReady->setValue(dist_ready);
    LEdHeadLandingExitDist->setValue(headn_land_exit_dist);
    LEdHeadLandingOffsetDist1->setValue(head_land_offset_dist_1);
    LEdHeadLandingOffsetDist3->setValue(head_land_offset_dist_3);

    LEdLandingSpeed->setValue(land_speed);
    LEdLandingSpeedClose->setValue(land_speed_close);
    LEdLandingSpeedCloseC->setValue(land_close_perc);
    CB_landing_up->setChecked(landUpward);
    CB_HeadLanding->setChecked(headLanding);
    CB_sysONMAF->setChecked(sysOnMAF);
}
void TOptionsForMotor::sl_update()
{
    up_speed = LEdUpSpeed->value();
    up_dist = LEdUpDist->value();
    dist_ready = LEdDistReady->value();
    headn_land_exit_dist = LEdHeadLandingExitDist->value();
    head_land_offset_dist_1 = LEdHeadLandingOffsetDist1->value();
    head_land_offset_dist_3 = LEdHeadLandingOffsetDist3->value();

    land_speed = LEdLandingSpeed->value();
    land_speed_close = LEdLandingSpeedClose->value();
    land_close_perc = LEdLandingSpeedCloseC->value();
    landUpward = CB_landing_up->isChecked();
    headLanding = CB_HeadLanding->isChecked();

    LEdHeadLandingExitDist->setEnabled(CB_HeadLanding->isChecked());
    LEdHeadLandingOffsetDist1->setEnabled(CB_HeadLanding->isChecked());
    LEdHeadLandingOffsetDist3->setEnabled(CB_HeadLanding->isChecked());
    sysOnMAF = CB_sysONMAF->isChecked();

    sl_save_to_file();
}


TOptionsForShuttle::TOptionsForShuttle()
{
    min_piezo_speed = 0.;
    max_piezo_speed = 0.;
    EPMinSpeed = new Edit(0, 100, 2);
    EPMaxSpeed = new Edit(0.01, 100, 1);
    connect(EPMinSpeed, SIGNAL(newValue()), this, SLOT(sl_update()));
    connect(EPMaxSpeed, SIGNAL(newValue()), this, SLOT(sl_update()));


    mainLay = new QVBoxLayout();
    //HBLParams->setSizeConstraint(QLayout::SetFixedSize);
    //HBLParams->setSpacing(3);
    //HBLParams->setContentsMargins(6, 3, 6, 6);

    GrBUp = new QGroupBox(tr("Piezo"));
    QFormLayout *VBLUpSpeed = makeFormLayout(GrBUp);

    GrBMap = new QGroupBox(tr("Keys"));
    QHBoxLayout *VBLMap = new QHBoxLayout(GrBMap);
    GrBMap->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Preferred );
    GrBUp->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Preferred );

    mainLay->addWidget(GrBUp, 1);
    mainLay->addWidget(GrBMap, 1);

    VBLUpSpeed->addRow(tr("Min speed") + " ("+QObject::tr("micro")+QObject::tr("m")+"/"+QObject::tr("s")+")", EPMinSpeed);
    VBLUpSpeed->addRow(tr("Max speed") + " ("+QObject::tr("micro")+QObject::tr("m")+"/"+QObject::tr("s")+")", EPMaxSpeed);

    QVBoxLayout **VBLMapRow = new QVBoxLayout*[5];
    for (int i=0; i<5; i++)
    {
        VBLMapRow[i] = new QVBoxLayout();
        VBLMap->addLayout(VBLMapRow[i], 1);
    }

    //header
    VBLMapRow[0]->addWidget(new QLabel(tr("Button #")), 0, Qt::AlignTop);
    VBLMapRow[1]->addWidget(new QLabel(tr("Role")), 0, Qt::AlignTop);
    VBLMapRow[2]->addWidget(new QLabel(tr("Single step") + " ("+QObject::tr("micro")+QObject::tr("m")+")"), 0, Qt::AlignTop);
    VBLMapRow[3]->addWidget(new QLabel(tr("Speed step") +
                            " ("+QObject::tr("micro")+QObject::tr("m")+"/" + QObject::tr("s") + ")"), 0, Qt::AlignTop);
    VBLMapRow[4]->addWidget(new QLabel(tr("Reverse")), 0, Qt::AlignTop);

    for (int i = 0; i < 15; i++)
    {
        VBLMapRow[0]->addWidget(new QLabel(tr("%1").arg(i+1)), 1, Qt::AlignHCenter);
        EKeyStep[i] = new Edit(0, 1000, 2);
        connect(EKeyStep[i], SIGNAL(newValue()), this, SLOT(sl_update()));
        VBLMapRow[2]->addWidget(EKeyStep[i], 1, Qt::AlignVCenter);

        EKeySpeed[i] = new Edit(1, 9999, 0);
        connect(EKeySpeed[i], SIGNAL(newValue()), this, SLOT(sl_update()));
        VBLMapRow[3]->addWidget(EKeySpeed[i], 1, Qt::AlignVCenter);

        CBKeyRole[i] = new QComboBox();
        connect(CBKeyRole[i], SIGNAL(activated(int)), this, SLOT(sl_update()));
        VBLMapRow[1]->addWidget(CBKeyRole[i], 1, Qt::AlignVCenter);
        CBKeyRole[i]->addItem(tr("Off"), -2);
        CBKeyRole[i]->addItem(tr("Feedback ON"), 2);
        CBKeyRole[i]->addItem(tr("Feedback OFF"), 3);
        CBKeyRole[i]->addItem(tr("Landing Toggle"), 8);
        CBKeyRole[i]->addItem(tr("M. Landing"), 4);
        CBKeyRole[i]->addItem(tr("M. Head X"), 151);
        CBKeyRole[i]->addItem(tr("M. Head Y"), 152);
        CBKeyRole[i]->addItem(tr("M. Stage X"), 9);
        CBKeyRole[i]->addItem(tr("M. Stage Y"), 14);
        CBKeyRole[i]->addItem(tr("M. Focus"), 0);

        CBKeyRole[i]->addItem(tr("Piezo Head Z"), 250);
        CBKeyRole[i]->addItem(tr("Piezo Head X"), 10);
        CBKeyRole[i]->addItem(tr("Piezo Head Y"), 13);
        CBKeyRole[i]->addItem(tr("Piezo Stage X"), 11);
        CBKeyRole[i]->addItem(tr("Piezo Stage Y"), 12);
        CBKeyRole[i]->addItem(tr("Piezo Focus Z"), 1);
//        CBKeyRole[i]->addItem(tr("Piezo Head X"), 252);
//        CBKeyRole[i]->addItem(tr("Piezo Head Y"), 253);
//        CBKeyRole[i]->addItem(tr("Piezo Stage X"), 254);
//        CBKeyRole[i]->addItem(tr("Piezo Stage Y"), 255);

        for (int j = 0; j < 10; j++)
            CBKeyRole[i]->addItem(tr("M.%1").arg(j+1), 100+j+1);

        CBKeyRole[i]->addItem(tr("M.1 Phys."), 5);
        CBKeyRole[i]->addItem(tr("M.2 Phys."), 6);
        CBKeyRole[i]->addItem(tr("M.3 Phys."), 7);

        CKeyReverse[i] = new QCheckBox();
        connect(CKeyReverse[i], SIGNAL(clicked(bool)), this, SLOT(sl_update()));
        VBLMapRow[4]->addWidget(CKeyReverse[i], 1, Qt::AlignHCenter|Qt::AlignVCenter);
    }

    sl_load_from_file();
}

void TOptionsForShuttle::sl_save_to_file()
{
    QSettings settings("Param/shuttle.conf", QSettings::IniFormat);
    settings.setValue("piezo/scale0", EPMinSpeed->value() / 0.1);
    settings.setValue("piezo/scale6", EPMaxSpeed->value() / 0.1);

    for (int i = 0; i < 15; i++)
    {
        settings.setValue("piezo/step/"+QString::number(i), EKeyStep[i]->value());

        int v = CBKeyRole[i]->itemData(CBKeyRole[i]->currentIndex()).toInt();
        if (v == i)
            v = -1;
        settings.setValue("remap/"+QString::number(i), v);
        settings.setValue("reverse/"+QString::number(i), CKeyReverse[i]->isChecked());
        settings.setValue("speed/" + QString::number(i), EKeySpeed[i]->value());
    }
}

void TOptionsForShuttle::sl_load_from_file()
{
    QSettings settings("Param/shuttle.conf", QSettings::IniFormat);

    EPMinSpeed->setValue(settings.value("piezo/scale0", 0.1).toDouble() * 0.1);
    EPMaxSpeed->setValue(settings.value("piezo/scale6", 10).toDouble() * 0.1);

    for (int i = 0; i < 15; i++)
    {
        double def = 0.1;
        if (i == 0)
            def = 10;
        if (i == 4)
            def = 1;
        if (i == 9)
            def = 10;
        if (i == 14)
            def = 10;
        EKeyStep[i]->setValue(settings.value("piezo/step/"+QString::number(i), def).toDouble());

        int v = settings.value("remap/"+QString::number(i), -1).toDouble();
        if (v == -2)
            v = 0;
        else
        {
            if (v == -1)
                v = i;
            v = CBKeyRole[i]->findData(v);
            if (v == -1)
                v = 0;
        }
        CBKeyRole[i]->setCurrentIndex(v);

        CKeyReverse[i]->setChecked(settings.value("reverse/"+QString::number(i), false).toBool());
        EKeySpeed[i]->setValue(settings.value("speed/"+QString::number(i), 1).toDouble());
    }

    double piezo_stepsps = 1000.0/settings_double(settings, "piezo/delay", 50);
    min_piezo_speed = EPMinSpeed->value() / piezo_stepsps;
    max_piezo_speed = EPMaxSpeed->value() / piezo_stepsps;
}

void TOptionsForShuttle::sl_update()
{
    QSettings settings("Param/shuttle.conf", QSettings::IniFormat);
    double piezo_stepsps = 1000.0/settings_double(settings, "piezo/delay", 50);
    min_piezo_speed = EPMinSpeed->value() / piezo_stepsps;
    max_piezo_speed = EPMaxSpeed->value() / piezo_stepsps;

    sl_save_to_file();
}



BYTE SPMTunning::getLandingMode()
{
    QList<int> l = Motors::getRoleMotors(Motors::RoleZ1Land);
    std::for_each(l.begin(), l.end(), Motors::mapVirtToPhys);
    if (!l.size())
        return lm04;
//    if (l.size() == 1 && l.contains(3))
//        return lm01;
//    if (l.size() == 3 && l.contains(1) && l.contains(2) && l.contains(3)
//            && 0) //TODO check same speeds/directions
//        return lm04;
    return lm05;
}
std::vector<int> SPMTunning::getLandingMotors()
{
    std::vector<int> v;
    int mode = getLandingMode();
    if (mode == lm01) {
        v.push_back(motors->mapPhysToVirt(3));
        return v;
    }
    if (mode == lm05) {
        QList<int> l = Motors::getRoleMotors(Motors::RoleZ1Land);
        return l.toVector().toStdVector();
    }

    v.push_back(motors->mapPhysToVirt(1));
    v.push_back(motors->mapPhysToVirt(2));
    v.push_back(motors->mapPhysToVirt(3));
    return v;
}



void SPMTunning::slot_mland_stopped()
{
    last_command = 0;
}

void SPMTunning::motors_stop()
{
    motors->goStop(250); //123 motors
    motors->goStop(motors->mapPhysToVirt(4)); //4 motor
    for (unsigned i = 0; i < getLandingMotors().size(); i++) {
        BYTE CurrentMotor = getLandingMotors()[i];
        motors->goStop(CurrentMotor);
    }
}

void SPMTunning::slot_DoMoveUp()
{
    BLanding->change_state_man(false);
    blanding_status = false;
    MobileServer::sendData(DataID::LandingOn, blanding_status);

    const int curr_cmd = 1;

    for (unsigned i = 0; i < getLandingMotors().size(); i++) {
        BYTE CurrentMotor = getLandingMotors()[i];
        if (last_command == curr_cmd)
            motors->goStop(CurrentMotor);
        else
            motors->goUp(CurrentMotor, OptionsForMotor->up_speed, OptionsForMotor->up_dist);
    }
    if (last_command == curr_cmd)
        last_command = 0;
    else
        last_command = curr_cmd;
}

void SPMTunning::slot_DoFastMoveUp()
{
    BLanding->change_state_man(false);
    blanding_status = false;
    MobileServer::sendData(DataID::LandingOn, blanding_status);

    const int curr_cmd = 2;

    for (unsigned i = 0; i < getLandingMotors().size(); i++) {
        BYTE CurrentMotor = getLandingMotors()[i];
        if (last_command == curr_cmd)
            motors->goStop(CurrentMotor);
        else
            motors->goUpE(CurrentMotor, OptionsForMotor->up_speed);
    }
    if (last_command == curr_cmd)
        last_command = 0;
    else
        last_command = curr_cmd;
}

void SPMTunning::slot_DoMoveDown()
{
    BLanding->change_state_man(false);
    blanding_status = false;
    MobileServer::sendData(DataID::LandingOn, blanding_status);

    auto scan = ScanPlot::GetIt();
    if (scan)
    {
        if (scan->smartLanding->getState() == 2)
            slot_DoLanding(false);
        scan->smartLanding->change_state_man(1);
    }


    const int curr_cmd = -1;

    for (unsigned i = 0; i < getLandingMotors().size(); i++) {
        BYTE CurrentMotor = getLandingMotors()[i];
        if (last_command == curr_cmd)
            motors->goStop(CurrentMotor);
        else
        {
            if (OptionsForMotor->headLanding)
                motors->goUp(CurrentMotor, OptionsForMotor->up_speed, OptionsForMotor->up_dist);
            else
                motors->goDown(CurrentMotor, OptionsForMotor->up_speed, OptionsForMotor->up_dist);
        }
    }
    if (last_command == curr_cmd)
        last_command = 0;
    else
        last_command = curr_cmd;
}

void SPMTunning::slot_DoFastMoveDown()
{
    BLanding->change_state_man(false);
    blanding_status = false;
    MobileServer::sendData(DataID::LandingOn, blanding_status);

    const int curr_cmd = -2;

    for (unsigned i = 0; i < getLandingMotors().size(); i++) {
        BYTE CurrentMotor = getLandingMotors()[i];
        if (last_command == curr_cmd)
            motors->goStop(CurrentMotor);
        else
            motors->goDownE(CurrentMotor, OptionsForMotor->up_speed);
    }
    if (last_command == curr_cmd)
        last_command = 0;
    else
        last_command = curr_cmd;
}

void SPMTunning::slot_RecedeLanding()
{
    if (OptionsForMotor->landUpward)
        slot_DoMoveDown();
    else
        slot_DoMoveUp();
}

void SPMTunning::scan_StartAutoLanding()
{
    if (!BFeedbackOnOff->isOn)
        BFeedbackOnOff->change_state();

    auto scan = ScanPlot::GetIt();
    if (scan && !scan->deviceSchemeWid->feedBack->turnOn)
        scan->deviceSchemeWid->feedBack->setState(true);

    doAutoLandConnect(true);
}

void SPMTunning::slot_DoLanding(bool st)
{
    QString text = tr("Landing Status: Waiting for response....");
    MobileServer::sendData(DataID::Land_status, text);

    auto scan = ScanPlot::GetIt();

    last_command = 0;
    if (st) {
        if (scan)
            scan->smartLanding->change_state_man(2);
        //SBT->setValue(1);
        int idx3 = motors->mapPhysToVirt(3);
        double k = motors->Estepsrev[idx3-1]->value() / motors->Epitch[idx3-1]->value() / 1e3;
        Landing_mode = getLandingMode();
        Landing_direction = (motors->orient[idx3-1] != OptionsForMotor->landUpward) ? 0 : 1;
        Landing_speed = OptionsForMotor->land_speed * k / 16;
        Landing_speed_close = OptionsForMotor->land_speed_close * k / 16;
        Landing_close_perc = OptionsForMotor->land_close_perc;
        if (Landing_mode == lm05) {
            std::vector<int> l = getLandingMotors();
            for (unsigned i = 0; i < 4; i++) {
                if (i >= l.size()) {
                    LandingMCfg[i].Idx = 0;
                    continue;
                }
                double k = motors->Estepsrev[l[i]-1]->value() / motors->Epitch[l[i]-1]->value() / 1e3;
                LandingMCfg[i].Idx = motors->mapVirtToPhys(l[i]);
                LandingMCfg[i].Dir = (motors->orient[l[i]-1] != OptionsForMotor->landUpward) ? 0 : 1;
                LandingMCfg[i].Speed = OptionsForMotor->land_speed * k / 16;
                LandingMCfg[i].Speed_near = OptionsForMotor->land_speed_close * k / 16;
                LandingMCfg[i].Steps_per_mkm = qRound(k);
            }
        } else {
            for (unsigned i = 0; i < 4; i++) {
                LandingMCfg[i].Idx = 0;
            }
        }
        updLanding = true;


        switch (FB_channel) {
        case srcDFL: {
            double cur = 1.1*nDFL;
            double sp = 2.2*DCfbSetPoint->value() * spm_sp_units_gain;
            double pad = fabs(input_gain(srcDFL)) * (B_PID_sign->isOn || cur < sp ? -1 : 1);
            OscYrange[0] = qMin(qMin(sp, pad), cur);
            OscYrange[1] = qMax(qMax(sp, cur), pad);
            OscYrange[0] /= input_gain(srcDFL);
            OscYrange[1] /= input_gain(srcDFL);
        }
            break;
        case srcLFL: {
            double cur = 1.1*nLFL;
            double sp = 2.2*DCfbSetPoint->value() * spm_sp_units_gain;
            double pad = fabs(input_gain(srcLFL)) * (B_PID_sign->isOn || cur < sp ? -1 : 1);
            OscYrange[0] = qMin(qMin(sp, pad), cur);
            OscYrange[1] = qMax(qMax(sp, cur), pad);
            OscYrange[0] /= input_gain(srcLFL);
            OscYrange[1] /= input_gain(srcLFL);
        }
            break;
        case srcMag1:
            OscYrange[0] = 0;
            OscYrange[1] = qMax(1.1*nMAG, 2.2*DCfbSetPoint->value() * spm_sp_units_gain);
            OscYrange[0] /= input_gain(srcMag1);
            OscYrange[1] /= input_gain(srcMag1);
            break;
        }

        PlotMode = mode_Osc;
        sl_setPlotMode();


        if (!landing_bar) {
            landing_bar = new StatusBar(text, 0);
            connect(landing_bar, SIGNAL(canceled()), BLanding, SLOT(click()));
            connect(landing_bar, SIGNAL(canceled()), this, SLOT(cancelWaitingResponce()));
            landing_bar->add();
        } else
            landing_bar->statusLabel->setText(text);
    } else {
        Landing_mode = lmOff;
        updLanding = true;

        destructStatus(landing_bar);
        auto osc = Oscilloscope::GetIt();
        if (osc)
            osc->acf->cancelOperation();
        autoSensorsCalibCanceled();

        BLanding->change_state_man(false);
        if (scan)
            scan->smartLanding->change_state(1);

    }
    destructStatus(landing_notification);
}

void SPMTunning::cancelWaitingResponce()
{
    auto scan = ScanPlot::GetIt();
    if (scan)
        scan->smartLanding->change_state(1);
}

void SPMTunning::blanding_sync()
{
    if (BLanding->isOn != blanding_status)
        doBeep();

    BLanding->change_state_man(blanding_status);
    MobileServer::sendData(DataID::LandingOn, blanding_status);
}

void SPMTunning::slot_set_Landing_status(int status)
{
    destructStatus(landing_notification);

    auto scan = ScanPlot::GetIt();

    QString text;
    switch (status)
    {
    case lndOff:
        text = tr("Landing Status: Canceled...");
        destructStatus(landing_bar);

        if (auto_land_state)
        {
            PBAutoLand->change_state_man_with_sig(false);
            if (scan)
                scan->smartLanding->change_state_man(1);
        }
        break;
    case lndLanding:
        text = tr("Landing Status: In progress...");
        if (landing_bar) {
            landing_bar->bar->setMaximum(0);
            Progress_timer->stop();
        }
        blanding_status = true;
        blanding_syncT->start(100);
        if (scan)
            scan->smartLanding->change_state_man(2);
        break;
    case lndDone:
        text = tr("Landing Status: Done...");
        destructStatus(landing_bar);
        landing_notification = new Notification(text, Notification::Good, 10);
        blanding_status = false;
        blanding_syncT->start(100);
        if (scan)
            scan->smartLanding->change_state_man(3);

        if (auto_land_state == 1)
            doAutoLand(); //to next step
        break;
    case lndError:
        text = tr("Landing Status: Error...");
        if (!BFeedbackOnOff->isOn) text += " " + tr("Feedback is disabled!");
        destructStatus(landing_bar);
        landing_notification = new Notification(text, Notification::Bad, 60);
        blanding_status = false;
        blanding_syncT->start(100);
        if (scan)
            scan->smartLanding->change_state_man(1);

        if (auto_land_state) {
            blanding_sync();
            PBAutoLand->change_state_man_with_sig(false);
        }
        break;
    case lndWaitDrift:
        text = tr("Landing Status: Checking stability");
        if (landing_bar) {
            landing_bar->bar->setMaximum(5*1000);
            landing_bar->bar->setValue(0);
            landing_bar->bar->setFormat("%p%");
            Progress_timer->start(100);
        }
        blanding_status = true;
        blanding_syncT->start(100);
        break;
    }

    if (landing_bar) landing_bar->statusLabel->setText(text);
    else Progress_timer->stop();
    MobileServer::sendData(DataID::Land_status, text);
}

void SPMTunning::s_updLandProgBar()
{
    if (!landing_bar)
        return;

    if (landing_bar->bar->maximum() && landing_bar->bar->value() < 5*1000)
        landing_bar->setValue(landing_bar->bar->value() + 100);
    else {
        Progress_timer->stop();
        landing_bar->bar->setMaximum(0);
    }
}

void SPMTunning::InitMotorControlSpace()
{
    last_command = 0;

    QPushButton *BMoveDown = new QPushButton(makeHugeIcon(":/SPMTuning_window/Up.png"), QString());
    BMoveDown->setToolTip(tr("Move Tip Up"));

    QPushButton *BMoveUp = new QPushButton(makeHugeIcon(":/SPMTuning_window/Down.png"), QString());
    BMoveUp->setToolTip(tr("Move Tip Down"));

    QPushButton *BFastMoveDown = new QPushButton(makeHugeIcon(":/SPMTuning_window/FastUp.png"), QString());
    BFastMoveDown->setToolTip(tr("Fast Tip Up till end"));

    QPushButton *BFastMoveUp = new QPushButton(makeHugeIcon(":/SPMTuning_window/FastDown.png"), QString());
    BFastMoveUp->setToolTip(tr("Fast Tip Down till end"));

    auto_land_state = 0;
    PBAutoLand = new TwoStateButton(":AutoStart.png", ":stop.png");
    PBAutoLand->setToolTip(tr("Smart landing"));
    connect(PBAutoLand, SIGNAL(state_changed(bool)), this, SLOT(doAutoLand()));
    PBAutoLand->hide();

    BLanding = new TwoStateButton(":start.png",":stop.png");
    BLanding->setProperty("id", DataID::LandingOn);
    BLanding->setToolTip(tr("Tip Landing"));

    QPushButton *BMotorsStop = new QPushButton();
    BMotorsStop->setIcon(makeHugeIcon(":/SPMTuning_window/approach/landing_done.png"));
    BMotorsStop->setToolTip(tr("Stop all motors"));

    connect(BMoveDown,     SIGNAL(clicked()), this, SLOT(slot_DoMoveDown()));
    connect(BMoveUp,       SIGNAL(clicked()), this, SLOT(slot_DoMoveUp()));
    connect(BFastMoveDown, SIGNAL(clicked()), this, SLOT(slot_DoFastMoveDown()));
    connect(BFastMoveUp,   SIGNAL(clicked()), this, SLOT(slot_DoFastMoveUp()));
    connect(BLanding,      SIGNAL(state_changed(bool)), this, SLOT(slot_DoLanding(bool)));
    connect(BMotorsStop, SIGNAL(clicked()), SLOT(motors_stop()));

    GrBMotorControl = new QGroupBox(this);
    QVBoxLayout *Motor_Layout = new QVBoxLayout(GrBMotorControl);
    //Motor_Layout->setContentsMargins(3, 3, 3, 3);
    //Motor_Layout->setSpacing(6);
    Motor_Layout->addWidget(PBAutoLand);
    Motor_Layout->addWidget(BLanding);
    Motor_Layout->addWidget(BMoveDown);
    Motor_Layout->addWidget(BMoveUp);
    Motor_Layout->addWidget(BFastMoveDown);
    Motor_Layout->addWidget(BFastMoveUp);
    Motor_Layout->addWidget(BMotorsStop);
}


void SPMTunning::shuttle__set_active(int num)
{
    active_shuttle_motors.resize(1);
    active_shuttle_motors[0] = num;
}

void SPMTunning::shuttle_button(int num)
{
    auto scan = ScanPlot::GetIt();

    {
        static int prevNum = -1;
        static QString prevStyle = "";
        auto column1 = OptionsForShuttle->GrBMap->layout()->itemAt(0)->layout();
        //auto column2 = OptionsForShuttle->GrBMap->layout()->itemAt(1)->layout();
        //auto column3 = OptionsForShuttle->GrBMap->layout()->itemAt(2)->layout();

        QString style = "QFrame, QLabel, QToolTip { \
                border: 2px solid red; \
                border-radius: 4px; \
                padding: 2px; \
            }";

        if (prevNum != -1)
            column1->itemAt(prevNum+1)->widget()->setStyleSheet(prevStyle);

        prevNum = num;
        prevStyle = column1->itemAt(num+1)->widget()->styleSheet();
        column1->itemAt(num+1)->widget()->setStyleSheet(style);
        lastShuttlePressedButton = num;
    }


    piezo_shuttle = 0;
    piezo_step = OptionsForShuttle->EKeyStep[qMin(num, 15-1)]->value();
    {
        QSettings settings("Param/shuttle.conf", QSettings::IniFormat);
        int remap = settings_double(settings, "remap/"+QString::number(num), -1);
        if (remap >= 0)
            num = remap;
    }

    //first row
    if (num == 0)
        shuttle__set_active(Motors::getRoleMotor(Motors::RoleZ2Focus));
    if (num == 1)
        shuttle__set_active(251);//piezo ZF

    if (num == 2)
    {
        BFeedbackOnOff->change_state_man_with_sig(true);
        if (scan)
            scan->deviceSchemeWid->feedBack->setState(true);
    }
    if (num == 3)
    {
        BFeedbackOnOff->change_state_man_with_sig(false);
        if (scan)
            scan->deviceSchemeWid->feedBack->setState(false);
    }


    //second row
    if (num == 4)
        active_shuttle_motors = getLandingMotors();
    if (num == 5)
        shuttle__set_active(motors->mapPhysToVirt(1));
    if (num == 6)
        shuttle__set_active(motors->mapPhysToVirt(2));
    if (num == 7)
        shuttle__set_active(motors->mapPhysToVirt(3));

    if (num == 8) {
        BLanding->click();
        blanding_status = BLanding->isOn;
    }

    if (num == 9)
        shuttle__set_active(Motors::getRoleMotor(Motors::RoleStageX));

    if (num == 10)
        shuttle__set_active(252);//piezo X
    if (num == 11)
        shuttle__set_active(254);//piezo X2
    if (num == 13)
        shuttle__set_active(253);//piezo Y
    if (num == 12)
        shuttle__set_active(255);//piezo Y2

    if (num == 14)
        shuttle__set_active(Motors::getRoleMotor(Motors::RoleStageY));


    if (num > 100 && num < 120)
        shuttle__set_active(qMin(16, num-100));

    if (num == 151)
        shuttle__set_active(Motors::getRoleMotor(Motors::RoleX1));
    if (num == 152)
        shuttle__set_active(Motors::getRoleMotor(Motors::RoleY1));

    if (num >= 250)
        shuttle__set_active(num);


    upd_shuttle_status();
}
void SPMTunning::upd_shuttle_status()
{
    if (!shuttleStatus)
        return;

    int idx = -1;
    QString name;

    if (active_shuttle_motors.size()) {
        if (active_shuttle_motors[0] == 250) //piezo Z
            name = "Z-P";
        if (active_shuttle_motors[0] == 251) //piezo ZF
            name = "ZF-P";
        if (active_shuttle_motors[0] == 252) //piezo X
            name = "X-P";
        if (active_shuttle_motors[0] == 254) //piezo X2
            name = "X2-P";
        if (active_shuttle_motors[0] == 253) //piezo Y
            name = "Y-P";
        if (active_shuttle_motors[0] == 255) //piezo Y2
            name = "Y2-P";

        if (active_shuttle_motors == getLandingMotors())
            name = "Land";

        if (active_shuttle_motors[0] <= 16 && active_shuttle_motors.size() == 1)
            idx = active_shuttle_motors[0];
    }

    QString n1 = tr("Shuttle: ");
    if (idx > 0) {
        QString stridx = QString::number(idx);
        //TODO get role name =

        n1 += stridx;
        if (!name.isEmpty()) n1 += " - ";
    }
    n1 += name;
    if (idx > 0 || !name.isEmpty())
        shuttleStatus->setText(n1);
}


#include "oscilloscope.h"
void SPMTunning::shuttle_wheel(int num) //small step-wheel
{
    if (!OptionsForShuttle->CKeyReverse[lastShuttlePressedButton]->isChecked())
        num *= -1;
    for (unsigned i = 0; i < active_shuttle_motors.size(); i++) {
        int active_shuttle_motor = active_shuttle_motors[i];
        if (active_shuttle_motor <= 16) {
            if (piezo_shuttle != 0)
                continue;

            //double speed = motors->getShuttleSpeed(1./5, active_shuttle_motor);
            double speed = OptionsForShuttle->EKeySpeed[lastShuttlePressedButton]->value();
            double steps = OptionsForShuttle->EKeyStep[lastShuttlePressedButton]->value();
            motors->go(active_shuttle_motor, num, speed, steps/*piezo_step*/);

            if (active_shuttle_motor == Motors::getRoleMotor(Motors::MotorRoles::RoleZ1Land)) //todo oplkill find better place
            {
                auto scan = ScanPlot::GetIt();
                if (scan)
                {
                    if (scan->smartLanding->getState() == 3)
                        scan->smartLanding->change_state_man(1);
                    if (scan->smartLanding->getState() == 2)
                        slot_DoLanding(false);
                }
            }
        } else {
            double sign = num > 0 ? 1 : -1;
            double speed = piezo_step;
            if (piezo_shuttle != 0) { //slider timer
                double p = (abs(num)-1) / (7.-1);
                p = p*p*p*p; //non-linear ramp
                double delta = OptionsForShuttle->min_piezo_speed - OptionsForShuttle->max_piezo_speed;
                speed = OptionsForShuttle->min_piezo_speed + delta * p;
            }

            auto osc = Oscilloscope::GetIt();
            if (!osc)
                return;
            DoubleChanger *dc = nullptr;

            switch (active_shuttle_motor) {
            case 250:
                dc = osc->DacsWidget->Z;
                break;
            case 251:
                dc = osc->DacsWidget->Zobj;
                break;
            case 252:
                dc = osc->DacsWidget->X;
                break;
            case 253:
                dc = osc->DacsWidget->Y;
                break;
            case 254:
                dc = osc->DacsWidget->X2;
                break;
            case 255:
                dc = osc->DacsWidget->Y2;
                break;
            default:
                return;
            }

            dc->setValue(dc->value() + sign*speed);
            dc->setNewValue();


        }
    }
}

void SPMTunning::shuttle_position_piezo_timeout()
{
    if (piezo_shuttle == 0)
        return;

    shuttle_wheel(piezo_shuttle);
}

void SPMTunning::shuttle_position(int num) //Big wheel
{
    piezo_shuttle = 0;
    if (!OptionsForShuttle->CKeyReverse[lastShuttlePressedButton]->isChecked())
        num *= -1;
    for (unsigned i = 0; i < active_shuttle_motors.size(); i++) {
        int active_shuttle_motor = active_shuttle_motors[i];

        if (active_shuttle_motor <= 16) {
            motors->goShuttle(num, active_shuttle_motor);
            if (active_shuttle_motor == Motors::getRoleMotor(Motors::MotorRoles::RoleZ1Land)) //todo oplkill find better place
            {
                auto scan = ScanPlot::GetIt();
                if (scan)
                {
                    if (scan->smartLanding->getState() == 3)
                        scan->smartLanding->change_state_man(1);
                    if (scan->smartLanding->getState() == 2)
                        slot_DoLanding(false);
                }
            }
        } else {
            piezo_shuttle = num;
        }
    }

}

void SPMTunning::InitShuttleGuiLink()
{
    shuttleStatus = new QLabel(tr("Shuttle: ") + "Land");
    //active_shuttle_motors = getLandingMotors(); //crash, no motors yet
    piezo_shuttle = 0;
    piezo_step = 0;

    QSettings settings("Param/shuttle.conf", QSettings::IniFormat);

    QTimer *piezo_shuttle_timer = new QTimer(this);
    connect(piezo_shuttle_timer, SIGNAL(timeout()), this, SLOT(shuttle_position_piezo_timeout()));
    piezo_shuttle_timer->start(settings_double(settings, "piezo/delay", 50));

    ShuttleControllerNative *hidinput = new ShuttleControllerNative(this);
    connect(hidinput, SIGNAL(button_press(int)), this, SLOT(shuttle_button(int)));
    connect(hidinput, SIGNAL(wheel_event(int)), this, SLOT(shuttle_wheel(int)));
    connect(hidinput, SIGNAL(shuttle_position(int)), this, SLOT(shuttle_position(int)));
}
#include <QMessageBox>
#include "autocalib.hpp"
void SPMTunning::doAutoLandConnect(bool on_off)
{
    if (!on_off)
    {
        PBAutoLand->change_state_man(false);
        autoSensorsCalibEnd(false);
        if (BLanding->isOn)
            BLanding->change_state();
        return;
    }

    auto osc = Oscilloscope::GetIt();
    if (!osc)
        return;

    connect(osc->acf, SIGNAL(sensorsRecalibDone(bool)), this, SLOT(autoSensorsCalibEnd(bool)));
    connect(osc->acf, SIGNAL(calibCanceled()), this, SLOT(autoSensorsCalibCanceled()));

    BYTE scanner = osc->CalibWidget->Scanner_type->currentIndex();
    auto spm = SPMTunning::GetIt();
    if (spm)
        scanner = !spm->OptionsForMotor->CB_HeadLanding->isChecked();
    osc->acf->SensorReCalc(scanner);
}

void SPMTunning::autoSensorsCalibCanceled()
{
    autoSensorsCalibEnd(false);
    auto scan = ScanPlot::GetIt();
    if (scan)
        scan->smartLanding->change_state_man(1);
}

void SPMTunning::autoSensorsCalibEnd(bool ok)
{
    auto osc = Oscilloscope::GetIt();
    if (!osc)
        return;
    auto scan = ScanPlot::GetIt();

    disconnect(osc->acf, SIGNAL(sensorsRecalibDone(bool)), this, SLOT(autoSensorsCalibEnd(bool)));
    disconnect(osc->acf, SIGNAL(calibCanceled()), this, SLOT(autoSensorsCalibCanceled()));

    if (!ok)
        return;

    s_FreqScanStartAuto();
    double oldmag = nMAG;
    if (scan && !scan->rectAmplitude->enableControl)
    {
        if (!findMag())
        return;
    }
    else {
        setEnabled(false);
        
        QElapsedTimer t; t.start();
        while (t.elapsed() < 250) {
            qApp->processEvents();
            SleepExport::msleep2(10);
        }
        setEnabled(true);
    }

    if (scan)
    {
        scan->oldMag = nMAG;
        scan->rectSliderSlFbiChanged(scan->rectSlFbi->Value());
    }
    if (qFabs(nMAG - oldmag) > 0.1 * nMAG) {
        DCfbSetPoint->setValue(nMAG * 0.6 / spm_sp_units_gain);
        DCfbSetPoint->setNewValue();
        if (scan)
        {
            scan->deviceSchemeWid->setPointEdit->setValue(nMAG * 0.6 / spm_sp_units_gain);
            scan->deviceSchemeWid->setPointEdit->setNewValue();
        }
    }

    slot_DoLanding(true);
}

bool SPMTunning::findMag()
{
    destructStatus(landing_notification);
    QString text = tr("Finding magnitude:");
    MobileServer::sendData(DataID::Land_status, text);
    if (!landing_bar) {
        landing_bar = new StatusBar(text, 0);
        landing_bar->add();
    }
    landing_bar->bar->setMaximum(0);
    Progress_timer->stop();
    landing_bar->statusLabel->setText(text);

    
    double freeAmplitude = 2.0;
    double stepSize = 0.1;
    int stepWait = 100;
    {
        QSettings settings("Param/tipMonitor.conf", QSettings::IniFormat);
        freeAmplitude = settings_double(settings, "auto_land_free_amplitude", 2.0);
        stepSize = settings_double(settings, "auto_land_step_size", 0.1);
        stepWait = settings.value("auto_land_step_wait_ms", 50).toInt();
    }
    
    #ifndef _OLD_AUTO_FINDING_MAG
    double gain = input_gain(srcMag1);
    double shift = input_shift(srcMag1);
    QPair<double, double> p = GetPairAnalogMapCache(11); //srcMAG
    wFreeAmplitude = (((freeAmplitude / gain) - shift - p.first)/(p.second - p.first)) * 65535.;
    fStepSize = stepSize;
    wStepWait = stepWait;
    sendFreeAmplitude = true;
    sendStepSize = true;
    sendStepWait = true;
#endif
    
    
    //----------------AutoMagFind
    int step = 0;
    while(step++ < 50)
    {
        qApp->processEvents();
        SleepExport::msleep2(10);
    }
    
#ifdef _OLD_AUTO_FINDING_MAG
    auto scan = ScanPlot::GetIt();
    step = 0;
    double oldDrive = driveInt->value();
    double oldmag2 = nMAG;
    while (step++ < 300)
    {
        double tMag = nMAG;
        
        if (tMag >= freeAmplitude)
        {
            oldDrive -= stepSize;
        }
        else
        {
            oldDrive += stepSize;
        }
        if (oldDrive < 0)
            oldDrive = 0;
        driveInt->setValue(oldDrive);
        if (scan)
        {
            scan->deviceSchemeWid->oscilator->edAmp->setValue(oldDrive);
            scan->tipDriveNewValue(oldDrive);
        }
        s_sendDriveInt();
        
        qApp->processEvents();
        SleepExport::msleep2(stepWait);
        
        if (oldmag2 >= freeAmplitude && tMag < freeAmplitude)
            break;
        if (oldmag2 < freeAmplitude && tMag > freeAmplitude)
            break;
    }
#else
    wStartAutoFindingMAG = 1;
    sendStartAutoFindingMAG = true;
    
    step = 0;
    while(step++ < 1000 && !autoFindingMAGEnded)
    {
        qApp->processEvents();
        SleepExport::msleep2(30);
        if (!landing_bar)
        {
            wStartAutoFindingMAG = 0;
            sendStartAutoFindingMAG = true;
            break;
        }
    }
    
    autoFindingMAGEnded = false;
    
    auto scan = ScanPlot::GetIt();
#endif
    
    destructStatus(landing_bar);

    setEnabled(false);

    QElapsedTimer t; t.start();
    while (t.elapsed() < 250) {
        qApp->processEvents();
        SleepExport::msleep2(10);
    }
    setEnabled(true);

    if (nMAG / input_gain(srcMag1) < 0.3)
    {
        autoland_notification = new Notification(tr("Smart landing") + ": " + tr("Weak resonance!"), Notification::Warn, 30);
        doBeep();
        PBAutoLand->change_state_man(false);
        if (scan)
            scan->smartLanding->change_state_man(1);
        return false;
    }
    
    return true;
}

void SPMTunning::doAutoLand()
{
    auto scan = ScanPlot::GetIt();

    destructStatus(autoland_notification);
    if (!PBAutoLand->isOn) {
        if (BLanding->isOn)
            BLanding->change_state_man_with_sig(false);
        if (BFeedbackOnOffMode->isOn && BFeedbackOnOff->isOn) {
            BFeedbackOnOff->change_state_man_with_sig(false);
            scan->deviceSchemeWid->feedBack->setState(false);
            BFeedbackOnOffMode->change_state_man_with_sig(false);
        }
        //TODO state-related reset
        if (auto_land_state > 0) {
            slot_RecedeLanding();
        }
        auto_land_state = 0;
        return;
    }

    //sanity checks, then landing
    if (auto_land_state == 0) {
        bool fault = false;
        QString error;

        if (BFeedbackOnOff->isOn && nZ < 0.9) {
            slot_RecedeLanding(); //up

            QElapsedTimer t; t.start();
            while (last_command == 1 && t.elapsed() < 250) {
                qApp->processEvents();
                SleepExport::msleep2(10);
            }
            if (last_command == 1)
                slot_RecedeLanding(); //stop
        }

        bool is_tuning_fork = currModeIdx == 10;
        bool is_mag = (currModeIdx >= 2 && currModeIdx <= 5) || (currModeIdx == 10);
        bool is_stm = currModeIdx >= 6 && currModeIdx <= 9;
        if (!is_stm && !is_tuning_fork && (nDFLr > 0.1 || nDFLr < -0.1 || nLFLr > 0.1 || nLFLr < -0.1)) {
            fault = true;
            error = tr("Laser not centered!");
        }
        if (!is_stm && !is_tuning_fork && nInt < 0.2) {
            fault = true;
            error = tr("Weak laser signal!");
        }
        if (is_mag && !fault) {
            double oldmag = nMAG;

            CB_fr_Channel->setCurrentIndex(CB_fr_Channel->findData(srcMag1));
            scan->deviceSchemeWid->oscilator->cbChannel->setCurrentIndex(CB_fr_Channel->findData(srcMag1));
            PlotMode = mode_ResCurve;
            sl_setPlotMode();

            if (!s_FreqScanStartAuto(true) && !s_FreqScanStartAuto()) {
                fault = true;
                error = tr("No resonance!");
            }
            else
            {
                setEnabled(false);

                QElapsedTimer t; t.start();
                while (t.elapsed() < 250) {
                    qApp->processEvents();
                    SleepExport::msleep2(10);
                }
                setEnabled(true);

                if (nMAG / input_gain(srcMag1) < 0.3) {
                    fault = true;
                    error = tr("Weak resonance!");
                }

                //scan->oldMag = nMAG;
                if (qFabs(nMAG - oldmag) > 0.1 * nMAG && !fault) {
                    DCfbSetPoint->setValue(nMAG * 0.6 / spm_sp_units_gain);
                    DCfbSetPoint->setNewValue();
                    scan->deviceSchemeWid->setPointEdit->setValue(nMAG * 0.6 / spm_sp_units_gain);
                    scan->deviceSchemeWid->setPointEdit->setNewValue();
                }
            }
        }

        if (BFeedbackOnOff->isOn && !fault) {
            bool spcheck = true;
            double dp = DCfbSetPoint->value() * spm_sp_units_gain;
            switch (FB_channel) {
            case srcDFL:
                dp -= nDFL;
                break;
            case srcLFL:
                dp -= nLFL;
                break;
            case srcMag1:
                dp -= nMAG;
                break;
            default:
                spcheck = false;
                break;
            }
            bool sign = Mode_data[currModeIdx][4];
            if (sign)
                dp *= -1;
            if (nZ < 0.2 || (spcheck && dp / input_gain(FB_channel) >= 0.01)) {
                fault = true;
                error = tr("Wrong feedback sign or setpoint!");
            }
        }

        auto scan = ScanPlot::GetIt();

        if (fault) {
            autoland_notification = new Notification(tr("Smart landing") + ": " + error, Notification::Warn, 30);
            doBeep();
            PBAutoLand->change_state_man(false);
            if (scan)
                scan->smartLanding->change_state_man(1);
            return;
        }

        if (!BFeedbackOnOffMode->isOn)
            BFeedbackOnOffMode->change_state_man_with_sig(true);
        if (!BFeedbackOnOff->isOn) {
            double sp = DCfbSetPoint->value() * spm_sp_units_gain;
            switch (FB_channel) {
            case srcDFL:
                sp = nDFL - 1 * input_gain(FB_channel);
                break;
            case srcMag1:
                sp = nMAG * 0.6;
                break;
            }
            DCfbSetPoint->setValue(sp / spm_sp_units_gain);
            DCfbSetPoint->setNewValue(true);
            BFeedbackOnOff->change_state_man_with_sig(true);
            if (scan)
            {
                scan->deviceSchemeWid->setPointEdit->setValue(sp / spm_sp_units_gain);
                scan->deviceSchemeWid->setPointEdit->setNewValue(true);
                scan->deviceSchemeWid->feedBack->setState(true);
            }

            setEnabled(false);
            QElapsedTimer t; t.start();
            while (t.elapsed() < 1000) {
                qApp->processEvents();
                SleepExport::msleep2(10);
            }
            setEnabled(true);
        }
        if (DCfbHeight->value() < 0.5) {
            DCfbHeight->setValue(0.5);
            DCfbHeight->setNewValue();
        }

        BLanding->change_state_man_with_sig(true);

        auto_land_state = 1;
        return;
    }

    //landing has completed, start spectro
    if (auto_land_state == 1) {
        PlotMode = mode_Spectroscopy;
        sl_setPlotMode();
        BSpectrJM->change_state_man_with_sig(true);
        JMdiv->setValue(qFloor(DCfbHeight->value() * 1e3 / DCfbPrec->value() / CB_Points_spectr->currentText().toInt()));
        JMdiv->setNewValue(true);
        PBSpectr_start_stop->change_state_man_with_sig(true);

        auto scan = ScanPlot::GetIt();
        if (scan)
            scan->PBSpectr_start_stop->change_state_man(true);

        auto_land_state = 2;
        return;
    }

    //spectroscopy has completed, calc PI
    if (auto_land_state == 2) {
        if (!doPICalib()) {
            PBAutoLand->change_state_man_with_sig(false);

            autoland_notification = new Notification(tr("Smart landing") + ": " + tr("PI calc error"), Notification::Bad, 30);
            doBeep();
            return;
        }

        BFeedbackOnOffMode->change_state_man_with_sig(false);

        auto scan = ScanPlot::GetIt();

        if (!doCheckFixFBosc()) {
            if (scan)
            {
                scan->deviceSchemeWid->feedBack->setState(false);
                scan->smartLanding->change_state_man(1);
            }
            BFeedbackOnOff->change_state_man_with_sig(false);
            PBAutoLand->change_state_man_with_sig(false);


            autoland_notification = new Notification(tr("Smart landing") + ": " + tr("FB is oscillating or noisy"), Notification::Bad, 30);
            doBeep();
            return;
        }

        autoland_notification = new Notification(tr("Smart landing") + ": " + tr("Success"), Notification::Good, 30);
        doBeep();

        auto_land_state = 0;
        PBAutoLand->change_state_man(false);
        if (scan)
            scan->smartLanding->change_state_man(3);
        return;
    }

}

