#include "spmtunning.h"
#include "spm_motors.hpp"
#include "defs.h"

#include <QCloseEvent>
#include "gui/status.hpp"
#include "gui/ttabwidget.hpp"
#include "nstslider.h"
#include "hardware/EG3000.h"
#include "mainwindow.h"

#include "flowlayout.hpp"
#include <qwt_legend.h>

#include "deflect_widget.hpp"
#include "mobile/remote_mobile.hpp"
#include <QTextStream>
#include <QCheckBox>
#include <QFormLayout>
#include <QSettings>
#include <QTimer>
#include <QApplication>
#include <QProgressBar>
#include <QSplitter>
#include "scanplot.h"

#include "2d/plot_2dtypes.h"


typedef QPair<int, int> FBinfo;
Q_DECLARE_METATYPE(FBinfo)


void SPMTunning::closeEvent(QCloseEvent *Event)
{

}

SPMTunning::SPMTunning(QWidget *parent): QWidget(parent)
{
    motors = nullptr;
    auto_land_state = 0;
    last_command = 0;
    piezo_shuttle = 0;
    piezo_step = 0.;
    AutoSend = false;
    spm_sp_units_gain = 0.;
    spm_sp_min = 0.;
    spm_sp_max = 0.;
    nDFLr = 0.;
    nLFLr = 0.;
    nDFL = 0.;
    nLFL = 0.;
    nMAG = 0.;
    nMAG2 = 0.;
    nInt = 0.;
    nZ = 0.;
    progDelta = 0;

    OptionsForMotor = new TOptionsForMotor;
    OptionsForShuttle = new TOptionsForShuttle;

    SWspmfb = new StatusWidget1("SPM FB");
    SWspmfb->status->set(StatusWidget::Circle, Qt::gray);
    SWspmfb->setToolTip(tr("Disabled"));
    prev_spmfb_state = 0;

    OscYrange[0] = -11;
    OscYrange[1] = 11;
    osc_y_postscale = 1;

    plot::ResizeOwner(curveData_freq, 1);
    curveData_freq[0]->Pen.setColor(Qt::red);

    plot::ResizeOwner(curveData_force, 2);
    curveData_force[0]->Pen.setColor(Qt::red);
    curveData_force[1]->Pen.setColor(Qt::blue);


    Spectr_Scan_On = false;
    Z1 = 0;
    Z2 = 0;
    Z_scanner=0;
    Zrange1 = Zrange2 = Zrange = 20;
    Up_lines_spectr = true;
    cz_updated = true;

    createFeedBackWorkSpace();
    createFreqWorkSpace();
    InitShuttleGuiLink();
    createLockInWorkSpace();
    createSpectroscopyWorkSpace();
    create_plot_mode_control();
    InitMotorControlSpace();
    createLayout();
    setWindowTitle(tr("SPM Tunning - NSpec"));


    const double sqrtOf2 = sqrt(2.); //lockin(demodulator) averaging compensation
    input_gain_set(srcMag1, sqrtOf2);
    input_gain_set(srcMag2, sqrtOf2);
    input_gain_set(srcMagCos1, sqrtOf2); //these need additional *2 on dla2k(if available)
    input_gain_set(srcMagSin1, sqrtOf2);
    double gain = 360. / 20; //degree/V
    input_gain_set(srcPhase1, gain);
    input_unit_set(srcPhase1, QChar(0x00B0));
    emit input_calib_updated();

    s_setCalibHeight();


    QAction *a;
    SWspmfb->setContextMenuPolicy(Qt::ActionsContextMenu);
    SWspmfb->addAction(a = new QAction(tr("Toggle"), SWspmfb));
    connect(a, SIGNAL(triggered()), BFeedbackOnOff, SLOT(click()));

    Progress_timer = new QTimer(this);
    connect(Progress_timer, SIGNAL(timeout()), this, SLOT(s_updFreqProgBar()));
    connect(Progress_timer, SIGNAL(timeout()), this, SLOT(s_updSpectrProgBar()));
    connect(Progress_timer, SIGNAL(timeout()), this, SLOT(s_updLandProgBar()));

    laserSpotTimeout.start();

    PlotMode = mode_ResCurve;
    connect(MyPl, SIGNAL(AddDataToContainer(plot::TCurvesDataOwner,plot::TMetaStorage)), SLOT(AddDataToContainer_slot(plot::TCurvesDataOwner,plot::TMetaStorage)));
    connect(MyPlScan, SIGNAL(AddDataToContainer(plot::TCurvesDataOwner,plot::TMetaStorage)), SLOT(AddDataToContainer_slot(plot::TCurvesDataOwner,plot::TMetaStorage)));

    blanding_status = false;
    connect(blanding_syncT = new QTimer(this), SIGNAL(timeout()), this, SLOT(blanding_sync()));
    blanding_syncT->setSingleShot(true);

    saveblock = true;
    sl_load_from_file();

#ifdef USE_EG3K_EMULATOR
    QSettings settings("Param/Emulator.conf", QSettings::IniFormat);
    if (settings.value("usb", false).toBool()) {
        QTimer *t = new QTimer;
        connect(t, SIGNAL(timeout()), this, SLOT(inject_test_packet()));
        t->start(50);
    }
#endif
}

void SPMTunning::AddDataToContainer_slot(plot::TCurvesDataOwner src, plot::TMetaStorage attributes)
{
    attributes.insert("Name", MyPl->Pure2dPlot->Title);
    attributes.insert("Source", "SPM");

    attributes.insert("Channel", CB_fr_Channel->currentText());
    attributes.insert("DFLgain", gain1->currentText());
    attributes.insert("LFLgain", gain2->currentText());
    attributes.insert("INTgain", gain3->currentText());

    attributes.insert("Spectroscopy points", CB_Points_spectr->currentText());
    attributes.insert("Spectroscopy speed, mkm/s", ev_Speed_spectr->value());
    attributes.insert("Spectroscopy dz1", ev_Z1_spectr->value());
    attributes.insert("Spectroscopy dz2", NS_Z2_spectr->value());

    appendAttrib(attributes);

    emit AddDataToContainer(src, attributes);
}
void SPMTunning::appendAttrib(plot::TMetaStorage &attributes)
{
    attributes.insert("SPM FB PID", FBZ_config);
    if (BFeedbackOnOff->isOn) {
        attributes.insert("SPM FB scanner", FB_Scanner);
        attributes.insert("SPM FB mode", fbs->currentText());

        attributes.insert("Set point", DCfbSetPoint->value());
        if (!BFeedbackOnOffMode->isOn) {
            attributes.insert("FB P", DCfbP->value());
            attributes.insert("FB I", DCfbI->value());
        } else {
            attributes.insert("FB JM Precision", DCfbPrec->value());
            attributes.insert("FB JM Height", DCfbHeight->value());
        }
    }

    attributes.insert("From, kHz", EFreqScanMin->value());
    attributes.insert("To, kHz", EFreqScanMax->value());
    attributes.insert("Current frequency, kHz", curFrequency->value());
    attributes.insert("Drive intensity, V", driveInt->value());
    attributes.insert("Low pass filter, kHz", lowPassFilter->value());
    attributes.insert("Phase shift, degree", phaseShift->value());

    attributes.insert("ChannelIdx", CB_fr_Channel->itemData(CB_fr_Channel->currentIndex()).toInt());
    attributes.insert("DFLIdx", gain1->currentIndex());
    attributes.insert("LFLIdx", gain2->currentIndex());
    attributes.insert("INTIdx", gain3->currentIndex());
}
void SPMTunning::restoreAttributes(plot::TMetaStorage attrib)
{
    auto scan = ScanPlot::GetIt();

    if (attrib.contains("FB P")) {
        DCfbP->setValue(attrib.value("FB P").toDouble());
        DCfbP->setNewValue(true);
        if (scan)
            scan->deviceSchemeWid->pid->edPr->setValue(attrib.value("FB P").toDouble());
    }
    if (attrib.contains("FB I")) {
        DCfbI->setValue(attrib.value("FB I").toDouble());
        DCfbI->setNewValue(true);
        if (scan)
            scan->deviceSchemeWid->pid->edInt->setValue(attrib.value("FB I").toDouble());
    }
    if (attrib.contains("FB JM Precision")) {
        DCfbPrec->setValue(attrib.value("FB JM Precision").toDouble());
        DCfbPrec->setNewValue(true);
    }
    if (attrib.contains("FB JM Height")) {
        DCfbHeight->setValue(attrib.value("FB JM Height").toDouble());
        DCfbHeight->setNewValue(true);
    }
    if (attrib.contains("Set point")) {
        DCfbSetPoint->setValue(attrib.value("Set point").toDouble());
        DCfbSetPoint->setNewValue(true);
        if (scan)
        {
            scan->deviceSchemeWid->setPointEdit->setValue(attrib.value("Set point").toDouble());
            scan->deviceSchemeWid->setPointEdit->setNewValue(true);
        }
    }

    if (attrib.contains("From, kHz"))
    {
        EFreqScanMin->setValue(attrib.value("From, kHz").toDouble());
        if (scan)
            scan->deviceSchemeWid->oscilator->edFreqScanMin->setValue(attrib.value("From, kHz").toDouble());
    }
    if (attrib.contains("To, kHz"))
    {
        EFreqScanMax->setValue(attrib.value("To, kHz").toDouble());
        if (scan)
            scan->deviceSchemeWid->oscilator->edFreqScanMax->setValue(attrib.value("To, kHz").toDouble());
    }
    if (attrib.contains("Current frequency, kHz")) {
        curFrequency->setValue(attrib.value("Current frequency, kHz").toDouble());
        curFrequency->setNewValue();
        if (scan)
            scan->deviceSchemeWid->oscilator->edFreq->setValue(attrib.value("Current frequency, kHz").toDouble());
    }
    if (attrib.contains("Drive intensity, V")) {
        driveInt->setValue(attrib.value("Drive intensity, V").toDouble());
        driveInt->setNewValue();
        if (scan)
            scan->deviceSchemeWid->oscilator->edAmp->setValue(attrib.value("Drive intensity, V").toDouble());
    }
    if (attrib.contains("Low pass filter, kHz")) {
        lowPassFilter->setValue(attrib.value("Low pass filter, kHz").toDouble());
        lowPassFilter->setNewValue();
        if (scan)
            scan->deviceSchemeWid->LockIn->edLP->setValue(attrib.value("Low pass filter, kHz").toDouble());
    }
    if (attrib.contains("Phase shift, degree")) {
        phaseShift->setValue(attrib.value("Phase shift, degree").toDouble());
        phaseShift->setNewValue();
        if (scan)
            scan->deviceSchemeWid->LockIn->edPSh->setValue(attrib.value("Phase shift, degree").toDouble());
    }

    if (attrib.contains("ChannelIdx")) {
        int idx = CB_fr_Channel->findData(attrib.value("ChannelIdx").toInt());
        if (idx != -1)
        {
            CB_fr_Channel->setCurrentIndex(idx);
            if (scan)
                scan->deviceSchemeWid->oscilator->cbChannel->setCurrentIndex(idx);
        }
    }

    if (attrib.contains("DFLIdx"))
    {
        gain1->setCurrentIndex(attrib.value("DFLIdx").toInt());
        if (scan)
            scan->deviceSchemeWid->inputGain->cbDFL->setCurrentIndex(attrib.value("DFLIdx").toInt());
    }
    if (attrib.contains("LFLIdx"))
    {
        gain2->setCurrentIndex(attrib.value("LFLIdx").toInt());
        if (scan)
            scan->deviceSchemeWid->inputGain->cbLF->setCurrentIndex(attrib.value("LFLIdx").toInt());
    }
    if (attrib.contains("INTIdx"))
    {
        gain3->setCurrentIndex(attrib.value("INTIdx").toInt());
        if (scan)
            scan->deviceSchemeWid->inputGain->cbINT->setCurrentIndex(attrib.value("INTIdx").toInt());
    }
}


void SPMTunning::sl_save_to_file()
{
    if (saveblock) return;

    QFile dataFile("Param/SPM.conf");
    dataFile.open(QIODevice::WriteOnly|QIODevice::Text);
    QTextStream outData(&dataFile);
    outData<<"Nano Scan Technologies Co." << endl;
    outData<<"Parameter file for spm settings" << endl << endl;
    outData<<"Laser on_off = " << 1 << endl;
    outData<<"Preamp Gain = " << 0 << endl;

    outData<<"Feed Back on_off = " << 0 << endl;
    outData<<"Feed Back pos_neg = " << Mode_data[currModeIdx][4] << endl;
    outData<<"Feed Back Signal = " << currModeIdx + 2 << endl;

    outData<<"Freqency scan Signal = " << CB_fr_Channel->itemData(CB_fr_Channel->currentIndex()).toInt() << endl;
    outData<<"Freqency scan from = " << EFreqScanMin->value() << endl;
    outData<<"Freqency scan to = " << EFreqScanMax->value() << endl;
    outData<<"Freqency current = " << curFrequency->value() << endl;
    outData<<"Low pass = " << lowPassFilter->value() << endl;
    outData<<"Phase Shift = " << phaseShift->value() << endl;

    const char *titles[11] = {"DFL data: ", "LFL data: ",
                       "MAG data: ", "MAGSIN data: ", "MAGCOS data: ", "Phase data: ",
                       "ADCAUX1 data: ", "ADCAUX2 data: ", "ITIP data: ", "ITIPlog data: ",
                       "MAG_TUN data: "};

    for (int i = 0; i < 11; i++) {
        outData << titles[qMin(i, 11-1)] << endl;
        outData << "SetP = " << Mode_data[i][0] << endl;
        outData << "FB_P = " << Mode_data[i][1] << endl;
        outData << "FB_I = " << Mode_data[i][2] << endl;
        outData << "FB_sign = " << Mode_data[i][4] << endl;
        outData << "Drive = " << Mode_data[i][3] << endl;
        outData << "In1 gain = " << Gain_data[i][0] << endl;
        outData << "In2 gain = " << Gain_data[i][1] << endl;
        outData << "In3 gain = " << Gain_data[i][2] << endl;
    }

    dataFile.close();
}
void SPMTunning::sl_load_from_file()
{
    auto scan = ScanPlot::GetIt();

    saveblock = true;

    QString parname;
    int fb_signal=0;
    int fr_signal=0;
    double fr_from=0;
    double fr_to=0;
    double fr_current=0;

    double LP=0;
    double Psh=0;

    QFile dataFile("Param/SPM.conf");
    dataFile.open(QIODevice::ReadOnly|QIODevice::Text);
    QTextStream inData(&dataFile);
    if (inData.atEnd() || inData.status() != QTextStream::Ok)
        goto nodata;

    inData>>parname>>parname>>parname>>parname;
    inData>>parname>>parname>>parname>>parname>>parname;
    inData>>parname>>parname>>parname>>parname;
    inData>>parname>>parname>>parname>>parname;
    inData>>parname>>parname>>parname>>parname>>parname;
    inData>>parname>>parname>>parname>>parname>>parname;
    inData>>parname>>parname>>parname>>parname>>fb_signal; fb_signal = qMax(0, fb_signal - 2);

    inData>>parname>>parname>>parname>>parname>>fr_signal;
    inData>>parname>>parname>>parname>>parname>>fr_from;
    inData>>parname>>parname>>parname>>parname>>fr_to;
    inData>>parname>>parname>>parname>>fr_current;
    inData>>parname>>parname>>parname>>LP;
    inData>>parname>>parname>>parname>>Psh;
    if (!inData.atEnd() && inData.status() == QTextStream::Ok) {
        CB_fr_Channel->setCurrentIndex(CB_fr_Channel->findData(fr_signal));
        if (scan)
        {
            scan->deviceSchemeWid->oscilator->cbChannel->setCurrentIndex(CB_fr_Channel->findData(fr_signal));
            scan->deviceSchemeWid->oscilator->edFreqScanMin->setValue(fr_from);
            scan->deviceSchemeWid->oscilator->edFreqScanMax->setValue(fr_to);
            scan->deviceSchemeWid->oscilator->edFreq->setValue(fr_current);
            scan->deviceSchemeWid->LockIn->edLP->setValue(LP);
            scan->deviceSchemeWid->LockIn->edPSh->setValue(Psh);
        }
        EFreqScanMin->setValue(fr_from);
        EFreqScanMax->setValue(fr_to);
        curFrequency->setValue(fr_current);
        lowPassFilter->setValue(LP);
        phaseShift->setValue(Psh);
    }

    for (int i = 0; i < 11; i++) {
        inData>>parname>>parname;
        if (inData.atEnd() || inData.status() != QTextStream::Ok)
            break;
        inData>>parname>>parname>>Mode_data[i][0];
        inData>>parname>>parname>>Mode_data[i][1];
        inData>>parname>>parname>>Mode_data[i][2];
        inData>>parname>>parname>>Mode_data[i][4];
        inData>>parname>>parname>>Mode_data[i][3];
        inData>>parname>>parname>>parname>>Gain_data[i][0];
        inData>>parname>>parname>>parname>>Gain_data[i][1];
        inData>>parname>>parname>>parname>>Gain_data[i][2];
    }

    dataFile.close();
nodata:

    for (int i = 0; i < fbs->count(); i++)
        if (fb_signal == fbs->itemData(i).value<QPair<int, int> >().first) {
            fbs->setCurrentIndex(i);
            break;
        }
    if (scan)
    {
        if (srcMag1 == fbs->itemData(fbs->currentIndex()).value<QPair<int, int> >().second)
            scan->deviceSchemeWid->PIDSwitch->currentConnectedIn = 3;
        else if (srcPhase1 == fbs->itemData(fbs->currentIndex()).value<QPair<int, int> >().second)
            scan->deviceSchemeWid->PIDSwitch->currentConnectedIn = 0;
        else if (srcDFL == fbs->itemData(fbs->currentIndex()).value<QPair<int, int> >().second)
            scan->deviceSchemeWid->PIDSwitch->currentConnectedIn = 4;
        else if (srcLFL == fbs->itemData(fbs->currentIndex()).value<QPair<int, int> >().second)
            scan->deviceSchemeWid->PIDSwitch->currentConnectedIn = 5;
    }
    setFBchannel();

    if (scan)
        scan->deviceSchemeWid->feedBack->setState(false);
    BFeedbackOnOff->change_state_man_with_sig(false);
    //sl_change_z_slider(); //resync zpos //disabled, breaks something

    curFrequency->setNewValue();
    lowPassFilter->setNewValue();
    phaseShift->setNewValue();

    saveblock = false;
}


static
double linear_calib(double value, double from_min, double from_max, double to_min, double to_max)
{
    return (value - from_min) / (from_max - from_min) * (to_max - to_min) + to_min;
}
void SPMTunning::setSPgui()
{
    auto scan = ScanPlot::GetIt();

    double min = input_calib(FB_channel, 0),
            max = input_calib(FB_channel, 65535);
    int decimals = 3;

    plot::TSIUnit tunit;
    tunit.SetSIUnit(input_units(FB_channel));
    tunit.UpdateLegend(min, max);
    spm_sp_units_gain = tunit.Multiplier;
    spm_sp_min = min = min / tunit.Multiplier;
    spm_sp_max = max = max / tunit.Multiplier;

    switch (FB_channel) {
    case srcMag1:
    case srcMag2:
        min = 0;
        max /= 3.;
        break;
    case srcPhase1:
        spm_sp_units_gain = 1;
        spm_sp_min = min = -180;
        spm_sp_max = max = 180;
        decimals = 1;
        tunit.Legend = QChar(0x00B0);
        break;
    }

    DCfbSetPoint->setDecimals(decimals);
    DCfbSetPoint->setMinimum(qMin(min, max));
    DCfbSetPoint->setMaximum(qMax(min, max));
    DCfbSetPointUnits->setText(tunit.Legend);

    if (scan)
    {
        scan->deviceSchemeWid->setPointEdit->setDecimals(decimals);
        scan->deviceSchemeWid->setPointEdit->setMinimum(qMin(min, max));
        scan->deviceSchemeWid->setPointEdit->setMaximum(qMax(min, max));
        scan->deviceSchemeWid->setPointEdit->setValue(linear_calib(Mode_data[currModeIdx][0], -10, 10, spm_sp_min, spm_sp_max));
    }

    DCfbSetPoint->setValue(linear_calib(Mode_data[currModeIdx][0], -10, 10, spm_sp_min, spm_sp_max));
}

void SPMTunning::setFBchannel()
{
    auto scan = ScanPlot::GetIt();

    currModeIdx = fbs->itemData(fbs->currentIndex()).value<QPair<int, int> >().first;
    FB_channel = fbs->itemData(fbs->currentIndex()).value<QPair<int, int> >().second;
    sendPID = true;


    //apply pid/gains
    setSPgui();

    DCfbP->setValue(Mode_data[currModeIdx][1]);
    DCfbI->setValue(Mode_data[currModeIdx][2]);
    driveInt->setValue(Mode_data[currModeIdx][3]);
    B_PID_sign->change_state_man(Mode_data[currModeIdx][4]);

    if (scan)
    {
        scan->deviceSchemeWid->pid->tsbFBSign->change_state_man(Mode_data[currModeIdx][4]);
        scan->deviceSchemeWid->pid->edPr->setValue(Mode_data[currModeIdx][1]);
        scan->deviceSchemeWid->pid->edInt->setValue(Mode_data[currModeIdx][2]);
        scan->deviceSchemeWid->oscilator->edAmp->setValue(Mode_data[currModeIdx][3]);
    }

    gain1->blockSignals(true);
    gain2->blockSignals(true);
    gain3->blockSignals(true);
    gain1->setCurrentIndex(Gain_data[currModeIdx][0]-1);
    gain2->setCurrentIndex(Gain_data[currModeIdx][1]-1);
    gain3->setCurrentIndex(Gain_data[currModeIdx][2]-1);
    if (scan)
    {
        scan->deviceSchemeWid->inputGain->cbDFL->setCurrentIndex(Gain_data[currModeIdx][0]-1);
        scan->deviceSchemeWid->inputGain->cbLF->setCurrentIndex(Gain_data[currModeIdx][1]-1);
        scan->deviceSchemeWid->inputGain->cbINT->setCurrentIndex(Gain_data[currModeIdx][2]-1);
    }
    gain1->blockSignals(false);
    gain2->blockSignals(false);
    gain3->blockSignals(false);

    s_sendGains();
    DCfbSetPoint->setNewValue();
    DCfbP->setNewValue();
    DCfbI->setNewValue();
    driveInt->setNewValue();


    bool is_tuning_fork = currModeIdx == 10;
    bool is_mag = (currModeIdx >= 2 && currModeIdx <= 5) || (currModeIdx == 10);
    bool is_stm = currModeIdx >= 6 && currModeIdx <= 9;
    bool is_stm_int = currModeIdx == 8 || currModeIdx == 9;

    //stm / tuning_fork
    if (is_mag)
        spmTabs->setCurrentWidget(driveInt->parentWidget()->parentWidget());
    if (is_stm)
        spmTabs->setCurrentWidget(Vtip->parentWidget()->parentWidget());

    LaserSpotWindow->setTuningFork(is_tuning_fork);
    LaserSpot->setTuningFork(is_tuning_fork);
    LaserSpotWindow->setSTM(is_stm_int);
    LaserSpot->setSTM(is_stm_int);

    lflCurrent->setChecked(is_stm_int);
    extCurrent->setChecked(is_stm_int || is_tuning_fork);

    if (scan)
    {
        scan->deviceSchemeWid->LFLSwitch->currentConnectedIn = is_stm_int;
        scan->deviceSchemeWid->DFLSwitch->currentConnectedIn = is_stm_int || is_tuning_fork;
    }

    if (is_stm || is_tuning_fork)
        CBcalibDFL_F->setChecked(false);


    AC_DC = FB_channel >= srcMag1 && FB_channel <= srcPhase1 ? 1 : 0;
    LaserOnOf = !(is_stm || is_tuning_fork);
    BLaserOnOff->change_state_man(!LaserOnOf);
    sendHead = true;


    sl_setPlotMode();
    sl_save_to_file();
}

void SPMTunning::s_setFBMode(bool state)
{
    DCfbP->parentWidget()->setVisible(!state);
    DCfbI->parentWidget()->setVisible(!state);
    DCfbPrec->parentWidget()->setVisible(state);
    DCfbHeight->parentWidget()->setVisible(state);

    BSpectrJM->setVisible(state);
    JMdiv->setVisible(state);
    LJMdiv->setVisible(state);
    if (!state)
        BSpectrJM->change_state_man_with_sig(false);
    else {
        DCfbHeight->setNewValue();
        DCfbPrec->setNewValue();
    }

    s_setFBOnOff(BFeedbackOnOff->isOn);
}
void SPMTunning::s_setFBOnOff(bool state)
{
    FBZ_config_masked = false;
    FBZ_config = state ? (BFeedbackOnOffMode->isOn ? fbZ_SPM_j :
                                                       fbZ_SPM) :
                         fbOff;
    sendFBon = true;

    if (prev_spmfb_state != (int)state) {
        prev_spmfb_state = state;
        SWspmfb->status->set(StatusWidget::Circle, state ? Qt::blue : Qt::gray);
        SWspmfb->setToolTip(state ? tr("Contact") : tr("Disabled"));
    }
}
void SPMTunning::s_setFBmasked(bool masked)
{
    FBZ_config_masked = masked;
    sendFBon = true;
}

void SPMTunning::sl_laser_on_of(bool value)
{
    LaserOnOf = (int)(!value);
    sendHead = true;
}
void SPMTunning::s_setFBScanner(int idx)
{
    Zrange = (idx == 0 ? Zrange1 : Zrange2);

    FB_Scanner = idx;
    sendPID = true;

    sl_set_XYZ_range_sync();
}

void SPMTunning::s_sendFBP()
{
    //auto scan = ScanPlot::GetIt();
    Mode_data[currModeIdx][1] = DCfbP->value();

    /*if (!show_additional_gui()) {
        QSettings settings2("Param/controller.conf", QSettings::IniFormat);
        DCfbI->setValue(DCfbP->value() / settings_double(settings2, "auto_i_divisor", 500));
        DCfbI->setNewValue();
        scan->deviceSchemeWid->pid->edInt->setValue(DCfbP->value() / settings_double(settings2, "auto_i_divisor", 500));
    }*/
    sendPID = true;

    sl_save_to_file();
}
void SPMTunning::s_sendFBI()
{
    Mode_data[currModeIdx][2] = DCfbI->value();
    sendPID = true;

    sl_save_to_file();
}
void SPMTunning::s_sendSP()
{
    Mode_data[currModeIdx][0] = linear_calib(DCfbSetPoint->value(), spm_sp_min, spm_sp_max, -10, 10);
    sendSetpoint = true;

    sl_save_to_file();
}
void SPMTunning::s_jm_prec()
{
    double val = DCfbPrec->value() * 1e-3 / Zrange;
    WORD msb = 0;
    WORD lsb = 0;
    if (!l2k_controller)
        msb = qMax<int>(1, val * 65535);
    else {
        if (qFuzzyIsNull(val))
            val = 0.01 * 1e-3 / Zrange;

        DWORD step = val * 65535 * 65535;
        msb = step >> 16;
        lsb = step & 0xFFFF;
    }
    m_jm_precision = msb;
    m_jm_precision_lsb = lsb;
    updateJM_Precision = true;
}
void SPMTunning::s_jm_height()
{
    m_jm_lift = qMax<int>(10, DCfbHeight->value() * 65535 / Zrange);
    updateJM_Lift = true;
}

void SPMTunning::s_setCurFrequency(double value)
{
    value *= 1e3; //khz->hz

    MyPl->Pure2dPlot->MoveVertLine(value, 0);
    MyPlScan->Pure2dPlot->MoveVertLine(value, 0);

    current_frequency = value;
    sendDriveFreq = true;

    //sl_save_to_file();
}

void SPMTunning::s_sendGains()
{
    Gain_data[currModeIdx][0] = gain1->currentIndex() + 1;
    Gain_data[currModeIdx][1] = gain2->currentIndex() + 1;
    Gain_data[currModeIdx][2] = gain3->currentIndex() + 1;
    sendGains = true;
    sl_save_to_file();

    s_setCalibLFI();
    s_setCalibDFL();
}
void SPMTunning::s_sendGains2()
{
    gain12v = gain12->currentIndex() + 1;
    gain22v = gain22->currentIndex() + 1;
    gain32v = gain32->currentIndex() + 1;
    sendGains2 = true;
}

void SPMTunning::s_sendDriveInt()
{
    Mode_data[currModeIdx][3] = driveInt->value();
    sendDriveV = true;

    sl_save_to_file();
}

void SPMTunning::s_set_Kelvin(bool val)
{
    auto scan = ScanPlot::GetIt();

    if (/*val ||*/ !piezoTip2->isChecked())
    {
        piezoTip2->setChecked(val);
        if (scan)
            scan->deviceSchemeWid->plusSwitch->currentConnectedIn = val;
    }

    kelvinOn = val;
    updateKelvin = true;
}
void SPMTunning::s_set_efmplus(bool val)
{
    auto scan = ScanPlot::GetIt();

    efmplus = val;
    updateEFMplus = true;

    if (val) {
        piezoTip->setChecked(false);
        capMode->setChecked(false);
        spmTabs2->setCurrentIndex(qMin(1, spmTabs2->count()-1));
    }
    piezoTip->setEnabled(!val);
    capMode->setEnabled(!val);

    curFrequency2->setValue(val ? 3 : curFrequency->value());
    curFrequency2->setNewValue();
    driveInt2->setValue(val ? 1 : 0);
    driveInt2->setNewValue();

    //disable not applicable
    lflCurrent->setEnabled(!val);
    extCurrent->setEnabled(!val);
    if (scan)
    {
        scan->deviceSchemeWid->LFLSwitch->setEnabled(!val);
        scan->deviceSchemeWid->DFLSwitch->setEnabled(!val);
    }

    QString nameMAG2 = !val ? QObject::tr("MAG(LFL)") : QObject::tr("MAG(EFM+)");
    changeCBname(srcMag2, nameMAG2);
}

void SPMTunning::updateTipDrive()
{
    driveInt->blockSignals(true);
    driveInt->slider->blockSignals(true);
    Mode_data[currModeIdx][3] = newTipDriveValue;
    driveInt->setValue(newTipDriveValue);
    driveInt->newValue();
    
    auto scan = ScanPlot::GetIt();
    if (scan)
    {
        scan->driveInt->blockSignals(true);
        scan->driveInt->slider->blockSignals(true);
        scan->driveInt->setValue(newTipDriveValue);
        scan->driveInt->newValue();
        
        scan->tipDriveNewValue(newTipDriveValue);
        
        scan->driveInt->blockSignals(false);
        scan->driveInt->slider->blockSignals(false);
    }
    
    driveInt->slider->blockSignals(false);
    driveInt->blockSignals(false);
}

void SPMTunning::set_spot_data()
{
    if (laserSpotTimeout.elapsed() < 99)
        return;
    laserSpotTimeout.restart();

    plot::TSIUnit tunit;
    tunit.SetSIUnit(input_units(srcDFL));
    tunit.UpdateLegend(nDFL);
    QString dfl = QString::number(nDFL / tunit.Multiplier, 'f', 1) + ' ' + tunit.Legend;
    tunit.SetSIUnit(input_units(srcLFL));
    tunit.UpdateLegend(nLFL);
    QString lfl = QString::number(nLFL / tunit.Multiplier, 'f', 1) + ' ' + tunit.Legend;
    tunit.SetSIUnit(input_units(srcMag1));
    tunit.UpdateLegend(nMAG);
    QString mag = QString::number(nMAG / tunit.Multiplier, 'f', 1) + ' ' + tunit.Legend;

    LaserSpotWindow->setXY(nLFLr, nDFLr);
    LaserSpotWindow->setInt(nInt);
    LaserSpotWindow->setDfl(dfl);
    LaserSpotWindow->setLfl(lfl);
    LaserSpotWindow->setMag(mag);
    LaserSpotWindow->setZpos(nZ);

    LaserSpot->setXY(nLFLr, nDFLr);
    LaserSpot->setInt(nInt);
    LaserSpot->setDfl(dfl);
    LaserSpot->setLfl(lfl);
    LaserSpot->setMag(mag);
    LaserSpot->setZpos(nZ);

    auto scan = ScanPlot::GetIt();
    if (scan)
    {
        scan->deviceSchemeWid->DFLtext->setValue(nDFL);
        scan->deviceSchemeWid->LFLtext->setValue(nLFL);
        scan->deviceSchemeWid->INTtext->setValue(nInt);
    }

    if (auto mob = MobileServer::GetIt()) {
        if (mob->any_need_data("deflect_need"))
        {
            auto list = QVariantList();
            list << nLFLr << nDFLr << nInt << 0 << nZ << dfl << lfl << mag;
            list << LaserSpot->qValue;
            mob->setPropIdVar(DataID::Deflect_data, list, 1);
        }
    }

    if (Up_lines_spectr) {
        MyPl->Pure2dPlot->MoveVertLine(nZ * Zrange * 1e-6, 2);
        MyPlScan->Pure2dPlot->MoveVertLine(nZ * Zrange * 1e-6, 2);
        Z_scanner = nZ * Zrange;

        if (!Spectr_Scan_On) {
            s_set_Z1_spectr(Z1);
            s_set_Z2_spectr(Z2);
        }
        //NS_Z_spectr->setValue((nZ/65535.0)*Zrange);
    }

    static int div = 0;
    div++;
    if (div >= 10) {
        div = 0;
        int state = 1;
        if (nZ < 0.05) state = 3;
        if (nZ > 0.97) state = 2;
        if (prev_spmfb_state > 0 && prev_spmfb_state != state) {
            prev_spmfb_state = state;
            SWspmfb->status->set(StatusWidget::Circle, state == 1 ? Qt::blue : state == 2 ? Qt::green : Qt::red);
            SWspmfb->setToolTip(state == 1 ? tr("Contact") : state == 2 ? tr("No contact") : tr("At limit"));
        }
    }
}


void SPMTunning::sl_set_XYZ_range2(double /*x*/, double /*y*/, double z)
{
    Zrange2 = z;
    if (FB_Scanner == 1) {
        Zrange = Zrange2;
        sl_set_XYZ_range_sync();
    }
}
void SPMTunning::sl_set_XYZ_range(double /*x*/, double /*y*/, double z)
{
    Zrange1 = z;
    if (FB_Scanner == 0) {
        Zrange = Zrange1;
        sl_set_XYZ_range_sync();
    }
}
void SPMTunning::sl_set_XYZ_range_sync()
{
    if (Zrange < Z_scanner)
        Z_scanner = Zrange;

    NS_Z1_spectr->setMainRangeAndStep(Zrange, 0, 0.001);
    NS_Z2_spectr->setMaximum(Zrange);
    NS_Z_spectr->setMaximum(Zrange);
    NS_Z1_spectr->setValue(0);
    NS_Z2_spectr->setValue(0);

    if (BFeedbackOnOffMode->isOn) {
        DCfbHeight->setNewValue();
        DCfbPrec->setNewValue();
    }

    if (PlotMode == mode_Spectroscopy) {
        MyPl->Pure2dPlot->SetAllXScales(0, Zrange / 1e6);
        MyPl->Refresh();
        MyPlScan->Pure2dPlot->SetAllXScales(0, Zrange / 1e6);
        MyPlScan->Refresh();
    }

    s_setCalibHeight();
}

void SPMTunning::s_set_Z1_spectr(double value)
{
    Z1 = value;
    if ((Z_scanner - Z1) <= 0) {
        Z1 = Z_scanner;
        NS_Z1_spectr->setValue(Z1);
        auto scan = ScanPlot::GetIt();
        if (scan)
            scan->ev_Z1_spectr->setValue(Z1);
    }

    ev_Z1_spectr->setValue(Z1);
    MyPl->Pure2dPlot->MoveVertLine((Z_scanner - Z1) / 1e6, 1);
    MyPlScan->Pure2dPlot->MoveVertLine((Z_scanner - Z1) / 1e6, 1);
}
void SPMTunning::s_set_Z2_spectr(double value)
{
    Z2 = value;
    if ((Z_scanner + Z2) >= Zrange) {
        Z2 = Zrange - Z_scanner;
        NS_Z2_spectr->setValue(Z2);
        auto scan = ScanPlot::GetIt();
        if (scan)
            scan->ev_Z2_spectr->setValue(Z2);
    }

    MyPl->Pure2dPlot->MoveVertLine((Z_scanner + Z2) / 1e6, 3);
    MyPlScan->Pure2dPlot->MoveVertLine((Z_scanner + Z2) / 1e6, 3);
}
void SPMTunning::s_set_Z_spectr(double value)
{
    Z_scanner = value;

    MyPl->Pure2dPlot->MoveVertLine(Z_scanner / 1e6, 2);
    MyPlScan->Pure2dPlot->MoveVertLine(Z_scanner / 1e6, 2);
}
void SPMTunning::s_send_Zpos()
{
    if (FB_Scanner == 0)
        emit sig_Zpos(NS_Z_spectr->value());
    else
        emit sig_Z2pos(NS_Z_spectr->value());
}
void SPMTunning::sl_change_z_slider()
{
    auto osc = Oscilloscope::GetIt();
    if (!osc)
        return;

    disconnect(osc->DacsWidget, SIGNAL(zUpdate(double)), NS_Z_spectr, SLOT(setValue(double)));
    disconnect(osc->DacsWidget, SIGNAL(z2Update(double)), NS_Z_spectr, SLOT(setValue(double)));

    if (FB_Scanner == 0) {
        connect(osc->DacsWidget, SIGNAL(zUpdate(double)), NS_Z_spectr, SLOT(setValue(double)));
        NS_Z_spectr->setValue(osc->DacsWidget->Zsp);
        box_Z_spectr->setTitle(tr("Z head, ")+tr("mkm"));
    } else {
        connect(osc->DacsWidget, SIGNAL(z2Update(double)), NS_Z_spectr, SLOT(setValue(double)));
        NS_Z_spectr->setValue(osc->DacsWidget->Zsp2);
        box_Z_spectr->setTitle(tr("Z stage, ")+tr("mkm"));
    }
}
void SPMTunning::Zpos_on_off(bool st)
{
    box_Z_spectr->setEnabled(!st);
    if (!st) {
        NS_Z_spectr->setValue(0);
        NS_Z_spectr->setNewValue();
    }
}


void SPMTunning::sl_up_lines_on()
{
    Up_lines_spectr=true;
}
void SPMTunning::sl_up_lines_of()
{
    Up_lines_spectr=false;
}


void SPMTunning::switchMode(int idx)
{
    if (PlotMode == mode_Osc) {
        OscYrange = MyPl->Pure2dPlot->CurrentYLimits;
        OscYrange[0] /= osc_y_postscale;
        OscYrange[1] /= osc_y_postscale;
    }

    PlotMode = modeBar->tabData(idx).toInt();
    sl_setPlotMode();
}

void SPMTunning::sl_setPlotMode()
{
    modeBar->blockSignals(true);
    int idx = 0;
    for (int i = 0; i < modeBar->count(); i++)
        if (PlotMode == modeBar->tabData(i).toInt()) {
            idx = i;
            break;
        }
    modeBar->setCurrentIndex(idx);
    modeBar->blockSignals(false);

    MyPl->Pure2dPlot->QPlot->insertLegend(PlotMode == mode_Spectroscopy ? new QwtLegend() : nullptr, QwtPlot::BottomLegend);

    MyPl->SetAutoScalingEnabled(PlotMode == mode_ResCurve || PlotMode == mode_Spectroscopy);
    MyPl->Pure2dPlot->SetAllYScales(-11, 11);

    MyPl->Pure2dPlot->setVertLineVisible(0, PlotMode == mode_ResCurve);
    MyPl->Pure2dPlot->setVertLineVisible(1, PlotMode == mode_Spectroscopy);
    MyPl->Pure2dPlot->setVertLineVisible(2, PlotMode == mode_Spectroscopy);
    MyPl->Pure2dPlot->setVertLineVisible(3, PlotMode == mode_Spectroscopy);

    MyPlScan->Pure2dPlot->QPlot->insertLegend(PlotMode == mode_Spectroscopy ? new QwtLegend() : nullptr, QwtPlot::BottomLegend);

    MyPlScan->SetAutoScalingEnabled(PlotMode == mode_ResCurve || PlotMode == mode_Spectroscopy);
    MyPlScan->Pure2dPlot->SetAllYScales(-11, 11);

    MyPlScan->Pure2dPlot->setVertLineVisible(0, PlotMode == mode_ResCurve);
    MyPlScan->Pure2dPlot->setVertLineVisible(1, PlotMode == mode_Spectroscopy);
    MyPlScan->Pure2dPlot->setVertLineVisible(2, PlotMode == mode_Spectroscopy);
    MyPlScan->Pure2dPlot->setVertLineVisible(3, PlotMode == mode_Spectroscopy);

    if (PlotMode != mode_Osc && grb_Av_rms->isVisible())
        emit sig_osc_stop();

    box_scan->setVisible(PlotMode == mode_ResCurve);
    grb_Av_rms->setVisible(PlotMode == mode_Osc);
    box_Z1_spectr->setVisible(PlotMode == mode_Spectroscopy);
    box_Z2_spectr->setVisible(PlotMode == mode_Spectroscopy);
    box_Spect_start->setVisible(PlotMode == mode_Spectroscopy);

    switch (PlotMode) {
    case mode_ResCurve:
        MyPl->Pure2dPlot->SetAutoScaleEnabled(true, true);
        MyPl->Pure2dPlot->singleAutoScale = false;
        MyPl->Pure2dPlot->SetAllXScales(0, 500000);
        MyPl->Pure2dPlot->XAxisUnit.SetDescAndSIUnit(tr("Frequency"), tr("Hz"));
        MyPl->Pure2dPlot->YAxisUnit.SetDescAndSIUnit(CB_fr_Channel->currentText(), input_units(CB_fr_Channel->itemData(CB_fr_Channel->currentIndex()).toInt()));
        MyPl->SetTitle(tr("Frequency curve"));
        MyPl->Pure2dPlot->SourceData = plot::getUnLock(curveData_freq);

        MyPlScan->Pure2dPlot->SetAutoScaleEnabled(true, true);
        MyPlScan->Pure2dPlot->singleAutoScale = false;
        MyPlScan->Pure2dPlot->SetAllXScales(0, 500000);
        MyPlScan->Pure2dPlot->XAxisUnit.SetDescAndSIUnit(tr("Frequency"), tr("Hz"));
        MyPlScan->Pure2dPlot->YAxisUnit.SetDescAndSIUnit(CB_fr_Channel->currentText(), input_units(CB_fr_Channel->itemData(CB_fr_Channel->currentIndex()).toInt()));
        MyPlScan->SetTitle(tr("Frequency curve"));
        MyPlScan->Pure2dPlot->SourceData = plot::getUnLock(curveData_freq);
        break;

    case mode_Osc: {
        emit sig_osc_channel(0); //to reset scale
        emit sig_osc_channel(FB_channel);
        emit sig_osc_channelB(1);
        emit sig_osc_start();

        auto osc = Oscilloscope::GetIt();
        if (!osc)
            return;

        double scale = osc->k_m_nm[0];
        osc_y_postscale = input_gain(FB_channel) * scale;

        plot::TSIUnit tunit;
        tunit.SetSIUnit(input_units(FB_channel));
        tunit.UpdateLegend(10 / scale);

        MyPl->Pure2dPlot->singleAutoScale = true;
        MyPl->Pure2dPlot->SetAllXScales(0, ePeriod->value());
        MyPl->Pure2dPlot->SetCurrentYScales(OscYrange[0] * osc_y_postscale,
                                            OscYrange[1] * osc_y_postscale);
        MyPl->Pure2dPlot->XAxisUnit.SetDescAndSIUnit(tr("time"), tr("s"));
        MyPl->Pure2dPlot->YAxisUnit.SetDescAndSIUnit(fbs->currentText(), tunit.Legend);
        MyPl->SetTitle(tr("Osc curve"));

        MyPl->Pure2dPlot->SourceData.clear();
        MyPl->Pure2dPlot->SourceData.append(osc->curveData.at(0));

        MyPlScan->Pure2dPlot->singleAutoScale = true;
        MyPlScan->Pure2dPlot->SetAllXScales(0, ePeriod->value());
        MyPlScan->Pure2dPlot->SetCurrentYScales(OscYrange[0] * osc_y_postscale,
                                            OscYrange[1] * osc_y_postscale);
        MyPlScan->Pure2dPlot->XAxisUnit.SetDescAndSIUnit(tr("time"), tr("s"));
        MyPlScan->Pure2dPlot->YAxisUnit.SetDescAndSIUnit(fbs->currentText(), tunit.Legend);
        MyPlScan->SetTitle(tr("Osc curve"));

        MyPlScan->Pure2dPlot->SourceData.clear();
        MyPlScan->Pure2dPlot->SourceData.append(osc->curveData.at(0));
    }
        break;

    case mode_Spectroscopy:
        CB_Channel->setCurrentIndex(CB_Channel->findData(FB_channel));

        MyPl->Pure2dPlot->singleAutoScale = false;
        MyPl->Pure2dPlot->SetAllXScales(0, Zrange/1e6);
        MyPl->Pure2dPlot->XAxisUnit.SetDescAndSIUnit("z --->"+tr("sample"), tr("m"));
        MyPl->Pure2dPlot->YAxisUnit.SetDescAndSIUnit(CB_Channel->currentText(), input_units(FB_channel));
        MyPl->SetTitle(tr("Spectroscopy curve"));

        MyPl->Pure2dPlot->SourceData = plot::getUnLock(curveData_force);
        MyPl->Refresh();

        MyPlScan->Pure2dPlot->singleAutoScale = false;
        MyPlScan->Pure2dPlot->SetAllXScales(0, Zrange/1e6);
        MyPlScan->Pure2dPlot->XAxisUnit.SetDescAndSIUnit("z --->"+tr("sample"), tr("m"));
        MyPlScan->Pure2dPlot->YAxisUnit.SetDescAndSIUnit(CB_Channel->currentText(), input_units(FB_channel));
        MyPlScan->SetTitle(tr("Spectroscopy curve"));

        MyPlScan->Pure2dPlot->SourceData = plot::getUnLock(curveData_force);
        MyPlScan->Refresh();
        if (MyPl->Pure2dPlot->QwtCurves.size() >= 2) {
            MyPl->Pure2dPlot->QwtCurves[0]->setTitle(tr("Approach"));
            MyPl->Pure2dPlot->QwtCurves[1]->setTitle(tr("Retract"));

            MyPlScan->Pure2dPlot->QwtCurves[0]->setTitle(tr("Approach"));
            MyPlScan->Pure2dPlot->QwtCurves[1]->setTitle(tr("Retract"));
        }
        break;
    }

    if (MyPl->PlotControl->BMarkers->isChecked())
        MyPl->PlotControl->BMarkers->click();

    MyPl->Refresh();

    if (MyPlScan->PlotControl->BMarkers->isChecked())
        MyPlScan->PlotControl->BMarkers->click();

    MyPlScan->Refresh();
}

void SPMTunning::osc_refresh()
{
    if (PlotMode != mode_Osc)
        return;

    auto osc = Oscilloscope::GetIt();
    if (!osc)
        return;

    ed_Aver->setValue(osc->LEdAverage[0]->text().toDouble());
    ed_RMS->setValue(osc->LEdStDev[0]->text().toDouble());
    Laver->setText(osc->LAverage[0]->text());
    Lrms->setText(osc->LStDev[0]->text());

    MyPl->Refresh();
    MyPlScan->Refresh();

    //if (BLanding->isOn)
    //    send2dPlot(PlotID::Landing, MyPl);
}
void SPMTunning::sl_set_osc_T(int T)
{
    if (PlotMode != mode_Osc)
        return;

    MyPl->Pure2dPlot->singleAutoScaleXLimit = T;
    MyPl->Pure2dPlot->SetAllXScales(0, T);
    MyPl->Refresh();

    MyPlScan->Pure2dPlot->singleAutoScaleXLimit = T;
    MyPlScan->Pure2dPlot->SetAllXScales(0, T);
    MyPlScan->Refresh();
}


void SPMTunning::s_setCalibHeight()
{
    //inversion (1-x)
    double hgain = -1. * Zrange * 1e-6 / 20.;
    double hshift = -10;
    input_gain_set(srcHeight, hgain);
    input_shift_set(srcHeight, hshift);
    input_unit_set(srcHeight, QObject::tr("m"));

    emit input_calib_updated();
}

void SPMTunning::s_setCalibLFI()
{
    QSettings settings("Param/gui.conf", QSettings::IniFormat);
    settings.setValue("spm/calib/lf_i", EcalibLF_I->value());

    if (CBcalibLF_I->isChecked()) {
        double gain = EcalibLF_I->value() * 1e-9 / gain2->itemData(gain2->currentIndex()).toDouble();
        input_gain_set(srcLFL, gain);
        input_gain_set(srcMag2, gain);
        input_unit_set(srcLFL, tr("A"));
        input_unit_set(srcMag2, tr("A"));
    } else {
        input_gain_clear(srcLFL);
        const double sqrtOf2 = sqrt(2.); //lockin(demodulator) averaging compensation
        input_gain_set(srcMag2, sqrtOf2);
        input_unit_clear(srcLFL);
        input_unit_clear(srcMag2);
    }

    setSPgui();
    emit input_calib_updated();
}
void SPMTunning::s_setCalibDFL()
{
    if (CBcalibDFL_I->isChecked() && CBcalibDFL_F->isChecked()) {
        if (sender() == CBcalibDFL_I)
            CBcalibDFL_F->setChecked(false);
        else
            CBcalibDFL_I->setChecked(false);
        return;
    }

    QSettings settings("Param/gui.conf", QSettings::IniFormat);
    settings.setValue("spm/calib/dfl_i", EcalibDFL_I->value());
    settings.setValue("spm/calib/dfl_f", EcalibDFL_F->value());

    const double sqrtOf2 = sqrt(2.); //lockin(demodulator) averaging compensation

    if (CBcalibDFL_I->isChecked()) {
        double gain = EcalibDFL_I->value() * 1e-9 / gain1->itemData(gain1->currentIndex()).toDouble();
        input_gain_set(srcDFL, gain);
        input_gain_set(srcMag1, gain*sqrtOf2);
        input_unit_set(srcDFL, tr("A"));
        input_unit_set(srcMag1, tr("A"));
    } else if (CBcalibDFL_F->isChecked()) {
        double gain = EcalibDFL_F->value() * 1e-9 / gain1->itemData(gain1->currentIndex()).toDouble();
        input_gain_set(srcDFL, gain);
        input_gain_set(srcMag1, gain*sqrtOf2);
        input_unit_set(srcDFL, tr("N"));
        input_unit_set(srcMag1, tr("N"));
    } else {
        input_gain_clear(srcDFL);
        input_gain_set(srcMag1, sqrtOf2);
        input_unit_clear(srcDFL);
        input_unit_clear(srcMag1);
    }

    setSPgui();
    emit input_calib_updated();
}
void SPMTunning::s_Imode()
{
    bool is_tuning_fork = currModeIdx == 10;
    bool isSTM = extCurrent->isChecked() && (!is_tuning_fork || lflCurrent->isChecked()); //distinguish stm mode by 2 flags at same time (from tuning fork)

    CBcalibLF_I->setChecked(lflCurrent->isChecked() && !extCurrent->isChecked());
    CBcalibDFL_I->setChecked(isSTM);

    QString nameDFL = isSTM ? QObject::tr("I-STM") : QObject::tr("DFL");
    changeCBname(srcDFL, nameDFL);

    QString nameLF = !lflCurrent->isChecked() ? QObject::tr("LFL") :
                                                isSTM ? QObject::tr("I-STM log") :
                                                        QObject::tr("ITip");
    changeCBname(srcLFL, nameLF);

    emit input_calib_updated();
}


void SPMTunning::Set_FB_sign(bool sign)
{
    Mode_data[currModeIdx][4] = sign ? 1 : 0;
    sendPID = true;

    sl_save_to_file();
}

void SPMTunning::setKelvinP(double val)
{
    QSettings settings("Param/gui.conf", QSettings::IniFormat);
    settings.setValue("kelvin/p", val);

    kelvinP = val;
    updateKelvin = true;
}

void SPMTunning::setDFLXtalk(double val)
{
    dfl_xtalk = analog_calib(val / input_gain(srcDFL), "adc/v1") / 20 * 65535;
    updateDFLXtalk = true;
}
void SPMTunning::setMAGXtalk(double val)
{
    mag_xtalk = analog_calib(val / input_gain(srcMag1), "adc/v1mag") / 20 * 65535;
    updateMAGXtalk = true;
}

void SPMTunning::setQcontrolGain(double val)
{
    qcontrol_gain = val;
    updateQcontrolGain = true;
}

void SPMTunning::setPiezoTip(bool on)
{
    piezoTipv = on;
    updatePiezoTip = true;
}
void SPMTunning::setPiezoTip2(bool on)
{
    piezoTip2v = on;
    updatePiezoTip2 = true;
}
void SPMTunning::setLFLCurrent(bool on)
{
    lflCurrentv = on;
    updateLflCurrent = true;
}
void SPMTunning::setExtCurrent(bool on)
{
    extCurrentv = on;
    updateExtCurrent = true;
}
void SPMTunning::setCapMode(bool on)
{
    capmode = on;
    updateCapMode = true;
}
void SPMTunning::setCapMode2(bool on)
{
    capmode2 = on;
    updateCapMode2 = true;
}
void SPMTunning::setDrive2(double val)
{
    drive2 = analog_calib(val, "dac/vdrive", 0.);
    updateDrive2 = true;
}
void SPMTunning::setFreq2(double val)
{
    freq2 = val * 1e3; //from khz
    updateFreq2 = true;
}
void SPMTunning::Set_headDAC(double val)
{
    val = analog_calib(val, "dac/vtip");
    head_dac = 65535.0 * (val + 10.0) / 20.0;
    updateHeadDAC = true;
}
void SPMTunning::Set_headDAC2(double val)
{
    val = analog_calib(val, "dac/vtip");
    head_dac2 = 65535.0 * (val + 10.0) / 20.0;
    updateHeadDAC2 = true;
}
void SPMTunning::set_cutoff_freq(double cutoff_n)
{
    cutoff = cutoff_n;
    sendLPF = true;

    sl_save_to_file();
}
void SPMTunning::set_phase(double phase_n)
{
    phase = phase_n;
    sendPhaseShift = true;

    sl_save_to_file();
}
void SPMTunning::spm_jm_div(int div)
{
    m_jm_div = div;
    updateJM_div = true;
}

SPMModel::SPMModel()
{
    #ifndef _OLD_AUTO_FINDING_MAG
	wFreeAmplitude = 0;
	sendFreeAmplitude = false;
	wStartAutoFindingMAG = 0;
	sendStartAutoFindingMAG = false;
    autoFindingMAGEnded = false;
#endif
    current_frequency = 0.;
    cutoff = 0.;
    fr_start = 0.;
    fr_end = 0.;
    phase = 0.;
    fr_channel = 0;
    Landing_mode = 0;
    Landing_direction = 0;
    Landing_speed = 0.;
    Landing_speed_close = 0.;
    Landing_close_perc = 0.;
    SpeedSpectr = 0.;
    SpectrChannel = 0;
    SpectrPoints = 0;
    spect_onn_off = 0;
    Z1spectr = 0.;
    Z2spectr = 0.;
    kelvinOn = false;
    kelvinP = 0.;
    m_jm_precision = 0;
    m_jm_precision_lsb = 0;
    m_jm_lift = 0;
    m_jm_div = 0;
    l2k_controller = false;

    currModeIdx = 0;
    for (int i = 0; i < 11; i++) {
        bool is_mag = (i >= 2 && i <= 5) || (i == 10);
        bool is_stm = i >= 6 && i <= 9;

        Mode_data[i][0] = is_mag || is_stm ? 1 : -1; //sp
        Mode_data[i][1] = 1; //P
        Mode_data[i][2] = 0.001; //I
        Mode_data[i][3] = is_mag ? 1 : 0; //v drive
        Mode_data[i][4] = is_stm ? 1 : 0; //sign

        Gain_data[i][0] = 1;
        Gain_data[i][1] = 1;
        Gain_data[i][2] = 1;
    }

    FB_Scanner = scHead;
    FBZ_config = fbOff;
    FBZ_config_masked = false;
    FB_channel = srcDFL;
    LaserOnOf = 1;
    AC_DC = 0;

    gain12v = gain22v = gain32v = 0 + 1;
    drive2 = 0;
    freq2 = 0;
    dfl_xtalk = 0;
    mag_xtalk = 0;
    head_dac2 = head_dac = 32767;
    qcontrol_gain = 0;
    efmplus = capmode = capmode2 =
            piezoTipv = piezoTip2v =
            lflCurrentv = extCurrentv = false;
}

#include <i_eg3000.h>
bool SPMModel::sendData(const void *handle)
{
    bool wrote = false;

    #ifndef _OLD_AUTO_FINDING_MAG
    if (sendFreeAmplitude.claimMessage()) {
        wrote = true;
        eg3000_send_word_packet(handle, ADR_CPU, CPU_AUTO_MAG_FREE_AMPLITUDE, wFreeAmplitude);
    }
    if (sendStartAutoFindingMAG.claimMessage()) {
        wrote = true;
        eg3000_send_word_packet(handle, ADR_CPU, CPU_AUTO_MAG_START, wStartAutoFindingMAG);
    }
    if (sendStepSize.claimMessage()) {
        wrote = true;
        eg3000_send_packet(handle, ADR_CPU, CPU_AUTO_MAG_MOVE_STEP, &fStepSize, sizeof(float));
    }
    if (sendStepWait.claimMessage()) {
        wrote = true;
        eg3000_send_word_packet(handle, ADR_CPU, CPU_AUTO_MAG_STEP_WAIT, wStepWait);
    }
#endif
    
    //config
    if (updateHeadDAC.claimMessage()) {
        wrote = true;
        eg3000_set_head_dac((void *)handle, head_dac); //set tip voltage
    }
    if (updateHeadDAC2.claimMessage()) {
        wrote = true;
        eg3000_send_word_packet(handle, ADR_CPU, CPU_HEAD_DAC_2, head_dac2); //set tip voltage
    }
    if (updateKelvin.claimMessage()) {
        wrote = true;
        eg3000_kelvin_params((void *)handle, kelvinOn, kelvinP / 100.);
    }
    if (updateEFMplus.claimMessage()) {
        wrote = true;
        eg3000_send_word_packet(handle, ADR_CPU, CPU_EFM_PLUS, efmplus);
    }
    if (updatePiezoTip.claimMessage()) {
        wrote = true;
        eg3000_set_head_piezotip((void *)handle, piezoTipv);
    }
    if (updatePiezoTip2.claimMessage()) {
        wrote = true;
        eg3000_send_word_packet(handle, ADR_CPU, CPU_PIEZO_TIP_2, piezoTip2v);
    }
    if (updateCapMode.claimMessage()) {
        wrote = true;
        eg3000_set_cap_mode((void *)handle, capmode);
    }
    if (updateCapMode2.claimMessage()) {
        wrote = true;
        eg3000_set_cap_mode2((void *)handle, capmode2);
    }
    if (updateLflCurrent.claimMessage()) {
        wrote = true;
        eg3000_set_head_lfl_current((void *)handle, lflCurrentv);
    }
    if (updateExtCurrent.claimMessage()) {
        wrote = true;
        eg3000_set_head_ext_current((void *)handle, extCurrentv);
    }
    if (updateDFLXtalk.claimMessage()) {
        wrote = true;
        eg3000_send_packet(handle, ADR_CPU, CPU_DFL_XTALK, &dfl_xtalk, sizeof(float));
    }
    if (updateMAGXtalk.claimMessage()) {
        wrote = true;
        eg3000_send_packet(handle, ADR_CPU, CPU_MAG_XTALK, &mag_xtalk, sizeof(float));
    }
    if (updateQcontrolGain.claimMessage()) {
        wrote = true;
        float qcoeff = -qcontrol_gain;
        if (qcontrol_gain > 0)
            qcoeff /= 5;
        eg3000_send_packet(handle, ADR_CPU, CPU_SET_QCONTROL, &qcoeff, sizeof(float));
    }

    if (sendDriveFreq.claimMessage()) {
        wrote = true;
        eg3000_set_frequency((void *)handle, current_frequency);
    }
    if (sendDriveV.claimMessage()) {
        wrote = true;
        eg3000_set_piezo_drive_voltage((void *)handle, analog_calib(Mode_data[currModeIdx][3], "dac/vdrive", 0.));
    }
    if (updateDrive2.claimMessage()) {
        wrote = true;
        eg3000_set_piezo_drive_voltage_lift((void *)handle, drive2);
    }
    if (updateFreq2.claimMessage()) {
        wrote = true;
        eg3000_set_frequency2((void *)handle, freq2);
    }

    if (sendHead.claimMessage()) {
        wrote = true;
        eg3000_head_params((void *)handle, LaserOnOf, AC_DC);
    }
    if (sendLPF.claimMessage()) {
        wrote = true;
        eg3000_set_cutoff_frequency((void *)handle, cutoff);
    }
    if (sendPhaseShift.claimMessage()) {
        wrote = true;
        eg3000_set_phase((void *)handle, phase);
    }
    if (sendGains.claimMessage()) {
        wrote = true;
        eg3000_set_ampl_gains((void *)handle, Gain_data[currModeIdx][0], Gain_data[currModeIdx][1], Gain_data[currModeIdx][2]);
    }
    if (sendGains2.claimMessage()) {
        wrote = true;
        eg3000_set_ampl_gains2((void *)handle, gain12v, gain22v, gain32v);
    }

    //FB
    if (updateJM_Precision.claimMessage()) {
        wrote = true;
        eg3000_send_word_packet(handle, ADR_CPU, CPU_JM_PREC_LSB, m_jm_precision_lsb);
        eg3000_send_word_packet(handle, ADR_CPU, CPU_JM_PREC, m_jm_precision);
    }
    if (updateJM_Lift.claimMessage()) {
        wrote = true;
        eg3000_send_packet(handle, ADR_CPU, CPU_JM_LIFT, &m_jm_lift, sizeof(DWORD));
    }
    if (updateJM_div.claimMessage()) {
        wrote = true;
        BYTE d =  m_jm_div;
        eg3000_send_packet(handle, ADR_CPU, CPU_JM_DIV, &d, sizeof(BYTE));
    }

    if (sendPID.claimMessage()) {
        wrote = true;
        QSettings settings("Param/controller.conf", QSettings::IniFormat);
        double spmP = (Mode_data[currModeIdx][1] / (l2k_controller ? 100 : 1)) / 100.;
        double spmI = (Mode_data[currModeIdx][2] / (l2k_controller ? 100 : 1)) / 10000.;
        double scale = settings_double(settings, (FB_Scanner == scHead) ? "mult_spm_pid_head" : "mult_spm_pid_stage", 1.0);
        eg3000_fb_pid_params((void *)handle, FB_Scanner, spmP*scale, spmI*scale, 0, FB_channel, Mode_data[currModeIdx][4]);
    }
    if (sendSetpoint.claimMessage()) {
        wrote = true;
        eg3000_fb_pid_setpoint((void *)handle, Mode_data[currModeIdx][0]);
    }
    if (sendFBon.claimMessage()) {
        wrote = true;
        eg3000_fb_control((void *)handle, !FBZ_config_masked ? FBZ_config : fbOff);
    }

    //func
    if (sendFreqScan.claimMessage()) {
        wrote = true;
        eg3000_frequency_scan_start((void *)handle, fr_start, fr_end, fr_channel);
    }
    if (sendSpectroscopy.claimMessage()) {
        wrote = true;
        eg3000_set_spectroscopy_params((void *)handle, spect_onn_off, Z1spectr, Z2spectr, SpeedSpectr, SpectrChannel, SpectrPoints);
    }
    if (updLanding.claimMessage()) {
        wrote = true;
        eg3000_set_landing_ex((void *)handle, Landing_mode, Landing_direction, Landing_speed, Landing_speed_close, Landing_close_perc,
                              LandingMCfg, 4);
    }
    if (doGetCZ.claimMessage()) {
        wrote = true;
        eg3000_send_word_packet(handle, ADR_CPU, CPU_GET_DATA, srcCZ);
        eg3000_send_word_packet(handle, ADR_CPU, CPU_GET_DATA, srcCZ2);
    }

    return wrote;
}

void SPMModel::resetWriter()
{
    FBZ_config = fbOff;
    FBZ_config_masked = false;
    sendFBon = true;

    updateHeadDAC = true;
    updateHeadDAC2 = true;
    updateEFMplus = true;
    updateKelvin = true;
    updatePiezoTip = true;
    updatePiezoTip2 = true;
    updateCapMode = true;
    updateCapMode2 = true;
    updateLflCurrent = true;
    updateExtCurrent = true;
    updateQcontrolGain = true;
    updateDFLXtalk = true;
    updateMAGXtalk = true;

    sendDriveFreq = true;
    sendDriveV = true;
    updateDrive2 = true;
    updateFreq2 = true;

    updateJM_Lift = false;
    updateJM_Precision = false;
    updateJM_div = false;
    sendPID = true;
    sendSetpoint = true;
    sendHead = true;
    sendLPF = true;
    sendPhaseShift = true;
    sendGains = true;
    sendGains2 = true;

    sendFreqScan = false;
    sendSpectroscopy = false;
    updLanding = false;

    doGetCZ = false;
}
void SPMTunning::resetWriter()
{
    SPMModel::resetWriter();

    QMetaObject::invokeMethod(this, "reset");
}
#include "scanplot.h"
void SPMTunning::reset()
{
    auto scan = ScanPlot::GetIt();

    saveblock = true;

    setFBchannel();

    if (scan)
        scan->deviceSchemeWid->feedBack->setState(false);
    BFeedbackOnOff->change_state_man_with_sig(false);

    destructStatus(landing_bar);
    destructStatus(landing_notification);

    s_Imode();

    Vtip->setNewValue();
    Vtip2->setNewValue();

    if (PBFreqStart->isOn)
    {
        PBFreqStart->change_state_man_with_sig(false);
        if (scan)
            scan->deviceSchemeWid->oscilator->btnFreqStartScan->change_state_man(false);
    }

    last_command = 0;
    if (auto_land_state)
    {
        PBAutoLand->change_state_man_with_sig(false);
        if (scan)
            scan->smartLanding->change_state_man(1);
    }

    saveblock = false;
}

void SPMTunning::capBits(int n, long bits)
{
    if (n == 0) {
        bool features1 = bits & 1;
        if (!features1) {
            BFeedbackOnOffMode->change_state_man_with_sig(false);
        }
        BFeedbackOnOffMode->setVisible(features1);

        l2k_controller = bits & 4;
        bool l2k_controller_v2 = bits & 8;
        if (l2k_controller)
            sendPID = true;
        PBAutoLand->setVisible(l2k_controller && l2k_controller_v2);
        box_qcontrol->setVisible(l2k_controller);

        DCfbP->setNewValue(); //sync l2k divisor
        DCfbI->setNewValue();

        auto scan = ScanPlot::GetIt();

        lowPassFilter->setMinimum(l2k_controller ? 8 : 0);
        lowPassFilter->setMaximum(l2k_controller ? 100 : 20); //TODO fix save for freq > 20
        if (scan)
        {
            scan->deviceSchemeWid->LockIn->edLP->setMinimum(l2k_controller ? 8 : 0);
            scan->deviceSchemeWid->LockIn->edLP->setMaximum(l2k_controller ? 100 : 20); //TODO fix save for freq > 20
        }

        DCfbPrec->edit->setDecimals(l2k_controller ? 2 : 1);
        DCfbPrec->slider->stepV = pow(10., -(l2k_controller ? 2 : 1));
        DCfbPrec->setMaximum(l2k_controller ? 2 : 5);

        efmSinglePass->setVisible(l2k_controller);
        if (efmSinglePass->isChecked() && !l2k_controller)
            efmSinglePass->setChecked(false);


        QList<QComboBox *> gainslist1, gainslist2;
        gainslist1 << gain1 << gain2 << gain12 << gain22;
        gainslist2 << gain3 << gain32;

        foreach (QComboBox *box, gainslist1) {
            int c = 3;//0;
            /*box->setItemText(c++, "x1");
            box->setItemText(c++, "x2");
            box->setItemText(c++, "x4");*/
            box->setItemText(c++, !l2k_controller ? "x5" : "x4");
            box->setItemText(c++, !l2k_controller ? "x10" : "x8");
            box->setItemText(c++, !l2k_controller ? "x20" : "x16");
            box->setItemText(c++, !l2k_controller ? "x25" : "x16");
            box->setItemText(c++, !l2k_controller ? "x50" : "x32");
            box->setItemText(c++, !l2k_controller ? "x100" : "x64");
            c = 3;//0;
            /*box->setItemData(c++, 1);
            box->setItemData(c++, 2);
            box->setItemData(c++, 4);*/
            box->setItemData(c++, !l2k_controller ? 5 : 4);
            box->setItemData(c++, !l2k_controller ? 10 : 8);
            box->setItemData(c++, !l2k_controller ? 20 : 16);
            box->setItemData(c++, !l2k_controller ? 25 : 16);
            box->setItemData(c++, !l2k_controller ? 50 : 32);
            box->setItemData(c++, !l2k_controller ? 100 : 64);
        }
        foreach (QComboBox *box, gainslist2) {
            int c = 2;//0;
            /*box->setItemText(c++, "x1");
            box->setItemText(c++, "x2");*/
            box->setItemText(c++, !l2k_controller ? "x5" : "x4");
            box->setItemText(c++, !l2k_controller ? "x10" : "x8");
            c = 2;//0;
            /*box->setItemData(c++, 1);
            box->setItemData(c++, 2);*/
            box->setItemData(c++, !l2k_controller ? 5 : 4);
            box->setItemData(c++, !l2k_controller ? 10 : 8);
        }
    }
}

void SPMTunning::inject_test_packet()
{
#ifdef USE_EG3K_EMULATOR
    static int deflec_ind = 0;
    if (deflec_ind < 100000) deflec_ind += 3;
    else deflec_ind = 0;
    EG_Status Deflectomter;
    Deflectomter.Scanning = mdIdle;
    Deflectomter.ErrorCode = ecOK;
    Deflectomter.DFL  = (WORD)(32000+20000*sin(0.01*deflec_ind));
    Deflectomter.LFL  = (WORD)(32000+20000*cos(0.01*deflec_ind));
    Deflectomter.DInt = (WORD)(32768+30000*cos(0.01*deflec_ind)*cos(0.01*deflec_ind));
    Deflectomter.MAG  = (WORD)(30000*cos(0.01*deflec_ind)*cos(0.01*deflec_ind))+32768;
    Deflectomter.Zpos = (WORD)(65000*cos(0.01*deflec_ind)*cos(0.01*deflec_ind));
    processPacket(ADR_CPU, CPU_STATUS, &Deflectomter, 0);
#endif
}

void SPMTunning::initTwoDPlot(plot::T2dPlot *plot)
{
    plot->Pure2dPlot->blockRefresh = true;
    plot->SetGridVisible(true);
    plot->Pure2dPlot->SetAllXScales(0,500000);
    plot->Pure2dPlot->SetAllYScales(-11,11);
    plot->Pure2dPlot->SetNumOfVertLines(4);
    plot->Pure2dPlot->setVertLineVisible(0, true);

    plot->Pure2dPlot->MoveVertLine(250, 0);
    plot->Pure2dPlot->MoveVertLine(30/1e6,1);
    plot->Pure2dPlot->MoveVertLine(50/1e6,2);
    plot->Pure2dPlot->MoveVertLine(70/1e6,3);

    QPen ZscannerLine;
    ZscannerLine.setWidth(4);
    ZscannerLine.setColor(QColor(255, 64, 255));
    plot->Pure2dPlot->SetVertLinePen(ZscannerLine,0);
    plot->Pure2dPlot->SetVertLinePen(ZscannerLine,2);

    QPen RightLine;
    RightLine.setWidth(3);
    RightLine.setColor(QColor(32, 164, 255));
    plot->Pure2dPlot->SetVertLinePen(RightLine,1);
    plot->Pure2dPlot->SetVertLinePen(RightLine,3);


    plot->Pure2dPlot->XAxisUnit.SetDescAndSIUnit(tr("Frequency"), tr("Hz"));
    plot->Pure2dPlot->YAxisUnit.SetDescAndSIUnit(tr("Amplifier Out"), tr("V"));
    plot->SetTitle(tr("Frequency curve"));
}

void SPMTunning::pidSwitch_newValue(int index)
{
    EG_DataSources dt = srcNone;

    if (index == 0)
        dt = srcPhase1;
    else if (index == 3)
        dt = srcMag1;
    else if (index == 4)
        dt = srcDFL;
    else if (index == 5)
        dt = srcLFL;

    if (dt == srcNone)
        return;

    for (int i = 0; i < fbs->count(); i++)
        if (dt == fbs->itemData(i).value<QPair<int, int> >().second) {
            fbs->setCurrentIndex(i);
            break;
        }
    auto scan = ScanPlot::GetIt();
    if (scan)
    {
        if (srcMag1 == fbs->itemData(fbs->currentIndex()).value<QPair<int, int> >().second)
            scan->deviceSchemeWid->PIDSwitch->currentConnectedIn = 3;
        else if (srcPhase1 == fbs->itemData(fbs->currentIndex()).value<QPair<int, int> >().second)
            scan->deviceSchemeWid->PIDSwitch->currentConnectedIn = 0;
        else if (srcDFL == fbs->itemData(fbs->currentIndex()).value<QPair<int, int> >().second)
            scan->deviceSchemeWid->PIDSwitch->currentConnectedIn = 4;
        else if (srcLFL == fbs->itemData(fbs->currentIndex()).value<QPair<int, int> >().second)
            scan->deviceSchemeWid->PIDSwitch->currentConnectedIn = 5;
    }
}

void SPMTunning::plusSwitch_newValue(int index)
{
    piezoTip->setChecked(!index);
}

void SPMTunning::edLP_newValue(double value)
{
    lowPassFilter->setValue(value);
    lowPassFilter->setNewValue();
}
void SPMTunning::edPSh_newValue(double value)
{
    phaseShift->setValue(value);
    phaseShift->setNewValue();
}

void SPMTunning::edPr_DCfbP_newValue(double value)
{
    DCfbP->setValue(value);
    DCfbP->setNewValue();
}
void SPMTunning::edInt_DCfbI_newValue(double value)
{
    DCfbI->setValue(value);
    DCfbI->setNewValue();
}

void SPMTunning::setPoint_DCfbSetPoint_newValue(double value)
{
    DCfbSetPoint->setValue(value);
    DCfbSetPoint->setNewValue();
}

void SPMTunning::edFreq_curFrequency_newValue(double value)
{
    curFrequency->setValue(value);
    curFrequency->setNewValue();
}

void SPMTunning::edAmp_driveInt_newValue(double value)
{
    driveInt->setValue(value);
    driveInt->setNewValue();
}

void SPMTunning::edBias_Vtip_newValue(double value)
{
    Vtip->setValue(value);
    Vtip->setNewValue();
}

void SPMTunning::lflSwitch_lflCurrent_newValue(int )
{
    lflCurrent->toggle();
}

void SPMTunning::dflSwitch_extCurrent_newValue(int )
{
    extCurrent->toggle();
}

void SPMTunning::createLockInWorkSpace()
{
    MyPl = new plot::T2dPlot;
    MyPlScan = new plot::T2dPlot;

    initTwoDPlot(MyPl);
    MyPl->Pure2dPlot->SourceData = plot::getUnLock(curveData_freq);
    MyPl->Pure2dPlot->blockRefresh = false;
    MyPl->Refresh();

    initTwoDPlot(MyPlScan);
    MyPlScan->Pure2dPlot->SourceData = *MyPl->Pure2dPlot->cloneSourceData();
    MyPlScan->Pure2dPlot->blockRefresh = false;
    MyPlScan->Refresh();

    ed_Aver = new Edit(-1e9, 1e9, 3); ed_Aver->showButtons(false); ed_Aver->setReadOnly(true);
    ed_RMS  = new Edit(-1e9, 1e9, 3); ed_RMS->showButtons(false); ed_RMS->setReadOnly(true);

    ePeriod = new Edit(1, 60);
    ePeriod->setImmediateMode();
    ePeriod->setValue(10);
    connect(ePeriod, SIGNAL(newValue(int)), this, SLOT(sl_set_osc_T(int)));


    QList<QComboBox *> gainslist1, gainslist2;

    gainslist1 << (gain1 = new QComboBox);
    gainslist1 << (gain2 = new QComboBox);
    gainslist2 << (gain3 = new QComboBox);

    gainslist1 << (gain12 = new QComboBox);
    gainslist1 << (gain22 = new QComboBox);
    gainslist2 << (gain32 = new QComboBox);

    foreach (QComboBox *box, gainslist1) {
        box->addItem("x1", 1);
        box->addItem("x2", 2);
        box->addItem("x4", 4);
        box->addItem("x5", 5);
        box->addItem("x10", 10);
        box->addItem("x20", 20);
        box->addItem("x25", 25);
        box->addItem("x50", 50);
        box->addItem("x100", 100);
    }
    foreach (QComboBox *box, gainslist2) {
        box->addItem("x1", 1);
        box->addItem("x2", 2);
        box->addItem("x5", 5);
        box->addItem("x10", 10);
    }

    auto scan = ScanPlot::GetIt();

    connect(gain1, SIGNAL(currentIndexChanged(int)), SLOT(s_sendGains()));
    connect(gain2, SIGNAL(currentIndexChanged(int)), SLOT(s_sendGains()));
    connect(gain3, SIGNAL(currentIndexChanged(int)), SLOT(s_sendGains()));
    connect(gain12, SIGNAL(currentIndexChanged(int)), SLOT(s_sendGains2()));
    connect(gain22, SIGNAL(currentIndexChanged(int)), SLOT(s_sendGains2()));
    connect(gain32, SIGNAL(currentIndexChanged(int)), SLOT(s_sendGains2()));

    if (scan)
    {
        connect(scan->deviceSchemeWid->inputGain->cbDFL, SIGNAL(currentIndexChanged(int)), gain1, SLOT(setCurrentIndex(int)));
        connect(scan->deviceSchemeWid->inputGain->cbLF, SIGNAL(currentIndexChanged(int)), gain2, SLOT(setCurrentIndex(int)));
        connect(scan->deviceSchemeWid->inputGain->cbINT, SIGNAL(currentIndexChanged(int)), gain3, SLOT(setCurrentIndex(int)));
    }

    QSettings settings("Param/analog.conf", QSettings::IniFormat);
    double a0 = settings_double(settings, "dac/vtip_min", -10.);
    double b0 = settings_double(settings, "dac/vtip_max", 10.);
    double a1 = settings_double(settings, "dac/vdrive_min", 0.);
    double b1 = settings_double(settings, "dac/vdrive_max", 10.);

    freqScanSet = new QCheckBox(tr("Set on max"));
    freqScanSet->setToolTip(tr("Set excitation frequency on maximum in scan"));
    freqScanSet->setChecked(true);

    curFrequency = new DoubleChanger(0, 1000, 3);
    lowPassFilter = new DoubleChanger(0, 20, 1);
    lowPassFilter->edit->setSpecialValueText(tr("min"));
    phaseShift = new DoubleChanger(-180, 180, 1);
    driveInt = new DoubleChanger(a1, b1, 3);

    driveInt->setProperty("id", DataID::SPM_drive);
    curFrequency->setProperty("id", DataID::SPM_freq);
    if (scan)
    {
        scan->deviceSchemeWid->oscilator->edFreq->setProperty("id", DataID::SPM_freq);
        scan->deviceSchemeWid->oscilator->edAmp->setProperty("id", DataID::SPM_drive);
        connect(scan->deviceSchemeWid->oscilator->edFreq, SIGNAL(newValue(double)), this, SLOT(edFreq_curFrequency_newValue(double)));
        connect(scan->deviceSchemeWid->oscilator->edAmp, SIGNAL(newValue(double)), this, SLOT(edAmp_driveInt_newValue(double)));
        connect(scan->deviceSchemeWid->LockIn->edLP, SIGNAL(newValue(double)), this, SLOT(edLP_newValue(double)));
        connect(scan->deviceSchemeWid->LockIn->edPSh, SIGNAL(newValue(double)), this, SLOT(edPSh_newValue(double)));
        connect(scan->deviceSchemeWid->oscilator->chbSetOn, SIGNAL(toggled(bool)), freqScanSet, SLOT(setChecked(bool)));
        connect(scan->deviceSchemeWid->oscilator->edFreqScanMin, SIGNAL(newValue(double)), EFreqScanMin, SLOT(setValue(double)));
        connect(scan->deviceSchemeWid->oscilator->edFreqScanMax, SIGNAL(newValue(double)), EFreqScanMax, SLOT(setValue(double)));
    }

    connect(curFrequency->slider, SIGNAL(valueChanged(double)), this, SLOT(s_setCurFrequency(double)));
    connect(curFrequency, SIGNAL(newValue(double)), this, SLOT(s_setCurFrequency(double)));
    connect(curFrequency, SIGNAL(newValue()), this, SLOT(sl_save_to_file()));
    connect(lowPassFilter, SIGNAL(newValue(double)), this, SLOT(set_cutoff_freq(double)));
    connect(phaseShift, SIGNAL(newValue(double)), this, SLOT(set_phase(double)));
    connect(driveInt, SIGNAL(newValue()), this, SLOT(s_sendDriveInt()));

    driveInt2 = new DoubleChanger(a1, b1, 3);
    connect(driveInt2, SIGNAL(newValue(double)), this, SLOT(setDrive2(double)));

    curFrequency2 = new DoubleChanger(0, 1000, 3);
    connect(curFrequency2, SIGNAL(newValue(double)), this, SLOT(setFreq2(double)));

    Vtip = new DoubleChanger(a0, b0, 3);
    Vtip2 = new DoubleChanger(a0, b0, 3);
    Vtip->setProperty("id", DataID::SPM_Vtip);
    connect(Vtip, SIGNAL(newValue(double)), this, SLOT(Set_headDAC(double)));
    connect(Vtip2, SIGNAL(newValue(double)), this, SLOT(Set_headDAC2(double)));

    piezoTip = new QCheckBox(tr("Piezo/Tip voltage"));
    piezoTip->setToolTip(tr("Apply drive AC voltage to tip instead of piezo"));
    connect(piezoTip, SIGNAL(toggled(bool)), this, SLOT(setPiezoTip(bool)));

    capMode = new QCheckBox(tr("2nd Harmonic"));
    capMode->setToolTip(tr("Detects 2nd harmonic of DFL and LFL signals"));
    connect(capMode, SIGNAL(toggled(bool)), this, SLOT(setCapMode(bool)));

    piezoTip2 = new QCheckBox(tr("Piezo/Tip voltage"));
    piezoTip2->setToolTip(tr("Apply drive AC voltage to tip instead of piezo"));
    connect(piezoTip2, SIGNAL(toggled(bool)), this, SLOT(setPiezoTip2(bool)));

    capMode2 = new QCheckBox(tr("2nd Harmonic"));
    capMode2->setToolTip(tr("Detects 2nd harmonic of DFL and LFL signals"));
    connect(capMode2, SIGNAL(toggled(bool)), this, SLOT(setCapMode2(bool)));

    efmSinglePass = new QCheckBox(tr("EFM+"));
    efmSinglePass->setToolTip(tr("Simultaneous topography and EFM measurements.") + "\n" +
                              tr("Applies 2nd electrical drive with 'second pass' settings, optional kelvin mode"));
    connect(efmSinglePass, SIGNAL(toggled(bool)), this, SLOT(s_set_efmplus(bool)));

    lflCurrent = new QCheckBox(tr("LF/I-Tip"));
    lflCurrent->setToolTip(tr("Show tip current in place of LF signal"));
    connect(lflCurrent, SIGNAL(toggled(bool)), this, SLOT(setLFLCurrent(bool)));
    connect(lflCurrent, SIGNAL(toggled(bool)), this, SLOT(s_Imode()));

    extCurrent = new QCheckBox(tr("I-STM"));
    extCurrent->setToolTip(tr("Show STM current (tip connector)"));
    connect(extCurrent, SIGNAL(toggled(bool)), this, SLOT(setExtCurrent(bool)));
    connect(extCurrent, SIGNAL(toggled(bool)), this, SLOT(s_Imode()));
}

void SPMTunning::createFeedBackWorkSpace()
{
    CBFBScanner = new QComboBox;
    CBFBScanner->addItem(makeHugeIcon(":/SPMTuning_window/FbTip.png"),"");
    CBFBScanner->addItem(makeHugeIcon(":/SPMTuning_window/FbSmp.png"),"");
    CBFBScanner->setToolTip(tr("Set Z piezo activator"));
    connect(CBFBScanner, SIGNAL(currentIndexChanged(int)), this, SLOT(s_setFBScanner(int)));
    connect(CBFBScanner, SIGNAL(currentIndexChanged(int)), this, SLOT(sl_change_z_slider()));

    fbs = new QComboBox;
    fbs->setProperty("id", DataID::SPM_FB_ch);
    fbs->addItem(tr("Contact - Normal DFL"), QVariant::fromValue(QPair<int, int>(0, srcDFL)));
    fbs->addItem(tr("Tapping - MAG"), QVariant::fromValue(QPair<int, int>(2, srcMag1)));
    fbs->addItem(tr("STM - Itip"), QVariant::fromValue(QPair<int, int>(8, srcDFL)));
    fbs->addItem(tr("STM - Itip log"), QVariant::fromValue(QPair<int, int>(9, srcLFL)));
    fbs->addItem(tr("Tuning fork - MAG"), QVariant::fromValue(QPair<int, int>(10, srcMag1)));
    fbs->addItem(tr("Contact - Lateral DFL"), QVariant::fromValue(QPair<int, int>(1, srcLFL)));
    fbs->addItem(tr("Custom - ADC AUX 1"), QVariant::fromValue(QPair<int, int>(6, srcADCHR1)));
    fbs->addItem(tr("Tapping - Phase"), QVariant::fromValue(QPair<int, int>(5, srcPhase1)));

    connect(fbs, SIGNAL(currentIndexChanged(int)), this, SLOT(setFBchannel()));

    B_PID_sign = new TwoStateButton(":/SPMTuning_window/FB_plus.png",":/SPMTuning_window/FB_minus.png");
    B_PID_sign->setToolTip(tr("Feed Back sign"));
    connect(B_PID_sign, SIGNAL(state_changed(bool)), this, SLOT(Set_FB_sign(bool)));

    BLaserOnOff = new TwoStateButton(":/SPMTuning_window/laser_on.png",":/SPMTuning_window/laser_off.png");
    BLaserOnOff->setToolTip(tr("SPM laser on/off"));
    connect(BLaserOnOff, SIGNAL(state_changed(bool)), this, SLOT(sl_laser_on_of(bool)));

    DCfbP = new DoubleChanger(0, 10, 2);
    DCfbI = new DoubleChanger(0, 1, 3);
    DCfbSetPoint = new DoubleChanger(-10, 10, 2);
    DCfbSetPointUnits = new QLabel(tr("V")); spm_sp_units_gain = 1; spm_sp_min = -10; spm_sp_max = 10;
    DCfbPrec = new DoubleChanger(0, 5, 1); DCfbPrec->setValue(0.1); DCfbPrec->edit->setSpecialValueText(tr("min"));
    DCfbHeight = new DoubleChanger(0, 5, 2); DCfbHeight->setValue(0.5);
    connect(DCfbPrec, SIGNAL(newValue()), this, SLOT(s_jm_prec()));
    connect(DCfbHeight, SIGNAL(newValue()), this, SLOT(s_jm_height()));

    DCfbP->setProperty("id", DataID::SPM_P);
    DCfbI->setProperty("id", DataID::SPM_I);
    DCfbSetPoint->setProperty("id", DataID::SPM_SetPoint);
    DCfbPrec->setProperty("id", DataID::SPM_jm_p);
    DCfbHeight->setProperty("id", DataID::SPM_jm_h);

    auto scan = ScanPlot::GetIt();

    connect(DCfbP, SIGNAL(newValue()), this, SLOT(s_sendFBP()));
    connect(DCfbI, SIGNAL(newValue()), this, SLOT(s_sendFBI()));
    connect(DCfbSetPoint, SIGNAL(newValue()), this, SLOT(s_sendSP()));

    if (scan)
    {
        connect(scan->deviceSchemeWid->pid->edPr, SIGNAL(valueChanged(double)), this, SLOT(edPr_DCfbP_newValue(double)));
        connect(scan->deviceSchemeWid->pid->edInt, SIGNAL(valueChanged(double)), this, SLOT(edInt_DCfbI_newValue(double)));
    }


    BFeedbackOnOff = new TwoStateButton(":/off.png",":/onn2.png");
    BFeedbackOnOff->setToolTip(tr("Feed Back : on/off button"));
    BFeedbackOnOff->setProperty("id", DataID::SPM_FB_On);
    connect(BFeedbackOnOff, SIGNAL(state_changed(bool)), this, SLOT(s_setFBOnOff(bool)));
    connect(BFeedbackOnOff, SIGNAL(state_changed(bool)), this, SLOT(Zpos_on_off(bool)));

    BFeedbackOnOffMode = new TwoStateButton(":/SPMTuning_window/JmOf.png",":/SPMTuning_window/JmOn.png");
    BFeedbackOnOffMode->setToolTip(tr("Feed Back jumping mode: on/off button"));
    connect(BFeedbackOnOffMode, SIGNAL(state_changed(bool)), this, SLOT(s_setFBMode(bool)));

    LaserSpotWindow = new DeflectWid;

    LaserSpot = new DeflectWid;

    connect(CBFBScanner, SIGNAL(currentIndexChanged(int)), LaserSpot, SLOT(setFbscanner(int)));
    connect(CBFBScanner, SIGNAL(currentIndexChanged(int)), LaserSpotWindow, SLOT(setFbscanner(int)));
}

void SPMTunning::feedBackClickedInScan(bool state)
{
    auto scan = ScanPlot::GetIt();
    if (scan)
        scan->deviceSchemeWid->feedBack->setState(state);
    BFeedbackOnOff->change_state_man(state);
    s_setFBOnOff(state);
    Zpos_on_off(state);
}

QIcon rotateIcon(QIcon src, int degrees)
{
    return QIcon(src.pixmap(1000).transformed(QTransform().rotate(degrees)));
}

void SPMTunning::create_plot_mode_control()
{
    modeBar = new QTabBar(this);
    modeBar->setShape(QTabBar::RoundedWest);

    modeBar->addTab(rotateIcon(makeHugeIcon(":/SPMTuning_window/RESMODE.png"), 90), tr("Resonance"));
    modeBar->addTab(rotateIcon(makeHugeIcon(":/SPMTuning_window/OSC_MODE.png"), 90), tr("Oscilloscope"));
    modeBar->addTab(rotateIcon(makeHugeIcon(":/SPMTuning_window/SPECT_MODE.png"), 90), tr("SPM Spectroscopy"));

    modeBar->setTabToolTip(0, tr("Resonance Curves Plot mode"));
    modeBar->setTabToolTip(1, tr("Oscilloscope Plot mode"));
    modeBar->setTabToolTip(2, tr("SPM Spectroscopy Plot mode"));

    modeBar->setTabData(0, mode_ResCurve);
    modeBar->setTabData(1, mode_Osc);
    modeBar->setTabData(2, mode_Spectroscopy);

    connect(modeBar, SIGNAL(currentChanged(int)), this, SLOT(switchMode(int)));
}

void SPMTunning::createLayout()
{
    QHBoxLayout *layout_main = new QHBoxLayout(this);
    layout_main->setSpacing(6);
    layout_main->setContentsMargins(6, 0, 0, 3);

    QVBoxLayout *layout_left0 = new QVBoxLayout;
    layout_left0->setSpacing(3);
    layout_left0->setContentsMargins(3, 9, 0, 6);

    QScrollArea *LeftWidScroll = new TScrollArea(this);
    LeftWidScroll->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Minimum);

    QSplitter *rspmspl = new QSplitter(Qt::Vertical, this);

    layout_main->addLayout(layout_left0);
    layout_main->addWidget(LeftWidScroll, 2);
    layout_main->addWidget(modeBar, 0, Qt::AlignTop);
    layout_main->addWidget(rspmspl, 5);


    layout_left0->addWidget(GrBMotorControl);


    QVBoxLayout *control_layout = (QVBoxLayout *)(GrBMotorControl->layout());
    control_layout->insertStretch(0, 3);
    control_layout->insertWidget(0, BLaserOnOff);


    QWidget *LeftWid = new QWidget(LeftWidScroll);
    QVBoxLayout *layout_left = new QVBoxLayout(LeftWid);
    layout_left->setSpacing(3);
    layout_left->setContentsMargins(6, 12, 6, 9);

    LeftWidScroll->setWidget(LeftWid);


    RW = new QWidget(rspmspl);
    QVBoxLayout *layout_plot = new QVBoxLayout(RW);
    layout_plot->setSpacing(6);
    layout_plot->setContentsMargins(0, 6, 6, 0);

    spmTabs = new TTabWidget(RW);
    spmTabs->setDocumentMode(true);
    spmTabs2 = new TTabWidget(rspmspl);
    spmTabs2->setDocumentMode(true);

    rspmspl->addWidget(RW); rspmspl->setCollapsible(rspmspl->count()-1-1, false);
    rspmspl->addWidget(spmTabs2); rspmspl->setCollapsible(rspmspl->count()-1, true);



    QGroupBox *box_FBS = new QGroupBox(tr("FB INPUT SIGNAL (SPM MODE)"));
    QVBoxLayout *layout_boxbutton = new QVBoxLayout(box_FBS);
    layout_boxbutton->setContentsMargins(6, 6, 6, 6);
    layout_boxbutton->setSpacing(6);

    QVBoxLayout *layout_sliders = new QVBoxLayout;
    layout_sliders->setSpacing(3);
    layout_sliders->setContentsMargins(0, 0, 0, 0);

    box_Z_spectr = new QGroupBox(tr("Z head, ")+tr("mkm"));
    QHBoxLayout *layout_spect_Z = new QHBoxLayout(box_Z_spectr);
    layout_spect_Z->setContentsMargins(6, 6, 6, 6);
    layout_spect_Z->addWidget(NS_Z_spectr);

    layout_left->addWidget(LaserSpot);
    layout_left->addWidget(box_FBS);
    layout_left->addLayout(layout_sliders);
    layout_left->addWidget(box_Z_spectr);

    grb_Av_rms = new QGroupBox;
    grb_Av_rms->hide();
    QHBoxLayout *layout_Av_rms = new QHBoxLayout(grb_Av_rms);
    layout_Av_rms->setContentsMargins(6, 3, 6, 6);

    box_scan = new QGroupBox(tr("Frequency Scan"));
    QHBoxLayout *ScanStr1 = new QHBoxLayout(box_scan);
    ScanStr1->setAlignment(Qt::AlignLeft);
    ScanStr1->setContentsMargins(6, 3, 6, 6);

    box_Spect_start = new QGroupBox(tr("Spectroscopy"));
    box_Spect_start->hide();
    QHBoxLayout *layout_spect_start = new QHBoxLayout(box_Spect_start);
    layout_spect_start->setAlignment(Qt::AlignLeft);
    layout_spect_start->setSpacing(6);
    layout_spect_start->setContentsMargins(6, 3, 6, 6);

    QHBoxLayout *layout_Spectr_Z12 = new QHBoxLayout;
    layout_Spectr_Z12->setSpacing(6);
    layout_Spectr_Z12->setContentsMargins(0, 0, 0, 0);

    layout_plot->addWidget(MyPl, 1);
    layout_plot->addWidget(grb_Av_rms);
    layout_plot->addWidget(box_scan);
    layout_plot->addWidget(box_Spect_start);
    layout_plot->addLayout(layout_Spectr_Z12);
    layout_plot->addWidget(spmTabs);



    QWidget *t1 = new QWidget();
    QHBoxLayout *layout_Fr_Drive = new QHBoxLayout(t1);
    layout_Fr_Drive->setSpacing(6);
    layout_Fr_Drive->setContentsMargins(0, 0, 0, 0);

    QWidget *t12 = new QWidget();
    QHBoxLayout *layout_Fr_Drive2 = new QHBoxLayout(t12);
    layout_Fr_Drive2->setSpacing(6);
    layout_Fr_Drive2->setContentsMargins(0, 0, 0, 0);

    QWidget *t2 = new QWidget();
    QHBoxLayout *layout_opt2 = new QHBoxLayout(t2);
    layout_opt2->setSpacing(6);
    layout_opt2->setContentsMargins(0, 0, 0, 0);

    QWidget *t22 = new QWidget();
    QHBoxLayout *layout_opt22 = new QHBoxLayout(t22);
    layout_opt22->setSpacing(6);
    layout_opt22->setContentsMargins(0, 0, 0, 0);

    QWidget *DownWid = new QWidget;
    QHBoxLayout *layout_Dacs = new QHBoxLayout(DownWid);
    layout_Dacs->setSpacing(6);
    layout_Dacs->setContentsMargins(0, 0, 0, 0);

    QWidget *tcalib = new QWidget();
    QFormLayout *layout_tcalib = makeFormLayout(tcalib);

    QWidget *txtalk = new QWidget();
    QFormLayout *layout_txtalk = makeFormLayout(txtalk);

    QGroupBox *opt_box = new QGroupBox(tr("Switches"));
    QVBoxLayout *layout_spm_options = new QVBoxLayout(opt_box);
    layout_spm_options->setContentsMargins(3, 3, 3, 3);

    QGroupBox *opt_box11 = new QGroupBox(tr("Add."));
    QVBoxLayout *layout_spm_options11 = new QVBoxLayout(opt_box11);
    layout_spm_options11->setContentsMargins(3, 3, 3, 3);

    QGroupBox *opt_box2 = new QGroupBox(tr("Switches"));
    QVBoxLayout *layout_spm_options2 = new QVBoxLayout(opt_box2);
    layout_spm_options2->setContentsMargins(3, 3, 3, 3);

    QGroupBox *box_fbi = new QGroupBox(tr("FB I"));


    spmTabs->addTab(t1, tr("Drive"));
    spmTabs->addTab(t2, tr("Electric"));

    spmTabs2->addTab(DownWid, tr("Lockin"));
    spmTabs2->addTab(t12, tr("Drive") + ", " + tr("Second pass"));
    spmTabs2->addTab(t22, tr("Electric") + ", " + tr("Second pass"));
    spmTabs2->addTab(tcalib, tr("Calibration"));
    spmTabs2->addTab(txtalk, tr("Y head crosstalk"));

    {
        QSettings settings("Param/gui.conf", QSettings::IniFormat);
        bool head_on = settings.value("display/oscilloscope/head", true).toBool();
        if (!head_on)
            spmTabs2->setTabVisible(4, false);
    }

    layout_boxbutton->addWidget(fbs);

    QHBoxLayout *layout_boxbutton2 = new QHBoxLayout();
    layout_boxbutton2->setContentsMargins(0, 0, 0, 0);
    layout_boxbutton->addLayout(layout_boxbutton2);

    layout_boxbutton2->addWidget(BFeedbackOnOff);
    layout_boxbutton2->addWidget(BFeedbackOnOffMode);
    layout_boxbutton2->addWidget(B_PID_sign);
    layout_boxbutton2->addWidget(CBFBScanner);
    layout_boxbutton2->addStretch(1);

    QGroupBox *box_fbp = new QGroupBox(tr("FB P"));
    QHBoxLayout *layout_fbp = new QHBoxLayout(box_fbp);
    layout_fbp->setContentsMargins(6, 6, 6, 6);
    layout_fbp->addWidget(DCfbP);

    QHBoxLayout *layout_fbi = new QHBoxLayout(box_fbi);
    layout_fbi->setContentsMargins(6, 6, 6, 6);
    layout_fbi->addWidget(DCfbI);

    QGroupBox *box_setPoint = new QGroupBox(tr("Set point"));
    QHBoxLayout *layout_setPoint = new QHBoxLayout(box_setPoint);
    layout_setPoint->setContentsMargins(6, 6, 6, 6);
    layout_setPoint->addWidget(DCfbSetPoint);
    layout_setPoint->addWidget(DCfbSetPointUnits);

    QGroupBox *box_fbprec = new QGroupBox(tr("Precision, nm"));
    box_fbprec->hide();
    QHBoxLayout *layout_prec = new QHBoxLayout(box_fbprec);
    layout_prec->setContentsMargins(6, 6, 6, 6);
    layout_prec->addWidget(DCfbPrec);

    QGroupBox *box_fbheight = new QGroupBox(tr("Jump height, ")+tr("mkm"));
    box_fbheight->hide();
    QHBoxLayout *layout_height = new QHBoxLayout(box_fbheight);
    layout_height->setContentsMargins(6, 6, 6, 6);
    layout_height->addWidget(DCfbHeight);



    layout_sliders->addWidget(box_setPoint);
    layout_sliders->addWidget(box_fbp);
    layout_sliders->addWidget(box_fbi);
    layout_sliders->addWidget(box_fbprec);
    layout_sliders->addWidget(box_fbheight);

    QGroupBox *box_gains = new QGroupBox(tr("Input Gains"));
    QFormLayout *layout_gains = makeFormLayout(box_gains);
    layout_gains->addRow(tr("DFL (ADC 1)"), gain1);
    layout_gains->addRow(tr("LFL (ADC 2)"), gain2);
    layout_gains->addRow(tr("INT (ADC 3)"), gain3);
    layout_gains->setSpacing(3);
    layout_gains->setContentsMargins(6, 3, 6, 0);

    QGroupBox *box_gains2 = new QGroupBox(tr("Input Gains") + ", " + tr("Second pass"));
    QFormLayout *layout_gains2 = makeFormLayout(box_gains2);
    layout_gains2->addRow(tr("DFL (ADC 1)"), gain12);
    layout_gains2->addRow(tr("LFL (ADC 2)"), gain22);
    layout_gains2->addRow(tr("INT (ADC 3)"), gain32);
    layout_gains2->setSpacing(3);
    layout_gains2->setContentsMargins(6, 3, 6, 0);

    QGroupBox *box_lowPassFilter = new QGroupBox(tr("Low Pass Filter, kHz"));
    QHBoxLayout *layout_lowPassFilter = new QHBoxLayout(box_lowPassFilter);
    layout_lowPassFilter->setContentsMargins(6, 6, 6, 6);
    layout_lowPassFilter->addWidget(lowPassFilter);

    QGroupBox *box_phaseShift = new QGroupBox(tr("Phase Shift, Degree"));
    QHBoxLayout *layout_phaseShift = new QHBoxLayout(box_phaseShift);
    layout_phaseShift->setContentsMargins(6, 6, 6, 6);
    layout_phaseShift->addWidget(phaseShift);

    layout_Dacs->addWidget(box_gains);
    layout_Dacs->addWidget(box_gains2);
    layout_Dacs->addWidget(box_lowPassFilter);
    layout_Dacs->addWidget(box_phaseShift);

//---------

    QGroupBox *box_curFr = new QGroupBox(tr("Frequency") + ", " + tr("kHz"));
    QHBoxLayout *layout_curFr = new QHBoxLayout(box_curFr);
    layout_curFr->setContentsMargins(6, 6, 6, 6);
    layout_curFr->addWidget(curFrequency);

    QGroupBox *box_curFr2 = new QGroupBox(tr("Frequency") + ", " + tr("kHz"));
    QHBoxLayout *layout_curFr2 = new QHBoxLayout(box_curFr2);
    layout_curFr2->setContentsMargins(6, 6, 6, 6);
    layout_curFr2->addWidget(curFrequency2);


    QGroupBox *box_driveInt = new QGroupBox(tr("Amplitude") + ", " + tr("V"));
    QHBoxLayout *layout_driveInt = new QHBoxLayout(box_driveInt);
    layout_driveInt->setContentsMargins(6, 6, 6, 6);
    layout_driveInt->addWidget(driveInt);

    QGroupBox *box_driveInt2 = new QGroupBox(tr("Amplitude") + ", " + tr("V"));
    QHBoxLayout *layout_driveInt2 = new QHBoxLayout(box_driveInt2);
    layout_driveInt2->setContentsMargins(6, 6, 6, 6);
    layout_driveInt2->addWidget(driveInt2);

    EQControlGain = new Edit(-10, 10, 2);
    connect(EQControlGain, SIGNAL(newValue(double)), this, SLOT(setQcontrolGain(double)));

    box_qcontrol = new QGroupBox(tr("Q Control"));
    QHBoxLayout *layout_qcontrol = new QHBoxLayout(box_qcontrol);
    layout_qcontrol->setContentsMargins(6, 6, 6, 6);
    layout_qcontrol->addWidget(EQControlGain);

    QGroupBox *box_Vtip = new QGroupBox(tr("V Tip") + ", " + tr("V"));
    QHBoxLayout *layout_vtip = new QHBoxLayout(box_Vtip);
    layout_vtip->setContentsMargins(6, 6, 6, 6);
    layout_vtip->addWidget(Vtip);

    QGroupBox *box_Vtip2 = new QGroupBox(tr("V Tip") + ", " + tr("V"));
    QHBoxLayout *layout_vtip2 = new QHBoxLayout(box_Vtip2);
    layout_vtip2->setContentsMargins(6, 6, 6, 6);
    layout_vtip2->addWidget(Vtip2);


    ScanStr1->addWidget(PBFreqStartAuto);
    ScanStr1->addWidget(PBFreqStart);
    ScanStr1->addWidget(new QLabel(tr("Channel")));
    ScanStr1->addWidget(CB_fr_Channel);
    ScanStr1->addWidget(new QLabel(tr("From")));
    ScanStr1->addWidget(EFreqScanMin);
    ScanStr1->addWidget(new QLabel(tr("To")));
    ScanStr1->addWidget(EFreqScanMax);
    ScanStr1->addWidget(new QLabel(tr("kHz")));
    ScanStr1->addWidget(freqScanSet);
    addDelimeterV(ScanStr1, PBFreqStartAuto->sizeHint().height());
    ScanStr1->addWidget(PBForceCalib);
    ScanStr1->addWidget(PBForceCalibConfig);
    QWidget::setTabOrder(EFreqScanMax, PBFreqStart);

    //==========Spectroscopy=============================
    box_Z1_spectr = new QGroupBox(QLatin1String("<---")+QChar(0x0394)+tr("Z1, ")+tr("mkm"));
    box_Z1_spectr->setAlignment(Qt::AlignRight);
    box_Z1_spectr->hide();
    QHBoxLayout *layout_spect_Z1 = new QHBoxLayout(box_Z1_spectr);
    layout_spect_Z1->setContentsMargins(6, 6, 6, 6);

    box_Z2_spectr = new QGroupBox(QChar(0x0394)+tr("Z2, ")+tr("mkm")+"--->");
    box_Z2_spectr->setAlignment(Qt::AlignLeft);
    box_Z2_spectr->hide();
    QHBoxLayout *layout_spect_Z2 = new QHBoxLayout(box_Z2_spectr);
    layout_spect_Z2->setContentsMargins(6, 6, 6, 6);

    layout_Spectr_Z12->addWidget(box_Z1_spectr);
    layout_Spectr_Z12->addWidget(box_Z2_spectr);

    layout_spect_Z1->addWidget(ev_Z1_spectr);
    layout_spect_Z1->addWidget(NS_Z1_spectr);

    layout_spect_Z2->addWidget(NS_Z2_spectr);


    layout_spect_start->addWidget(PBSpectr_start_stop);
    layout_spect_start->addWidget(BSpectrJM);
    layout_spect_start->addWidget(new QLabel(tr("Channel")));
    layout_spect_start->addWidget(CB_Channel);
    layout_spect_start->addWidget(new QLabel(tr("Points")));
    layout_spect_start->addWidget(CB_Points_spectr);
    layout_spect_start->addWidget(new QLabel(tr("Speed, ") + tr("mkm")+"/"+tr("s")));
    layout_spect_start->addWidget(ev_Speed_spectr);
    QWidget::setTabOrder(ev_Speed_spectr, PBSpectr_start_stop);
    addDelimeterV(layout_spect_start, PBSpectr_start_stop->sizeHint().height());
    layout_spect_start->addWidget(LJMdiv = new QLabel(tr("JM points div")));
    layout_spect_start->addWidget(JMdiv);
    layout_spect_start->addWidget(PBPIDCalib);

    //====================================================
    layout_Av_rms->addWidget(new QLabel(tr("T, s")));
    layout_Av_rms->addWidget(ePeriod);
    layout_Av_rms->addStretch(1);
    layout_Av_rms->addWidget(Laver = new QLabel(tr("Average, V")));
    layout_Av_rms->addWidget(ed_Aver);
    layout_Av_rms->addStretch(1);
    layout_Av_rms->addWidget(Lrms = new QLabel(tr("RMS, V")));
    layout_Av_rms->addWidget(ed_RMS);
    layout_Av_rms->addStretch(5);

    layout_spm_options->addWidget(piezoTip);
    layout_spm_options->addWidget(capMode);

    layout_spm_options11->addWidget(lflCurrent);
    layout_spm_options11->addWidget(extCurrent);
    layout_spm_options11->addWidget(efmSinglePass);

    layout_spm_options2->addWidget(piezoTip2);
    layout_spm_options2->addWidget(capMode2);

    layout_Fr_Drive->addWidget(box_curFr);
    layout_Fr_Drive->addWidget(box_driveInt);
    layout_Fr_Drive->addWidget(box_qcontrol);

    layout_Fr_Drive2->addWidget(box_curFr2);
    layout_Fr_Drive2->addWidget(box_driveInt2);


    kelvinOn = false;
    Kelvin_on = new QCheckBox(tr("Enable"));
    Kelvin_on->setToolTip(tr("Enable VTip(MAG) feedback in lift mode second pass"));
    connect(Kelvin_on, SIGNAL(toggled(bool)), this, SLOT(s_set_Kelvin(bool)));

    Kelvin_P = new Edit(-10, 10, 4); //6 digits
    connect(Kelvin_P, SIGNAL(newValue(double)), this, SLOT(setKelvinP(double)));

    QGroupBox *kelvin_box = new QGroupBox(tr("Kelvin Mode"));
    QGridLayout *layout_opt3 = new QGridLayout(kelvin_box);
    layout_opt3->setContentsMargins(3, 6, 3, 6);
    layout_opt3->addWidget(Kelvin_on, 0, 0, 1, 2);
    layout_opt3->addWidget(new QLabel(tr("Scale:")), 1, 0, 1, 1);
    layout_opt3->addWidget(Kelvin_P, 1, 1, 1, 1);

    layout_opt2->addWidget(opt_box);
    layout_opt2->addWidget(opt_box11);
    layout_opt2->addWidget(box_Vtip, 1);

    layout_opt22->addWidget(opt_box2);
    layout_opt22->addWidget(kelvin_box);
    layout_opt22->addWidget(box_Vtip2, 1);


    layout_txtalk->addRow(tr("DFL Y full-range xtalk"), Edfl_xtalk = new Edit(-100, 100, 1));
    layout_txtalk->addRow(tr("MAG Y full-range xtalk"), Emag_xtalk = new Edit(-100, 100, 1));
    connect(Edfl_xtalk, SIGNAL(newValue(double)), this, SLOT(setDFLXtalk(double)));
    connect(Emag_xtalk, SIGNAL(newValue(double)), this, SLOT(setMAGXtalk(double)));


    layout_tcalib->addRow(CBcalibLF_I = new QCheckBox(tr("Show LF as current, nA/V: ")), EcalibLF_I = new Edit(-1e10, 1e10, 2));
    layout_tcalib->addRow(CBcalibDFL_I = new QCheckBox(tr("Show DFL as current, nA/V: ")), EcalibDFL_I = new Edit(-1e10, 1e10, 2));
    layout_tcalib->addRow(CBcalibDFL_F = new QCheckBox(tr("Show DFL/MAG as force, nN/V: ")), EcalibDFL_F = new Edit(-1e10, 1e10, 2));
    {
        QSettings settings("Param/gui.conf", QSettings::IniFormat);
        Kelvin_P->setValue(settings_double(settings, "kelvin/p", 1));
        Kelvin_P->setNewValue();
        EcalibLF_I->setValue(settings_double(settings, "spm/calib/lf_i", 25));
        EcalibDFL_I->setValue(settings_double(settings, "spm/calib/dfl_i", 0.5));
        EcalibDFL_F->setValue(settings_double(settings, "spm/calib/dfl_f", 1000));
    }

    connect(CBcalibLF_I, SIGNAL(toggled(bool)), this, SLOT(s_setCalibLFI()));
    connect(EcalibLF_I, SIGNAL(newValue()), this, SLOT(s_setCalibLFI()));

    connect(CBcalibDFL_I, SIGNAL(toggled(bool)), this, SLOT(s_setCalibDFL()));
    connect(EcalibDFL_I, SIGNAL(newValue()), this, SLOT(s_setCalibDFL()));

    connect(CBcalibDFL_F, SIGNAL(toggled(bool)), this, SLOT(s_setCalibDFL()));
    connect(EcalibDFL_F, SIGNAL(newValue()), this, SLOT(s_setCalibDFL()));


    spmTabs2->setFixedHeight(spmTabs2->sizeHint().height());
}

SPMTunning* SPMTunning::GetIt() noexcept
{
    auto spm = ModuleManager::instance->find1VoidPtr("spm");
    return static_cast<SPMTunning*>(spm);
}
