#pragma once

#include "eg3000/defs.h"
#include <QWidget>
#include <QPointer>
class QCheckBox;
#include <plot.h>
#include "controls.h"
#include "edit.hpp"
#include "tools.hpp"


class SPMModel : public IWriter
{
public:
    SPMModel();
    bool sendData(const void *handle) override;
    void resetWriter() override;

    bool l2k_controller;

    int currModeIdx;
    double Mode_data[11][5];//0-setpoint; 1-fbp; 2-fbi; 3-drive; 4-sign
    int    Gain_data[11][3];//0-g1; 1-g2; 2-g3;

    TMessageFlag sendFBon, sendPID, sendSetpoint,
                sendDriveFreq, sendFreqScan, sendLPF, sendPhaseShift,
                sendDriveV, sendGains, sendGains2,
                updLanding, sendSpectroscopy, sendHead;

    int FB_channel;
    int LaserOnOf, AC_DC;
    char FB_Scanner;
    BYTE FBZ_config;
    bool FBZ_config_masked;
    int gain12v, gain22v, gain32v;

    double current_frequency, cutoff, fr_start, fr_end, phase;
    int fr_channel;

    BYTE Landing_mode;
    BYTE Landing_direction;
    double Landing_speed;
    double Landing_speed_close;
    double Landing_close_perc;
    EG_LandingMCfg LandingMCfg[4];

    double SpeedSpectr;
    int SpectrChannel;
    int SpectrPoints;
    int spect_onn_off;
    double Z1spectr, Z2spectr;

    #ifndef _OLD_AUTO_FINDING_MAG
    WORD wFreeAmplitude;
    TMessageFlag sendFreeAmplitude;
    
    float fStepSize = 0.1;
    TMessageFlag sendStepSize;
    
    WORD wStepWait = 50;
    TMessageFlag sendStepWait;
    
    bool autoFindingMAGEnded;
    WORD wStartAutoFindingMAG;
    TMessageFlag sendStartAutoFindingMAG;
#endif
    
    WORD head_dac;
    TMessageFlag updateHeadDAC;
    WORD head_dac2;
    TMessageFlag updateHeadDAC2;

    bool kelvinOn;
    double kelvinP;
    TMessageFlag updateKelvin;

    float dfl_xtalk;
    TMessageFlag updateDFLXtalk;
    float mag_xtalk;
    TMessageFlag updateMAGXtalk;

    float qcontrol_gain;
    TMessageFlag updateQcontrolGain;

    bool piezoTipv;
    TMessageFlag updatePiezoTip;
    bool piezoTip2v;
    TMessageFlag updatePiezoTip2;

    bool lflCurrentv;
    TMessageFlag updateLflCurrent;
    bool extCurrentv;
    TMessageFlag updateExtCurrent;

    bool capmode;
    TMessageFlag updateCapMode;
    bool capmode2;
    TMessageFlag updateCapMode2;
    bool efmplus;
    TMessageFlag updateEFMplus;

    double drive2;
    TMessageFlag updateDrive2;
    double freq2;
    TMessageFlag updateFreq2;

    WORD m_jm_precision, m_jm_precision_lsb;
    TMessageFlag updateJM_Precision;
    DWORD m_jm_lift;
    TMessageFlag updateJM_Lift;
    int m_jm_div;
    TMessageFlag updateJM_div;

    TMessageFlag doGetCZ;
};


class StatusBar;
class Notification;
class TOptionsForMotor;
class TOptionsForShuttle;
class DeflectWid;
class ShuttleController;
class Motors;
class NSTSlider;
class StatusWidget1;
class TTabWidget;
struct ResPeakInfo;
class SPMTunning: public QWidget, public IProcessPacket, public SPMModel
{
    Q_OBJECT

    enum plot_mode{mode_ResCurve, mode_Osc, mode_Spectroscopy};

public:
    //HW
    bool processPacket(const BYTE addr, const BYTE sub_addr, const void *data, const DWORD size) override;
    void resetWriter() override;
    static SPMTunning* GetIt() noexcept;

public slots:
    void capBits(int, long);
    void reset();
    void inject_test_packet();
    void doAutoLandConnect(bool on_off);
    void autoSensorsCalibEnd(bool ok);
    void autoSensorsCalibCanceled();
    bool findMag();
    void scan_StartAutoLanding();

    void sl_save_to_file();
    void sl_load_from_file();
    void setFBchannel();
    void s_setFBOnOff(bool state);
    void s_setFBmasked(bool);
    void s_setFBMode(bool state);
    void sl_laser_on_of(bool);
    void s_setFBScanner(int idx);
    void s_setCalibHeight();

    void setSPgui(); //
    void s_sendFBP();
    void s_sendFBI();
    void s_sendSP();
    void s_jm_prec();
    void s_jm_height();

    void s_set_Kelvin(bool);
    void setKelvinP(double);

    void s_set_efmplus(bool);
    void s_setCalibLFI();
    void s_setCalibDFL();
    void s_Imode();

    void s_sendGains();
    void s_sendGains2();
    void s_sendDriveInt();

    void Set_headDAC(double);
    void Set_headDAC2(double);

    void setDrive2(double);
    void setFreq2(double);

    void spm_jm_div(int div);
    void Set_FB_sign(bool);
    void set_cutoff_freq(double);
    void set_phase(double);

    void setPiezoTip(bool);
    void setPiezoTip2(bool);
    void setCapMode(bool);
    void setCapMode2(bool);
    void setLFLCurrent(bool);
    void setExtCurrent(bool);
    void setDFLXtalk(double);
    void setMAGXtalk(double);
    void setQcontrolGain(double);


public slots:
    bool s_FreqScanStartAuto(bool use_current_range = false);
    void s_FreqScanStart(bool st);
    void s_setFrequencyScanData(QByteArray data);
    void s_setCurFrequency(double value);
    void s_updFreqProgBar();

    void sl_get_data(int ch, long val);
    void s_spectr_start(bool);
    void s_spectr_jm(bool);
    void s_setForceCurveData(QByteArray data);
    void s_updSpectrProgBar();

    void set_spot_data();
    void doForceCalib();
    void doForceCalibConfig();
    bool doPICalib();
    bool doCheckFixFBosc();
    void doAutoLand();
    
    void updateTipDrive();

    void s_set_Z1_spectr(double value);
    void s_set_Z2_spectr(double value);
    void s_set_Z_spectr(double value);
    void s_send_Zpos();
    void Zpos_on_off(bool);
    void sl_change_z_slider();
    void sl_set_XYZ_range(double x, double y, double z);
    void sl_set_XYZ_range2(double x, double y, double z);
    void sl_set_XYZ_range_sync();
    void sl_up_lines_on();
    void sl_up_lines_of();

    void upd_shuttle_status();
    void shuttle_button(int num);// 0..15
    void shuttle_wheel(int num);// -1 left 1 right
    void shuttle_position(int num);// -7..7 current position
    void shuttle_position_piezo_timeout();
    void shuttle__set_active(int num);

    void motors_stop();
    void slot_mland_stopped();
    void slot_DoMoveUp();
    void slot_DoFastMoveUp();
    void slot_DoMoveDown();
    void slot_DoFastMoveDown();
    void slot_RecedeLanding();
    void feedBackClickedInScan(bool);
    void slot_DoLanding(bool);
    void slot_set_Landing_status(int);
    void blanding_sync();
    void s_updLandProgBar();
    void cancelWaitingResponce();

    void sl_setPlotMode();
    void switchMode(int idx);

    void osc_refresh();
    void sl_set_osc_T(int);

    void AddDataToContainer_slot(plot::TCurvesDataOwner, plot::TMetaStorage);
    void appendAttrib(plot::TMetaStorage &);
    void restoreAttributes(plot::TMetaStorage);

    void pidSwitch_newValue(int index);
    void plusSwitch_newValue(int index);

    //todo temp things, delete it when SPM GUI will be removed
    void edLP_newValue(double value);
    void edPSh_newValue(double value);
    void edPr_DCfbP_newValue(double value);
    void edInt_DCfbI_newValue(double value);
    void setPoint_DCfbSetPoint_newValue(double value);
    void edFreq_curFrequency_newValue(double value);
    void edAmp_driveInt_newValue(double value);
    void edBias_Vtip_newValue(double value);
    void lflSwitch_lflCurrent_newValue(int value);
    void dflSwitch_extCurrent_newValue(int value);
    //------------

signals:
    void AddDataToContainer(plot::TCurvesDataOwner, plot::TMetaStorage);

    void sig_osc_channel(int);
    void sig_osc_channelB(int);
    void sig_osc_start();
    void sig_osc_stop();

    void sig_Zpos(double);
    void sig_Z2pos(double);
    void input_calib_updated();

public:
    QTimer *Progress_timer;

    QGroupBox *GrBMotorControl;

    TwoStateButton *BLanding;
    TwoStateButton *BLaserOnOff;
    QTabBar *modeBar;

    Edit *EQControlGain;
    QCheckBox *CBcalibDFL_F;
    Edit *EcalibDFL_F;

    int lastShuttlePressedButton = 0; // 0..15

    DoubleChanger *Vtip;
    float newTipDriveValue = 0;

    QCheckBox *capMode;
    QPointer<StatusBar> landing_bar;
    QPointer<Notification> landing_notification;
    bool blanding_status;
    QTimer *blanding_syncT;

    QComboBox *gain1;
    QComboBox *gain2;
    QComboBox *gain3;

    QComboBox *CB_fr_Channel;

    Edit *ePeriod;
    Edit *ed_Aver;
    Edit *ed_RMS;
    

    //==============================================
    TwoStateButton *PBAutoLand;
    int auto_land_state;
    QPointer<Notification> autoland_notification;

    //=======Spectroscopy======================

    Edit *ev_Z1_spectr;
    Edit *ev_Speed_spectr;

    QComboBox *CB_Points_spectr;
    QComboBox *CB_Channel;
    TwoStateButton *PBSpectr_start_stop;
    TwoStateButton *BSpectrJM;
    Edit *JMdiv;
    QLabel *LJMdiv;
    QPushButton *PBPIDCalib;

    NSTSlider *NS_Z1_spectr;
    DoubleChanger *NS_Z2_spectr;

    QGroupBox *box_Spect_start;
    QGroupBox *box_Z2_spectr;
    QGroupBox *box_Z1_spectr;
    QGroupBox *box_Z_spectr;

    QPointer<StatusBar> Spect_progBar;

    //==============================================

    Motors *motors;
    QPointer<QLabel> shuttleStatus;
    std::vector<int> active_shuttle_motors;
    int last_command;
    int piezo_shuttle;
    double piezo_step;

    void InitShuttleGuiLink();
    static BYTE getLandingMode();
    std::vector<int> getLandingMotors();

    //==================================================

    DoubleChanger *curFrequency;
    DoubleChanger *lowPassFilter;
    DoubleChanger *phaseShift;
    DoubleChanger *driveInt;

    QPushButton *PBForceCalib;
    QPushButton *PBForceCalibConfig;
    TwoStateButton *PBFreqStart;
    QPushButton *PBFreqStartAuto;
    Edit *EFreqScanMin;
    Edit *EFreqScanMax;
    QCheckBox *freqScanSet;

    unsigned char Spectr_Scan_On;
    bool AutoSend;
    bool isSPMHidded = false;

    void createFeedBackWorkSpace();
    void createLockInWorkSpace();
    void createFreqWorkSpace();
    void createSpectroscopyWorkSpace();
    void create_plot_mode_control();
    void InitMotorControlSpace();
    void createLayout();
    void closeEvent(QCloseEvent *) override;

    void initTwoDPlot(plot::T2dPlot *plot);

public:
    DoubleChanger *DCfbP;
    DoubleChanger *DCfbI;
    DoubleChanger *DCfbSetPoint;
    double spm_sp_units_gain;
    
    DoubleChanger *DCfbPrec;
    DoubleChanger *DCfbHeight;

    DoubleChanger *NS_Z_spectr;

    TwoStateButton *B_PID_sign;

    DeflectWid *LaserSpotWindow;
    DeflectWid *LaserSpot;
    TOptionsForMotor *OptionsForMotor;
    TOptionsForShuttle *OptionsForShuttle;
    QComboBox *fbs;
    
    QComboBox *CBFBScanner;

    double nDFLr, nLFLr;
    double nDFL, nLFL, nMAG, nMAG2, nInt, nZ;

    Range OscYrange;
    double osc_y_postscale;
    int PlotMode;

    TwoStateButton *BFeedbackOnOff;
    TwoStateButton *BFeedbackOnOffMode;

    double Z1, Z2, Z_scanner, Zrange, Zrange1, Zrange2;
    bool cz_updated;
    int progDelta;
    QPointer<StatusWidget1> SWspmfb;

    plot::TCurvesDataOwner curveData_freq;
    plot::TCurvesDataOwner curveData_force;

    ResPeakInfo findPeakInfo(bool extended_slopes = true);

    plot::T2dPlot *MyPl;
    plot::T2dPlot *MyPlScan;

    SPMTunning(QWidget *parent = nullptr);
    
private:
    QLabel *DCfbSetPointUnits;
    double spm_sp_min, spm_sp_max;
    
    int prev_spmfb_state;
    bool Up_lines_spectr;
    QPointer<StatusBar> progBar;
    
    QWidget *RW;
    QElapsedTimer laserSpotTimeout;
    Edit *Kelvin_P;
    QCheckBox *Kelvin_on;
    Edit *Edfl_xtalk;
    Edit *Emag_xtalk;
    QGroupBox *box_qcontrol;
    QCheckBox *CBcalibLF_I;
    Edit *EcalibLF_I;
    QCheckBox *CBcalibDFL_I;
    Edit *EcalibDFL_I;
    TTabWidget *spmTabs;
    TTabWidget *spmTabs2;
    DoubleChanger *curFrequency2;
    DoubleChanger *driveInt2;
    DoubleChanger *Vtip2;
    QCheckBox *piezoTip;
    QCheckBox *piezoTip2;
    QCheckBox *capMode2;
    QCheckBox *efmSinglePass;
    QCheckBox *lflCurrent;
    QCheckBox *extCurrent;
    QComboBox *gain12;
    QComboBox *gain22;
    QComboBox *gain32;
    QGroupBox *grb_Av_rms;
    QGroupBox *box_scan;
    QLabel *Lrms;
    QLabel *Laver;
    
    bool saveblock;
};
