#pragma once

#include <QtGlobal>


void doBeep();

class QSettings;
class QString;
double settings_double(QSettings &sett, const QString &path, double def);

//----

#include <QThread>
#include <QApplication>
#include <QElapsedTimer>
class SleepExport : public QThread
{
public:
    static void msleep2(unsigned long ms) { msleep(ms); }
    static void msleepWithProcessEvents(unsigned long ms)
    {
        qApp->processEvents();
        msleep(ms);
    }
    static void msleep_gui(unsigned long ms) {
        const unsigned long time = qBound<unsigned long>(1, ms, 10);
        QElapsedTimer timer;

        timer.start();
        while (!timer.hasExpired(ms)) {
            msleepWithProcessEvents(time);
        }
    }
};

//----
#ifdef Q_CC_MSVC
#include <intrin.h>
#define MemoryBarrier0() _ReadWriteBarrier()
#else
#if defined(Q_CC_GNU) || defined(Q_CC_CLANG)
#define MemoryBarrier0() {asm volatile("" ::: "memory");}
#else
#define MemoryBarrier0()
#pragma message ("fix this memory barrier please")
#endif
#endif

#include <QAtomicInt>
class TMessageFlag : public QAtomicInt
{
public:
    //send
    inline
    QAtomicInt &operator= (int value)
    {
        MemoryBarrier0();
        fetchAndStoreRelease(value);
        return *this;
    }
#ifndef Q_ATOMIC_INT_FETCH_AND_STORE_IS_WAIT_FREE
#pragma message ("fetchAndStore is slow. ok?")
#endif

    //check & clear
    inline
    bool claimMessage()
    {
        bool ret = testAndSetAcquire(true, false);
        MemoryBarrier0();
        return ret;
    }
#ifndef Q_ATOMIC_INT_TEST_AND_SET_IS_WAIT_FREE
#pragma message ("testAndSet is slow. ok?")
#endif
};

//----
class TNodeContainer;
class TNode
{
    friend class TNodeContainer;
public:
    TNode() { container = nullptr; }
    ~TNode();

private:
    TNodeContainer *container;
};

#include <QVector>
class TNodeContainer
{ //not thread safe
    friend class TNode;
public:
    ~TNodeContainer()
    {
        for (int i = 0; i < nodes.size(); i++)
            nodes[i]->container = nullptr;
    }
    void addNode(TNode *node)
    {
        if (node->container)
            return;

        nodes.append(node);
        node->container = this;
    }
    void removeNode(TNode *node)
    {
        if (node->container != this)
            return;

        nodes.remove(nodes.indexOf(node));
        node->container = nullptr;
    }
    int count() const { return nodes.count(); }
    TNode *nodeAt(int i) const { return nodes.at(i); }

private:
    QVector<TNode *> nodes;
};

template <class T>
class TNodeContainer2 : public TNodeContainer
{ //use as interface
public:
    void add(T *node) { addNode(static_cast<TNode *>(node)); }
    T *at(int i) const { return static_cast<T *>(nodeAt(i)); }
};

//----

//for eg3k read
#include "eg3000/defs.h" //fuck this :/
class IProcessPacket : public TNode
{
public:
    //should be thread safe
    virtual bool processPacket(const BYTE addr, const BYTE sub_addr, const void *data, const DWORD size) = 0;
};

//& write
class IWriter : public TNode
{
public:
    class IEG3KWrite;
    //should be thread safe
    virtual bool sendData(const void *handle) { Q_UNUSED(handle) return false; }
    virtual bool sendDataIndirect(IWriter::IEG3KWrite *const writer) { Q_UNUSED(writer) return false; }
    virtual void resetWriter() { }

public:
    class IEG3KWrite {
    public:
        virtual
        BYTE raw_send(const void *buffer, WORD data_size) = 0;
        virtual
        BYTE send_packet(BYTE Address, BYTE SubAddress, const void *buffer, WORD data_size) = 0;

        inline
        BYTE send_word_packet(BYTE Address, BYTE SubAddress, WORD Data)
        {
            return send_packet(Address, SubAddress, &Data, sizeof(WORD));
        }
        inline
        BYTE send_empty_packet(BYTE Address, BYTE SubAddress)
        {
            return send_packet(Address, SubAddress, nullptr, 0);
        }
    };
};
