#pragma once

#include <QWidget>
#include <QPainter>
class QCheckBox;
class QComboBox;
class QTabWidget;
class QLabel;
#include "controls.h"
#include "edit.hpp"
#include "eg3000/defs.h"
#include "tools.hpp"
#include <QHBoxLayout>
#include "3d/plot_3dpure.h"
#include <QTableWidget>

class NLCWidget;
class Oscilloscope;
class XYZCalibrations : public QWidget, public IProcessPacket, public IWriter
{
    Q_OBJECT
public:
    XYZCalibrations(QWidget *parent = nullptr, Oscilloscope *osc = nullptr);

    bool processPacket(const BYTE addr, const BYTE sub_addr, const void *data, const DWORD size) override;
    bool sendData(const void *handle) override;
    void resetWriter() override;
    bool eventFilter(QObject *target, QEvent *event) override;

signals:
    void sigScanError();
    void sigSetpointError(int);
    void program_controller(QString);

    void sig_XYZ_range(double,double,double);
    void sig_XYZ_range2(double,double,double);
    void sig_XYZ_range3(double,double,double);

    void input_calib_updated();

public slots:
    void LVMLoad();
    void LVMSave();

    void nlcEdit();
    void nlcEditRst();
    void nlcEditUpd();
    void nlcEditApply();

    void WriteConfigToDeviceMem();
    void z3Save();
    void z3Load();
    void AutoDataLoad();
    void sl_Load_from_File();
    void read_calib_par(QString file);
    void save_To_File();
    void write_calib_par(QString file, bool need_old = true);
    void apply_calib_data();
    void sl_XYZ_range();
    void sl_setCalibVC();

    void s_sendData();
    void updateCalibDisplay();

    void lvmClicked();
    void s_setCLStatus(bool status);
    void s_setCLStatusZ(bool status);
    void set_nlc_status_cl(bool);
    void set_nlc_status_ol(bool);
    void CL_NLS_Lamps(int scanner);

    void capBits(int, long);
    void reset();
    void program_controller();

    void SensorAutocalib();
    void PIAutocalib();

public:
    QList<QPushButton*> priborMenuButtons;

    Oscilloscope *osc;
    QPushButton *BSensorAutocalib;
    QPushButton *BPIDAutocalib;

    //Calibrations
    QString defaultFileName;

    plot::TPure3dPlot *threeDPlot = nullptr;
    QGroupBox *threeDPlotGB = nullptr;
    QComboBox *pointSync = nullptr;

    QCheckBox *CBlvm[2];
    QTabWidget *Scanner_type;

    TwoStateButton *fb_on_off_xy;
    TwoStateButton *fb_on_off_z;
    QCheckBox *nl_on_off_cl;
    QCheckBox *nl_on_off_ol;
    QCheckBox *nl_on_off_cl2;
    QCheckBox *nl_on_off_ol2;

    QVBoxLayout *stageLayout;
    QWidget *stageCalibWid;
    QWidget *headCalibWid;
    QGridLayout *layout_config2;
    QTableWidget *calibTable[2];
    QGroupBox *calibVoltageGB[2];
    QVBoxLayout *vertCalibLay;
    QGroupBox *GrBCalibrations;

    bool headConnected;
    bool stageConnected;
    volatile bool scannersConnectedUpdated;

    //
    TMessageFlag doSendCalData, doWriteConfigToDeviceMem, doSendCSBSpeed;
    TMessageFlag sendStartup;

    BYTE SensorOFon;
    BYTE updateSensorOF;
    void setSensorOverflowMode(BYTE scanner, BYTE mode);

    TMessageFlag updLVM;
    BYTE lvm_bits;
    void setLVM(BYTE head, BYTE stage);

    int csb_speed_idx;
    bool csb_speed_extended;
    double get_csb_speed(); //hz

    //-----------------------------------------------------------------
    bool Nlc_status_cl_xy;
    bool Nlc2_status_cl_xy;
    bool Nlc_status_cl_z;
    bool Nlc2_status_cl_z;

    bool Nlc_status_ol_xy;
    bool Nlc2_status_ol_xy;
    bool Nlc_status_ol_z;
    bool Nlc2_status_ol_z;

    bool CL_status_xy;
    bool CL2_status_xy;
    bool CL_status_z;
    bool CL2_status_z;
    bool CL3_status_z;

    float free_move_speed;
    DoubleEdit *lineFreeSpeed;

    //nlc edit
    DoubleEdit *lineNLCEdit[6];
    NLCWidget *viewNLCEdit;
    QWidget *NLCEditWid;
    QLabel *NLCEditValid;
    bool checkNLCform(double calib[6]);

    //reverse nlc - linear if not enabled
    float reverse_nlc_sensor(int axis, float c); //c is [0..65535], returns mkm
    float reverse_nlc_voltage(int axis, float v); //v is [0..65535], returns mkm
    float CalibRangeCL[2][3];
    float ShiftCL[2][3];
    bool nlc_valid_cl;
    float CalibRangeOL[2][3];
    float ShiftOL[2][3];
    bool nlc_valid_ol;
    void init_nlc_sensor(); //private
    void init_nlc_voltage(); //private

    //xyz1
    DoubleEdit *linePID[3][3];

    DoubleEdit *lineScannerRangeOL[3];
    DoubleEdit *lineScannerRangeCL[3];
    DoubleEdit *lineSensorScale[3]; //todo rename those
    DoubleEdit *lineSensorScale0[3]; //shift

    double linePiezoCalibration[3][6];
    double lineSensorCalibration[3][6];
    NLCWidget *viewPiezoCalibration[3];
    NLCWidget *viewSensorCalibration[3];

    float Scanner_data_pid_cal_range[48]; //don't change order of fields!! it's same as in EG_CalData
    DWORD Scanner_data_scale[3];
    long Scanner_data_Shift[3];

    //xyz2
    DoubleEdit *linePID2[3][3];

    DoubleEdit *lineScannerRangeOL2[3];
    DoubleEdit *lineScannerRangeCL2[3];
    DoubleEdit *lineSensorScale2[3];
    DoubleEdit *lineSensorScale02[3]; //shift

    double linePiezoCalibration2[3][6];
    double lineSensorCalibration2[3][6];
    NLCWidget *viewPiezoCalibration2[3];
    NLCWidget *viewSensorCalibration2[3];

    float Scanner2_data_pid_cal_range[48]; //don't change order of fields!! it's same as in EG_CalData
    DWORD Scanner2_data_scale[3];
    long Scanner2_data_Shift[3];

    //Z3
    DoubleEdit *linePIDProportionalZ3;
    DoubleEdit *linePIDIntegralZ3;

    DoubleEdit *lineScannerRangeOLZ3;
    DoubleEdit *lineScannerRangeCLZ3;
    DoubleEdit *lineSensorScaleZ3;
    DoubleEdit *lineSensorScaleZ03;

    float Scanner3_data_pid_cal_range[48]; //don't change order of fields!! it's same as in EG_CalData
    DWORD Scanner3_data_scale[3];
    long Scanner3_data_Shift[3];
};


QVector<double> buildNlcPic(double k[6]);

class NLCWidget: public QFrame
{
    Q_OBJECT
signals:
    void clicked();
public:
    QVector<double> m_pic; //0..1 range
    void SetPic(const QVector<double> &c) { m_pic = c; update(); }

    int heightForWidth(int w) const override { return w*0.66; }
    void paintEvent(QPaintEvent *) override;
    NLCWidget() {
        setStyleSheet("background-color: rgb(230, 255, 230); border: 1px solid #0000DF; border-radius: 3px;");

        setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Minimum);
        QSizePolicy p = sizePolicy();
        p.setHeightForWidth(true);
        setSizePolicy(p);
        setMinimumSize(10, 13);
    }
    void mousePressEvent(QMouseEvent *) override { emit clicked(); }
};
